'use strict';
var context = require('context');
var router = require('router');
const PROGRAM = 'http';

function Extension() {
	this.robots = {};
}

Extension.prototype.getComponents = function() {
	if(!this.components) {
		this.components = context.getComponents(PROGRAM);
	}
	return this.components;
};

Extension.prototype.getRobot = function(route) {
	var key = router.multi ? route.module + route.index : route.module;
	var robot = this.robots[key];
	if(!robot) {
		var component = context.getComponentByRoute(PROGRAM, route);
		robot = component.createRobot(route.state);
		if(robot) {
			this.robots[key] = robot;
		}
	}
	return robot;
};

Extension.prototype.start = function(express) {
	if(express) {
		var components = this.getComponents();
		var robots = this.robots;
		for(var i in components) {
			components[i].start(express, robots);
		}
		express.get('*', function(request, response) {
			response.writeHead(200, { 'Content-Type': 'text/plain' });
			response.write('error');
			response.end();
		});
	}
};

Extension.prototype.stop = function() {
	router.reset();
	this.robots = {};
};

Extension.prototype.setState = function(route) {
	if(router.multi || route == router.selectedRoute) {
		var robot = this.getRobot(route);
		if(robot) {
			var component = context.getComponentByRoute(PROGRAM, route);
			component.setState(robot, route.state);
		}
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleSensory = function(data, route) {
	var robot = this.getRobot(route);
	if(robot) {
		var component = context.getComponentByRoute(PROGRAM, route);
		component.handleSensory(robot, data, route.state);
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	var robot = this.getRobot(route);
	if(robot) {
		var roboid = robot.roboid;
		roboid.handleMotoring(data);
		roboid.updateMotoring();
	}
};

Extension.prototype.requestMotoring = function(route) {
	var robot = this.getRobot(route);
	if(robot) {
		return robot.roboid.requestMotoring();
	}
};

Extension.prototype.reset = function(route) {
	if(router.multi || route == router.selectedRoute) {
		var robot = this.getRobot(route);
		if(robot) {
			robot.roboid.reset();
		}
	}
};

module.exports.create = function() {
	return new Extension();
};