'use strict';
var context = require('context');
var router = require('router');
const PROGRAM = 'playbot';

function Extension() {
	this.robots = {};
	this.connectionState = {
		type: 0,
		state: 0
	};
}

Extension.prototype.getRobotByModule = function(module) {
	var robot = this.robots[module];
	if(!robot) {
		var component = context.getComponentByModule(PROGRAM, module);
		robot = component.createRobot();
		if(robot) {
			this.robots[module] = robot;
		}
	}
	return robot;
};

Extension.prototype.getRobotByRoute = function(route) {
	return this.getRobotByModule(route.module);
};

Extension.prototype.start = function(connection) {
	var self = this;
	self.connection = connection;
	connection.on('message', function(message) {
		if(message.type === 'utf8') {
			self.handleRemoteData(message.utf8Data);
		}
	});
};

Extension.prototype.stop = function() {
	this.connection = undefined;
	router.reset();
	this.robots = {};
};

Extension.prototype.setState = function(route) {
	if(route == router.selectedRoute && this.connection) {
		this.connectionState.module = route.module;
		this.connectionState.state = router.getConnectionState(route.state);
		try {
			this.connection.send(JSON.stringify(this.connectionState));
		} catch (e) {
		}
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleRemoteData = function(utf8Data) {
	var selectedRoute = router.selectedRoute;
	if(selectedRoute) {
		try {
			var data = JSON.parse(utf8Data);
			if(data.module == selectedRoute.module) {
				var robot = this.getRobotByModule(data.module);
				if(robot) {
					var component = context.getComponentByModule(PROGRAM, data.module);
					component.handleRemoteData(robot, data);
				}
			}
		} catch (e) {
		}
	}
};

Extension.prototype.handleSensory = function(data, route) {
	var robot = this.getRobotByRoute(route);
	if(robot) {
		var component = context.getComponentByRoute(PROGRAM, route);
		component.handleSensory(robot, data, this.connection);
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	var robot = this.getRobotByRoute(route);
	if(robot) {
		var roboid = robot.roboid;
		roboid.handleMotoring(data);
		roboid.updateMotoring();
	}
};

Extension.prototype.requestMotoring = function(route) {
	var robot = this.getRobotByRoute(route);
	if(robot) {
		return robot.roboid.requestMotoring();
	}
};

Extension.prototype.reset = function(route) {
	if(route == router.selectedRoute) {
		var robot = this.getRobotByRoute(route);
		if(robot) {
			robot.roboid.reset();
		}
	}
};

module.exports.create = function() {
	return new Extension();
};