'use strict';
var context = require('context');
var router = require('router');
const PROGRAM = 'scratchx';

function Extension() {
	this.robots = {};
	this.group2module = {};
	this.connectionPacket = {
		type: 0,
		state: 0
	};
	this.masterVersion = 2;
}

Extension.prototype.getRobotByModule = function(module) {
	var robot = this.robots[module];
	if(!robot) {
		var component = context.getComponentByModule(PROGRAM, module);
		robot = component.createRobot(0);
		if(robot) {
			this.robots[module] = robot;
		}
	}
	return robot;
};

Extension.prototype.getRobotByRoute = function(route) {
	this.group2module[route.group] = route.module;
	return this.getRobotByModule(route.module);
};

Extension.prototype.start = function(connection) {
	var self = this;
	self.connection = connection;
	connection.on('message', function(message) {
		if(message.type === 'utf8') {
			self.handleRemoteData(message.utf8Data);
		}
	});
};

Extension.prototype.stop = function() {
	this.connection = undefined;
	router.reset();
	this.robots = {};
	this.group2module = {};
};

Extension.prototype.setState = function(route) {
	if(route == router.firstRoute && this.connection) {
		this.connectionPacket.module = route.group; // for compatibility
		this.connectionPacket.state = router.getConnectionState(route.state);
		try {
			this.connection.send(JSON.stringify(this.connectionPacket));
		} catch (e) {
		}
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleRemoteDataByModuleOld = function(data) {
	var module = data.module;
	if(module) {
		if(module == 'hamster') {
			if(this.group2module['hamster'] == 'hamsterS') {
				module = 'hamsterS';
				var robot = this.getRobotByModule(module);
				if(robot) {
					var component = context.getComponentByModule(PROGRAM, module);
					component.handleRemoteDataHamster(robot, data);
				}
			} else {
				var robot = this.getRobotByModule(module);
				if(robot) {
					var component = context.getComponentByModule(PROGRAM, module);
					component.handleRemoteData(robot, data);
				}
			}
		} else {
			var robot = this.getRobotByModule(module);
			if(robot) {
				var component = context.getComponentByModule(PROGRAM, module);
				component.handleRemoteData(robot, data);
			}
		}
	}
};

Extension.prototype.handleRemoteDataByModule = function(data) {
	var module = data.module;
	if(module) {
		var robot = this.getRobotByModule(module);
		if(robot) {
			var component = context.getComponentByModule(PROGRAM, module);
			component.handleRemoteData(robot, data);
		}
	}
};

Extension.prototype.handleRemoteData = function(utf8Data) {
	try {
		var received = JSON.parse(utf8Data);
		var version = received.version || 0;
		if(version == 2) {
			for(var i in received) {
				this.handleRemoteDataByModule(received[i]);
			}
		} else {
			for(var i in received) {
				this.handleRemoteDataByModuleOld(received[i]);
			}
		}
		this.masterVersion = version;
	} catch (e) {
	}
};

Extension.prototype.handleSensory = function(data, route) {
	var robot = this.getRobotByRoute(route);
	if(robot) {
		var component = context.getComponentByRoute(PROGRAM, route);
		var packet = {};
		component.handleSensory(packet, robot, data);
		if(this.masterVersion == 0) {
			packet = packet.robot;
		}
		packet.type = 1;
		packet.version = 1;
		if(this.connection) {
			try {
				this.connection.send(JSON.stringify(packet));
			} catch (e) {
			}
		}
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	var robot = this.getRobotByRoute(route);
	if(robot) {
		var roboid = robot.roboid;
		roboid.handleMotoring(data);
		roboid.updateMotoring();
	}
};

Extension.prototype.requestMotoring = function(route) {
	var robot = this.getRobotByRoute(route);
	if(robot) {
		return robot.roboid.requestMotoring();
	}
};

Extension.prototype.reset = function(route) {
	var robot = this.getRobotByRoute(route);
	if(robot) {
		robot.roboid.reset();
	}
};

module.exports.create = function() {
	return new Extension();
};