'use strict';
var Cheese = require('./cheese');

function CheeseRoboid(parent) {
	var io = require('roboid.io');
	var Device = io.Device;
	var DeviceType = io.DeviceType;
	var DataType = io.DataType;
	
	var sensory = {
		devices: [],
		readData: [],
		callbacks: []
	};
	var motoring = {
		map1: 0,
		map2: 0,
		devices: [],
		readData: [],
		writeData: [],
		callbacks: []
	};
	var writeMotoringFlag1 = 0;
	var writeMotoringFlag2 = 0;
	
	this.bind = function(device, type, data, callback) {
		if(type == DeviceType.SENSOR || type == DeviceType.EVENT) {
			sensory.devices.push(device);
			sensory.readData.push(data.read);
			sensory.callbacks.push(callback);
		} else {
			motoring.devices.push(device);
			motoring.readData.push(data.read);
			motoring.writeData.push(data.write);
			motoring.callbacks.push(callback);
		}
	};
	
	var devices = {};
	devices[Cheese.OUTPUT_SA] = new Device(this, Cheese.OUTPUT_SA, 'OutputSA', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.OUTPUT_SB] = new Device(this, Cheese.OUTPUT_SB, 'OutputSB', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.OUTPUT_SC] = new Device(this, Cheese.OUTPUT_SC, 'OutputSC', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.OUTPUT_LA] = new Device(this, Cheese.OUTPUT_LA, 'OutputLA', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.OUTPUT_LB] = new Device(this, Cheese.OUTPUT_LB, 'OutputLB', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.OUTPUT_LC] = new Device(this, Cheese.OUTPUT_LC, 'OutputLC', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.OUTPUT_LX] = new Device(this, Cheese.OUTPUT_LX, 'OutputLX', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.OUTPUT_MAB] = new Device(this, Cheese.OUTPUT_MAB, 'OutputMAB', DeviceType.EFFECTOR, DataType.INTEGER, 1, -100, 255, 0);
	devices[Cheese.OUTPUT_MCD] = new Device(this, Cheese.OUTPUT_MCD, 'OutputMCD', DeviceType.EFFECTOR, DataType.INTEGER, 1, -100, 255, 0);
	devices[Cheese.BUZZER] = new Device(this, Cheese.BUZZER, 'Buzzer', DeviceType.EFFECTOR, DataType.FLOAT, 1, 0, 6500.0, 0);
	devices[Cheese.VELOCITY] = new Device(this, Cheese.VELOCITY, 'Velocity', DeviceType.EFFECTOR, DataType.INTEGER, 1, -32768, 32767, 0);
	devices[Cheese.STEP] = new Device(this, Cheese.STEP, 'Step', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 65535, 0);
	devices[Cheese.NOTE] = new Device(this, Cheese.NOTE, 'Note', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 88, 0);
	devices[Cheese.SOUND] = new Device(this, Cheese.SOUND, 'Sound', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 127, 0);
	devices[Cheese.MODE_SOUND] = new Device(this, Cheese.MODE_SOUND, 'ModeSound', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.MODE_SA] = new Device(this, Cheese.MODE_SA, 'ModeSA', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.MODE_SB] = new Device(this, Cheese.MODE_SB, 'ModeSB', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.MODE_SC] = new Device(this, Cheese.MODE_SC, 'ModeSC', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.MODE_LA] = new Device(this, Cheese.MODE_LA, 'ModeLA', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.MODE_LB] = new Device(this, Cheese.MODE_LB, 'ModeLB', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.MODE_LC] = new Device(this, Cheese.MODE_LC, 'ModeLC', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.MODE_M] = new Device(this, Cheese.MODE_M, 'ModeM', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.RED_LED] = new Device(this, Cheese.RED_LED, 'RedLed', DeviceType.COMMAND, DataType.INTEGER, 1, -1, 1, -1);
	devices[Cheese.BLUE_LED] = new Device(this, Cheese.BLUE_LED, 'BlueLed', DeviceType.COMMAND, DataType.INTEGER, 1, -1, 1, -1);
	devices[Cheese.HAT_OA] = new Device(this, Cheese.HAT_OA, 'HatOA', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 0);
	devices[Cheese.HAT_OB] = new Device(this, Cheese.HAT_OB, 'HatOB', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 0);
	devices[Cheese.RESET_ENCODER] = new Device(this, Cheese.RESET_ENCODER, 'ResetEncoder', DeviceType.COMMAND, DataType.INTEGER, 0, 0, 0, 0);
	devices[Cheese.WRITE_I2C] = new Device(this, Cheese.WRITE_I2C, 'WriteI2c', DeviceType.COMMAND, DataType.INTEGER, 20, 0, 255, 0);
	devices[Cheese.WRITE_SERIAL] = new Device(this, Cheese.WRITE_SERIAL, 'WriteSerial', DeviceType.COMMAND, DataType.INTEGER, 19, 0, 255, 0);
	devices[Cheese.NEOPIXEL] = new Device(this, Cheese.NEOPIXEL, 'Neopixel', DeviceType.COMMAND, DataType.INTEGER, 20, 0, 255, 0);
	devices[Cheese.CONFIG_ADC_SA] = new Device(this, Cheese.CONFIG_ADC_SA, 'ConfigAdcSA', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 1);
	devices[Cheese.CONFIG_ADC_SB] = new Device(this, Cheese.CONFIG_ADC_SB, 'ConfigAdcSB', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 1);
	devices[Cheese.CONFIG_ADC_SC] = new Device(this, Cheese.CONFIG_ADC_SC, 'ConfigAdcSC', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 1);
	devices[Cheese.CONFIG_ADC_LA] = new Device(this, Cheese.CONFIG_ADC_LA, 'ConfigAdcLA', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 1);
	devices[Cheese.CONFIG_ADC_LB] = new Device(this, Cheese.CONFIG_ADC_LB, 'ConfigAdcLB', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 1);
	devices[Cheese.CONFIG_ADC_LC] = new Device(this, Cheese.CONFIG_ADC_LC, 'ConfigAdcLC', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 1);
	devices[Cheese.CONFIG_GRAVITY] = new Device(this, Cheese.CONFIG_GRAVITY, 'ConfigGravity', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 3, 0);
	devices[Cheese.CONFIG_BAND_WIDTH] = new Device(this, Cheese.CONFIG_BAND_WIDTH, 'ConfigBandWidth', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 7, 0);
	devices[Cheese.SIGNAL_STRENGTH] = new Device(this, Cheese.SIGNAL_STRENGTH, 'SignalStrength', DeviceType.SENSOR, DataType.INTEGER, 1, -128, 0, 0);
	devices[Cheese.ACCELERATION] = new Device(this, Cheese.ACCELERATION, 'Acceleration', DeviceType.SENSOR, DataType.INTEGER, 3, -32768, 32767, 0);
	devices[Cheese.TEMPERATURE] = new Device(this, Cheese.TEMPERATURE, 'Temperature', DeviceType.SENSOR, DataType.INTEGER, 1, -40, 88, 0);
	devices[Cheese.INPUT_SA] = new Device(this, Cheese.INPUT_SA, 'InputSA', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.INPUT_SB] = new Device(this, Cheese.INPUT_SB, 'InputSB', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.INPUT_SC] = new Device(this, Cheese.INPUT_SC, 'InputSC', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Cheese.INPUT_LA] = new Device(this, Cheese.INPUT_LA, 'InputLA', DeviceType.SENSOR, DataType.INTEGER, 1, -65536, 65535, 0);
	devices[Cheese.INPUT_LB] = new Device(this, Cheese.INPUT_LB, 'InputLB', DeviceType.SENSOR, DataType.INTEGER, 1, -65536, 65535, 0);
	devices[Cheese.INPUT_LC] = new Device(this, Cheese.INPUT_LC, 'InputLC', DeviceType.SENSOR, DataType.INTEGER, 1, -65536, 65535, 0);
	devices[Cheese.HAT_IA] = new Device(this, Cheese.HAT_IA, 'HatIA', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[Cheese.HAT_IB] = new Device(this, Cheese.HAT_IB, 'HatIB', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[Cheese.FREE_FALL] = new Device(this, Cheese.FREE_FALL, 'FreeFall', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[Cheese.TAP] = new Device(this, Cheese.TAP, 'Tap', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[Cheese.READ_I2C] = new Device(this, Cheese.READ_I2C, 'ReadI2c', DeviceType.EVENT, DataType.INTEGER, 20, 0, 255, 0);
	devices[Cheese.READ_SERIAL] = new Device(this, Cheese.READ_SERIAL, 'ReadSerial', DeviceType.EVENT, DataType.INTEGER, 19, 0, 255, 0);
	devices[Cheese.STEP_COUNT] = new Device(this, Cheese.STEP_COUNT, 'StepCount', DeviceType.EVENT, DataType.INTEGER, 1, 0, 65535, 0);
	devices[Cheese.STEP_STATE] = new Device(this, Cheese.STEP_STATE, 'StepState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[Cheese.SOUND_STATE] = new Device(this, Cheese.SOUND_STATE, 'SoundState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[Cheese.BATTERY_STATE] = new Device(this, Cheese.BATTERY_STATE, 'BatteryState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 2, 2);
	devices[Cheese.I2C_STATE] = new Device(this, Cheese.I2C_STATE, 'I2cState', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[Cheese.SERIAL_STATE] = new Device(this, Cheese.SERIAL_STATE, 'SerialState', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[Cheese.NEOPIXEL_STATE] = new Device(this, Cheese.NEOPIXEL_STATE, 'NeopixelState', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	
	this.findDeviceByName = function(name) {
		name = '' + name;
		name = name.toLowerCase();
		var dot = name.search('\\.');
		if(dot < 0) {
			for(var i in devices) {
				if(name == devices[i].getName().toLowerCase())
					return devices[i];
			}
		}
		return undefined;
	};
	
	this.findDeviceById = function(deviceId) {
		var device = devices[deviceId];
		if(device) {
			return device;
		}
		return undefined;
	};
	
	this.addDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].addDeviceDataChangedListener(listener);
		}
	};
	
	this.removeDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].removeDeviceDataChangedListener(listener);
		}
	};
	
	this.clearDeviceDataChangedListeners = function() {
		for(var i in devices) {
			devices[i].clearDeviceDataChangedListeners();
		}
	};
	
	this.onMotoringDeviceWritten = function(device) {
		switch(device.getId()) {
			case Cheese.STEP:
				writeMotoringFlag1 |= 0x00080000;
				break;
			case Cheese.NOTE:
				writeMotoringFlag1 |= 0x00040000;
				break;
			case Cheese.SOUND:
				writeMotoringFlag1 |= 0x00020000;
				break;
			case Cheese.MODE_SOUND:
				writeMotoringFlag1 |= 0x00010000;
				break;
			case Cheese.MODE_SA:
				writeMotoringFlag1 |= 0x00008000;
				break;
			case Cheese.MODE_SB:
				writeMotoringFlag1 |= 0x00004000;
				break;
			case Cheese.MODE_SC:
				writeMotoringFlag1 |= 0x00002000;
				break;
			case Cheese.MODE_LA:
				writeMotoringFlag1 |= 0x00001000;
				break;
			case Cheese.MODE_LB:
				writeMotoringFlag1 |= 0x00000800;
				break;
			case Cheese.MODE_LC:
				writeMotoringFlag1 |= 0x00000400;
				break;
			case Cheese.MODE_M:
				writeMotoringFlag1 |= 0x00000200;
				break;
			case Cheese.RED_LED:
				writeMotoringFlag1 |= 0x00000100;
				break;
			case Cheese.BLUE_LED:
				writeMotoringFlag1 |= 0x00000080;
				break;
			case Cheese.HAT_OA:
				writeMotoringFlag1 |= 0x00000040;
				break;
			case Cheese.HAT_OB:
				writeMotoringFlag1 |= 0x00000020;
				break;
			case Cheese.RESET_ENCODER:
				writeMotoringFlag1 |= 0x00000010;
				break;
			case Cheese.WRITE_I2C:
				writeMotoringFlag1 |= 0x00000008;
				break;
			case Cheese.WRITE_SERIAL:
				writeMotoringFlag1 |= 0x00000004;
				break;
			case Cheese.NEOPIXEL:
				writeMotoringFlag1 |= 0x00000002;
				break;
			case Cheese.CONFIG_ADC_SA:
				writeMotoringFlag1 |= 0x00000001;
				break;
			case Cheese.CONFIG_ADC_SB:
				writeMotoringFlag2 |= 0x80000000;
				break;
			case Cheese.CONFIG_ADC_SC:
				writeMotoringFlag2 |= 0x40000000;
				break;
			case Cheese.CONFIG_ADC_LA:
				writeMotoringFlag2 |= 0x20000000;
				break;
			case Cheese.CONFIG_ADC_LB:
				writeMotoringFlag2 |= 0x10000000;
				break;
			case Cheese.CONFIG_ADC_LC:
				writeMotoringFlag2 |= 0x08000000;
				break;
			case Cheese.CONFIG_GRAVITY:
				writeMotoringFlag2 |= 0x04000000;
				break;
			case Cheese.CONFIG_BAND_WIDTH:
				writeMotoringFlag2 |= 0x02000000;
				break;
		}
	};
	
	this.handleSensory = function(data) {
		var map = data.map;
		var readData = sensory.readData;
		readData[0][0] = data.signalStrength;
		readData[1][0] = data.acceleration.x;
		readData[1][1] = data.acceleration.y;
		readData[1][2] = data.acceleration.z;
		readData[2][0] = data.internalTemperature;
		readData[3][0] = data.inputSA;
		readData[4][0] = data.inputSB;
		readData[5][0] = data.inputSC;
		readData[6][0] = data.inputLA;
		readData[7][0] = data.inputLB;
		readData[8][0] = data.inputLC;
		
		var callbacks = sensory.callbacks;
		var i;
		for(i = 0; i < 9; ++i) {
			callbacks[i].fire();
		}
		
		if(map & 0x00200000) {
			readData[9][0] = data.hatA;
			callbacks[9].fire();
		}
		if(map & 0x00100000) {
			readData[10][0] = data.hatB;
			callbacks[10].fire();
		}
		if(map & 0x00080000) {
			callbacks[11].fire(); // free fall
		}
		if(map & 0x00040000) {
			callbacks[12].fire(); // tap
		}
		if(map & 0x00020000) {
			var r = readData[13];
			var i2c = data.i2c;
			if(i2c) {
				for(var i = 0; i < 20; ++i) {
					r[i] = i2c[i];
				}
				callbacks[13].fire();
			}
		}
		if(map & 0x00010000) {
			var r = readData[14];
			var serial = data.serial;
			if(serial) {
				for(var i = 0; i < 19; ++i) {
					r[i] = serial[i];
				}
				callbacks[14].fire();
			}
		}
		if(map & 0x00008000) {
			readData[15][0] = data.stepCount;
			callbacks[15].fire();
		}
		if(map & 0x00004000) {
			readData[16][0] = data.stepState;
			callbacks[16].fire();
		}
		if(map & 0x00002000) {
			readData[17][0] = data.soundState;
			callbacks[17].fire();
		}
		if(map & 0x00001000) {
			readData[18][0] = data.batteryState;
			callbacks[18].fire();
		}
		if(map & 0x00000800) { // i2c state
			callbacks[19].fire();
		}
		if(map & 0x00000400) { // serial state
			callbacks[20].fire();
		}
		if(map & 0x00000200) { // neopixel state
			callbacks[21].fire();
		}
		
		for(i = 0; i < 9; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map & 0x00200000) {
			callbacks[9].notifyDeviceDataChanged();
		}
		if(map & 0x00100000) {
			callbacks[10].notifyDeviceDataChanged();
		}
		if(map & 0x00080000) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map & 0x00040000) {
			callbacks[12].notifyDeviceDataChanged();
		}
		if(map & 0x00020000) {
			callbacks[13].notifyDeviceDataChanged();
		}
		if(map & 0x00010000) {
			callbacks[14].notifyDeviceDataChanged();
		}
		if(map & 0x00008000) {
			callbacks[15].notifyDeviceDataChanged();
		}
		if(map & 0x00004000) {
			callbacks[16].notifyDeviceDataChanged();
		}
		if(map & 0x00002000) {
			callbacks[17].notifyDeviceDataChanged();
		}
		if(map & 0x00001000) {
			callbacks[18].notifyDeviceDataChanged();
		}
		if(map & 0x00000800) {
			callbacks[19].notifyDeviceDataChanged();
		}
		if(map & 0x00000400) {
			callbacks[20].notifyDeviceDataChanged();
		}
		if(map & 0x00000200) {
			callbacks[21].notifyDeviceDataChanged();
		}
	};
	
	this.handleMotoring = function(data) {
		var map1 = data.map1;
		var map2 = data.map2;
		var readData = motoring.readData;
		readData[0][0] = data.outputSA;
		readData[1][0] = data.outputSB;
		readData[2][0] = data.outputSC;
		readData[3][0] = data.outputLA;
		readData[4][0] = data.outputLB;
		readData[5][0] = data.outputLC;
		readData[6][0] = data.outputLX;
		readData[7][0] = data.outputMAB;
		readData[8][0] = data.outputMCD;
		readData[9][0] = data.buzzer;
		readData[10][0] = data.velocity;
		
		var callbacks = motoring.callbacks;
		var i;
		for(i = 0; i < 11; ++i) {
			callbacks[i].fire();
		}
		
		if(map1 & 0x00080000) {
			readData[11][0] = data.step;
			callbacks[11].fire();
		}
		if(map1 & 0x00040000) {
			readData[12][0] = data.note;
			callbacks[12].fire();
		}
		if(map1 & 0x00020000) {
			readData[13][0] = data.sound;
			callbacks[13].fire();
		}
		if(map1 & 0x00010000) {
			readData[14][0] = data.modeSound;
			callbacks[14].fire();
		}
		if(map1 & 0x00008000) {
			readData[15][0] = data.modeSA;
			callbacks[15].fire();
		}
		if(map1 & 0x00004000) {
			readData[16][0] = data.modeSB;
			callbacks[16].fire();
		}
		if(map1 & 0x00002000) {
			readData[17][0] = data.modeSC;
			callbacks[17].fire();
		}
		if(map1 & 0x00001000) {
			readData[18][0] = data.modeLA;
			callbacks[18].fire();
		}
		if(map1 & 0x00000800) {
			readData[19][0] = data.modeLB;
			callbacks[19].fire();
		}
		if(map1 & 0x00000400) {
			readData[20][0] = data.modeLC;
			callbacks[20].fire();
		}
		if(map1 & 0x00000200) {
			readData[21][0] = data.modeM;
			callbacks[21].fire();
		}
		if(map1 & 0x00000100) {
			readData[22][0] = data.redLed;
			callbacks[22].fire();
		}
		if(map1 & 0x00000080) {
			readData[23][0] = data.blueLed;
			callbacks[23].fire();
		}
		if(map1 & 0x00000040) {
			readData[24][0] = data.hatA;
			callbacks[24].fire();
		}
		if(map1 & 0x00000020) {
			readData[25][0] = data.hatB;
			callbacks[25].fire();
		}
		if(map1 & 0x00000010) { // reset encoder
			callbacks[26].fire();
		}
		if(map1 & 0x00000008) {
			var r = readData[27];
			var i2c = data.i2c;
			if(i2c) {
				for(var i = 0; i < 20; ++i) {
					r[i] = i2c[i];
				}
				callbacks[27].fire();
			}
		}
		if(map1 & 0x00000004) {
			var r = readData[28];
			var serial = data.serial;
			if(serial) {
				for(var i = 0; i < 19; ++i) {
					r[i] = serial[i];
				}
				callbacks[28].fire();
			}
		}
		if(map1 & 0x00000002) {
			var r = readData[29];
			var neopixel = data.neopixel;
			if(neopixel) {
				for(var i = 0; i < 20; ++i) {
					r[i] = neopixel[i];
				}
				callbacks[29].fire();
			}
		}
		if(map1 & 0x00000001) {
			readData[30][0] = data.configAdcSA;
			callbacks[30].fire();
		}
		if(map2 & 0x80000000) {
			readData[31][0] = data.configAdcSB;
			callbacks[31].fire();
		}
		if(map2 & 0x40000000) {
			readData[32][0] = data.configAdcSC;
			callbacks[32].fire();
		}
		if(map2 & 0x20000000) {
			readData[33][0] = data.configAdcLA;
			callbacks[33].fire();
		}
		if(map2 & 0x10000000) {
			readData[34][0] = data.configAdcLB;
			callbacks[34].fire();
		}
		if(map2 & 0x08000000) {
			readData[35][0] = data.configAdcLC;
			callbacks[35].fire();
		}
		if(map2 & 0x04000000) {
			readData[36][0] = data.configGravity;
			callbacks[36].fire();
		}
		if(map2 & 0x02000000) {
			readData[37][0] = data.configBandWidth;
			callbacks[37].fire();
		}
		
		for(i = 0; i < 11; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map1 & 0x00080000) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map1 & 0x00040000) {
			callbacks[12].notifyDeviceDataChanged();
		}
		if(map1 & 0x00020000) {
			callbacks[13].notifyDeviceDataChanged();
		}
		if(map1 & 0x00010000) {
			callbacks[14].notifyDeviceDataChanged();
		}
		if(map1 & 0x00008000) {
			callbacks[15].notifyDeviceDataChanged();
		}
		if(map1 & 0x00004000) {
			callbacks[16].notifyDeviceDataChanged();
		}
		if(map1 & 0x00002000) {
			callbacks[17].notifyDeviceDataChanged();
		}
		if(map1 & 0x00001000) {
			callbacks[18].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000800) {
			callbacks[19].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000400) {
			callbacks[20].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000200) {
			callbacks[21].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000100) {
			callbacks[22].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000080) {
			callbacks[23].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000040) {
			callbacks[24].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000020) {
			callbacks[25].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000010) {
			callbacks[26].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000008) {
			callbacks[27].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000004) {
			callbacks[28].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000002) {
			callbacks[29].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000001) {
			callbacks[30].notifyDeviceDataChanged();
		}
		if(map2 & 0x80000000) {
			callbacks[31].notifyDeviceDataChanged();
		}
		if(map2 & 0x40000000) {
			callbacks[32].notifyDeviceDataChanged();
		}
		if(map2 & 0x20000000) {
			callbacks[33].notifyDeviceDataChanged();
		}
		if(map2 & 0x10000000) {
			callbacks[34].notifyDeviceDataChanged();
		}
		if(map2 & 0x08000000) {
			callbacks[35].notifyDeviceDataChanged();
		}
		if(map2 & 0x04000000) {
			callbacks[36].notifyDeviceDataChanged();
		}
		if(map2 & 0x02000000) {
			callbacks[37].notifyDeviceDataChanged();
		}
	};
	
	this.requestMotoring = function() {
		motoring.map1 = 0xfff00000 | writeMotoringFlag1;
		motoring.map2 = writeMotoringFlag2;
		writeMotoringFlag1 = 0;
		writeMotoringFlag2 = 0;
		return motoring;
	};
	
	this.updateSensory = function() {
		var callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.updateMotoring = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.reset = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
		callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
	};
}

CheeseRoboid.prototype.getId = function() {
	return Cheese.ID;
};

CheeseRoboid.prototype.getName = function() {
	return 'Cheese';
};

module.exports.create = function(parent) {
	return new CheeseRoboid(parent);
};