'use strict';
var Cheese = require('./cheese');

const PACKET_NORMAL = 1;
const PACKET_I2C = 2;
const PACKET_SERIAL = 3;
const PACKET_NEOPIXEL = 4;

const I2C_LED_MATRIX = 1;

function Controller() {
	this.sensory = {
		map: 0,
		signalStrength: 0,
		acceleration: { x: 0, y: 0, z: 0 },
		internalTemperature: 0,
		inputSA: 0,
		inputSB: 0,
		inputSC: 0,
		inputLA: 0,
		inputLB: 0,
		inputLC: 0,
		ultrasonic: 0,
		encoder: 0,
		temperature: 0,
		humidity: 0,
		hatA: 0,
		hatB: 0,
		i2c: undefined,
		serial: undefined,
		stepCount: 0,
		stepState: 0,
		soundState: 0,
		batteryState: 2
	};
	this.motoring = {
		map1: 0,
		map2: 0,
		outputSA: 0,
		outputSB: 0,
		outputSC: 0,
		outputLA: 0,
		outputLB: 0,
		outputLC: 0,
		outputLX: 0,
		outputMAB: 0,
		outputMCD: 0,
		buzzer: 0,
		velocity: 0,
		step: 0,
		note: 0,
		sound: 0,
		modeSound: 0,
		modeSA: 0,
		modeSB: 0,
		modeSC: 0,
		modeLA: 0,
		modeLB: 0,
		modeLC: 0,
		modeM: 0,
		redLed: -1,
		blueLed: -1,
		hatA: 0,
		hatB: 0,
		i2c: undefined,
		serial: undefined,
		neopixel: undefined,
		configAdcSA: 1,
		configAdcSB: 1,
		configAdcSC: 1,
		configAdcLA: 1,
		configAdcLB: 1,
		configAdcLC: 1,
		configGravity: 0,
		configBandWidth: 0
	};
	this.acceleration = {
		x: new Array(10),
		y: new Array(10),
		z: new Array(10),
		sumx: 0.0,
		sumy: 0.0,
		sumz: 0.0,
		index: 0,
		count: 0
	};
	this.step = {
		id: 0,
		prev: -1,
		event: 0,
		state: 0,
		count: 0
	};
	this.sound = {
		flag: 0,
		event: 0,
		state: 0,
		count: 0
	};
	this.event = {
		hatA: -1,
		hatB: -1,
		freeFallId: -1,
		tapId: -1,
		serialId: -1,
		stepCount: -1,
		batteryState: -1
	};
	this.command = {
		serialId: 0,
		encoderId: 0
	};
	this.portL = {
		ackId: -1,
		serial: false,
		neopixel: false
	};
	this.i2c = {
		sendId: 0,
		sendPrevId: 0,
		activated: 0
	};
	this.serial = {
		sendId: 0,
		sendPrevId: 0
	};
	this.neopixel = {
		sendId: 0,
		sendPrevId: 0,
		activated: false
	};
	this.packetSent = 0;
	this.packetReceived = 0;
	this.sensorMonitor = {
		count: 0,
		index: 0,
		duration: 0,
		averageIndex: 0,
		signalStrength: { current: 0, sum: 0.0, values: new Array(100) },
		accelerationX: { current: 0, sum: 0.0, values: new Array(100) },
		accelerationY: { current: 0, sum: 0.0, values: new Array(100) },
		accelerationZ: { current: 0, sum: 0.0, values: new Array(100) },
		inputSA: { current: 0, sum: 0.0, values: new Array(100) },
		inputSB: { current: 0, sum: 0.0, values: new Array(100) },
		inputSC: { current: 0, sum: 0.0, values: new Array(100) },
		inputLA: { current: 0, sum: 0.0, values: new Array(100) },
		inputLB: { current: 0, sum: 0.0, values: new Array(100) },
		inputLC: { current: 0, sum: 0.0, values: new Array(100) },
		ultrasonic: { current: 0, sum: 0.0, values: new Array(100) },
		encoder: { current: 0, sum: 0.0, values: new Array(100) },
		temperature: { current: 0, sum: 0.0, values: new Array(100) },
		humidity: { current: 0, sum: 0.0, values: new Array(100) },
		freeFall: { current: 0, sum: 0.0, values: new Array(100) },
		tap: { current: 0, sum: 0.0, values: new Array(100) },
		stepCount: { current: 0, sum: 0.0, values: new Array(100) },
		batteryState: { current: 0, sum: 0.0, values: new Array(100) },
		reset: function() {
			this.signalStrength.current = 0;
			this.signalStrength.sum = 0.0;
			this.accelerationX.current = 0;
			this.accelerationX.sum = 0.0;
			this.accelerationY.current = 0;
			this.accelerationY.sum = 0.0;
			this.accelerationZ.current = 0;
			this.accelerationZ.sum = 0.0;
			this.inputSA.current = 0;
			this.inputSA.sum = 0.0;
			this.inputSB.current = 0;
			this.inputSB.sum = 0.0;
			this.inputSC.current = 0;
			this.inputSC.sum = 0.0;
			this.inputLA.current = 0;
			this.inputLA.sum = 0.0;
			this.inputLB.current = 0;
			this.inputLB.sum = 0.0;
			this.inputLC.current = 0;
			this.inputLC.sum = 0.0;
			this.ultrasonic.current = 0;
			this.ultrasonic.sum = 0.0;
			this.encoder.current = 0;
			this.encoder.sum = 0.0;
			this.temperature.current = 0;
			this.temperature.sum = 0.0;
			this.humidity.current = 0;
			this.humidity.sum = 0.0;
			this.freeFall.current = 0;
			this.freeFall.sum = 0.0;
			this.tap.current = 0;
			this.tap.sum = 0.0;
			this.stepCount.current = 0;
			this.stepCount.sum = 0.0;
			this.batteryState.current = 0;
			this.batteryState.sum = 0.0;
			this.count = 0;
			this.index = 0;
			this.averageIndex = 0;
		},
		put: function(device, value) {
			if(this.duration > 0) {
				device.sum += value;
				if(this.averageIndex >= this.duration - 1) {
					device.current = parseInt(device.sum * 100 / this.duration) / 100;
					device.sum = 0.0;
					device.values[this.index] = device.current;
				}
			} else {
				device.current = value;
				device.values[this.index] = value;
			}
		},
		update: function() {
			if(this.duration > 0) {
				++ this.averageIndex;
				if(this.averageIndex >= this.duration) {
					this.averageIndex = 0;
					if(this.count < 100) ++ this.count;
					++ this.index;
					if(this.index >= 100) this.index = 0;
				}
			} else {
				if(this.count < 100) ++ this.count;
				++ this.index;
				if(this.index >= 100) this.index = 0;
			}
		}
	};
	this.sensorNames = [
		'acceleration sensor', 'free fall and tap', 'signal strength', 'step count', 'battery state', 'input S', 'input L', 'ultrasonic and encoder', 'temperature and humidity'
	];
}

Controller.prototype.toHex = function(number) {
	var value = parseInt(number);
	if(value < 0) value += 0x100;

	value = value.toString(16).toUpperCase();
	if(value.length > 1) return value;
	else return '0' + value;
};

Controller.prototype.toHex2 = function(number) {
	var value = parseInt(number);
	if(value < 0) value += 0x10000;
	
	value = value.toString(16).toUpperCase();
	var result = '';
	for(var i = value.length; i < 4; ++i) {
		result += '0';
	}
	return result + value;
};
	
Controller.prototype.load = function(connector, route, callback) {
	callback();
};

Controller.prototype.getOrCreateI2cArray = function(target) {
	if(target.i2c == undefined) target.i2c = new Array(20);
	return target.i2c;
};

Controller.prototype.getOrCreateSerialArray = function(target) {
	if(target.serial == undefined) target.serial = new Array(19);
	return target.serial;
};

Controller.prototype.getOrCreateNeopixelArray = function(target) {
	if(target.neopixel == undefined) target.neopixel = new Array(20);
	return target.neopixel;
};

Controller.prototype.handleLocalData = function(data) { // data: string
	if(data.length != 53) return;
	
	var sensory = this.sensory;
	sensory.map = 0xffc00000;
	
	this.packetReceived = 0;
	var str = data.slice(0, 1);
	var value = parseInt(str, 16);
	if(value == 1) { // normal
		var motoring = this.motoring;
		var sensorMonitor = this.sensorMonitor;
		
		// input S
		str = data.slice(2, 4);
		value = parseInt(str, 16);
		if(motoring.modeSA == Cheese.MODE_S_BUTTON_INPUT) {
			if(value == 0) sensory.inputSA = 1;
			else sensory.inputSA = 0;
		} else {
			sensory.inputSA = value;
		}
		str = data.slice(4, 6);
		value = parseInt(str, 16);
		if(motoring.modeSB == Cheese.MODE_S_BUTTON_INPUT) {
			if(value == 0) sensory.inputSB = 1;
			else sensory.inputSB = 0;
		} else {
			sensory.inputSB = value;
		}
		str = data.slice(6, 8);
		value = parseInt(str, 16);
		if(motoring.modeSC == Cheese.MODE_S_BUTTON_INPUT) {
			if(value == 0) sensory.inputSC = 1;
			else sensory.inputSC = 0;
		} else {
			sensory.inputSC = value;
		}
		sensorMonitor.put(sensorMonitor.inputSA, sensory.inputSA);
		sensorMonitor.put(sensorMonitor.inputSB, sensory.inputSB);
		sensorMonitor.put(sensorMonitor.inputSC, sensory.inputSC);
		
		// input L or ultrasonic, count
		switch(motoring.modeLA) {
			case Cheese.MODE_L_SERIAL_9600:
			case Cheese.MODE_L_SERIAL_14400:
			case Cheese.MODE_L_SERIAL_19200:
			case Cheese.MODE_L_SERIAL_28800:
			case Cheese.MODE_L_SERIAL_38400:
			case Cheese.MODE_L_SERIAL_57600:
			case Cheese.MODE_L_SERIAL_76800:
			case Cheese.MODE_L_SERIAL_115200:
			case Cheese.MODE_L_NEOPIXEL_CLEAR:
			case Cheese.MODE_L_NEOPIXEL_28_ONOFF:
			case Cheese.MODE_L_NEOPIXEL_1_GRB:
			case Cheese.MODE_L_NEOPIXEL_1_GRBW:
			case Cheese.MODE_L_NEOPIXEL_GRB_PATTERN:
			case Cheese.MODE_L_NEOPIXEL_GRBW_PATTERN:
			case Cheese.MODE_L_NEOPIXEL_I2C:
			case Cheese.MODE_L_NEOPIXEL_WRITE:
			case Cheese.MODE_L_NEOPIXEL_UPDATE:
				sensory.inputLA = 0;
				sensory.inputLB = 0;
				sensory.inputLC = 0;
				sensory.ultrasonic = 0;
				sensory.encoder = 0;
				sensory.temperature = 0;
				sensory.humidity = 0;
				break;
			case Cheese.MODE_L_ULTRASONIC:
				str = data.slice(10, 14);
				sensory.inputLA = 0;
				sensory.inputLB = 0;
				sensory.inputLC = 0;
				sensory.ultrasonic = parseInt(str, 16) / 10;
				sensory.encoder = 0;
				sensory.temperature = 0;
				sensory.humidity = 0;
				break;
			case Cheese.MODE_L_ENCODER:
				str = data.slice(10, 14);
				value = parseInt(str, 16);
				if(value > 0x7fff) value -= 0x10000;
				sensory.inputLA = 0;
				sensory.inputLB = 0;
				sensory.inputLC = 0;
				sensory.ultrasonic = 0;
				sensory.encoder = value;
				sensory.temperature = 0;
				sensory.humidity = 0;
				break;
			case Cheese.MODE_L_DS18B20:
				str = data.slice(8, 12);
				value = parseInt(str, 16);
				if(value > 0x7fff) value -= 0x10000;
				value = value * 0.0625;
				sensory.inputLA = 0;
				sensory.inputLB = 0;
				sensory.inputLC = 0;
				sensory.ultrasonic = 0;
				sensory.encoder = 0;
				sensory.temperature = value;
				sensory.humidity = 0;
				break;
			case Cheese.MODE_L_DHT11:
				str = data.slice(8, 10);
				sensory.humidity = parseInt(str, 16);
				str = data.slice(10, 12);
				sensory.temperature = parseInt(str, 16);
				sensory.inputLA = 0;
				sensory.inputLB = 0;
				sensory.inputLC = 0;
				sensory.ultrasonic = 0;
				sensory.encoder = 0;
				break;
			case Cheese.MODE_L_DHT21:
			case Cheese.MODE_L_DHT22:
				str = data.slice(8, 10);
				sensory.humidity = parseInt(str, 16);
				str = data.slice(10, 12);
				sensory.temperature = parseInt(str, 16);
				str = data.slice(12, 14);
				value = parseInt(str, 16);
				if(sensory.temperature & 0x80) {
					sensory.temperature = -(((sensory.temperature & 0x7f) << 4) | (value & 0x0f)) / 10;
				} else {
					sensory.temperature = ((sensory.temperature << 4) | (value & 0x0f)) / 10;
				}
				sensory.humidity = ((sensory.humidity << 4) | ((value >> 4) & 0x0f)) / 10;
				sensory.inputLA = 0;
				sensory.inputLB = 0;
				sensory.inputLC = 0;
				sensory.ultrasonic = 0;
				sensory.encoder = 0;
				break;
			case Cheese.MODE_L_PEAK_DETECTOR:
				break;
			case Cheese.MODE_L_IR_REMOTE_CONTROL:
			case Cheese.MODE_L_IR_NEC:
				break;
			default:
				str = data.slice(8, 10);
				value = parseInt(str, 16);
				if(motoring.modeLA == Cheese.MODE_L_BUTTON_INPUT) {
					if(value == 0) sensory.inputLA = 1;
					else sensory.inputLA = 0;
				} else {
					sensory.inputLA = value;
				}
				str = data.slice(10, 12);
				value = parseInt(str, 16);
				if(motoring.modeLB == Cheese.MODE_L_BUTTON_INPUT) {
					if(value == 0) sensory.inputLB = 1;
					else sensory.inputLB = 0;
				} else {
					sensory.inputLB = value;
				}
				str = data.slice(12, 14);
				value = parseInt(str, 16);
				if(motoring.modeLC == Cheese.MODE_L_BUTTON_INPUT) {
					if(value == 0) sensory.inputLC = 1;
					else sensory.inputLC = 0;
				} else {
					sensory.inputLC = value;
				}
				sensory.ultrasonic = 0;
				sensory.encoder = 0;
				sensory.temperature = 0;
				sensory.humidity = 0;
				break;
		}
		sensorMonitor.put(sensorMonitor.inputLA, sensory.inputLA);
		sensorMonitor.put(sensorMonitor.inputLB, sensory.inputLB);
		sensorMonitor.put(sensorMonitor.inputLC, sensory.inputLC);
		sensorMonitor.put(sensorMonitor.ultrasonic, sensory.ultrasonic);
		sensorMonitor.put(sensorMonitor.encoder, sensory.encoder);
		sensorMonitor.put(sensorMonitor.temperature, sensory.temperature);
		sensorMonitor.put(sensorMonitor.humidity, sensory.humidity);
		
		// acceleration
		var acc = this.acceleration;
		if(acc.count < 10) {
			++ acc.count;
		} else {
			acc.index %= 10;
			acc.sumx -= acc.x[acc.index];
			acc.sumy -= acc.y[acc.index];
			acc.sumz -= acc.z[acc.index];
		}
		// acceleration y
		str = data.slice(14, 18);
		value = parseInt(str, 16);
		if(value > 0x7fff) value -= 0x10000;
		value *= 16;
		acc.sumy += value;
		acc.y[acc.index] = value;
		// acceleration x
		str = data.slice(18, 22);
		value = parseInt(str, 16);
		if(value > 0x7fff) value -= 0x10000;
		value *= -16;
		acc.sumx += value;
		acc.x[acc.index] = value;
		// acceleration z
		str = data.slice(22, 26);
		value = parseInt(str, 16);
		if(value > 0x7fff) value -= 0x10000;
		value *= -16;
		acc.sumz += value;
		acc.z[acc.index] = value;
		++ acc.index;
		sensory.acceleration.x = Math.round(acc.sumx / acc.count);
		sensory.acceleration.y = Math.round(acc.sumy / acc.count);
		sensory.acceleration.z = Math.round(acc.sumz / acc.count);
		sensorMonitor.put(sensorMonitor.accelerationX, sensory.acceleration.x);
		sensorMonitor.put(sensorMonitor.accelerationY, sensory.acceleration.y);
		sensorMonitor.put(sensorMonitor.accelerationZ, sensory.acceleration.z);

		var event = this.event;
		// step count
		str = data.slice(26, 30);
		value = parseInt(str, 16);
		if(value != event.stepCount) {
			if(event.stepCount != -1) {
				sensory.map |= 0x00008000;
			}
			event.stepCount = value;
			sensory.stepCount = value;
		}
		sensorMonitor.put(sensorMonitor.stepCount, value);
		
		// free fall
		str = data.slice(30, 32);
		value = parseInt(str, 16);
		var id = (value >> 6) & 0x03;
		var sensor = 0;
		if(id != event.freeFallId) {
			if(event.freeFallId != -1) {
				sensory.map |= 0x00080000;
				sensor = -1;
			}
			event.freeFallId = id;
		}
		sensorMonitor.put(sensorMonitor.freeFall, sensor);
		
		// tap
		id = (value >> 4) & 0x03;
		sensor = 0;
		if(id != event.tapId) {
			if(event.tapId != -1) {
				if((value & 0x07) != 0) {
					sensor = 1;
					sensory.map |= 0x00040000;
				}
			}
			event.tapId = id;
		}
		sensorMonitor.put(sensorMonitor.tap, sensor);
		
		// step state
		str = data.slice(32, 34);
		value = parseInt(str, 16);
		var state = (value >> 7) & 0x01;
		var step = this.step;
		if(step.event == 1) {
			if(state == 0) {
				if(++step.count > 5) step.event = 2;
			} else {
				step.event = 2;
			}
		}
		if(step.event == 2) {
			if(state != step.state || step.count > 5) {
				step.state = state;
				sensory.stepState = state;
				sensory.map |= 0x00004000;
				if(state == 0) {
					step.event = 0;
					step.count = 0;
				}
			}
		}
		if(step.event == -1) {
			step.state = state;
			sensory.stepState = 0;
			sensory.map |= 0x00004000;
			step.event = 0;
			step.count = 0;
		}
		
		// sound state
		state = (value >> 4) & 0x01;
		var sound = this.sound;
		if(sound.event == 1) {
			if(state == 0) {
				if(++sound.count > 5) sound.event = 2;
			} else {
				sound.event = 2;
			}
		}
		if(sound.event == 2) {
			if(state != sound.state || sound.count > 5) {
				sound.state = state;
				sensory.soundState = state;
				sensory.map |= 0x00002000;
				if(state == 0) {
					sound.event = 0;
					sound.count = 0;
				}
			}
		}
		
		// battery state
		state = value & 0x03;
		if(state == 0) state = 2; // normal
		else if(state == 2) state = 0; // empty
		if(state != event.batteryState) {
			if(event.batteryState != -1) {
				sensory.map |= 0x00001000;
			}
			event.batteryState = state;
			sensory.batteryState = state;
		}
		sensorMonitor.put(sensorMonitor.batteryState, state);
		
		// hatA
		state = (value >> 6) & 0x01;
		state = 1 - state;
		if(state != event.hatA) {
			if(event.hatA != -1) {
				sensory.map |= 0x00200000;
			}
			sensory.hatA = state;
			event.hatA = state;
		}
		// hatB
		state = (value >> 5) & 0x01;
		state = 1 - state;
		if(state != event.hatB) {
			if(event.hatB != -1) {
				sensory.map |= 0x00100000;
			}
			sensory.hatB = state;
			event.hatB = state;
		}
		
		// ack
		var portL = this.portL;
		id = (value >> 2) & 0x01;
		if(id != portL.ackId) {
			if(portL.ackId != -1) {
				switch(motoring.modeLA) {
					case Cheese.MODE_L_SERIAL_9600:
					case Cheese.MODE_L_SERIAL_14400:
					case Cheese.MODE_L_SERIAL_19200:
					case Cheese.MODE_L_SERIAL_28800:
					case Cheese.MODE_L_SERIAL_38400:
					case Cheese.MODE_L_SERIAL_57600:
					case Cheese.MODE_L_SERIAL_76800:
					case Cheese.MODE_L_SERIAL_115200:
						portL.serial = true;
						break;
					case Cheese.MODE_L_NEOPIXEL_CLEAR:
					case Cheese.MODE_L_NEOPIXEL_28_ONOFF:
					case Cheese.MODE_L_NEOPIXEL_1_GRB:
					case Cheese.MODE_L_NEOPIXEL_1_GRBW:
					case Cheese.MODE_L_NEOPIXEL_GRB_PATTERN:
					case Cheese.MODE_L_NEOPIXEL_GRBW_PATTERN:
					case Cheese.MODE_L_NEOPIXEL_I2C:
					case Cheese.MODE_L_NEOPIXEL_WRITE:
					case Cheese.MODE_L_NEOPIXEL_UPDATE:
						portL.neopixel = true;
						break;
					default: {
						portL.serial = false;
						portL.neopixel = false;
						break;
					}
				}
			}
			portL.ackId = id;
		}
		
		// temperature
		str = data.slice(34, 36);
		value = parseInt(str, 16);
		if(value > 0x7f) value -= 0x100;
		value = value / 2.0 + 24;
		sensory.internalTemperature = parseInt(value);
		
		// signal strength
		str = data.slice(36, 38);
		value = parseInt(str, 16);
		value -= 0x100;
		sensory.signalStrength = value;
		sensorMonitor.put(sensorMonitor.signalStrength, value);
		
		sensorMonitor.update();
		this.packetReceived = PACKET_NORMAL;
		this.handleExtensionSensory(sensory);
		return sensory;
	} else if(value == 2) { // i2c
		var i2c = this.getOrCreateI2cArray(sensory);
		str = data.slice(1, 2);
		i2c[0] = parseInt(str, 16);
		for(var i = 1, j = 2; i < 20; ++i, j += 2) {
			str = data.slice(j, j + 2);
			i2c[i] = parseInt(str, 16);
		}
		sensory.map |= 0x00020000;
		this.packetReceived = PACKET_I2C;
		this.handleExtensionSensory(sensory);
		return sensory;
	} else if(value == 3) { // serial
		str = data.slice(1, 2);
		var id = parseInt(str, 16);
		var event = this.event;
		if(id != event.serialId) {
			if(event.serialId != -1) {
				sensory.map |= 0x00010000;
				str = data.slice(2, 4);
				var len = parseInt(str, 16);
				if(len > 18) len = 18;
				var serial = this.getOrCreateSerialArray(sensory);
				serial[0] = len;
				for(var i = 1, j = 4; i <= len; ++i, j += 2) {
					str = data.slice(j, j + 2);
					serial[i] = parseInt(str, 16);
				}
				for(var i = len + 1; i <= 18; ++i) {
					serial[i] = 0;
				}
			}
			event.serialId = id;
		}
		this.packetReceived = PACKET_SERIAL;
		this.handleExtensionSensory(sensory);
		return sensory;
	}
};

Controller.prototype.handleExtensionSensory = function(sensory) {
	var i2c = this.i2c;
	if(i2c.sendId != i2c.sendPrevId) {
		i2c.sendPrevId = i2c.sendId;
		sensory.map |= 0x00000800;
	}
	var serial = this.serial;
	if(serial.sendId != serial.sendPrevId) {
		serial.sendPrevId = serial.sendId;
		sensory.map |= 0x00000400;
	}
	var neopixel = this.neopixel;
	if(neopixel.sendId != neopixel.sendPrevId) {
		neopixel.sendPrevId = neopixel.sendId;
		sensory.map |= 0x00000200;
	}
};

Controller.prototype.write = function(data) {
	if(!data) return this.motoring;
	
	var map1 = data.map1;
	var map2 = data.map2;
	var writeData = data.writeData;
	var motoring = this.motoring;
	
	if(map1 & 0x00008000) motoring.modeSA = writeData[15][0];
	if(map1 & 0x00004000) motoring.modeSB = writeData[16][0];
	if(map1 & 0x00002000) motoring.modeSC = writeData[17][0];
	if(map1 & 0x00001000) motoring.modeLA = writeData[18][0];
	if(map1 & 0x00000800) motoring.modeLB = writeData[19][0];
	if(map1 & 0x00000400) motoring.modeLC = writeData[20][0];
	if(map1 & 0x00000200) motoring.modeM = writeData[21][0];
	
	// output S
	var t = writeData[0][0];
	switch(motoring.modeSA) {
		case Cheese.MODE_S_DIGITAL_OUTPUT:
			break;
		case Cheese.MODE_S_PWM_OUTPUT:
			t += motoring.outputSA;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case Cheese.MODE_S_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputSA;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputSA;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputSA = t;
	
	t = writeData[1][0];
	switch(motoring.modeSB) {
		case Cheese.MODE_S_DIGITAL_OUTPUT:
			break;
		case Cheese.MODE_S_PWM_OUTPUT:
			t += motoring.outputSB;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case Cheese.MODE_S_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputSB;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputSB;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputSB = t;
	
	t = writeData[2][0];
	switch(motoring.modeSC) {
		case Cheese.MODE_S_DIGITAL_OUTPUT:
			break;
		case Cheese.MODE_S_PWM_OUTPUT:
			t += motoring.outputSC;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case Cheese.MODE_S_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputSC;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputSC;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputSC = t;
	
	// output L
	t = writeData[3][0];
	switch(motoring.modeLA) {
		case Cheese.MODE_L_DIGITAL_OUTPUT:
			break;
		case Cheese.MODE_L_PWM_OUTPUT:
			t += motoring.outputLA;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case Cheese.MODE_L_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputLA;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputLA;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputLA = t;
	
	t = writeData[4][0];
	switch(motoring.modeLB) {
		case Cheese.MODE_L_DIGITAL_OUTPUT:
			break;
		case Cheese.MODE_L_PWM_OUTPUT:
			t += motoring.outputLB;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case Cheese.MODE_L_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputLB;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputLB;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputLB = t;
	
	t = writeData[5][0];
	switch(motoring.modeLC) {
		case Cheese.MODE_L_DIGITAL_OUTPUT:
			break;
		case Cheese.MODE_L_PWM_OUTPUT:
			t += motoring.outputLC;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case Cheese.MODE_L_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputLC;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputLC;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputLC = t;
	
	motoring.outputLX = writeData[6][0];
	
	// output M
	switch(motoring.modeM) {
		case Cheese.MODE_M_DC:
			t = motoring.outputMAB + writeData[7][0];
			if(t < -100) t = -100;
			else if(t > 100) t = 100;
			motoring.outputMAB = t;
			t = motoring.outputMCD + writeData[8][0];
			if(t < -100) t = -100;
			else if(t > 100) t = 100;
			motoring.outputMCD = t;
			break;
		case Cheese.MODE_M_SERVO_MONO:
			t = writeData[7][0];
			if(t != 0) {
				t += motoring.outputMAB;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			motoring.outputMAB = t;
			break;
		case Cheese.MODE_M_SERVO_DUAL:
			t = writeData[7][0];
			if(t != 0) {
				t += motoring.outputMAB;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			motoring.outputMAB = t;
			t = writeData[8][0];
			if(t != 0) {
				t += motoring.outputMCD;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			motoring.outputMCD = t;
			break;
		default:
			t = motoring.outputMAB + writeData[7][0];
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			motoring.outputMAB = t;
			t = motoring.outputMCD + writeData[8][0];
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			motoring.outputMCD = t;
			break;
	}
	
	// buzzer;
	t = writeData[9][0];
	if(t == 0) {
		motoring.buzzer = 0;
	} else {
		t = motoring.buzzer + t;
		if(t < 0.1) t = 0.1;
		else if(t > 6500.0) t = 6500.0;
		motoring.buzzer = t;
	}
	
	// velocity
	t = motoring.velocity + writeData[10][0];
	if(t < -1000) t = -1000;
	else if(t > 1000) t = 1000;
	motoring.velocity = t;
	
	// commands
	if(map1 & 0x00080000) motoring.step = writeData[11][0];
	if(map1 & 0x00040000) motoring.note = writeData[12][0];
	if(map1 & 0x00020000) motoring.sound = writeData[13][0];
	if(map1 & 0x00010000) motoring.modeSound = writeData[14][0];
	
	if(map1 & 0x00000100) motoring.redLed = writeData[22][0];
	if(map1 & 0x00000080) motoring.blueLed = writeData[23][0];
	if(map1 & 0x00000040) motoring.hatA = writeData[24][0];
	if(map1 & 0x00000020) motoring.hatB = writeData[25][0];
	
	if(map1 & 0x00000008) {
		var i2c = this.getOrCreateI2cArray(motoring);
		var w = writeData[27];
		for(var i = 0; i < 20; ++i) {
			i2c[i] = w[i];
		}
		
		if(w[0] == 0x22 && w[1] == 0x01) {
			this.i2c.activated = I2C_LED_MATRIX;
		}
	}
	if(map1 & 0x00000004) {
		var serial = this.getOrCreateSerialArray(motoring);
		var w = writeData[28];
		for(var i = 0; i < 19; ++i) {
			serial[i] = w[i];
		}
	}
	if(map1 & 0x00000002) {
		var neopixel = this.getOrCreateNeopixelArray(motoring);
		var w = writeData[29];
		for(var i = 0; i < 20; ++i) {
			neopixel[i] = w[i];
		}
		this.neopixel.activated = true;
	}
	if(map1 & 0x00000001) motoring.configAdcSA = writeData[30][0];
	if(map2 & 0x80000000) motoring.configAdcSB = writeData[31][0];
	if(map2 & 0x40000000) motoring.configAdcSC = writeData[32][0];
	if(map2 & 0x20000000) motoring.configAdcLA = writeData[33][0];
	if(map2 & 0x10000000) motoring.configAdcLB = writeData[34][0];
	if(map2 & 0x08000000) motoring.configAdcLC = writeData[35][0];
	if(map2 & 0x04000000) motoring.configGravity = writeData[36][0];
	if(map2 & 0x02000000) motoring.configBandWidth = writeData[37][0];
	
	motoring.map1 |= map1;
	motoring.map2 |= map2;
	return motoring;
};

Controller.prototype.requestLocalData = function(address) {
	var self = this;
	var motoring = self.motoring;
	var portL = self.portL;
	
	// i2c
	if((motoring.map1 & 0x00000008) && self.packetSent != PACKET_I2C) {
		motoring.map1 &= 0xfffffff7;
		self.packetSent = PACKET_I2C;
		self.i2c.sendId = (self.i2c.sendId % 255) + 1;
		
		var i2c = motoring.i2c;
		if(i2c) {
			var str = '';
			for(var i = 0; i < 20; ++i) {
				str += self.toHex(i2c[i]);
			}
			str += '-';
			str += address;
			str += '\r';
			return str;
		}
	}
	
	// serial
	if(portL.serial && (motoring.map1 & 0x00000004) && self.packetSent != PACKET_SERIAL) {
		motoring.map1 &= 0xfffffffb;
		self.packetSent = PACKET_SERIAL;
		self.serial.sendId = (self.serial.sendId % 255) + 1;
		
		var serial = motoring.serial;
		if(serial) {
			var len = serial[0];
			if(len > 0) {
				if(len > 18) len = 18;
				var command = self.command;
				command.serialId = (command.serialId % 15) + 1;
				var tmp = 0x30 | (command.serialId & 0x0f);
				var str = self.toHex(tmp);
				str += self.toHex(len);
				for(var i = 1; i <= len; ++i) {
					str += self.toHex(serial[i]);
				}
				for(var i = len + 1; i <= 18; ++i) {
					str += '00';
				}
				str += '-';
				str += address;
				str += '\r';
				return str;
			}
		}
	}
	
	// neopixel
	if(portL.neopixel && (motoring.map1 & 0x00000002) && self.packetSent != PACKET_NEOPIXEL) {
		motoring.map1 &= 0xfffffffd;
		self.packetSent = PACKET_NEOPIXEL;
		self.neopixel.sendId = (self.neopixel.sendId % 255) + 1;
		
		var neopixel = motoring.neopixel;
		if(neopixel) {
			var str = '';
			for(var i = 0; i < 20; ++i) {
				str += self.toHex(neopixel[i]);
			}
			str += '-';
			str += address;
			str += '\r';
			return str;
		}
	}
	
	var str = '10';
	// mode S
	var tmp = 0;
	switch(motoring.modeSA) {
		case Cheese.MODE_S_DIGITAL_INPUT:
		case Cheese.MODE_S_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_S_MAKEY_INPUT:
		case Cheese.MODE_S_BUTTON_INPUT: break;
		case Cheese.MODE_S_ANALOG_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT_PULL_UP: tmp |= 0x01; break;
		case Cheese.MODE_S_DIGITAL_OUTPUT:
		case Cheese.MODE_S_PWM_OUTPUT: tmp |= 0x02; break;
		case Cheese.MODE_S_SERVO_OUTPUT: tmp |= 0x03; break;
	}
	switch(motoring.modeSB) {
		case Cheese.MODE_S_DIGITAL_INPUT:
		case Cheese.MODE_S_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_S_MAKEY_INPUT:
		case Cheese.MODE_S_BUTTON_INPUT: break;
		case Cheese.MODE_S_ANALOG_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT_PULL_UP: tmp |= 0x04; break;
		case Cheese.MODE_S_DIGITAL_OUTPUT:
		case Cheese.MODE_S_PWM_OUTPUT: tmp |= 0x08; break;
		case Cheese.MODE_S_SERVO_OUTPUT: tmp |= 0x0c; break;
	}
	switch(motoring.modeSC) {
		case Cheese.MODE_S_DIGITAL_INPUT:
		case Cheese.MODE_S_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_S_MAKEY_INPUT:
		case Cheese.MODE_S_BUTTON_INPUT: break;
		case Cheese.MODE_S_ANALOG_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT_PULL_UP: tmp |= 0x10; break;
		case Cheese.MODE_S_DIGITAL_OUTPUT:
		case Cheese.MODE_S_PWM_OUTPUT: tmp |= 0x20; break;
		case Cheese.MODE_S_SERVO_OUTPUT: tmp |= 0x30; break;
	}
	tmp |= (motoring.modeSound & 0xc0);
	str += self.toHex(tmp);
	
	// mode L
	tmp = 0;
	if(motoring.modeLA >= 0 && motoring.modeLA <= Cheese.MODE_L_SERVO_OUTPUT) {
		switch(motoring.modeLA) {
			case Cheese.MODE_L_DIGITAL_INPUT:
			case Cheese.MODE_L_DIGITAL_INPUT_PULL_UP:
			case Cheese.MODE_L_MAKEY_INPUT:
			case Cheese.MODE_L_BUTTON_INPUT: break;
			case Cheese.MODE_L_ANALOG_INPUT:
			case Cheese.MODE_L_ANALOG_INPUT_PULL_UP: tmp |= 0x01; break;
			case Cheese.MODE_L_DIGITAL_OUTPUT:
			case Cheese.MODE_L_PWM_OUTPUT: tmp |= 0x02; break;
			case Cheese.MODE_L_SERVO_OUTPUT: tmp |= 0x03; break;
		}
		switch(motoring.modeLB) {
			case Cheese.MODE_L_DIGITAL_INPUT:
			case Cheese.MODE_L_DIGITAL_INPUT_PULL_UP:
			case Cheese.MODE_L_MAKEY_INPUT:
			case Cheese.MODE_L_BUTTON_INPUT: break;
			case Cheese.MODE_L_ANALOG_INPUT:
			case Cheese.MODE_L_ANALOG_INPUT_PULL_UP: tmp |= 0x04; break;
			case Cheese.MODE_L_DIGITAL_OUTPUT:
			case Cheese.MODE_L_PWM_OUTPUT: tmp |= 0x08; break;
			case Cheese.MODE_L_SERVO_OUTPUT: tmp |= 0x0c; break;
		}
		switch(motoring.modeLC) {
			case Cheese.MODE_L_DIGITAL_INPUT:
			case Cheese.MODE_L_DIGITAL_INPUT_PULL_UP:
			case Cheese.MODE_L_MAKEY_INPUT:
			case Cheese.MODE_L_BUTTON_INPUT: break;
			case Cheese.MODE_L_ANALOG_INPUT:
			case Cheese.MODE_L_ANALOG_INPUT_PULL_UP: tmp |= 0x10; break;
			case Cheese.MODE_L_DIGITAL_OUTPUT:
			case Cheese.MODE_L_PWM_OUTPUT: tmp |= 0x20; break;
			case Cheese.MODE_L_SERVO_OUTPUT: tmp |= 0x30; break;
		}
	} else if(motoring.modeLA == Cheese.MODE_L_NEOPIXEL_28_ONOFF) {
		tmp = 0xc0;
	} else {
		switch(motoring.modeLA) {
			case Cheese.MODE_L_SERIAL_9600: tmp = 0x40; break;
			case Cheese.MODE_L_SERIAL_14400: tmp = 0x41; break;
			case Cheese.MODE_L_SERIAL_19200: tmp = 0x42; break;
			case Cheese.MODE_L_SERIAL_28800: tmp = 0x43; break;
			case Cheese.MODE_L_SERIAL_38400: tmp = 0x44; break;
			case Cheese.MODE_L_SERIAL_57600: tmp = 0x45; break;
			case Cheese.MODE_L_SERIAL_76800: tmp = 0x46; break;
			case Cheese.MODE_L_SERIAL_115200: tmp = 0x47; break;
			case Cheese.MODE_L_ULTRASONIC: tmp = 0x81; break;
			case Cheese.MODE_L_ENCODER: tmp = 0x82; break;
			case Cheese.MODE_L_DS18B20: tmp = 0x83; break;
			case Cheese.MODE_L_DHT11: tmp = 0x84; break;
			case Cheese.MODE_L_DHT21: tmp = 0x85; break;
			case Cheese.MODE_L_DHT22: tmp = 0x86; break;
			case Cheese.MODE_L_PEAK_DETECTOR: tmp = 0x87; break;
			case Cheese.MODE_L_IR_REMOTE_CONTROL: tmp = 0x90; break;
			case Cheese.MODE_L_IR_NEC: tmp = 0x91; break;
			case Cheese.MODE_L_NEOPIXEL_CLEAR: tmp = 0xdc; break;
			case Cheese.MODE_L_NEOPIXEL_1_GRB: tmp = 0xd0; break;
			case Cheese.MODE_L_NEOPIXEL_1_GRBW: tmp = 0xd1; break;
			case Cheese.MODE_L_NEOPIXEL_GRB_PATTERN: tmp = 0xd8; break;
			case Cheese.MODE_L_NEOPIXEL_GRBW_PATTERN: tmp = 0xd9; break;
			case Cheese.MODE_L_NEOPIXEL_I2C: tmp = 0xdd; break;
			case Cheese.MODE_L_NEOPIXEL_WRITE: tmp = 0xde; break;
			case Cheese.MODE_L_NEOPIXEL_UPDATE: tmp = 0xdf; break;
		}
	}
	str += self.toHex(tmp);
	
	// mode M
	tmp = motoring.modeM;
	tmp |= (motoring.hatA & 0x01) << 7;
	tmp |= (motoring.hatB & 0x01) << 6;
	str += self.toHex(tmp);
	
	// config gravity / band width
	tmp = (motoring.configBandWidth & 0x07) << 4;
	tmp |= (motoring.configGravity & 0x03);
	str += self.toHex(tmp);
	
	// output S
	switch(motoring.modeSA) {
		case Cheese.MODE_S_MAKEY_INPUT:
			tmp = 0x00;
			if(motoring.configAdcSA != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_S_BUTTON_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT_PULL_UP:
			tmp = 0x01;
			if(motoring.configAdcSA != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT:
			tmp = 0x02;
			if(motoring.configAdcSA != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_OUTPUT:
			if(motoring.outputSA == 0) tmp = 0;
			else tmp = 255;
			break;
		default:
			tmp = motoring.outputSA;
			break;
	}
	str += self.toHex(tmp);
	switch(motoring.modeSB) {
		case Cheese.MODE_S_MAKEY_INPUT:
			tmp = 0x00;
			if(motoring.configAdcSB != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_S_BUTTON_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT_PULL_UP:
			tmp = 0x01;
			if(motoring.configAdcSB != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT:
			tmp = 0x02;
			if(motoring.configAdcSB != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_OUTPUT:
			if(motoring.outputSB == 0) tmp = 0;
			else tmp = 255;
			break;
		default:
			tmp = motoring.outputSB;
			break;
	}
	str += self.toHex(tmp);
	switch(motoring.modeSC) {
		case Cheese.MODE_S_MAKEY_INPUT:
			tmp = 0x00;
			if(motoring.configAdcSC != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_S_BUTTON_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT_PULL_UP:
			tmp = 0x01;
			if(motoring.configAdcSC != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_INPUT:
		case Cheese.MODE_S_ANALOG_INPUT:
			tmp = 0x02;
			if(motoring.configAdcSC != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_S_DIGITAL_OUTPUT:
			if(motoring.outputSC == 0) tmp = 0;
			else tmp = 255;
			break;
		default:
			tmp = motoring.outputSC;
			break;
	}
	str += self.toHex(tmp);
	
	// output L
	switch(motoring.modeLA) {
		case Cheese.MODE_L_MAKEY_INPUT:
			tmp = 0x00;
			if(motoring.configAdcLA != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_L_BUTTON_INPUT:
		case Cheese.MODE_L_ANALOG_INPUT_PULL_UP:
			tmp = 0x01;
			if(motoring.configAdcLA != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_INPUT:
		case Cheese.MODE_L_ANALOG_INPUT:
			tmp = 0x02;
			if(motoring.configAdcLA != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_OUTPUT:
			if(motoring.outputLA == 0) tmp = 0;
			else tmp = 255;
			break;
		default:
			tmp = motoring.outputLA;
			break;
	}
	str += self.toHex(tmp);
	switch(motoring.modeLB) {
		case Cheese.MODE_L_MAKEY_INPUT:
			tmp = 0x00;
			if(motoring.configAdcLB != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_L_BUTTON_INPUT:
		case Cheese.MODE_L_ANALOG_INPUT_PULL_UP:
			tmp = 0x01;
			if(motoring.configAdcLB != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_INPUT:
		case Cheese.MODE_L_ANALOG_INPUT:
			tmp = 0x02;
			if(motoring.configAdcLB != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_OUTPUT:
			if(motoring.outputLB == 0) tmp = 0;
			else tmp = 255;
			break;
		default:
			tmp = motoring.outputLB;
			break;
	}
	str += self.toHex(tmp);
	switch(motoring.modeLC) {
		case Cheese.MODE_L_MAKEY_INPUT:
			tmp = 0x00;
			if(motoring.configAdcLC != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_INPUT_PULL_UP:
		case Cheese.MODE_L_BUTTON_INPUT:
		case Cheese.MODE_L_ANALOG_INPUT_PULL_UP:
			tmp = 0x01;
			if(motoring.configAdcLC != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_INPUT:
		case Cheese.MODE_L_ANALOG_INPUT:
			tmp = 0x02;
			if(motoring.configAdcLC != 0) tmp |= 0x10;
			break;
		case Cheese.MODE_L_DIGITAL_OUTPUT:
			if(motoring.outputLC == 0) tmp = 0;
			else tmp = 255;
			break;
		default:
			tmp = motoring.outputLC;
			break;
	}
	str += self.toHex(tmp);
	
	// Lx
	str += self.toHex(motoring.outputLX);
	
	// output M
	switch(motoring.modeM) {
		case Cheese.MODE_M_STEP_OFF:
		case Cheese.MODE_M_STEP_WAVE:
		case Cheese.MODE_M_STEP_FULL:
		case Cheese.MODE_M_STEP_HALF:
			str += self.toHex2(motoring.velocity);
			break;
		default:
			str += self.toHex(motoring.outputMAB);
			str += self.toHex(motoring.outputMCD);
			break;
	}
	
	// step
	var step = self.step;
	if(motoring.map1 & 0x00080000) {
		if(motoring.step != 0 || step.prev != 0) {
			step.id = (step.id % 3) + 1;
		}
		step.count = 0;
		if(motoring.step > 0) {
			step.event = 1;
		} else if(step.event != -1) {
			step.event = 0;
		}
		step.prev = motoring.step;
	}
	
	// reset encoder
	var command = self.command;
	if(motoring.map1 & 0x00000010) {
		command.encoderId = (command.encoderId % 3) + 1;
	}
	tmp = (command.encoderId & 0x03) << 2;
	tmp |= (step.id & 0x03);
	if(motoring.map1 & 0x00000100) {
		if(motoring.redLed == 0) {
			tmp |= 0x80;
		} else if(motoring.redLed == 1) {
			tmp |= 0xc0;
		}
	}
	if(motoring.map1 & 0x00000080) {
		if(motoring.blueLed == 0) {
			tmp |= 0x20;
		} else if(motoring.blueLed == 1) {
			tmp |= 0x30;
		}
	}
	str += self.toHex(tmp);
	str += self.toHex2(motoring.step);
	
	// sound
	var sound = self.sound;
	var sndid = 0;
	switch(motoring.sound) {
		case Cheese.SOUND_OFF: sndid = 0x00; break;
		case Cheese.SOUND_BEEP: sndid = 0x01; break;
		case Cheese.SOUND_RANDOM_BEEP: sndid = 0x05; break;
		case Cheese.SOUND_NOISE: sndid = 0x07; break;
		case Cheese.SOUND_SIREN: sndid = 0x09; break;
		case Cheese.SOUND_ENGINE: sndid = 0x0b; break;
		case Cheese.SOUND_CHOP: sndid = 0x12; break;
		case Cheese.SOUND_ROBOT: sndid = 0x20; break;
		case Cheese.SOUND_DIBIDIBIDIP: sndid = 0x21; break;
		case Cheese.SOUND_GOOD_JOB: sndid = 0x23; break;
		case Cheese.SOUND_HAPPY: sndid = 0x30; break;
		case Cheese.SOUND_ANGRY: sndid = 0x31; break;
		case Cheese.SOUND_SAD: sndid = 0x32; break;
		case Cheese.SOUND_SLEEP: sndid = 0x33; break;
		case Cheese.SOUND_MARCH: sndid = 0x34; break;
		case Cheese.SOUND_BIRTHDAY: sndid = 0x35; break;
	}
	if(motoring.map1 & 0x00020000) {
		sound.count = 0;
		if(sndid > 0) {
			sound.flag ^= 0x80;
			sound.event = 1;
		} else {
			sound.event = 0;
		}
	}
	if(sndid > 0) { // sound
		str += '00';
		str += self.toHex(sndid | sound.flag);
	} else if(motoring.note > 0) { // note
		str += '01';
		str += self.toHex(motoring.note);
	} else { // buzzer
		tmp = parseInt(motoring.buzzer * 10) + 512;
		str += self.toHex2(tmp);
	}
	str += '00-';
	str += address;
	str += '\r';
	self.packetSent = PACKET_NORMAL;
	return str;
};

Controller.prototype.clear = function() {
	var motoring = this.motoring;
	if(this.packetSent == PACKET_NORMAL) {
		motoring.map1 &= 0x8000000e;
		motoring.map2 = 0;
	}
	motoring.outputSA = 0;
	motoring.outputSB = 0;
	motoring.outputSC = 0;
	motoring.outputLA = 0;
	motoring.outputLB = 0;
	motoring.outputLC = 0;
	motoring.outputLX = 0;
	motoring.outputMAB = 0;
	motoring.outputMCD = 0;
	motoring.buzzer = 0.0;
	motoring.velocity = 0;
	
	var sensory = this.sensory;
	switch(this.packetReceived) {
		case PACKET_NORMAL: sensory.map &= 0x80030000; break;
		case PACKET_I2C: sensory.map &= 0xfffdffff; break;
		case PACKET_SERIAL: sensory.map &= 0xfffeffff; break;
	}
	sensory.stepCount = 0;
	sensory.stepState = 0;
	sensory.soundState = 0;
	sensory.batteryState = 2;
};

Controller.prototype.reset = function() {
	this.clear();
	
	var motoring = this.motoring;
	var i2c = this.i2c;
	var neopixel = this.neopixel;
	motoring.step = 0;
	motoring.note = 0;
	motoring.sound = 0;
	motoring.modeSound = 0;
	motoring.modeSA = 0;
	motoring.modeSB = 0;
	motoring.modeSC = 0;
	if(neopixel.activated) {
		neopixel.activated = false;
		motoring.modeLA = Cheese.MODE_L_NEOPIXEL_CLEAR;
	} else {
		motoring.modeLA = 0;
	}
	motoring.modeLB = 0;
	motoring.modeLC = 0;
	motoring.modeM = 0;
	motoring.redLed = -1;
	motoring.blueLed = -1;
	motoring.hatA = 0;
	motoring.hatB = 0;
	if(i2c.activated == 0) {
		motoring.i2c = undefined;
	}
	motoring.serial = undefined;
	motoring.neopixel = undefined;
	motoring.configAdcSA = 1;
	motoring.configAdcSB = 1;
	motoring.configAdcSC = 1;
	motoring.configAdcLA = 1;
	motoring.configAdcLB = 1;
	motoring.configAdcLC = 1;
	motoring.configGravity = 0;
	motoring.configBandWidth = 0;
	
	var sensory = this.sensory;
	sensory.hatA = 0;
	sensory.hatB = 0;
	sensory.i2c = undefined;
	sensory.serial = undefined;
	
	var acc = this.acceleration;
	acc.sumx = 0.0;
	acc.sumy = 0.0;
	acc.sumz = 0.0;
	acc.index = 0;
	acc.count = 0;
	var step = this.step;
	step.prev = -1;
	step.event = 0;
	step.state = 0;
	step.count = 0;
	var sound = this.sound;
	sound.event = 0;
	sound.state = 0;
	sound.count = 0;
	var event = this.event;
	event.hatA = -1;
	event.hatB = -1;
	event.freeFallId = -1;
	event.tapId = -1;
	event.serialId = -1;
	event.stepCount = -1;
	event.batteryState = -1;
	i2c.sendId = 0;
	i2c.sendPrevId = 0;
	this.serial.sendId = 0;
	this.serial.sendPrevId = 0;
	neopixel.sendId = 0;
	neopixel.sendPrevId = 0;
	var portL = this.portL;
	portL.ackId = -1;
	portL.serial = false;
	portL.neopixel = false;
	this.packetSent = 0;
	this.packetReceived = 0;
	
	if(i2c.activated == I2C_LED_MATRIX) {
		i2c.activated = 0;
		var packet = this.getOrCreateI2cArray(motoring);
		packet[0] = 0x22;
		packet[1] = 0x01;
		for(var i = 2; i < 20; ++i) {
			packet[i] = 0;
		}
		packet[17] = 100;
		motoring.map1 |= 0x00000008;
	}
};

Controller.prototype.getSensorMonitor = function() {
	return this.sensorMonitor;
};

Controller.prototype.getSensorNames = function() {
	return this.sensorNames;
};

Controller.prototype.setDuration = function(duration) {
	var sensorMonitor = this.sensorMonitor;
	sensorMonitor.reset();
	sensorMonitor.duration = duration;
};

module.exports.create = function() {
	return new Controller();
};