'use strict';
var Turtle = require('../turtle');
var factory = require('../turtle-roboid');
const SENSORY_NAMES = ['signalStrength', 'colorRed', 'colorGreen', 'colorBlue', 'colorClear', 'floor', 'accelerationX', 'accelerationY', 'accelerationZ', 'temperature', 'button', 'clicked', 'clickedId', 'doubleClicked', 'doubleClickedId', 'longPressed', 'longPressedId', 'colorNumber', 'colorPattern', 'colorPatternId', 'pulseCount', 'wheelState', 'wheelStateId', 'soundState', 'soundStateId', 'lineTracerState', 'lineTracerStateId'];

function Component() {
}

Component.prototype.createRobot = function() {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.module = 'cheese';
	robot.sensory = {
		version: 0x01,
		company: 0x02,
		model: 0x09,
		variation: 0x01,
		network: 0x00,
		protocol: 'json',
		signalStrength: 0,
		colorRed: 0,
		colorGreen: 0,
		colorBlue: 0,
		colorClear: 0,
		floor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		temperature: 0,
		button: 0,
		clicked: 0,
		clickedId: 0,
		doubleClicked: 0,
		doubleClickedId: 0,
		longPressed: 0,
		longPressedId: 0,
		colorNumber: -1,
		colorPattern: -1,
		colorPatternId: 0,
		pulseCount: 0,
		wheelState: -1,
		wheelStateId: 0,
		soundState: -1,
		soundStateId: 0,
		lineTracerState: 0,
		lineTracerStateId: 0
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(Turtle.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(Turtle.RIGHT_WHEEL),
		led: roboid.findDeviceById(Turtle.LED),
		buzzer: roboid.findDeviceById(Turtle.BUZZER),
		pulse: roboid.findDeviceById(Turtle.PULSE),
		note: roboid.findDeviceById(Turtle.NOTE),
		sound: roboid.findDeviceById(Turtle.SOUND),
		lineTracerMode: roboid.findDeviceById(Turtle.LINE_TRACER_MODE),
		lineTracerGain: roboid.findDeviceById(Turtle.LINE_TRACER_GAIN),
		lineTracerSpeed: roboid.findDeviceById(Turtle.LINE_TRACER_SPEED),
		lamp: roboid.findDeviceById(Turtle.LAMP),
		lock: roboid.findDeviceById(Turtle.LOCK),
		motion: roboid.findDeviceById(Turtle.MOTION)
	};
	robot.pulseId = -1;
	robot.sound = 0;
	robot.soundRepeat = 1;
	robot.soundId = -1;
	robot.lineTracerModeId = -1;
	robot.motionId = -1;
	return robot;
};

Component.prototype.runSound = function(robot, sound, count) {
	if(typeof count != 'number') count = 1;
	if(count < 0) count = -1;
	if(count) {
		robot.sound = sound;
		robot.soundRepeat = count;
		robot.motoring.sound.write(sound);
	}
};

Component.prototype.handleRemoteData = function(robot, data) {
	var motoring = robot.motoring;
	if(data.leftWheel != undefined) motoring.leftWheel.write(data.leftWheel);
	if(data.rightWheel != undefined) motoring.rightWheel.write(data.rightWheel);
	if(data.ledRed != undefined) motoring.led.write(0, data.ledRed);
	if(data.ledGreen != undefined) motoring.led.write(1, data.ledGreen);
	if(data.ledBlue != undefined) motoring.led.write(2, data.ledBlue);
	if(data.buzzer != undefined) motoring.buzzer.write(data.buzzer);
	if(data.pulseId != undefined) {
		var t = data.pulseId;
		if(t != robot.pulseId) {
			robot.pulseId = t;
			if(data.pulse != undefined) motoring.pulse.write(data.pulse);
		}
	}
	
	if(data.note != undefined) motoring.note.write(data.note);
	if(data.soundId != undefined) {
		var t = data.soundId;
		if(t != robot.soundId) {
			robot.soundId = t;
			if(data.sound != undefined && data.soundRepeat != undefined) {
				this.runSound(robot, data.sound, data.soundRepeat);
			}
		}
	}
	if(data.lineTracerModeId != undefined) {
		var t = data.lineTracerModeId;
		if(t != robot.lineTracerModeId) {
			robot.lineTracerModeId = t;
			if(data.lineTracerMode != undefined) motoring.lineTracerMode.write(data.lineTracerMode);
		}
	}
	if(data.lineTracerGain != undefined) motoring.lineTracerGain.write(data.lineTracerGain);
	if(data.lineTracerSpeed != undefined) motoring.lineTracerSpeed.write(data.lineTracerSpeed);
	if(data.motionId != undefined) {
		var t = data.motionId;
		if(t != robot.motionId) {
			robot.motionId = t;
			if(data.motionType != undefined) motoring.motion.write(0, data.motionType);
			if(data.motionUnit != undefined) motoring.motion.write(1, data.motionUnit);
			if(data.motionSpeed != undefined) motoring.motion.write(2, data.motionSpeed);
			if(data.motionValue != undefined) motoring.motion.write(3, data.motionValue);
			if(data.motionRadius != undefined) motoring.motion.write(4, data.motionRadius);
		}
	}
};

Component.prototype.updateSensoryEvent = function(robot) {
	var sensory = robot.sensory;
	sensory.clicked = 0;
	sensory.doubleClicked = 0;
	sensory.longPressed = 0;
	sensory.colorPattern = -1;
	sensory.wheelState = -1;
	sensory.soundState = -1;
	sensory.lineTracerState = 0;
};

Component.prototype.prepareSensory = function(robot, data) {
	var sensory = robot.sensory;
	sensory.signalStrength = data.signalStrength;
	sensory.colorRed = data.color.r;
	sensory.colorGreen = data.color.g;
	sensory.colorBlue = data.color.b;
	sensory.colorClear = data.color.c;
	sensory.floor = data.floor;
	sensory.accelerationX = data.acceleration.x;
	sensory.accelerationY = data.acceleration.y;
	sensory.accelerationZ = data.acceleration.z;
	sensory.temperature = data.temperature;
	if(data.map & 0x00001000) {
		sensory.button = data.button;
	}
	if(data.map & 0x00000800) {
		sensory.clicked = 1;
		sensory.clickedId = (sensory.clickedId % 255) + 1;
	}
	if(data.map & 0x00000400) {
		sensory.doubleClicked = 1;
		sensory.doubleClickedId = (sensory.doubleClickedId % 255) + 1;
	}
	if(data.map & 0x00000200) {
		sensory.longPressed = 1;
		sensory.longPressedId = (sensory.longPressedId % 255) + 1;
	}
	if(data.map & 0x00000100) {
		sensory.colorNumber = data.colorNumber;
	}
	if(data.map & 0x00000080) {
		sensory.colorPattern = data.colorPattern;
		sensory.colorPatternId = (sensory.colorPatternId % 255) + 1;
	}
	if(data.map & 0x00000040) {
		sensory.pulseCount = data.pulseCount;
	}
	if(data.map & 0x00000020) {
		sensory.wheelState = data.wheelState;
		sensory.wheelStateId = (sensory.wheelStateId % 255) + 1;
	}
	if(data.map & 0x00000010) {
		if(data.soundState == 0) {
			if(robot.sound > 0) {
				if(robot.soundRepeat < 0) {
					this.runSound(robot, robot.sound, -1);
				} else if(robot.soundRepeat > 1) {
					robot.soundRepeat --;
					this.runSound(robot, robot.sound, robot.soundRepeat);
				} else {
					robot.sound = 0;
					robot.soundRepeat = 1;
					sensory.soundState = data.soundState;
					sensory.soundStateId = (sensory.soundStateId % 255) + 1;
				}
			} else {
				robot.sound = 0;
				robot.soundRepeat = 1;
				sensory.soundState = data.soundState;
				sensory.soundStateId = (sensory.soundStateId % 255) + 1;
			}
		} else if(data.soundState != sensory.soundState) {
			sensory.soundState = data.soundState;
			sensory.soundStateId = (sensory.soundStateId % 255) + 1;
		}
	}
	if(data.map & 0x00000008) {
		sensory.lineTracerState = data.lineTracerState;
		sensory.lineTracerStateId = (sensory.lineTracerStateId % 255) + 1;
	}
};

Component.prototype.prepareSensoryMulti = function(sensory, key, robot, data) {
	this.prepareSensory(robot, data);
	var name;
	for(var i in SENSORY_NAMES) {
		name = SENSORY_NAMES[i];
		sensory[key + name] = robot.sensory[name];
	}
};

Component.prototype.updateSensory = function(robot) {
};

Component.prototype.reset = function(robot) {
	robot.sound = 0;
	robot.soundRepeat = 1;
	robot.roboid.reset();
};

module.exports = new Component();