'use strict';
var router = require('router');
var Cheese = require('../cheese');
var factory = require('../cheese-roboid');
const MODULE = 'cheese';
const HEADER = { 'Content-Type': 'text/plain' };

function Component() {
}

Component.prototype.createRobot = function(state) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.connectionState = state;
	robot.sensory = {
		signalStrength: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		temperature: 0,
		inputSA: 0,
		inputSB: 0,
		inputSC: 0,
		inputLA: 0,
		inputLB: 0,
		inputLC: 0,
		ultrasonic: 0,
		encoder: 0,
		tap: 0,
		i2c: '',
		uart: '',
		stepCount: 0
	};
	robot.motoring = {
		Sa: roboid.findDeviceById(Cheese.OUTPUT_SA),
		Sb: roboid.findDeviceById(Cheese.OUTPUT_SB),
		Sc: roboid.findDeviceById(Cheese.OUTPUT_SC),
		La: roboid.findDeviceById(Cheese.OUTPUT_LA),
		Lb: roboid.findDeviceById(Cheese.OUTPUT_LB),
		Lc: roboid.findDeviceById(Cheese.OUTPUT_LC),
		Ma: roboid.findDeviceById(Cheese.OUTPUT_MA),
		Mb: roboid.findDeviceById(Cheese.OUTPUT_MB),
		buzzer: roboid.findDeviceById(Cheese.BUZZER),
		velocity: roboid.findDeviceById(Cheese.VELOCITY),
		step: roboid.findDeviceById(Cheese.STEP),
		note: roboid.findDeviceById(Cheese.NOTE),
		sound: roboid.findDeviceById(Cheese.SOUND),
		modeSound: roboid.findDeviceById(Cheese.MODE_SOUND),
		modeSa: roboid.findDeviceById(Cheese.MODE_SA),
		modeSb: roboid.findDeviceById(Cheese.MODE_SB),
		modeSc: roboid.findDeviceById(Cheese.MODE_SC),
		modeLa: roboid.findDeviceById(Cheese.MODE_LA),
		modeLb: roboid.findDeviceById(Cheese.MODE_LB),
		modeLc: roboid.findDeviceById(Cheese.MODE_LC),
		modeM: roboid.findDeviceById(Cheese.MODE_M),
		resetEncoder: roboid.findDeviceById(Cheese.RESET_ENCODER),
		writeI2c: roboid.findDeviceById(Cheese.WRITE_I2C),
		writeUart: roboid.findDeviceById(Cheese.WRITE_UART)
	};
	robot.soundId = 0;
	robot.soundRepeat = 1;
	return robot;
};

Component.prototype.start = function(express, robots) {
	var self = this;
	var getRoute = function(index) {
		if(router.multi) {
			return router.findRouteByIndex(MODULE, index);
		} else {
			var selectedRoute = router.selectedRoute;
			if(selectedRoute && selectedRoute.module == MODULE) {
				return selectedRoute;
			}
		}
	};
	
	var getRobot = function(index) {
		var key = router.multi ? MODULE + index : MODULE;
		var robot = robots[key];
		if(!robot) {
			var route = getRoute(index);
			if(route) {
				robot = self.createRobot(route.state);
				if(robot) {
					robots[key] = robot;
				}
			}
		}
		return robot;
	};
	/*
	express.get('/turtle/:index/wheels/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__wheels(getRobot(index), leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/turtle/:index/wheels/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__wheels(getRobot(index), speed, speed);
		response.end();
	});
	
	express.get('/turtle/:index/leftWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__leftWheel(getRobot(index), speed);
		response.end();
	});
	
	express.get('/turtle/:index/rightWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__rightWheel(getRobot(index), speed);
		response.end();
	});
	
	express.get('/turtle/:index/moveForward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveForward(getRobot(index), cm, speed);
		response.end();
	});
	
	express.get('/turtle/:index/moveForward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		self.__moveForward(getRobot(index), cm, 0);
		response.end();
	});
	
	express.get('/turtle/:index/moveBackward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveBackward(getRobot(index), cm, speed);
		response.end();
	});
	
	express.get('/turtle/:index/moveBackward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		self.__moveBackward(getRobot(index), cm, 0);
		response.end();
	});
	
	express.get('/turtle/:index/turnLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnLeft(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/turtle/:index/turnLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__turnLeft(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/turtle/:index/turnRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnRight(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/turtle/:index/turnRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__turnRight(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/turtle/:index/pivotLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotLeft(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/turtle/:index/pivotLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__pivotLeft(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/turtle/:index/pivotRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotRight(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/turtle/:index/pivotRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__pivotRight(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/turtle/:index/swingLeft/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingLeft(getRobot(index), deg, cm, speed);
		response.end();
	});
	
	express.get('/turtle/:index/swingLeft/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingLeft(getRobot(index), deg, cm, 0);
		response.end();
	});
	
	express.get('/turtle/:index/swingRight/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingRight(getRobot(index), deg, cm, speed);
		response.end();
	});
	
	express.get('/turtle/:index/swingRight/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingRight(getRobot(index), deg, cm, 0);
		response.end();
	});
	
	express.get('/turtle/:index/pulse/:pulse/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var pulse = parseFloat(request.params.pulse);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__pulse(getRobot(index), pulse, leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/turtle/:index/pulse/:pulse/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var pulse = parseFloat(request.params.pulse);
		var speed = parseFloat(request.params.speed);
		self.__pulse(getRobot(index), pulse, speed, speed);
		response.end();
	});
	
	express.get('/turtle/:index/stop', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__stop(getRobot(index));
		response.end();
	});
	
	express.get('/turtle/:index/lineTracerMode/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var mode = parseInt(request.params.mode);
		self.__lineTracerMode(getRobot(index), mode);
		response.end();
	});
	
	express.get('/turtle/:index/lineTracerGain/:gain', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var gain = parseInt(request.params.gain);
		self.__lineTracerGain(getRobot(index), gain);
		response.end();
	});
	
	express.get('/turtle/:index/lineTracerSpeed/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseInt(request.params.speed);
		self.__lineTracerSpeed(getRobot(index), speed);
		response.end();
	});
	
	express.get('/turtle/:index/led/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__led(getRobot(index), red, green, blue);
		response.end();
	});
	
	express.get('/turtle/:index/led/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var gray = parseInt(request.params.gray);
		self.__led(getRobot(index), gray, gray, gray);
		response.end();
	});
	
	express.get('/turtle/:index/buzzer/:hz', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var hz = parseFloat(request.params.hz);
		self.__buzzer(self, getRobot(index), hz);
		response.end();
	});
	
	express.get('/turtle/:index/note/:pitch', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var pitch = parseInt(request.params.pitch);
		self.__note(self, getRobot(index), pitch);
		response.end();
	});
	
	express.get('/turtle/:index/sound/:sound/:count', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var sound = parseInt(request.params.sound);
		var count = parseInt(request.params.count);
		self.__sound(self, getRobot(index), sound, count);
		response.end();
	});
	
	express.get('/turtle/:index/sound/:sound', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var sound = parseInt(request.params.sound);
		self.__sound(self, getRobot(index), sound, 1);
		response.end();
	});
	
	express.get('/turtle/:index/lamp/:onoff', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var onoff = parseInt(request.params.onoff);
		self.__lamp(getRobot(index), onoff);
		response.end();
	});
	
	express.get('/turtle/:index/lock/:onoff', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var onoff = parseInt(request.params.onoff);
		self.__lock(getRobot(index), onoff);
		response.end();
	});
	
	express.get('/turtle/:index/signalStrength', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__signalStrength(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/floor', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__floor(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/accelerationX', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationX(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/accelerationY', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationY(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/accelerationZ', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationZ(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/temperature', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__temperature(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/button', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__button(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/buttonId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__buttonId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/clickedId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__clickedId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/doubleClickedId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__doubleClickedId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/longPressedId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__longPressedId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/colorNumber', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__colorNumber(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/colorNumberId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__colorNumberId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/colorPattern', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__colorPattern(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/colorPatternId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__colorPatternId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/pulseCount', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__pulseCount(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/pulseCountId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__pulseCountId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/wheelState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__wheelState(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/wheelStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__wheelStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/soundState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__soundState(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/soundStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__soundStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/lineTracerState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__lineTracerState(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/lineTracerStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__lineTracerStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/sensors', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write(self.__sensors(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/connectionState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__connectionState(getRobot(index)));
		response.end();
	});
	
	express.get('/turtle/:index/reset', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__reset(getRobot(index), getRoute(index));
		response.end();
	});
	
	express.get('/turtle/wheels/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__wheels(getRobot(0), leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/turtle/wheels/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__wheels(getRobot(0), speed, speed);
		response.end();
	});
	
	express.get('/turtle/leftWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__leftWheel(getRobot(0), speed);
		response.end();
	});
	
	express.get('/turtle/rightWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__rightWheel(getRobot(0), speed);
		response.end();
	});
	
	express.get('/turtle/moveForward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveForward(getRobot(0), cm, speed);
		response.end();
	});
	
	express.get('/turtle/moveForward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		self.__moveForward(getRobot(0), cm, 0);
		response.end();
	});
	
	express.get('/turtle/moveBackward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveBackward(getRobot(0), cm, speed);
		response.end();
	});
	
	express.get('/turtle/moveBackward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		self.__moveBackward(getRobot(0), cm, 0);
		response.end();
	});
	
	express.get('/turtle/turnLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnLeft(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/turtle/turnLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__turnLeft(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/turtle/turnRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnRight(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/turtle/turnRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__turnRight(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/turtle/pivotLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotLeft(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/turtle/pivotLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__pivotLeft(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/turtle/pivotRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotRight(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/turtle/pivotRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__pivotRight(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/turtle/swingLeft/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingLeft(getRobot(0), deg, cm, speed);
		response.end();
	});
	
	express.get('/turtle/swingLeft/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingLeft(getRobot(0), deg, cm, 0);
		response.end();
	});
	
	express.get('/turtle/swingRight/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingRight(getRobot(0), deg, cm, speed);
		response.end();
	});
	
	express.get('/turtle/swingRight/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingRight(getRobot(0), deg, cm, 0);
		response.end();
	});
	
	express.get('/turtle/pulse/:pulse/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var pulse = parseFloat(request.params.pulse);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__pulse(getRobot(0), pulse, leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/turtle/pulse/:pulse/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var pulse = parseFloat(request.params.pulse);
		var speed = parseFloat(request.params.speed);
		self.__pulse(getRobot(0), pulse, speed, speed);
		response.end();
	});
	
	express.get('/turtle/stop', function(request, response) {
		response.writeHead(200, HEADER);
		self.__stop(getRobot(0));
		response.end();
	});
	
	express.get('/turtle/lineTracerMode/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var mode = parseInt(request.params.mode);
		self.__lineTracerMode(getRobot(0), mode);
		response.end();
	});
	
	express.get('/turtle/lineTracerGain/:gain', function(request, response) {
		response.writeHead(200, HEADER);
		var gain = parseInt(request.params.gain);
		self.__lineTracerGain(getRobot(0), gain);
		response.end();
	});
	
	express.get('/turtle/lineTracerSpeed/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseInt(request.params.speed);
		self.__lineTracerSpeed(getRobot(0), speed);
		response.end();
	});
	
	express.get('/turtle/led/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__led(getRobot(0), red, green, blue);
		response.end();
	});
	
	express.get('/turtle/led/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var gray = parseInt(request.params.gray);
		self.__led(getRobot(0), gray, gray, gray);
		response.end();
	});
	
	express.get('/turtle/buzzer/:hz', function(request, response) {
		response.writeHead(200, HEADER);
		var hz = parseFloat(request.params.hz);
		self.__buzzer(self, getRobot(0), hz);
		response.end();
	});
	
	express.get('/turtle/note/:pitch', function(request, response) {
		response.writeHead(200, HEADER);
		var pitch = parseInt(request.params.pitch);
		self.__note(self, getRobot(0), pitch);
		response.end();
	});
	
	express.get('/turtle/sound/:sound/:count', function(request, response) {
		response.writeHead(200, HEADER);
		var sound = parseInt(request.params.sound);
		var count = parseInt(request.params.count);
		self.__sound(self, getRobot(0), sound, count);
		response.end();
	});
	
	express.get('/turtle/sound/:sound', function(request, response) {
		response.writeHead(200, HEADER);
		var sound = parseInt(request.params.sound);
		self.__sound(self, getRobot(0), sound, 1);
		response.end();
	});
	
	express.get('/turtle/lamp/:onoff', function(request, response) {
		response.writeHead(200, HEADER);
		var onoff = parseInt(request.params.onoff);
		self.__lamp(getRobot(0), onoff);
		response.end();
	});
	
	express.get('/turtle/lock/:onoff', function(request, response) {
		response.writeHead(200, HEADER);
		var onoff = parseInt(request.params.onoff);
		self.__lock(getRobot(0), onoff);
		response.end();
	});
	
	express.get('/turtle/signalStrength', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__signalStrength(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/floor', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__floor(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/accelerationX', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationX(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/accelerationY', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationY(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/accelerationZ', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationZ(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/temperature', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__temperature(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/button', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__button(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/buttonId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__buttonId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/clickedId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__clickedId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/doubleClickedId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__doubleClickedId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/longPressedId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__longPressedId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/colorNumber', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__colorNumber(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/colorNumberId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__colorNumberId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/colorPattern', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__colorPattern(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/colorPatternId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__colorPatternId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/pulseCount', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__pulseCount(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/pulseCountId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__pulseCountId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/wheelState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__wheelState(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/wheelStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__wheelStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/soundState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__soundState(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/soundStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__soundStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/lineTracerState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__lineTracerState(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/lineTracerStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__lineTracerStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/sensors', function(request, response) {
		response.writeHead(200, HEADER);
		response.write(self.__sensors(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/connectionState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__connectionState(getRobot(0)));
		response.end();
	});
	
	express.get('/turtle/reset', function(request, response) {
		response.writeHead(200, HEADER);
		self.__reset(getRobot(0), getRoute(0));
		response.end();
	});*/
};
/*
Component.prototype.__wheels = function(robot, leftSpeed, rightSpeed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		motoring.motion.write(0);
		if(typeof leftSpeed == 'number') {
			motoring.leftWheel.write(leftSpeed);
		}
		if(typeof rightSpeed == 'number') {
			motoring.rightWheel.write(rightSpeed);
		}
	}
};

Component.prototype.__leftWheel = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		motoring.motion.write(0);
		if(typeof speed == 'number') {
			motoring.leftWheel.write(speed);
		}
	}
};

Component.prototype.__rightWheel = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		motoring.motion.write(0);
		if(typeof speed == 'number') {
			motoring.rightWheel.write(speed);
		}
	}
};

Component.prototype.__moveForward = function(robot, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof cm == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: cm
			motoring.motion.write(2, speed);
			if(cm < 0) {
				motoring.motion.write(3, -cm);
				motoring.motion.write(0, Turtle.MOTION_MOVE_BACKWARD);
			} else {
				motoring.motion.write(3, cm);
				motoring.motion.write(0, Turtle.MOTION_MOVE_FORWARD);
			}
		}
	}
};

Component.prototype.__moveBackward = function(robot, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof cm == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: cm
			motoring.motion.write(2, speed);
			if(cm < 0) {
				motoring.motion.write(3, -cm);
				motoring.motion.write(0, Turtle.MOTION_MOVE_FORWARD);
			} else {
				motoring.motion.write(3, cm);
				motoring.motion.write(0, Turtle.MOTION_MOVE_BACKWARD);
			}
		}
	}
};

Component.prototype.__turnLeft = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, Turtle.MOTION_TURN_RIGHT);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, Turtle.MOTION_TURN_LEFT);
			}
		}
	}
};

Component.prototype.__turnRight = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, Turtle.MOTION_TURN_LEFT);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, Turtle.MOTION_TURN_RIGHT);
			}
		}
	}
};

Component.prototype.__pivotLeft = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, Turtle.MOTION_PIVOT_LEFT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, Turtle.MOTION_PIVOT_LEFT_HEAD);
			}
		}
	}
};

Component.prototype.__pivotRight = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, Turtle.MOTION_PIVOT_RIGHT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, Turtle.MOTION_PIVOT_RIGHT_HEAD);
			}
		}
	}
};

Component.prototype.__swingLeft = function(robot, deg, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof cm == 'number') && cm >= 0 && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			motoring.motion.write(4, cm); // radius
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, Turtle.MOTION_SWING_LEFT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, Turtle.MOTION_SWING_LEFT_HEAD);
			}
		}
	}
};

Component.prototype.__swingRight = function(robot, deg, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof cm == 'number') && cm >= 0 && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			motoring.motion.write(4, cm); // radius
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, Turtle.MOTION_SWING_RIGHT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, Turtle.MOTION_SWING_RIGHT_HEAD);
			}
		}
	}
};

Component.prototype.__pulse = function(robot, pulse, leftSpeed, rightSpeed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		motoring.motion.write(0);
		if(typeof leftSpeed == 'number') {
			motoring.leftWheel.write(leftSpeed);
		}
		if(typeof rightSpeed == 'number') {
			motoring.rightWheel.write(rightSpeed);
		}
		if(typeof pulse == 'number') {
			motoring.pulse.write(pulse);
		}
	}
};

Component.prototype.__stop = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		motoring.motion.write(0);
	}
};

Component.prototype.__lineTracerMode = function(robot, mode) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.motion.write(0);
		if(typeof mode == 'number') {
			motoring.lineTracerMode.write(mode);
		}
	}
};

Component.prototype.__lineTracerGain = function(robot, gain) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof gain == 'number') {
			motoring.lineTracerGain.write(gain);
		}
	}
};

Component.prototype.__lineTracerSpeed = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof speed == 'number') {
			motoring.lineTracerSpeed.write(speed);
		}
	}
};

Component.prototype.__led = function(robot, red, green, blue) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof red == 'number') {
			motoring.led.write(0, red);
		}
		if(typeof green == 'number') {
			motoring.led.write(1, green);
		}
		if(typeof blue == 'number') {
			motoring.led.write(2, blue);
		}
	}
};

Component.prototype.__buzzer = function(self, robot, hz) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof hz == 'number') {
			motoring.buzzer.write(hz);
		}
		motoring.note.write(0);
		self.runSound(robot, 0);
	}
};

Component.prototype.__note = function(self, robot, pitch) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.buzzer.write(0);
		if(typeof pitch == 'number') {
			motoring.note.write(pitch);
		}
		self.runSound(robot, 0);
	}
};

Component.prototype.runSound = function(robot, sound, count) {
	if(typeof count != 'number') count = 1;
	if(count < 0) count = -1;
	if(count) {
		robot.soundId = sound;
		robot.soundRepeat = count;
		robot.motoring.sound.write(sound);
	}
};

Component.prototype.__sound = function(self, robot, sound, count) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.buzzer.write(0);
		motoring.note.write(0);
		if((typeof sound == 'number') && count) {
			self.runSound(robot, sound, count);
		}
	}
};

Component.prototype.__lamp = function(robot, onoff) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof onoff == 'number') {
			motoring.lamp.write(onoff);
		}
	}
};

Component.prototype.__lock = function(robot, onoff) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof onoff == 'number') {
			motoring.lock.write(onoff);
		}
	}
};

Component.prototype.__signalStrength = function(robot) {
	if(robot) {
		return robot.sensory.signalStrength;
	}
};

Component.prototype.__floor = function(robot) {
	if(robot) {
		return robot.sensory.floor;
	}
};

Component.prototype.__accelerationX = function(robot) {
	if(robot) {
		return robot.sensory.accelerationX;
	}
};

Component.prototype.__accelerationY = function(robot) {
	if(robot) {
		return robot.sensory.accelerationY;
	}
};

Component.prototype.__accelerationZ = function(robot) {
	if(robot) {
		return robot.sensory.accelerationZ;
	}
};

Component.prototype.__temperature = function(robot) {
	if(robot) {
		return robot.sensory.temperature;
	}
};

Component.prototype.__button = function(robot) {
	if(robot) {
		return robot.sensory.button;
	}
};

Component.prototype.__buttonId = function(robot) {
	if(robot) {
		return robot.sensory.buttonId;
	}
};

Component.prototype.__clickedId = function(robot) {
	if(robot) {
		return robot.sensory.clickedId;
	}
};

Component.prototype.__doubleClickedId = function(robot) {
	if(robot) {
		return robot.sensory.doubleClickedId;
	}
};

Component.prototype.__longPressedId = function(robot) {
	if(robot) {
		return robot.sensory.longPressedId;
	}
};

Component.prototype.__colorNumber = function(robot) {
	if(robot) {
		return robot.sensory.colorNumber;
	}
};

Component.prototype.__colorNumberId = function(robot) {
	if(robot) {
		return robot.sensory.colorNumberId;
	}
};

Component.prototype.__colorPattern = function(robot) {
	if(robot) {
		return robot.sensory.colorPattern;
	}
};

Component.prototype.__colorPatternId = function(robot) {
	if(robot) {
		return robot.sensory.colorPatternId;
	}
};

Component.prototype.__pulseCount = function(robot) {
	if(robot) {
		return robot.sensory.pulseCount;
	}
};

Component.prototype.__pulseCountId = function(robot) {
	if(robot) {
		return robot.sensory.pulseCountId;
	}
};

Component.prototype.__wheelState = function(robot) {
	if(robot) {
		return robot.sensory.wheelState;
	}
};

Component.prototype.__wheelStateId = function(robot) {
	if(robot) {
		return robot.sensory.wheelStateId;
	}
};

Component.prototype.__soundState = function(robot) {
	if(robot) {
		return robot.sensory.soundState;
	}
};

Component.prototype.__soundStateId = function(robot) {
	if(robot) {
		return robot.sensory.soundStateId;
	}
};

Component.prototype.__lineTracerState = function(robot) {
	if(robot) {
		return robot.sensory.lineTracerState;
	}
};

Component.prototype.__lineTracerStateId = function(robot) {
	if(robot) {
		return robot.sensory.lineTracerStateId;
	}
};

Component.prototype.__sensors = function(robot) {
	if(robot) {
		try {
			return JSON.stringify(robot.sensory);
		} catch (e) {
		}
	}
};

Component.prototype.__connectionState = function(robot) {
	if(robot) {
		return robot.connectionState;
	}
};

Component.prototype.__reset = function(robot, route) {
	if(robot) {
		robot.soundId = 0;
		robot.soundRepeat = 1;
	}
	if(route) {
		router.resetRoute(route);
	}
};
*/
Component.prototype.setState = function(robot, state) {
	robot.connectionState = state;
};

Component.prototype.handleSensory = function(robot, data, state) {
	robot.connectionState = state;
	/*var sensory = robot.sensory;
	sensory.signalStrength = data.signalStrength;
	sensory.floor = data.floor;
	sensory.accelerationX = data.acceleration.x;
	sensory.accelerationY = data.acceleration.y;
	sensory.accelerationZ = data.acceleration.z;
	sensory.temperature = data.temperature;
	if(data.map & 0x00001000) {
		sensory.button = data.button;
		sensory.buttonId = (sensory.buttonId % 255) + 1;
	}
	if(data.map & 0x00000800) {
		sensory.clickedId = (sensory.clickedId % 255) + 1;
	}
	if(data.map & 0x00000400) {
		sensory.doubleClickedId = (sensory.doubleClickedId % 255) + 1;
	}
	if(data.map & 0x00000200) {
		sensory.longPressedId = (sensory.longPressedId % 255) + 1;
	}
	if(data.map & 0x00000100) {
		sensory.colorNumber = data.colorNumber;
		sensory.colorNumberId = (sensory.colorNumberId % 255) + 1;
	}
	if(data.map & 0x00000080) {
		sensory.colorPattern = data.colorPattern;
		sensory.colorPatternId = (sensory.colorPatternId % 255) + 1;
	}
	if(data.map & 0x00000040) {
		sensory.pulseCount = data.pulseCount;
		sensory.pulseCountId = (sensory.pulseCountId % 255) + 1;
	}
	if(data.map & 0x00000020) {
		sensory.wheelState = data.wheelState;
		sensory.wheelStateId = (sensory.wheelStateId % 255) + 1;
	}
	if(data.map & 0x00000010) {
		if(data.soundState == 0) {
			if(robot.soundId > 0) {
				if(robot.soundRepeat < 0) {
					this.runSound(robot, robot.soundId, -1);
				} else if(robot.soundRepeat > 1) {
					robot.soundRepeat --;
					this.runSound(robot, robot.soundId, robot.soundRepeat);
				} else {
					robot.soundId = 0;
					robot.soundRepeat = 1;
					sensory.soundState = data.soundState;
					sensory.soundStateId = (sensory.soundStateId % 255) + 1;
				}
			} else {
				robot.soundId = 0;
				robot.soundRepeat = 1;
				sensory.soundState = data.soundState;
				sensory.soundStateId = (sensory.soundStateId % 255) + 1;
			}
		} else if(data.soundState != sensory.soundState) {
			sensory.soundState = data.soundState;
			sensory.soundStateId = (sensory.soundStateId % 255) + 1;
		}
	}
	if(data.map & 0x00000008) {
		sensory.lineTracerState = data.lineTracerState;
		sensory.lineTracerStateId = (sensory.lineTracerStateId % 255) + 1;
	}*/
};

module.exports = new Component();