'use strict';

const LED_PER_BLOCK = 4;

const TYPE_GRB = 0;
const TYPE_GRBW = 1;

const COLOR_TYPE_GRB = 0;

function Neopixel() {
	this.packet = new Array(20);
	this.wrgb = new Array(4);
	this.reset();
}

Neopixel.prototype.clear = function() {
};

Neopixel.prototype.reset = function() {
	var packet = this.packet;
	for(var i = 0; i < 20; ++i) {
		packet[i] = 0;
	}
	this.auto = true;
	this.hasWhite = false;
	this.colorType = COLOR_TYPE_GRB;
	this.leds = undefined;
	this.numLeds = 0;
	this.numBlocks = 0;
	this.brightness = 7;
	this.brightnessPerc = 50;
	this.startBlock = 0;
	this.endBlock = 0;
	this.currentBlock = 0;
	this.next = false;
};

Neopixel.prototype.setType = function(type) {
	switch(type) {
		case TYPE_GRB:
			this.hasWhite = false;
			break;
		case TYPE_GRBW:
			this.hasWhite = true;
			break;
	}
	switch(type) {
		case TYPE_GRB:
		case TYPE_GRBW:
			this.colorType = COLOR_TYPE_GRB;
			break;
	}
};

Neopixel.prototype.setNumLeds = function(value) {
	if(value < 0) value = 0;
	else if(value > 144) value = 144;
	this.numLeds = value;
	this.numBlocks = parseInt((value + (LED_PER_BLOCK-1)) / LED_PER_BLOCK);
	
	var leds = new Array(value);
	var led;
	for(var i = 0; i < value; ++i) {
		led = leds[i] = new Array(7);
		for(var j = 0; j < 7; ++j) {
			led[j] = 0;
		}
	}
	this.leds = leds;
};
/*
Neopixel.prototype.checkRgb = function(wrgb, r, g, b, bright, hasWhite) {
	var rf = r / 255.0;
	var gf = g / 255.0;
	var bf = b / 255.0;
	var max = Math.max(rf, gf, bf);
	var min = Math.min(rf, gf, bf);
	var c = max - min;
	var l = (max + min) * bright / 200.0;
	
	var h, s;
	if(max == min) {
		h = s = 0;
	} else {
		if(max == rf) {
			h = ((gf - bf) / c) % 6;
		} else if (max == gf) {
			h = ((bf - rf) / c) + 2;
		} else {
			h = ((rf - gf) / c) + 4;
		}
		if(l == 1) {
			s = 0;
		} else {
			s = c / (1 - Math.abs(2 * l - 1));
		}
	}
	h = (h * 60) % 360;
	if(h < 0) {
		h += 360;
	}
	
	c = (1 - Math.abs(2 * l - 1)) * s;
	var m = l - 0.5 * c;
	var x = c * (1 - Math.abs((h / 60.0 % 2) - 1));
	var hueSegment = parseInt(h / 60.0);
	switch(hueSegment) {
		case 0:
			r = Math.round(255 * (c + m));
			g = Math.round(255 * (x + m));
			b = Math.round(255 * m);
			break;
		case 1:
			r = Math.round(255 * (x + m));
			g = Math.round(255 * (c + m));
			b = Math.round(255 * m);
			break;
		case 2:
			r = Math.round(255 * m);
			g = Math.round(255 * (c + m));
			b = Math.round(255 * (x + m));
			break;
		case 3:
			r = Math.round(255 * m);
			g = Math.round(255 * (x + m));
			b = Math.round(255 * (c + m));
			break;
		case 4:
			r = Math.round(255 * (x + m));
			g = Math.round(255 * m);
			b = Math.round(255 * (c + m));
			break;
		case 5:
		case 6:
			r = Math.round(255 * (c + m));
			g = Math.round(255 * m);
			b = Math.round(255 * (x + m));
			break;
		default:
			r = 0; g = 0; b = 0;
			break;
	}
	if(hasWhite && r == g && r == b) {
		wrgb[0] = (r >> 4) & 0x0f;
		wrgb[1] = 0;
		wrgb[2] = 0;
		wrgb[3] = 0;
	} else {
		wrgb[0] = 0;
		wrgb[1] = (r >> 4) & 0x0f;
		wrgb[2] = (g >> 4) & 0x0f;
		wrgb[3] = (b >> 4) & 0x0f;
	}
};
*/
Neopixel.prototype.checkRgb = function(wrgb, r, g, b, bright, hasWhite) {
	r = parseInt(r * bright / 100.0);
	g = parseInt(g * bright / 100.0);
	b = parseInt(b * bright / 100.0);
	
	if(r < 0) r = 0;
	else if(r > 255) r = 255;
	if(g < 0) g = 0;
	else if(g > 255) g = 255;
	if(b < 0) b = 0;
	else if(b > 255) b = 255;
	
	if(hasWhite && r == g && r == b) {
		wrgb[0] = r;
		wrgb[1] = 0;
		wrgb[2] = 0;
		wrgb[3] = 0;
	} else {
		wrgb[0] = 0;
		wrgb[1] = r;
		wrgb[2] = g;
		wrgb[3] = b;
	}
};

Neopixel.prototype.setAllRgb = function(r, g, b) {
	var leds = this.leds;
	if(leds) {
		if(r < 0) r = 0;
		else if(r > 255) r = 255;
		if(g < 0) g = 0;
		else if(g > 255) g = 255;
		if(b < 0) b = 0;
		else if(b > 255) b = 255;
		var wrgb = this.wrgb;
		this.checkRgb(wrgb, r, g, b, this.brightnessPerc, this.hasWhite);
		var w = wrgb[0];
		var rr = wrgb[1];
		var gg = wrgb[2];
		var bb = wrgb[3];
		var led;
		for(var i in leds) {
			led = leds[i];
			if(led) {
				led[0] = r;
				led[1] = g;
				led[2] = b;
				led[3] = w;
				led[4] = rr;
				led[5] = gg;
				led[6] = bb;
			}
		}
		this.startBlock = 0;
		this.endBlock = this.numBlocks - 1;
		this.currentBlock = 0;
	}
};

Neopixel.prototype.changeAllRgb = function(r, g, b) {
	var leds = this.leds;
	if(leds) {
		var wrgb = this.wrgb;
		var hasWhite = this.hasWhite;
		var bright = this.brightnessPerc;
		var led, rr, gg, bb;
		for(var i in leds) {
			led = leds[i];
			if(led) {
				rr = r + led[0];
				gg = g + led[1];
				bb = b + led[2];
				if(rr < 0) rr = 0;
				else if(rr > 255) rr = 255;
				if(gg < 0) gg = 0;
				else if(gg > 255) gg = 255;
				if(bb < 0) bb = 0;
				else if(bb > 255) bb = 255;
				this.checkRgb(wrgb, rr, gg, bb, bright, hasWhite);
				led[0] = rr;
				led[1] = gg;
				led[2] = bb;
				led[3] = wrgb[0];
				led[4] = wrgb[1];
				led[5] = wrgb[2];
				led[6] = wrgb[3];
			}
		}
		this.startBlock = 0;
		this.endBlock = this.numBlocks - 1;
		this.currentBlock = 0;
	}
};

Neopixel.prototype.setRgbAt = function(index, r, g, b) {
	if(index < 0 || index >= this.numLeds) return;
	
	var led = this.leds[index];
	if(led) {
		if(r < 0) r = 0;
		else if(r > 255) r = 255;
		if(g < 0) g = 0;
		else if(g > 255) g = 255;
		if(b < 0) b = 0;
		else if(b > 255) b = 255;
		var wrgb = this.wrgb;
		this.checkRgb(wrgb, r, g, b, this.brightnessPerc, this.hasWhite);
		led[0] = r;
		led[1] = g;
		led[2] = b;
		led[3] = wrgb[0];
		led[4] = wrgb[1];
		led[5] = wrgb[2];
		led[6] = wrgb[3];
		
		this.startBlock = this.endBlock = this.currentBlock = parseInt(index / LED_PER_BLOCK);
	}
};

Neopixel.prototype.changeRgbAt = function(index, r, g, b) {
	if(index < 0 || index >= this.numLeds) return;
	
	var led = this.leds[index];
	if(led) {
		r += led[0];
		g += led[1];
		b += led[2];
		if(r < 0) r = 0;
		else if(r > 255) r = 255;
		if(g < 0) g = 0;
		else if(g > 255) g = 255;
		if(b < 0) b = 0;
		else if(b > 255) b = 255;
		var wrgb = this.wrgb;
		this.checkRgb(wrgb, r, g, b, this.brightnessPerc, this.hasWhite);
		led[0] = r;
		led[1] = g;
		led[2] = b;
		led[3] = wrgb[0];
		led[4] = wrgb[1];
		led[5] = wrgb[2];
		led[6] = wrgb[3];
		
		this.startBlock = this.endBlock = this.currentBlock = parseInt(index / LED_PER_BLOCK);
	}
};

Neopixel.prototype.setRgbFromTo = function(from, to, interval, r, g, b) {
	if(from < 0 || from >= this.numLeds) return;
	if(to < 0 || to >= this.numLeds) return;
	
	var leds = this.leds;
	if(leds) {
		if(r < 0) r = 0;
		else if(r > 255) r = 255;
		if(g < 0) g = 0;
		else if(g > 255) g = 255;
		if(b < 0) b = 0;
		else if(b > 255) b = 255;
		var wrgb = this.wrgb;
		this.checkRgb(wrgb, r, g, b, this.brightnessPerc, this.hasWhite);
		var w = wrgb[0];
		var rr = wrgb[1];
		var gg = wrgb[2];
		var bb = wrgb[3];
		var led;
		for(var i = from; i <= to; i += interval) {
			led = leds[i];
			if(led) {
				led[0] = r;
				led[1] = g;
				led[2] = b;
				led[3] = w;
				led[4] = rr;
				led[5] = gg;
				led[6] = bb;
			}
		}
		this.startBlock = this.currentBlock = parseInt(from / LED_PER_BLOCK);
		this.endBlock = parseInt(to / LED_PER_BLOCK);
	}
};

Neopixel.prototype.changeRgbFromTo = function(from, to, interval, r, g, b) {
	if(from < 0 || from >= this.numLeds) return;
	if(to < 0 || to >= this.numLeds) return;
	
	var leds = this.leds;
	if(leds) {
		var wrgb = this.wrgb;
		var hasWhite = this.hasWhite;
		var bright = this.brightnessPerc;
		var led, w, rr, gg, bb;
		for(var i = from; i <= to; i += interval) {
			led = leds[i];
			if(led) {
				rr = r + led[0];
				gg = g + led[1];
				bb = b + led[2];
				if(rr < 0) rr = 0;
				else if(rr > 255) rr = 255;
				if(gg < 0) gg = 0;
				else if(gg > 255) gg = 255;
				if(bb < 0) bb = 0;
				else if(bb > 255) bb = 255;
				this.checkRgb(wrgb, rr, gg, bb, bright, hasWhite);
				led[0] = rr;
				led[1] = gg;
				led[2] = bb;
				led[3] = wrgb[0];
				led[4] = wrgb[1];
				led[5] = wrgb[2];
				led[6] = wrgb[3];
			}
		}
		this.startBlock = this.currentBlock = parseInt(from / LED_PER_BLOCK);
		this.endBlock = parseInt(to / LED_PER_BLOCK);
	}
};

Neopixel.prototype.clearAll = function() {
	var leds = this.leds;
	if(leds) {
		var led;
		for(var i in leds) {
			led = leds[i];
			if(led) {
				led[0] = 0;
				led[1] = 0;
				led[2] = 0;
				led[3] = 0;
				led[4] = 0;
				led[5] = 0;
				led[6] = 0;
			}
		}
		this.startBlock = 0;
		this.endBlock = this.numBlocks - 1;
		this.currentBlock = 0;
	}
};

Neopixel.prototype.clearAt = function(index) {
	if(index < 0 || index >= this.numLeds) return;
	
	var led = this.leds[index];
	if(led) {
		led[0] = 0;
		led[1] = 0;
		led[2] = 0;
		led[3] = 0;
		led[4] = 0;
		led[5] = 0;
		led[6] = 0;
		
		this.startBlock = this.endBlock = this.currentBlock = parseInt(index / LED_PER_BLOCK);
	}
};

Neopixel.prototype.clearFromTo = function(from, to, interval) {
	if(from < 0 || from >= this.numLeds) return;
	if(to < 0 || to >= this.numLeds) return;
	
	var leds = this.leds;
	if(leds) {
		var led;
		for(var i = from; i <= to; i += interval) {
			led = leds[i];
			if(led) {
				led[0] = 0;
				led[1] = 0;
				led[2] = 0;
				led[3] = 0;
				led[4] = 0;
				led[5] = 0;
				led[6] = 0;
			}
		}
		this.startBlock = this.currentBlock = parseInt(from / LED_PER_BLOCK);
		this.endBlock = parseInt(to / LED_PER_BLOCK);
	}
};

Neopixel.prototype.changeBrightnessBy = function(value) {
	value += this.brightnessPerc;
	if(value < 0) value = 0;
	else if(value > 100) value = 100;
	this.brightnessPerc = value;
	this.brightness = parseInt(value * 0.15);
	
	this.startBlock = 0;
	this.endBlock = this.numBlocks - 1;
	this.currentBlock = 0;
};

Neopixel.prototype.setBrightnessTo = function(value) {
	if(value < 0) value = 0;
	else if(value > 100) value = 100;
	this.brightnessPerc = value;
	this.brightness = parseInt(value * 0.15);
	
	this.startBlock = 0;
	this.endBlock = this.numBlocks - 1;
	this.currentBlock = 0;
};

Neopixel.prototype.setAuto = function(auto) {
	this.auto = auto;
};

Neopixel.prototype.update = function() {
	var leds = this.leds;
	if(leds) {
		var packet = this.packet;
		packet[0] = 0x40;
		if(this.hasWhite) packet[0] |= 0x08;
		packet[0] |= 0x04;
		
		var currentBlock = this.currentBlock;
		packet[1] = currentBlock & 0xff;
		
		var startIndex = currentBlock * LED_PER_BLOCK;
		var endIndex = Math.min(startIndex + LED_PER_BLOCK, this.numLeds);
		var led, j = 2;
		for(var i = startIndex; i < endIndex; ++i, j += 4) {
			led = leds[i];
			if(led) {
				packet[j] = led[3];
				packet[j+1] = led[4];
				packet[j+2] = led[5];
				packet[j+3] = led[6];
			}
		}
		for(; j < 20; ++j) {
			packet[j] = 0;
		}
		if(currentBlock < this.endBlock) {
			this.currentBlock ++;
			this.next = true;
		} else {
			this.next = false;
		}
		return packet;
	}
};

Neopixel.prototype.hasNext = function() {
	return this.next;
};

module.exports.create = function() {
	return new Neopixel();
};