'use strict';
var Turtle = require('../turtle');
var factory = require('../turtle-roboid');
const MODULE = 'turtle';

function Component() {
}

Component.prototype.createRobot = function() {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.sensory = {
		type: 1,
		module: MODULE,
		map: 0,
		signalStrength: 0,
		colorRed: 0,
		colorGreen: 0,
		colorBlue: 0,
		colorClear: 0,
		floor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		temperature: 0,
		button: 0,
		colorNumber: -1,
		colorPattern: -1,
		pulseCount: 0,
		wheelState: 0,
		soundState: 0,
		lineTracerState: 0
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(Turtle.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(Turtle.RIGHT_WHEEL),
		led: roboid.findDeviceById(Turtle.LED),
		buzzer: roboid.findDeviceById(Turtle.BUZZER),
		pulse: roboid.findDeviceById(Turtle.PULSE),
		note: roboid.findDeviceById(Turtle.NOTE),
		sound: roboid.findDeviceById(Turtle.SOUND),
		lineTracerMode: roboid.findDeviceById(Turtle.LINE_TRACER_MODE),
		lineTracerGain: roboid.findDeviceById(Turtle.LINE_TRACER_GAIN),
		lineTracerSpeed: roboid.findDeviceById(Turtle.LINE_TRACER_SPEED),
		lamp: roboid.findDeviceById(Turtle.LAMP),
		lock: roboid.findDeviceById(Turtle.LOCK),
		motion: roboid.findDeviceById(Turtle.MOTION)
	};
	return robot;
}

Component.prototype.handleRemoteData = function(robot, data) {
	var motoring = robot.motoring;
	if(data.leftWheel !== undefined) motoring.leftWheel.write(data.leftWheel);
	if(data.rightWheel !== undefined) motoring.rightWheel.write(data.rightWheel);
	if(data.ledRed !== undefined) motoring.led.write(0, data.ledRed);
	if(data.ledGreen !== undefined) motoring.led.write(1, data.ledGreen);
	if(data.ledBlue !== undefined) motoring.led.write(2, data.ledBlue);
	if(data.buzzer !== undefined) motoring.buzzer.write(data.buzzer);
	var map = data.map;
	if((map & 0x04000000) && (data.pulse != undefined)) motoring.pulse.write(data.pulse);
	if((map & 0x02000000) && (data.note != undefined)) motoring.note.write(data.note);
	if((map & 0x01000000) && (data.sound != undefined)) motoring.sound.write(data.sound);
	if((map & 0x00800000) && (data.lineTracerMode != undefined)) motoring.lineTracerMode.write(data.lineTracerMode);
	if((map & 0x00400000) && (data.lineTracerGain != undefined)) motoring.lineTracerGain.write(data.lineTracerGain);
	if((map & 0x00200000) && (data.lineTracerSpeed != undefined)) motoring.lineTracerSpeed.write(data.lineTracerSpeed);
	if((map & 0x00100000) && (data.lamp != undefined)) motoring.lamp.write(data.lamp);
	if((map & 0x00080000) && (data.lock != undefined)) motoring.lock.write(data.lock);
	if(map & 0x00040000) {
		if(data.motionType != undefined) motoring.motion.write(0, data.motionType);
		if(data.motionUnit != undefined) motoring.motion.write(1, data.motionUnit);
		if(data.motionSpeed != undefined) motoring.motion.write(2, data.motionSpeed);
		if(data.motionValue != undefined) motoring.motion.write(3, data.motionValue);
		if(data.motionRadius != undefined) motoring.motion.write(4, data.motionRadius);
	}
};

Component.prototype.handleSensory = function(robot, data, connection) {
	if(connection) {
		var sensory = robot.sensory;
		sensory.map = data.map;
		sensory.signalStrength = data.signalStrength;
		sensory.colorRed = data.color.r;
		sensory.colorGreen = data.color.g;
		sensory.colorBlue = data.color.b;
		sensory.colorClear = data.color.c;
		sensory.floor = data.floor;
		sensory.accelerationX = data.acceleration.x;
		sensory.accelerationY = data.acceleration.y;
		sensory.accelerationZ = data.acceleration.z;
		sensory.temperature = data.temperature;
		if(data.map & 0x00001000) sensory.button = data.button;
		if(data.map & 0x00000100) sensory.colorNumber = data.colorNumber;
		if(data.map & 0x00000080) sensory.colorPattern = data.colorPattern;
		if(data.map & 0x00000040) sensory.pulseCount = data.pulseCount;
		if(data.map & 0x00000020) sensory.wheelState = data.wheelState;
		if(data.map & 0x00000010) sensory.soundState = data.soundState;
		if(data.map & 0x00000008) sensory.lineTracerState = data.lineTracerState;
		try {
			connection.send(JSON.stringify(sensory));
		} catch (e) {
		}
	}
};

module.exports = new Component();