'use strict';
var router = require('router');
var translator = require('translator');
var component = require('./component');
const MODULE = 'cheese';

function Extension() {
	this.robot = component.createRobot(0);
}

Extension.prototype.start = function(express) {
	var self = this;
	var robot = self.robot;
	var problemMessage = '';
	
	self.running = true;
	robot.clearWaits();
	var setup = function() {
		problemMessage = '_problem ' + translator.translate('Robot is not connected.') + '\n';
		component.updateLang(translator);
	};
	translator.on('cheese-scratch', function() {
		setup();
	});
	setup();
	
	if(express) {
		const HEADER = { 'Content-Type': 'text/plain' };
		
		express.get('/reset_all', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearWaits();
			robot.clearTimeouts();
			robot.reset();
			router.reset();
			response.end();
		});
		
		express.get('/poll', function(request, response) {
			if(!self.running) {
				response.set('Connection', 'close');
			}
			response.writeHead(200, HEADER);
			var waits = robot.getWaits();
			if(waits.length > 0) {
				response.write('_busy ' + waits.join(' ') + '\n');
			}
			if(self.state == 'connected') {
				robot.writeSingleSensory(response);
			} else {
				response.write(problemMessage);
			}
			response.end();
		});
		
		express.get('/cheeseSetInputPortTo/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setInputPortTo(request);
			response.end();
		});
		
		express.get('/cheeseSetInputRangeTo/:port/:lower1/:upper1/:lower2/:upper2', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setInputRangeTo(request);
			response.end();
		});
		
		express.get('/cheeseSetInputZeroAndRangeTo/:port/:zero/:lower1/:upper1/:lower2/:upper2', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setInputZeroAndRangeTo(request);
			response.end();
		});
		
		express.get('/cheeseStartSensor/:sensor', function(request, response) {
			response.writeHead(200, HEADER);
			robot.startSensor(request);
			response.end();
		});
		
		express.get('/cheeseResetEncoder', function(request, response) {
			response.writeHead(200, HEADER);
			robot.resetEncoder(request);
			response.end();
		});
		
		express.get('/cheeseSetSoundPortTo/:port', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setSoundPortTo(request);
			response.end();
		});
		
		express.get('/cheesePlaySound/:sound', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSound(request);
			response.end();
		});
		
		express.get('/cheesePlaySoundTimes/:sound/:count', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSoundTimes(request);
			response.end();
		});
		
		express.get('/cheesePlaySoundTimesUntilDone/:id/:sound/:count', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSoundTimesUntilDone(request);
			response.end();
		});
		
		express.get('/cheeseChangeBuzzerBy/:hz', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeBuzzerBy(request);
			response.end();
		});
		
		express.get('/cheeseSetBuzzerTo/:hz', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setBuzzerTo(request);
			response.end();
		});
		
		express.get('/cheeseClearSound', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearSound(request);
			response.end();
		});
		
		express.get('/cheesePlayNote/:note/:octave', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNote(request);
			response.end();
		});
		
		express.get('/cheesePlayNoteForBeats/:id/:note/:octave/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNoteForBeats(request);
			response.end();
		});
		
		express.get('/cheeseRestForBeats/:id/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.restForBeats(request);
			response.end();
		});
		
		express.get('/cheeseChangeTempoBy/:bpm', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeTempoBy(request);
			response.end();
		});
		
		express.get('/cheeseSetTempoTo/:bpm', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setTempoTo(request);
			response.end();
		});
		
		express.get('/cheeseChangeServoMotorAngleBy/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeServoMotorAngleBy(request);
			response.end();
		});
		
		express.get('/cheeseSetServoMotorAngleTo/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setServoMotorAngleTo(request);
			response.end();
		});
		
		express.get('/cheeseTurnOffServoMotor/:port', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnOffServoMotor(request);
			response.end();
		});
		
		express.get('/cheeseChangeDcMotorVelocityBy/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeDcMotorVelocityBy(request);
			response.end();
		});
		
		express.get('/cheeseSetDcMotorVelocityTo/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setDcMotorVelocityTo(request);
			response.end();
		});
		
		express.get('/cheeseStopDcMotor/:port', function(request, response) {
			response.writeHead(200, HEADER);
			robot.stopDcMotor(request);
			response.end();
		});
		
		express.get('/cheeseRotateStepMotor/:id/:value/:velocity', function(request, response) {
			response.writeHead(200, HEADER);
			robot.rotateStepMotor(request);
			response.end();
		});
		
		express.get('/cheeseChangeStepMotorVelocityBy/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeStepMotorVelocityBy(request);
			response.end();
		});
		
		express.get('/cheeseSetStepMotorVelocityTo/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setStepMotorVelocityTo(request);
			response.end();
		});
		
		express.get('/cheeseStopOffStepMotor/:action', function(request, response) {
			response.writeHead(200, HEADER);
			robot.stopOffStepMotor(request);
			response.end();
		});
		
		express.get('/cheeseSetStepMotorModeTo/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setStepMotorModeTo(request);
			response.end();
		});
		
		express.get('/cheeseSetDigitalOutputTo/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setDigitalOutputTo(request);
			response.end();
		});
		
		express.get('/cheeseChangePwmOutputBy/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changePwmOutputBy(request);
			response.end();
		});
		
		express.get('/cheeseSetPwmOutputTo/:port/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setPwmOutputTo(request);
			response.end();
		});
		
		express.get('/cheeseSetLedTypeTo/:led/:port/:type', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedTypeTo(request);
			response.end();
		});
		
		express.get('/cheeseSetLedToColor/:led/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedToColor(request);
			response.end();
		});
		
		express.get('/cheeseChangeLedByRGB/:led/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeLedByRGB(request);
			response.end();
		});
		
		express.get('/cheeseSetLedToRGB/:led/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedToRGB(request);
			response.end();
		});
		
		express.get('/cheeseClearLed/:led', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearLed(request);
			response.end();
		});
		
		express.get('/cheeseClearLedMatrixBackground/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearLedMatrixBackground(request);
			response.end();
		});
		
		express.get('/cheeseClearLedMatrix/:id/:target', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearLedMatrix(request);
			response.end();
		});
		
		express.get('/cheeseTurnLedMatrixOnOff/:id/:x/:y/:action', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnLedMatrixOnOff(request);
			response.end();
		});
		
		express.get('/cheeseDrawLedMatrixShape/:id/:shape', function(request, response) {
			response.writeHead(200, HEADER);
			robot.drawLedMatrixShape(request);
			response.end();
		});
		
		express.get('/cheeseDrawLedMatrixShapeAt/:id/:shape/:x/:y', function(request, response) {
			response.writeHead(200, HEADER);
			robot.drawLedMatrixShapeAt(request);
			response.end();
		});
		
		express.get('/cheeseDrawLedMatrixStringAt/:id/:text/:x/:y', function(request, response) {
			response.writeHead(200, HEADER);
			robot.drawLedMatrixStringAt(request);
			response.end();
		});
		
		express.get('/cheeseDrawLedMatrixBinaryAt/:id/:value/:x/:y', function(request, response) {
			response.writeHead(200, HEADER);
			robot.drawLedMatrixBinaryAt(request);
			response.end();
		});
		
		express.get('/cheeseMoveLedMatrixTo/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveLedMatrixTo(request);
			response.end();
		});
		
		express.get('/cheeseMoveLedMatrixBackgroundBy/:id/:x/:y', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveLedMatrixBackgroundBy(request);
			response.end();
		});
		
		express.get('/cheeseMoveLedMatrixBy/:id/:target/:x/:y', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveLedMatrixBy(request);
			response.end();
		});
		
		express.get('/cheeseClearShowHideLedMatrixSprite/:id/:sprite/:action', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearShowHideLedMatrixSprite(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixSpriteShape/:id/:sprite/:shape', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixSpriteShape(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixSpriteString/:id/:sprite/:text', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixSpriteString(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixSpriteBinary/:id/:sprite/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixSpriteBinary(request);
			response.end();
		});
		
		express.get('/cheeseChangeLedMatrixSpritePositionsBy/:id/:sprite/:x/:y', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeLedMatrixSpritePositionsBy(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixSpritePositionsTo/:id/:sprite/:x/:y', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixSpritePositionsTo(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixSpritePositionTo/:id/:sprite/:position/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixSpritePositionTo(request);
			response.end();
		});
		
		express.get('/cheeseRotateLedMatrixSprite/:id/:sprite/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.rotateLedMatrixSprite(request);
			response.end();
		});
		
		express.get('/cheeseFlipLedMatrixSprite/:id/:sprite/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.flipLedMatrixSprite(request);
			response.end();
		});
		
		express.get('/cheeseStampLedMatrixSprite/:id/:sprite', function(request, response) {
			response.writeHead(200, HEADER);
			robot.stampLedMatrixSprite(request);
			response.end();
		});
		
		express.get('/cheeseChangeLedMatrixBrightnessBy/:id/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeLedMatrixBrightnessBy(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixBrightnessTo/:id/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixBrightnessTo(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixDirectionTo/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixDirectionTo(request);
			response.end();
		});
		
		express.get('/cheeseSetLedMatrixModeTo/:id/:auto', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedMatrixModeTo(request);
			response.end();
		});
		
		express.get('/cheeseUpdateLedMatrix/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.updateLedMatrix(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetNumberAndTypeTo/:id/:num/:type', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetNumberAndTypeTo(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetAllLedsToColor/:id/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetAllLedsToColor(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelChangeAllLedsByRGB/:id/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelChangeAllLedsByRGB(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetAllLedsToRGB/:id/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetAllLedsToRGB(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelClearAllLeds/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelClearAllLeds(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetLedAtToColor/:id/:pos/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetLedAtToColor(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelChangeLedAtByRGB/:id/:pos/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelChangeLedAtByRGB(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetLedAtToRGB/:id/:pos/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetLedAtToRGB(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelClearLedAt/:id/:pos', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelClearLedAt(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetLedFromToColor/:id/:from/:to/:interval/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetLedFromToColor(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelChangeLedFromByRGB/:id/:from/:to/:interval/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelChangeLedFromByRGB(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetLedFromToRGB/:id/:from/:to/:interval/:r/:g/:b', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetLedFromToRGB(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelClearLedFromTo/:id/:from/:to/:interval', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelClearLedFromTo(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelChangeBrightnessBy/:id/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelChangeBrightnessBy(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetBrightnessTo/:id/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetBrightnessTo(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelSetModeTo/:id/:auto', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelSetModeTo(request);
			response.end();
		});
		
		express.get('/cheeseNeopixelUpdate/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.neopixelUpdate(request);
			response.end();
		});
		
		express.get('/cheeseWriteSerial/:id/:mode/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.writeSerial(request);
			response.end();
		});
		
		express.get('/cheeseReadSerial/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.readSerial(request);
			response.end();
		});
		
		express.get('/cheeseReadSerialUntil/:id/:delimiter', function(request, response) {
			response.writeHead(200, HEADER);
			robot.readSerialUntil(request);
			response.end();
		});
	}
};

Extension.prototype.stop = function() {
	this.running = false;
	translator.remove('cheese-scratch');
	router.reset();
};

Extension.prototype.setState = function(route) {
	if(route == router.selectedRoute && route.module == MODULE) {
		this.state = route.state;
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleSensory = function(data, route) {
	if(route.module == MODULE) {
		this.robot.handleSensory(data);
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	if(route.module == MODULE) {
		var roboid = this.robot.roboid;
		roboid.handleMotoring(data);
		roboid.updateMotoring();
	}
};

Extension.prototype.requestMotoring = function(route) {
	if(route.module == MODULE) {
		return this.robot.roboid.requestMotoring();
	}
};

Extension.prototype.reset = function(route) {
	if(route == router.selectedRoute && route.module == MODULE) {
		this.robot.roboid.reset();
		this.robot.motoring.reset();
	}
};

module.exports.create = function() {
	return new Extension();
};