'use strict';
var router = require('router');
var component = require('./component');
const MODULE = 'turtle';

function Extension() {
	this.robot = component.createRobot(0);
	this.connectionPacket = {
		type: 0,
		module: MODULE,
		state: 0
	};
	this.masterVersion = 1;
}

Extension.prototype.start = function(connection) {
	var self = this;
	self.connection = connection;
	connection.on('message', function(message) {
		if(message.type === 'utf8') {
			self.handleRemoteData(message.utf8Data);
		}
	});
};

Extension.prototype.stop = function() {
	this.connection = undefined;
	router.reset();
};

Extension.prototype.setState = function(route) {
	if(route == router.selectedRoute && route.module == MODULE && this.connection) {
		this.connectionPacket.state = router.getConnectionState(route.state);
		try {
			this.connection.send(JSON.stringify(this.connectionPacket));
		} catch (e) {
		}
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleRemoteDataByModule = function(data) {
	if(data.module == MODULE) {
		component.handleRemoteData(this.robot, data);
	}
};

Extension.prototype.handleRemoteData = function(utf8Data) {
	try {
		var received = JSON.parse(utf8Data);
		var version = received.version || 0;
		if(version == 1) {
			for(var i in received) {
				this.handleRemoteDataByModule(received[i]);
			}
		} else {
			this.handleRemoteDataByModule(received);
		}
		this.masterVersion = version;
	} catch (e) {
	}
};

Extension.prototype.handleSensory = function(data, route) {
	if(route.module == MODULE) {
		var packet = {};
		component.handleSensory(packet, this.robot, data);
		if(this.masterVersion == 0) {
			packet = packet.robot;
		}
		packet.type = 1;
		packet.version = 1;
		if(this.connection) {
			try {
				this.connection.send(JSON.stringify(packet));
			} catch (e) {
			}
		}
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	if(route.module == MODULE) {
		var roboid = this.robot.roboid;
		roboid.handleMotoring(data);
		roboid.updateMotoring();
	}
};

Extension.prototype.requestMotoring = function(route) {
	if(route.module == MODULE) {
		return this.robot.roboid.requestMotoring();
	}
};

Extension.prototype.reset = function(route) {
	if(route == router.selectedRoute && route.module == MODULE) {
		this.robot.roboid.reset();
	}
};

module.exports.create = function() {
	return new Extension();
};