'use strict';

function Component() {
	this.chat = undefined;
	this.chatSocket = undefined;
}

Component.prototype.getChat = function() {
	var chat = this.chat;
	if(chat === undefined) {
		chat = {
			state: 0,
			message: {}
		};
		this.chat = chat;
	}
	return chat;
};

Component.prototype.chatConnectTo = function(ip, port, name) {
	if('WebSocket' in window) {
		var self = this;
		self.chatDisconnect();
		try {
			var sock = new window.WebSocket('ws://' + ip + ':' + port);
			sock.binaryType = 'arraybuffer';
			sock.onopen = function() {
				self.chatSocket = sock;
				sock.onmessage = function(message) {
					try {
						var data = JSON.parse(message.data);
						if(data.command === 'send' || data.command === 'broadcast') {
							var chat = self.getChat();
							for(var i in data.message) {
								chat.message[i] = data.message[i];
							}
						}
					} catch (e) {
					}
				};
				sock.onclose = function() {
					self.chatSocket = undefined;
				};
				self.chatSend({
					command: 'register',
					name: name
				});
				var chat = self.getChat();
				chat.state = 1;
			};
		} catch (e) {
		}
	}
};

Component.prototype.chatDisconnect = function() {
	if(this.chatSocket) {
		this.chatSocket.close();
		this.chatSocket = undefined;
	}
};

Component.prototype.chatSend = function(data) {
	if(this.chatSocket) {
		try {
			this.chatSocket.send(JSON.stringify(data));
		} catch (e) {
		}
	}
};

Component.prototype.handleRemoteChat = function(data) {
	if(data) {
		if(data.command == 'connect') {
			if(data.param) {
				var param = data.param;
				this.chatConnectTo(param.ip, param.port, param.name);
			}
		} else if(data.command == 'disconnect') {
			this.chatDisconnect();
		} else if(data.command == 'send') {
			var param = data.param;
			this.chatSend({ command: 'send', to: param.to, message: param.message });
		} else if(data.command == 'broadcast') {
			var param = data.param;
			this.chatSend({ command: 'broadcast', message: param.message });
		}
	}
};

Component.prototype.prepareSensory = function(sensory) {
	sensory.chat = this.chat;
};

Component.prototype.updateSensory = function() {
	this.chat = undefined;
};

module.exports = new Component();