'use strict';
var context = require('context');
var Hamster = require('../hamster');
var factory = require('../hamster-roboid');
const MODULE = 'hamster';

function Component() {
}

Component.prototype.createRobot = function(index) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.sensory = {
		version: 0x01,
		company: 0x02,
		model: 0x04,
		variation: 0x02,
		network: 0x00,
		protocol: 'json',
		group: 'hamster',
		module: 'hamster',
		index: index,
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		tilt: 0,
		lineTracerState: 0,
		lineTracerStateId: 0,
		batteryState: 2,
		handFound: false
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(Hamster.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(Hamster.RIGHT_WHEEL),
		buzzer: roboid.findDeviceById(Hamster.BUZZER),
		outputA: roboid.findDeviceById(Hamster.OUTPUT_A),
		outputB: roboid.findDeviceById(Hamster.OUTPUT_B),
		topology: roboid.findDeviceById(Hamster.TOPOLOGY),
		leftLed: roboid.findDeviceById(Hamster.LEFT_LED),
		rightLed: roboid.findDeviceById(Hamster.RIGHT_LED),
		note: roboid.findDeviceById(Hamster.NOTE),
		lineTracerMode: roboid.findDeviceById(Hamster.LINE_TRACER_MODE),
		lineTracerSpeed: roboid.findDeviceById(Hamster.LINE_TRACER_SPEED),
		ioModeA: roboid.findDeviceById(Hamster.IO_MODE_A),
		ioModeB: roboid.findDeviceById(Hamster.IO_MODE_B),
		configProximity: roboid.findDeviceById(Hamster.CONFIG_PROXIMITY),
		configGravity: roboid.findDeviceById(Hamster.CONFIG_GRAVITY),
		configBandWidth: roboid.findDeviceById(Hamster.CONFIG_BAND_WIDTH)
	};
	robot.lineTracerModeId = -1;
	return robot;
};

Component.prototype.calculateInnerSpeed = function(speed, radius) {
	return speed * (radius - Hamster.WHEEL_CENTER_DISTANCE) / (radius + Hamster.WHEEL_CENTER_DISTANCE);
};

Component.prototype.handleRemoteData = function(robot, data) {
	var motoring = robot.motoring;
	if(data.motion == 1) {
		var speed = context.getMovementSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 2) {
		var speed = context.getMovementSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 3) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 4) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 5) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 6) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 7) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(0);
	} else if(data.motion == 8) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(0);
	} else if(data.motion == 9) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(this.calculateInnerSpeed(speed, data.radius));
		motoring.rightWheel.write(speed);
	} else if(data.motion == 10) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-this.calculateInnerSpeed(speed, data.radius));
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 11) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(this.calculateInnerSpeed(speed, data.radius));
	} else if(data.motion == 12) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-this.calculateInnerSpeed(speed, data.radius));
	} else if(data.motion == 13) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(speed);
	} else if(data.motion == 14) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 15) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
	} else if(data.motion == 16) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
	} else if(data.motion == 17) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(speed);
	} else if(data.motion == 18) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 19) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(speed);
			motoring.rightWheel.write(this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
		} else {
			motoring.leftWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
			motoring.rightWheel.write(-speed);
		}
	} else if(data.motion == 20) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(-speed);
			motoring.rightWheel.write(-this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
		} else {
			motoring.leftWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
			motoring.rightWheel.write(speed);
		}
	} else if(data.motion == 21) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
			motoring.rightWheel.write(speed);
		} else {
			motoring.leftWheel.write(-speed);
			motoring.rightWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
		}
	} else if(data.motion == 22) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(-this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
			motoring.rightWheel.write(-speed);
		} else {
			motoring.leftWheel.write(speed);
			motoring.rightWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
		}
	} else if(data.motion == 23) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
	} else if(data.motion == 24) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
	} else {
		if(data.leftWheel !== undefined) motoring.leftWheel.write(data.leftWheel);
		if(data.rightWheel !== undefined) motoring.rightWheel.write(data.rightWheel);
	}
	if(data.buzzer !== undefined) motoring.buzzer.write(data.buzzer);
	if(data.outputA !== undefined) motoring.outputA.write(data.outputA);
	if(data.outputB !== undefined) motoring.outputB.write(data.outputB);
	if(data.topology !== undefined) motoring.topology.write(data.topology);
	if(data.leftLed !== undefined) motoring.leftLed.write(data.leftLed);
	if(data.rightLed !== undefined) motoring.rightLed.write(data.rightLed);
	if(data.note !== undefined) motoring.note.write(data.note);
	if(data.lineTracerModeId !== undefined) {
		var t = data.lineTracerModeId;
		if(t != robot.lineTracerModeId) {
			robot.lineTracerModeId = t;
			if(data.lineTracerMode !== undefined) motoring.lineTracerMode.write(data.lineTracerMode);
		}
	}
	if(data.lineTracerSpeed !== undefined) motoring.lineTracerSpeed.write(data.lineTracerSpeed);
	if(data.ioModeA !== undefined) motoring.ioModeA.write(data.ioModeA);
	if(data.ioModeB !== undefined) motoring.ioModeB.write(data.ioModeB);
	if(data.configProximity !== undefined) motoring.configProximity.write(data.configProximity);
	if(data.configGravity !== undefined) motoring.configGravity.write(data.configGravity);
	if(data.configBandWidth !== undefined) motoring.configBandWidth.write(data.configBandWidth);
};

Component.prototype.prepareSensory = function(robot, data) {
	var sensory = robot.sensory;
	sensory.signalStrength = data.signalStrength;
	sensory.leftProximity = data.leftProximity;
	sensory.rightProximity = data.rightProximity;
	sensory.leftFloor = data.leftFloor;
	sensory.rightFloor = data.rightFloor;
	sensory.accelerationX = data.acceleration.x;
	sensory.accelerationY = data.acceleration.y;
	sensory.accelerationZ = data.acceleration.z;
	sensory.light = data.light;
	sensory.temperature = data.temperature;
	sensory.inputA = data.inputA;
	sensory.inputB = data.inputB;
	if(data.map & 0x00000010) {
		sensory.lineTracerState = data.lineTracerState;
		sensory.lineTracerStateId = (sensory.lineTracerStateId % 255) + 1;
	}
	if(data.map & 0x00000008) sensory.tilt = data.tilt;
	if(data.map & 0x00000004) sensory.batteryState = data.batteryState;
	var threshold = context.getHandFoundThreshold(MODULE);
	sensory.handFound = sensory.leftProximity > threshold || sensory.rightProximity > threshold;
};

Component.prototype.prepareSensoryMulti = function(sensory, key, robot, data) {
	this.prepareSensory(robot, data);
	var robotsensory = robot.sensory;
	sensory[key + 'signalStrength'] = robotsensory.signalStrength;
	sensory[key + 'leftProximity'] = robotsensory.leftProximity;
	sensory[key + 'rightProximity'] = robotsensory.rightProximity;
	sensory[key + 'leftFloor'] = robotsensory.leftFloor;
	sensory[key + 'rightFloor'] = robotsensory.rightFloor;
	sensory[key + 'accelerationX'] = robotsensory.accelerationX;
	sensory[key + 'accelerationY'] = robotsensory.accelerationY;
	sensory[key + 'accelerationZ'] = robotsensory.accelerationZ;
	sensory[key + 'light'] = robotsensory.light;
	sensory[key + 'temperature'] = robotsensory.temperature;
	sensory[key + 'inputA'] = robotsensory.inputA;
	sensory[key + 'inputB'] = robotsensory.inputB;
	sensory[key + 'lineTracerState'] = robotsensory.lineTracerState;
	sensory[key + 'lineTracerStateId'] = robotsensory.lineTracerStateId;
	sensory[key + 'tilt'] = robotsensory.tilt;
	sensory[key + 'batteryState'] = robotsensory.batteryState;
	sensory[key + 'handFound'] = robotsensory.handFound;
};

Component.prototype.updateSensory = function(robot) {
	robot.sensory.lineTracerState = 0;
};

Component.prototype.reset = function(robot) {
	robot.roboid.reset();
};

module.exports = new Component();