'use strict';
var Hamster = require('./hamster');

function HamsterRoboid(parent) {
	var io = require('roboid.io');
	var Device = io.Device;
	var DeviceType = io.DeviceType;
	var DataType = io.DataType;
	
	var sensory = {
		devices: [],
		readData: [],
		callbacks: []
	};
	var motoring = {
		map: 0,
		devices: [],
		readData: [],
		writeData: [],
		callbacks: []
	};
	var writeMotoringFlag = 0;
	
	this.bind = function(device, type, data, callback) {
		if(type == DeviceType.SENSOR || type == DeviceType.EVENT) {
			sensory.devices.push(device);
			sensory.readData.push(data.read);
			sensory.callbacks.push(callback);
		} else {
			motoring.devices.push(device);
			motoring.readData.push(data.read);
			motoring.writeData.push(data.write);
			motoring.callbacks.push(callback);
		}
	};
	
	var devices = {};
	devices[Hamster.LEFT_WHEEL] = new Device(this, Hamster.LEFT_WHEEL, 'LeftWheel', DeviceType.EFFECTOR, DataType.INTEGER, 1, -100, 100, 0);
	devices[Hamster.RIGHT_WHEEL] = new Device(this, Hamster.RIGHT_WHEEL, 'RightWheel', DeviceType.EFFECTOR, DataType.INTEGER, 1, -100, 100, 0);
	devices[Hamster.BUZZER] = new Device(this, Hamster.BUZZER, 'Buzzer', DeviceType.EFFECTOR, DataType.FLOAT, 1, 0, 167772.15, 0.0);
	devices[Hamster.OUTPUT_A] = new Device(this, Hamster.OUTPUT_A, 'OutputA', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.OUTPUT_B] = new Device(this, Hamster.OUTPUT_B, 'OutputB', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.TOPOLOGY] = new Device(this, Hamster.TOPOLOGY, 'Topology', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 15, 0);
	devices[Hamster.LEFT_LED] = new Device(this, Hamster.LEFT_LED, 'LeftLed', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 7, 0);
	devices[Hamster.RIGHT_LED] = new Device(this, Hamster.RIGHT_LED, 'RightLed', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 7, 0);
	devices[Hamster.NOTE] = new Device(this, Hamster.NOTE, 'Note', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 88, 0);
	devices[Hamster.LINE_TRACER_MODE] = new Device(this, Hamster.LINE_TRACER_MODE, 'LineTracerMode', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 15, 0);
	devices[Hamster.LINE_TRACER_SPEED] = new Device(this, Hamster.LINE_TRACER_SPEED, 'LineTracerSpeed', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 8, 5);
	devices[Hamster.IO_MODE_A] = new Device(this, Hamster.IO_MODE_A, 'IoModeA', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 15, 0);
	devices[Hamster.IO_MODE_B] = new Device(this, Hamster.IO_MODE_B, 'IoModeB', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 15, 0);
	devices[Hamster.CONFIG_PROXIMITY] = new Device(this, Hamster.CONFIG_PROXIMITY, 'ConfigProximity', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 7, 2);
	devices[Hamster.CONFIG_GRAVITY] = new Device(this, Hamster.CONFIG_GRAVITY, 'ConfigGravity', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 3, 0);
	devices[Hamster.CONFIG_BAND_WIDTH] = new Device(this, Hamster.CONFIG_BAND_WIDTH, 'ConfigBandWidth', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 8, 3);
	devices[Hamster.SIGNAL_STRENGTH] = new Device(this, Hamster.SIGNAL_STRENGTH, 'SignalStrength', DeviceType.SENSOR, DataType.INTEGER, 1, -128, 0, 0);
	devices[Hamster.LEFT_PROXIMITY] = new Device(this, Hamster.LEFT_PROXIMITY, 'LeftProximity', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.RIGHT_PROXIMITY] = new Device(this, Hamster.RIGHT_PROXIMITY, 'RightProximity', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.LEFT_FLOOR] = new Device(this, Hamster.LEFT_FLOOR, 'LeftFloor', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.RIGHT_FLOOR] = new Device(this, Hamster.RIGHT_FLOOR, 'RightFloor', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.ACCELERATION] = new Device(this, Hamster.ACCELERATION, 'Acceleration', DeviceType.SENSOR, DataType.INTEGER, 3, -32768, 32767, 0);
	devices[Hamster.LIGHT] = new Device(this, Hamster.LIGHT, 'Light', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 65535, 0);
	devices[Hamster.TEMPERATURE] = new Device(this, Hamster.TEMPERATURE, 'Temperature', DeviceType.SENSOR, DataType.INTEGER, 1, -40, 88, 0);
	devices[Hamster.INPUT_A] = new Device(this, Hamster.INPUT_A, 'InputA', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.INPUT_B] = new Device(this, Hamster.INPUT_B, 'InputB', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.LINE_TRACER_STATE] = new Device(this, Hamster.LINE_TRACER_STATE, 'LineTracerState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 255, 0);
	devices[Hamster.TILT] = new Device(this, Hamster.TILT, 'Tilt', DeviceType.EVENT, DataType.INTEGER, 1, -3, 3, 0);
	devices[Hamster.BATTERY_STATE] = new Device(this, Hamster.BATTERY_STATE, 'BatteryState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 2, 2);
	
	this.findDeviceByName = function(name) {
		name = '' + name;
		name = name.toLowerCase();
		var dot = name.search('\\.');
		if(dot < 0) {
			for(var i in devices) {
				if(name == devices[i].getName().toLowerCase())
					return devices[i];
			}
		}
		return undefined;
	};
	
	this.findDeviceById = function(deviceId) {
		var device = devices[deviceId];
		if(device) {
			return device;
		}
		return undefined;
	};
	
	this.addDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].addDeviceDataChangedListener(listener);
		}
	};
	
	this.removeDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].removeDeviceDataChangedListener(listener);
		}
	};
	
	this.clearDeviceDataChangedListeners = function() {
		for(var i in devices) {
			devices[i].clearDeviceDataChangedListeners();
		}
	};
	
	this.onMotoringDeviceWritten = function(device) {
		switch(device.getId()) {
			case Hamster.TOPOLOGY:
				writeMotoringFlag |= 0x02000000;
				break;
			case Hamster.LEFT_LED:
				writeMotoringFlag |= 0x01000000;
				break;
			case Hamster.RIGHT_LED:
				writeMotoringFlag |= 0x00800000;
				break;
			case Hamster.NOTE:
				writeMotoringFlag |= 0x00400000;
				break;
			case Hamster.LINE_TRACER_MODE:
				writeMotoringFlag |= 0x00200000;
				break;
			case Hamster.LINE_TRACER_SPEED:
				writeMotoringFlag |= 0x00100000;
				break;
			case Hamster.IO_MODE_A:
				writeMotoringFlag |= 0x00080000;
				break;
			case Hamster.IO_MODE_B:
				writeMotoringFlag |= 0x00040000;
				break;
			case Hamster.CONFIG_PROXIMITY:
				writeMotoringFlag |= 0x00020000;
				break;
			case Hamster.CONFIG_GRAVITY:
				writeMotoringFlag |= 0x00010000;
				break;
			case Hamster.CONFIG_BAND_WIDTH:
				writeMotoringFlag |= 0x00008000;
				break;
		}
	};
	
	this.handleSensory = function(data) {
		var map = data.map;
		var readData = sensory.readData;
		readData[0][0] = data.signalStrength;
		readData[1][0] = data.leftProximity;
		readData[2][0] = data.rightProximity;
		readData[3][0] = data.leftFloor;
		readData[4][0] = data.rightFloor;
		readData[5][0] = data.acceleration.x;
		readData[5][1] = data.acceleration.y;
		readData[5][2] = data.acceleration.z;
		readData[6][0] = data.light;
		readData[7][0] = data.temperature;
		readData[8][0] = data.inputA;
		readData[9][0] = data.inputB;
		
		var callbacks = sensory.callbacks;
		var i;
		for(i = 0; i < 10; ++i) {
			callbacks[i].fire();
		}
		
		if(map & 0x00000010) {
			readData[10][0] = data.lineTracerState;
			callbacks[10].fire();
		}
		if(map & 0x00000008) {
			readData[11][0] = data.tilt;
			callbacks[11].fire();
		}
		if(map & 0x00000004) {
			readData[12][0] = data.batteryState;
			callbacks[12].fire();
		}
		
		for(i = 0; i < 10; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map & 0x00000010) {
			callbacks[10].notifyDeviceDataChanged();
		}
		if(map & 0x00000008) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map & 0x00000004) {
			callbacks[12].notifyDeviceDataChanged();
		}
	};
	
	this.handleMotoring = function(data) {
		var map = data.map;
		var readData = motoring.readData;
		readData[0][0] = data.leftWheel;
		readData[1][0] = data.rightWheel;
		readData[2][0] = data.buzzer;
		readData[3][0] = data.outputA;
		readData[4][0] = data.outputB;
		
		var callbacks = motoring.callbacks;
		var i;
		for(i = 0; i < 5; ++i) {
			callbacks[i].fire();
		}
		
		if(map & 0x02000000) {
			readData[5][0] = data.topology;
			callbacks[5].fire();
		}
		if(map & 0x01000000) {
			readData[6][0] = data.leftLed;
			callbacks[6].fire();
		}
		if(map & 0x00800000) {
			readData[7][0] = data.rightLed;
			callbacks[7].fire();
		}
		if(map & 0x00400000) {
			readData[8][0] = data.note;
			callbacks[8].fire();
		}
		if(map & 0x00200000) {
			readData[9][0] = data.lineTracerMode;
			callbacks[9].fire();
		}
		if(map & 0x00100000) {
			readData[10][0] = data.lineTracerSpeed;
			callbacks[10].fire();
		}
		if(map & 0x00080000) {
			readData[11][0] = data.ioModeA;
			callbacks[11].fire();
		}
		if(map & 0x00040000) {
			readData[12][0] = data.ioModeB;
			callbacks[12].fire();
		}
		if(map & 0x00020000) {
			readData[13][0] = data.configProximity;
			callbacks[13].fire();
		}
		if(map & 0x00010000) {
			readData[14][0] = data.configGravity;
			callbacks[14].fire();
		}
		if(map & 0x00008000) {
			readData[15][0] = data.configBandWidth;
			callbacks[15].fire();
		}
		
		for(i = 0; i < 5; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map & 0x02000000) {
			callbacks[5].notifyDeviceDataChanged();
		}
		if(map & 0x01000000) {
			callbacks[6].notifyDeviceDataChanged();
		}
		if(map & 0x00800000) {
			callbacks[7].notifyDeviceDataChanged();
		}
		if(map & 0x00400000) {
			callbacks[8].notifyDeviceDataChanged();
		}
		if(map & 0x00200000) {
			callbacks[9].notifyDeviceDataChanged();
		}
		if(map & 0x00100000) {
			callbacks[10].notifyDeviceDataChanged();
		}
		if(map & 0x00080000) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map & 0x00040000) {
			callbacks[12].notifyDeviceDataChanged();
		}
		if(map & 0x00020000) {
			callbacks[13].notifyDeviceDataChanged();
		}
		if(map & 0x00010000) {
			callbacks[14].notifyDeviceDataChanged();
		}
		if(map & 0x00008000) {
			callbacks[15].notifyDeviceDataChanged();
		}
	};
	
	this.requestMotoring = function() {
		motoring.map = 0xfc000000 | writeMotoringFlag;
		writeMotoringFlag = 0;
		return motoring;
	};
	
	this.updateSensory = function() {
		var callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.updateMotoring = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.reset = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
		callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
	};
}

HamsterRoboid.prototype.getId = function() {
	return Hamster.ID;
};

HamsterRoboid.prototype.getName = function() {
	return 'Hamster';
};

module.exports.create = function(parent) {
	return new HamsterRoboid(parent);
};