'use strict';
var router = require('router');
var Hamster = require('../hamster');
var factory = require('../hamster-roboid');
const MODULE = 'hamster';
const HEADER = { 'Content-Type': 'text/plain' };

function Component() {
}

Component.prototype.createRobot = function(state) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.connectionState = state;
	robot.sensory = {
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		tilt: 0,
		lineTracerStateId: 0,
		lineTracerState: 0,
		batteryState: 2
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(Hamster.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(Hamster.RIGHT_WHEEL),
		buzzer: roboid.findDeviceById(Hamster.BUZZER),
		outputA: roboid.findDeviceById(Hamster.OUTPUT_A),
		outputB: roboid.findDeviceById(Hamster.OUTPUT_B),
		topology: roboid.findDeviceById(Hamster.TOPOLOGY),
		leftLed: roboid.findDeviceById(Hamster.LEFT_LED),
		rightLed: roboid.findDeviceById(Hamster.RIGHT_LED),
		note: roboid.findDeviceById(Hamster.NOTE),
		lineTracerMode: roboid.findDeviceById(Hamster.LINE_TRACER_MODE),
		lineTracerSpeed: roboid.findDeviceById(Hamster.LINE_TRACER_SPEED),
		ioModeA: roboid.findDeviceById(Hamster.IO_MODE_A),
		ioModeB: roboid.findDeviceById(Hamster.IO_MODE_B),
		configProximity: roboid.findDeviceById(Hamster.CONFIG_PROXIMITY),
		configGravity: roboid.findDeviceById(Hamster.CONFIG_GRAVITY),
		configBandWidth: roboid.findDeviceById(Hamster.CONFIG_BAND_WIDTH)
	};
	return robot;
};

Component.prototype.start = function(express, robots) {
	var self = this;
	
	var getRoute = function(index) {
		if(router.multi) {
			return router.findRouteByIndex(MODULE, index);
		} else {
			var selectedRoute = router.selectedRoute;
			if(selectedRoute && selectedRoute.module == MODULE) {
				return selectedRoute;
			}
		}
	};
	
	var getRobot = function(index) {
		var key = router.multi ? MODULE + index : MODULE;
		var robot = robots[key];
		if(!robot) {
			var route = getRoute(index);
			if(route) {
				robot = self.createRobot(route.state);
				if(robot) {
					robots[key] = robot;
				}
			}
		}
		return robot;
	};
	
	express.get('/hamster/:index/wheels/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__wheels(getRobot(index), leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/hamster/:index/wheels/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__wheels(getRobot(index), speed, speed);
		response.end();
	});
	
	express.get('/hamster/:index/leftWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__leftWheel(getRobot(index), speed);
		response.end();
	});
	
	express.get('/hamster/:index/rightWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__rightWheel(getRobot(index), speed);
		response.end();
	});
	
	express.get('/hamster/:index/stop', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__stop(getRobot(index));
		response.end();
	});
	
	express.get('/hamster/:index/lineTracerMode/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var mode = parseInt(request.params.mode);
		self.__lineTracerMode(getRobot(index), mode);
		response.end();
	});
	
	express.get('/hamster/:index/lineTracerSpeed/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseInt(request.params.speed);
		self.__lineTracerSpeed(getRobot(index), speed);
		response.end();
	});
	
	express.get('/hamster/:index/leds/:leftColor/:rightColor', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var leftColor = parseInt(request.params.leftColor);
		var rightColor = parseInt(request.params.rightColor);
		self.__leds(getRobot(index), leftColor, rightColor);
		response.end();
	});
	
	express.get('/hamster/:index/leds/:color', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var color = parseInt(request.params.color);
		self.__leds(getRobot(index), color, color);
		response.end();
	});
	
	express.get('/hamster/:index/leftLed/:color', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var color = parseInt(request.params.color);
		self.__leftLed(getRobot(index), color);
		response.end();
	});
	
	express.get('/hamster/:index/rightLed/:color', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var color = parseInt(request.params.color);
		self.__rightLed(getRobot(index), color);
		response.end();
	});
	
	express.get('/hamster/:index/buzzer/:hz', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var hz = parseFloat(request.params.hz);
		self.__buzzer(getRobot(index), hz);
		response.end();
	});
	
	express.get('/hamster/:index/note/:pitch', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var pitch = parseInt(request.params.pitch);
		self.__note(getRobot(index), pitch);
		response.end();
	});
	
	express.get('/hamster/:index/ioModeA/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var mode = parseInt(request.params.mode);
		self.__ioModeA(getRobot(index), mode);
		response.end();
	});
	
	express.get('/hamster/:index/ioModeB/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var mode = parseInt(request.params.mode);
		self.__ioModeB(getRobot(index), mode);
		response.end();
	});
	
	express.get('/hamster/:index/outputA/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var value = parseFloat(request.params.value);
		self.__outputA(getRobot(index), value);
		response.end();
	});
	
	express.get('/hamster/:index/outputB/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var value = parseFloat(request.params.value);
		self.__outputB(getRobot(index), value);
		response.end();
	});
	
	express.get('/hamster/:index/gripper/:action', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var action = parseInt(request.params.action);
		self.__gripper(getRobot(index), action);
		response.end();
	});
	
	express.get('/hamster/:index/openGripper', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__openGripper(getRobot(index));
		response.end();
	});
	
	express.get('/hamster/:index/closeGripper', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__closeGripper(getRobot(index));
		response.end();
	});
	
	express.get('/hamster/:index/releaseGripper', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__releaseGripper(getRobot(index));
		response.end();
	});
	
	express.get('/hamster/:index/signalStrength', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__signalStrength(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/leftProximity', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__leftProximity(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/rightProximity', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__rightProximity(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/leftFloor', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__leftFloor(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/rightFloor', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__rightFloor(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/accelerationX', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationX(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/accelerationY', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationY(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/accelerationZ', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationZ(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/light', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__light(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/temperature', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__temperature(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/inputA', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__inputA(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/inputB', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__inputB(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/tilt', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__tilt(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/lineTracerState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__lineTracerState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/lineTracerStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__lineTracerStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/batteryState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__batteryState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/sensors', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write(self.__sensors(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/connectionState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__connectionState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamster/:index/reset', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__reset(getRoute(index));
		response.end();
	});
	
	express.get('/hamster/wheels/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__wheels(getRobot(0), leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/hamster/wheels/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__wheels(getRobot(0), speed, speed);
		response.end();
	});
	
	express.get('/hamster/leftWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__leftWheel(getRobot(0), speed);
		response.end();
	});
	
	express.get('/hamster/rightWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__rightWheel(getRobot(0), speed);
		response.end();
	});
	
	express.get('/hamster/stop', function(request, response) {
		response.writeHead(200, HEADER);
		self.__stop(getRobot(0));
		response.end();
	});
	
	express.get('/hamster/lineTracerMode/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var mode = parseInt(request.params.mode);
		self.__lineTracerMode(getRobot(0), mode);
		response.end();
	});
	
	express.get('/hamster/lineTracerSpeed/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseInt(request.params.speed);
		self.__lineTracerSpeed(getRobot(0), speed);
		response.end();
	});
	
	express.get('/hamster/leds/:leftColor/:rightColor', function(request, response) {
		response.writeHead(200, HEADER);
		var leftColor = parseInt(request.params.leftColor);
		var rightColor = parseInt(request.params.rightColor);
		self.__leds(getRobot(0), leftColor, rightColor);
		response.end();
	});
	
	express.get('/hamster/leds/:color', function(request, response) {
		response.writeHead(200, HEADER);
		var color = parseInt(request.params.color);
		self.__leds(getRobot(0), color, color);
		response.end();
	});
	
	express.get('/hamster/leftLed/:color', function(request, response) {
		response.writeHead(200, HEADER);
		var color = parseInt(request.params.color);
		self.__leftLed(getRobot(0), color);
		response.end();
	});
	
	express.get('/hamster/rightLed/:color', function(request, response) {
		response.writeHead(200, HEADER);
		var color = parseInt(request.params.color);
		self.__rightLed(getRobot(0), color);
		response.end();
	});
	
	express.get('/hamster/buzzer/:hz', function(request, response) {
		response.writeHead(200, HEADER);
		var hz = parseFloat(request.params.hz);
		self.__buzzer(getRobot(0), hz);
		response.end();
	});
	
	express.get('/hamster/note/:pitch', function(request, response) {
		response.writeHead(200, HEADER);
		var pitch = parseInt(request.params.pitch);
		self.__note(getRobot(0), pitch);
		response.end();
	});
	
	express.get('/hamster/ioModeA/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var mode = parseInt(request.params.mode);
		self.__ioModeA(getRobot(0), mode);
		response.end();
	});
	
	express.get('/hamster/ioModeB/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var mode = parseInt(request.params.mode);
		self.__ioModeB(getRobot(0), mode);
		response.end();
	});
	
	express.get('/hamster/outputA/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var value = parseFloat(request.params.value);
		self.__outputA(getRobot(0), value);
		response.end();
	});
	
	express.get('/hamster/outputB/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var value = parseFloat(request.params.value);
		self.__outputB(getRobot(0), value);
		response.end();
	});
	
	express.get('/hamster/gripper/:action', function(request, response) {
		response.writeHead(200, HEADER);
		var action = parseInt(request.params.action);
		self.__gripper(getRobot(0), action);
		response.end();
	});
	
	express.get('/hamster/openGripper', function(request, response) {
		response.writeHead(200, HEADER);
		self.__openGripper(getRobot(0));
		response.end();
	});
	
	express.get('/hamster/closeGripper', function(request, response) {
		response.writeHead(200, HEADER);
		self.__closeGripper(getRobot(0));
		response.end();
	});
	
	express.get('/hamster/releaseGripper', function(request, response) {
		response.writeHead(200, HEADER);
		self.__releaseGripper(getRobot(0));
		response.end();
	});
	
	express.get('/hamster/signalStrength', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__signalStrength(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/leftProximity', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__leftProximity(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/rightProximity', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__rightProximity(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/leftFloor', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__leftFloor(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/rightFloor', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__rightFloor(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/accelerationX', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationX(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/accelerationY', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationY(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/accelerationZ', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationZ(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/light', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__light(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/temperature', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__temperature(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/inputA', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__inputA(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/inputB', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__inputB(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/tilt', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__tilt(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/lineTracerState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__lineTracerState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/lineTracerStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__lineTracerStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/batteryState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__batteryState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/sensors', function(request, response) {
		response.writeHead(200, HEADER);
		response.write(self.__sensors(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/connectionState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__connectionState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamster/reset', function(request, response) {
		response.writeHead(200, HEADER);
		self.__reset(getRoute(0));
		response.end();
	});
};

Component.prototype.__wheels = function(robot, leftSpeed, rightSpeed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		if(typeof leftSpeed == 'number') {
			motoring.leftWheel.write(leftSpeed);
		}
		if(typeof rightSpeed == 'number') {
			motoring.rightWheel.write(rightSpeed);
		}
	}
};

Component.prototype.__leftWheel = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		if(typeof speed == 'number') {
			motoring.leftWheel.write(speed);
		}
	}
};

Component.prototype.__rightWheel = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		if(typeof speed == 'number') {
			motoring.rightWheel.write(speed);
		}
	}
};

Component.prototype.__stop = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
	}
};

Component.prototype.__lineTracerMode = function(robot, mode) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		if(typeof mode == 'number') {
			motoring.lineTracerMode.write(mode);
		}
	}
};

Component.prototype.__lineTracerSpeed = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof speed == 'number') {
			motoring.lineTracerSpeed.write(speed);
		}
	}
};

Component.prototype.__leds = function(robot, leftColor, rightColor) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof leftColor == 'number') {
			motoring.leftLed.write(leftColor);
		}
		if(typeof rightColor == 'number') {
			motoring.rightLed.write(rightColor);
		}
	}
};

Component.prototype.__leftLed = function(robot, color) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof color == 'number') {
			motoring.leftLed.write(color);
		}
	}
};

Component.prototype.__rightLed = function(robot, color) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof color == 'number') {
			motoring.rightLed.write(color);
		}
	}
};

Component.prototype.__buzzer = function(robot, hz) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof hz == 'number') {
			motoring.buzzer.write(hz);
		}
		motoring.note.write(0);
	}
};

Component.prototype.__note = function(robot, pitch) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.buzzer.write(0);
		if(typeof pitch == 'number') {
			motoring.note.write(pitch);
		}
	}
};

Component.prototype.__ioModeA = function(robot, mode) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof mode == 'number') {
			motoring.ioModeA.write(mode);
		}
	}
};

Component.prototype.__ioModeB = function(robot, mode) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof mode == 'number') {
			motoring.ioModeB.write(mode);
		}
	}
};

Component.prototype.__outputA = function(robot, value) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof value == 'number') {
			motoring.outputA.write(value);
		}
	}
};

Component.prototype.__outputB = function(robot, value) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof value == 'number') {
			motoring.outputB.write(value);
		}
	}
};

Component.prototype.__gripper = function(robot, action) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof action == 'number') {
			motoring.ioModeA.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
			motoring.ioModeB.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
			if(action > 0) {
				motoring.outputA.write(1);
				motoring.outputB.write(0);
			} else if(action < 0) {
				motoring.outputA.write(0);
				motoring.outputB.write(1);
			} else {
				motoring.outputA.write(0);
				motoring.outputB.write(0);
			}
		}
	}
};

Component.prototype.__openGripper = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.ioModeA.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
		motoring.ioModeB.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
		motoring.outputA.write(1);
		motoring.outputB.write(0);
	}
};

Component.prototype.__closeGripper = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.ioModeA.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
		motoring.ioModeB.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
		motoring.outputA.write(0);
		motoring.outputB.write(1);
	}
};

Component.prototype.__releaseGripper = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.ioModeA.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
		motoring.ioModeB.write(Hamster.IO_MODE_DIGITAL_OUTPUT);
		motoring.outputA.write(0);
		motoring.outputB.write(0);
	}
};

Component.prototype.__signalStrength = function(robot) {
	if(robot) {
		return robot.sensory.signalStrength;
	}
};

Component.prototype.__leftProximity = function(robot) {
	if(robot) {
		return robot.sensory.leftProximity;
	}
};

Component.prototype.__rightProximity = function(robot) {
	if(robot) {
		return robot.sensory.rightProximity;
	}
};

Component.prototype.__leftFloor = function(robot) {
	if(robot) {
		return robot.sensory.leftFloor;
	}
};

Component.prototype.__rightFloor = function(robot) {
	if(robot) {
		return robot.sensory.rightFloor;
	}
};

Component.prototype.__accelerationX = function(robot) {
	if(robot) {
		return robot.sensory.accelerationX;
	}
};

Component.prototype.__accelerationY = function(robot) {
	if(robot) {
		return robot.sensory.accelerationY;
	}
};

Component.prototype.__accelerationZ = function(robot) {
	if(robot) {
		return robot.sensory.accelerationZ;
	}
};

Component.prototype.__light = function(robot) {
	if(robot) {
		return robot.sensory.light;
	}
};

Component.prototype.__temperature = function(robot) {
	if(robot) {
		return robot.sensory.temperature;
	}
};

Component.prototype.__inputA = function(robot) {
	if(robot) {
		return robot.sensory.inputA;
	}
};

Component.prototype.__inputB = function(robot) {
	if(robot) {
		return robot.sensory.inputB;
	}
};

Component.prototype.__tilt = function(robot) {
	if(robot) {
		return robot.sensory.tilt;
	}
};

Component.prototype.__lineTracerState = function(robot) {
	if(robot) {
		return robot.sensory.lineTracerState;
	}
};

Component.prototype.__lineTracerStateId = function(robot) {
	if(robot) {
		return robot.sensory.lineTracerStateId;
	}
};

Component.prototype.__batteryState = function(robot) {
	if(robot) {
		return robot.sensory.batteryState;
	}
};

Component.prototype.__sensors = function(robot) {
	if(robot) {
		try {
			return JSON.stringify(robot.sensory);
		} catch (e) {
		}
	}
};

Component.prototype.__connectionState = function(robot) {
	if(robot) {
		return robot.connectionState;
	}
};

Component.prototype.__reset = function(route) {
	if(route) {
		router.resetRoute(route);
	}
};

Component.prototype.setState = function(robot, state) {
	robot.connectionState = state;
};

Component.prototype.handleSensory = function(robot, data, state) {
	robot.connectionState = state;
	var sensory = robot.sensory;
	sensory.signalStrength = data.signalStrength;
	sensory.leftProximity = data.leftProximity;
	sensory.rightProximity = data.rightProximity;
	sensory.leftFloor = data.leftFloor;
	sensory.rightFloor = data.rightFloor;
	sensory.accelerationX = data.acceleration.x;
	sensory.accelerationY = data.acceleration.y;
	sensory.accelerationZ = data.acceleration.z;
	sensory.light = data.light;
	sensory.temperature = data.temperature;
	sensory.inputA = data.inputA;
	sensory.inputB = data.inputB;
	if(data.map & 0x00000008) sensory.tilt = data.tilt;
	if(data.map & 0x00000010) {
		sensory.lineTracerState = data.lineTracerState;
		sensory.lineTracerStateId = (sensory.lineTracerStateId % 255) + 1;
	}
	if(data.map & 0x00000004) sensory.batteryState = data.batteryState;
};

module.exports = new Component();