'use strict';
var router = require('router');
var translator = require('translator');
var component = require('./component');
const MODULE = 'hamster';

function Extension() {
	this.robot = component.createRobot(0);
}

Extension.prototype.start = function(express) {
	var self = this;
	var robot = self.robot;
	var problemMessage = '';
	
	self.running = true;
	robot.clearWaits();
	var setup = function() {
		problemMessage = '_problem ' + translator.translate('Robot is not connected.') + '\n';
		component.updateLang(translator);
	};
	translator.on('hamster-scratch', function() {
		setup();
	});
	setup();
	
	if(express) {
		const HEADER = { 'Content-Type': 'text/plain' };
		
		express.get('/reset_all', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearWaits();
			robot.clearTimeouts();
			robot.reset();
			router.reset();
			response.end();
		});
		
		express.get('/poll', function(request, response) {
			if(!self.running) {
				response.set('Connection', 'close');
			}
			response.writeHead(200, HEADER);
			var waits = robot.getWaits();
			if(waits.length > 0) {
				response.write('_busy ' + waits.join(' ') + '\n');
			}
			if(self.state == 'connected') {
				robot.writeSingleSensory(response);
			} else {
				response.write(problemMessage);
			}
			response.end();
		});
		
		express.get('/boardMoveForward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.boardMoveForward(request);
			response.end();
		});
		
		express.get('/boardTurn/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.boardTurn(request);
			response.end();
		});
		
		express.get('/moveForward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveForward(request);
			response.end();
		});
		
		express.get('/moveBackward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveBackward(request);
			response.end();
		});
		
		express.get('/turn/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turn(request);
			response.end();
		});
		
		express.get('/moveForwardForSecs/:id/:sec', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveForwardForSecs(request);
			response.end();
		});
		
		express.get('/moveBackwardForSecs/:id/:sec', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveBackwardForSecs(request);
			response.end();
		});
		
		express.get('/turnForSecs/:id/:direction/:sec', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnForSecs(request);
			response.end();
		});
		
		express.get('/changeBothWheelsBy/:left/:right', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeBothWheelsBy(request);
			response.end();
		});
		
		express.get('/setBothWheelsTo/:left/:right', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setBothWheelsTo(request);
			response.end();
		});
		
		express.get('/changeWheelBy/:which/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeWheelBy(request);
			response.end();
		});
		
		express.get('/setWheelTo/:which/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setWheelTo(request);
			response.end();
		});
		
		express.get('/followLineUsingFloorSensor/:color/:which', function(request, response) {
			response.writeHead(200, HEADER);
			robot.followLineUsingFloorSensor(request);
			response.end();
		});
		
		express.get('/followLineUntilIntersection/:id/:color/:which', function(request, response) {
			response.writeHead(200, HEADER);
			robot.followLineUntilIntersection(request);
			response.end();
		});
		
		express.get('/setFollowingSpeedTo/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setFollowingSpeedTo(request);
			response.end();
		});
		
		express.get('/stop', function(request, response) {
			response.writeHead(200, HEADER);
			robot.stop(request);
			response.end();
		});
		
		express.get('/setLedTo/:which/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedTo(request);
			response.end();
		});
		
		express.get('/clearLed/:which', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearLed(request);
			response.end();
		});
		
		express.get('/beep/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.beep(request);
			response.end();
		});
		
		express.get('/changeBuzzerBy/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeBuzzerBy(request);
			response.end();
		});
		
		express.get('/setBuzzerTo/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setBuzzerTo(request);
			response.end();
		});
		
		express.get('/clearBuzzer', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearBuzzer(request);
			response.end();
		});
		
		express.get('/playNote/:note/:octave', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNote(request);
			response.end();
		});
		
		express.get('/playNoteFor/:id/:note/:octave/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNoteFor(request);
			response.end();
		});
		
		express.get('/restFor/:id/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.restFor(request);
			response.end();
		});
		
		express.get('/changeTempoBy/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeTempoBy(request);
			response.end();
		});
		
		express.get('/setTempoTo/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setTempoTo(request);
			response.end();
		});
		
		express.get('/setPortTo/:which/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setPortTo(request);
			response.end();
		});
		
		express.get('/changeOutputBy/:which/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeOutputBy(request);
			response.end();
		});
		
		express.get('/setOutputTo/:which/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setOutputTo(request);
			response.end();
		});
		
		express.get('/gripper/:id/:action', function(request, response) {
			response.writeHead(200, HEADER);
			robot.gripper(request);
			response.end();
		});
		
		express.get('/releaseGripper', function(request, response) {
			response.writeHead(200, HEADER);
			robot.releaseGripper(request);
			response.end();
		});
		
		express.get('/sBoardMoveForward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.boardMoveForward(request);
			response.end();
		});
		
		express.get('/sBoardTurn/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.boardTurn(request);
			response.end();
		});
		
		express.get('/sMoveForward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveForward(request);
			response.end();
		});
		
		express.get('/sMoveBackward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveBackward(request);
			response.end();
		});
		
		express.get('/sTurn/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turn(request);
			response.end();
		});
		
		express.get('/sMoveForwardUnit/:id/:value/:unit', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveForwardUnit(request);
			response.end();
		});

		express.get('/sMoveBackwardUnit/:id/:value/:unit', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveBackwardUnit(request);
			response.end();
		});

		express.get('/sTurnUnitInPlace/:id/:direction/:value/:unit', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnUnitInPlace(request);
			response.end();
		});
		
		express.get('/sPivotAroundWheelUnitInDirection/:id/:wheel/:value/:unit/:forward', function(request, response) {
			response.writeHead(200, HEADER);
			robot.pivotAroundWheelUnitInDirection(request);
			response.end();
		});
		
		express.get('/sTurnBodyUnitWithRadiusInDirection/:id/:direction/:value/:unit/:radius/:forward', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnBodyUnitWithRadiusInDirection(request);
			response.end();
		});
		
		express.get('/sPivotAroundUnitInDirection/:id/:part/:value/:unit/:forward', function(request, response) {
			response.writeHead(200, HEADER);
			robot.pivotAroundUnitInDirection(request);
			response.end();
		});
		
		express.get('/sTurnUnitWithRadiusInDirection/:id/:part/:direction/:value/:unit/:radius/:forward', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnUnitWithRadiusInDirection(request);
			response.end();
		});
		
		express.get('/sChangeBothWheelsBy/:left/:right', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeBothWheelsBy(request);
			response.end();
		});
		
		express.get('/sSetBothWheelsTo/:left/:right', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setBothWheelsTo(request);
			response.end();
		});
		
		express.get('/sChangeWheelBy/:which/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeWheelBy(request);
			response.end();
		});
		
		express.get('/sSetWheelTo/:which/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setWheelTo(request);
			response.end();
		});
		
		express.get('/sFollowLineUsingFloorSensor/:color/:which', function(request, response) {
			response.writeHead(200, HEADER);
			robot.followLineUsingFloorSensor(request);
			response.end();
		});
		
		express.get('/sFollowLineUntilIntersection/:id/:color/:which', function(request, response) {
			response.writeHead(200, HEADER);
			robot.followLineUntilIntersection(request);
			response.end();
		});
		
		express.get('/sSetFollowingSpeedTo/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setFollowingSpeedTo(request);
			response.end();
		});
		
		express.get('/sSetFollowingGainTo/:gain', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setFollowingGainTo(request);
			response.end();
		});
		
		express.get('/sStop', function(request, response) {
			response.writeHead(200, HEADER);
			robot.stop(request);
			response.end();
		});
		
		express.get('/sSetLedTo/:which/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedTo(request);
			response.end();
		});
		
		express.get('/sChangeLedByRGB/:which/:red/:green/:blue', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeLedByRGB(request);
			response.end();
		});
		
		express.get('/sSetLedToRGB/:which/:red/:green/:blue', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setLedToRGB(request);
			response.end();
		});
		
		express.get('/sClearLed/:which', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearLed(request);
			response.end();
		});
		
		express.get('/sPlaySound/:sound', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSound(request);
			response.end();
		});
		
		express.get('/sPlaySoundTimes/:sound/:count', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSoundTimes(request);
			response.end();
		});
		
		express.get('/sPlaySoundTimesUntilDone/:id/:sound/:count', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSoundTimesUntilDone(request);
			response.end();
		});
		
		express.get('/sChangeBuzzerBy/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeBuzzerBy(request);
			response.end();
		});
		
		express.get('/sSetBuzzerTo/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setBuzzerTo(request);
			response.end();
		});
		
		express.get('/sClearSound', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearSound(request);
			response.end();
		});
		
		express.get('/sPlayNote/:note/:octave', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNote(request);
			response.end();
		});
		
		express.get('/sPlayNoteFor/:id/:note/:octave/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNoteFor(request);
			response.end();
		});
		
		express.get('/sRestFor/:id/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.restFor(request);
			response.end();
		});
		
		express.get('/sChangeTempoBy/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeTempoBy(request);
			response.end();
		});
		
		express.get('/sSetTempoTo/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setTempoTo(request);
			response.end();
		});
		
		express.get('/sSetPortTo/:which/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setPortTo(request);
			response.end();
		});
		
		express.get('/sChangeOutputBy/:which/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeOutputBy(request);
			response.end();
		});
		
		express.get('/sSetOutputTo/:which/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setOutputTo(request);
			response.end();
		});
		
		express.get('/sGripper/:id/:action', function(request, response) {
			response.writeHead(200, HEADER);
			robot.gripper(request);
			response.end();
		});
		
		express.get('/sReleaseGripper', function(request, response) {
			response.writeHead(200, HEADER);
			robot.releaseGripper(request);
			response.end();
		});
		
		express.get('/sWriteSerial/:id/:mode/:value', function(request, response) {
			response.writeHead(200, HEADER);
			robot.writeSerial(request);
			response.end();
		});
		
		express.get('/sReadSerialUntil/:id/:delimiter', function(request, response) {
			response.writeHead(200, HEADER);
			robot.readSerialUntil(request);
			response.end();
		});
		
		express.get('/sSetSerialRateTo/:baud', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setSerialRateTo(request);
			response.end();
		});
	}
};

Extension.prototype.stop = function() {
	this.running = false;
	translator.remove('hamster-scratch');
	router.reset();
};

Extension.prototype.setState = function(route) {
	if(route == router.selectedRoute && route.module == MODULE) {
		this.state = route.state;
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleSensory = function(data, route) {
	if(route.module == MODULE) {
		this.robot.handleSensory(data);
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	if(route.module == MODULE) {
		var roboid = this.robot.roboid;
		roboid.handleMotoring(data);
		roboid.updateMotoring();
	}
};

Extension.prototype.requestMotoring = function(route) {
	if(route.module == MODULE) {
		return this.robot.roboid.requestMotoring();
	}
};

Extension.prototype.reset = function(route) {
	if(route == router.selectedRoute && route.module == MODULE) {
		this.robot.roboid.reset();
		this.robot.motoring.reset();
	}
};

module.exports.create = function() {
	return new Extension();
};