'use strict';
var context = require('context');
var router = require('router');
var Hamster = require('../hamster');
var factory = require('../hamster-roboid');
const GROUP = 'hamster';
const MODULE = 'hamster';

function Component() {
}

Component.prototype.createRobot = function(index, state) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	state = router.getConnectionState(state);
	robot.sensory = {
		group: GROUP,
		module: MODULE,
		model: Hamster.ID,
		index: index,
		connectionState: state,
		map: 0,
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		tilt: 0,
		lineTracerState: 0,
		batteryState: 2,
		handFound: false
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(Hamster.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(Hamster.RIGHT_WHEEL),
		buzzer: roboid.findDeviceById(Hamster.BUZZER),
		outputA: roboid.findDeviceById(Hamster.OUTPUT_A),
		outputB: roboid.findDeviceById(Hamster.OUTPUT_B),
		topology: roboid.findDeviceById(Hamster.TOPOLOGY),
		leftLed: roboid.findDeviceById(Hamster.LEFT_LED),
		rightLed: roboid.findDeviceById(Hamster.RIGHT_LED),
		note: roboid.findDeviceById(Hamster.NOTE),
		lineTracerMode: roboid.findDeviceById(Hamster.LINE_TRACER_MODE),
		lineTracerSpeed: roboid.findDeviceById(Hamster.LINE_TRACER_SPEED),
		ioModeA: roboid.findDeviceById(Hamster.IO_MODE_A),
		ioModeB: roboid.findDeviceById(Hamster.IO_MODE_B),
		configProximity: roboid.findDeviceById(Hamster.CONFIG_PROXIMITY),
		configGravity: roboid.findDeviceById(Hamster.CONFIG_GRAVITY),
		configBandWidth: roboid.findDeviceById(Hamster.CONFIG_BAND_WIDTH)
	};
	return robot;
};

Component.prototype.setState = function(robot, state) {
	robot.sensory.connectionState = router.getConnectionState(state);
};

Component.prototype.calculateInnerSpeed = function(speed, radius) {
	return speed * (radius - Hamster.WHEEL_CENTER_DISTANCE) / (radius + Hamster.WHEEL_CENTER_DISTANCE);
};

Component.prototype.handleRemoteData = function(robot, data) {
	var motoring = robot.motoring;
	if(data.motion == 1) {
		var speed = context.getMovementSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 2) {
		var speed = context.getMovementSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 3) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 4) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 5) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 6) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 7) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(0);
	} else if(data.motion == 8) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(0);
	} else if(data.motion == 9) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(this.calculateInnerSpeed(speed, data.radius));
		motoring.rightWheel.write(speed);
	} else if(data.motion == 10) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-this.calculateInnerSpeed(speed, data.radius));
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 11) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(this.calculateInnerSpeed(speed, data.radius));
	} else if(data.motion == 12) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-this.calculateInnerSpeed(speed, data.radius));
	} else if(data.motion == 13) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(speed);
	} else if(data.motion == 14) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 15) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
	} else if(data.motion == 16) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE));
	} else if(data.motion == 17) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(speed);
	} else if(data.motion == 18) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 19) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(speed);
			motoring.rightWheel.write(this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
		} else {
			motoring.leftWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
			motoring.rightWheel.write(-speed);
		}
	} else if(data.motion == 20) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(-speed);
			motoring.rightWheel.write(-this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
		} else {
			motoring.leftWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
			motoring.rightWheel.write(speed);
		}
	} else if(data.motion == 21) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
			motoring.rightWheel.write(speed);
		} else {
			motoring.leftWheel.write(-speed);
			motoring.rightWheel.write(-this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
		}
	} else if(data.motion == 22) {
		var speed = context.getTurningSpeed(MODULE);
		if(data.radius >= Hamster.PEN_CENTER_DISTANCE) {
			motoring.leftWheel.write(-this.calculateInnerSpeed(speed, data.radius - Hamster.PEN_CENTER_DISTANCE));
			motoring.rightWheel.write(-speed);
		} else {
			motoring.leftWheel.write(speed);
			motoring.rightWheel.write(this.calculateInnerSpeed(speed, Hamster.PEN_CENTER_DISTANCE - data.radius));
		}
	} else if(data.motion == 23) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
	} else if(data.motion == 24) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-this.calculateInnerSpeed(speed, data.radius + Hamster.PEN_CENTER_DISTANCE));
	} else {
		if(data.leftWheel !== undefined) motoring.leftWheel.write(data.leftWheel);
		if(data.rightWheel !== undefined) motoring.rightWheel.write(data.rightWheel);
	}
	if(data.buzzer !== undefined) motoring.buzzer.write(data.buzzer);
	if(data.outputA !== undefined) motoring.outputA.write(data.outputA);
	if(data.outputB !== undefined) motoring.outputB.write(data.outputB);
	var map = data.map;
	if((map & 0x02000000) && (data.topology !== undefined)) motoring.topology.write(data.topology);
	if((map & 0x01000000) && (data.leftLed !== undefined)) motoring.leftLed.write(data.leftLed);
	if((map & 0x00800000) && (data.rightLed !== undefined)) motoring.rightLed.write(data.rightLed);
	if((map & 0x00400000) && (data.note !== undefined)) motoring.note.write(data.note);
	if((map & 0x00200000) && (data.lineTracerMode !== undefined)) motoring.lineTracerMode.write(data.lineTracerMode);
	if((map & 0x00100000) && (data.lineTracerSpeed !== undefined)) motoring.lineTracerSpeed.write(data.lineTracerSpeed);
	if((map & 0x00080000) && (data.ioModeA !== undefined)) motoring.ioModeA.write(data.ioModeA);
	if((map & 0x00040000) && (data.ioModeB !== undefined)) motoring.ioModeB.write(data.ioModeB);
	if((map & 0x00020000) && (data.configProximity !== undefined)) motoring.configProximity.write(data.configProximity);
	if((map & 0x00010000) && (data.configGravity !== undefined)) motoring.configGravity.write(data.configGravity);
	if((map & 0x00008000) && (data.configBandWidth !== undefined)) motoring.configBandWidth.write(data.configBandWidth);
};

Component.prototype.handleSensory = function(robot, data, state, connection) {
	if(connection) {
		var sensory = robot.sensory;
		sensory.connectionState = router.getConnectionState(state);
		sensory.map = data.map;
		sensory.signalStrength = data.signalStrength;
		sensory.leftProximity = data.leftProximity;
		sensory.rightProximity = data.rightProximity;
		sensory.leftFloor = data.leftFloor;
		sensory.rightFloor = data.rightFloor;
		sensory.accelerationX = data.acceleration.x;
		sensory.accelerationY = data.acceleration.y;
		sensory.accelerationZ = data.acceleration.z;
		sensory.light = data.light;
		sensory.temperature = data.temperature;
		sensory.inputA = data.inputA;
		sensory.inputB = data.inputB;
		if(data.map & 0x00000010) sensory.lineTracerState = data.lineTracerState;
		if(data.map & 0x00000008) sensory.tilt = data.tilt;
		if(data.map & 0x00000004) sensory.batteryState = data.batteryState;
		var threshold = context.getHandFoundThreshold(MODULE);
		sensory.handFound = sensory.leftProximity > threshold || sensory.rightProximity > threshold;
		try {
			connection.send(JSON.stringify(sensory));
		} catch (e) {
		}
	}
};

module.exports = new Component();