'use strict';
var context = require('context');
var router = require('router');
var HamsterS = require('./hamsterS');
const MODULE = 'hamsterS';

const PACKET_NORMAL = 1;
const PACKET_SERIAL = 3;
const HEX_LEDS = [
	'#ff0000', '#ffff00', '#00ff00', '#00ffff', '#0000ff', '#ff00ff', '#ffffff'
];
const RGB_LEDS = [
	[255, 0, 0], [255, 255, 0], [0, 255, 0], [0, 255, 255], [0, 0, 255], [255, 0, 255], [255, 255, 255]
];

function Controller() {
	this.sensory = {
		map1: 0,
		map2: 0,
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		acceleration: { x: 0, y: 0, z: 0 },
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		tilt: 0,
		serial: undefined,
		pulseCount: 0,
		wheelState: 0,
		soundState: 0,
		lineTracerState: 0,
		batteryState: 2
	};
	this.motoring = {
		map1: 0,
		map2: 0,
		leftWheel: 0,
		rightWheel: 0,
		leftRgb: { r: 0, g: 0, b: 0 },
		rightRgb: { r: 0, g: 0, b: 0 },
		buzzer: 0,
		outputA: 0,
		outputB: 0,
		pulse: 0,
		note: 0,
		sound: 0,
		lineTracerMode: 0,
		lineTracerGain: 4,
		lineTracerSpeed: 5,
		ioModeA: 0,
		ioModeB: 0,
		serial: undefined,
		motion: { type: 0, unit: 0, speed: 0, value: 0, radius: 0 },
		motorMode: 0,
		configProximity: 2,
		configGravity: 0,
		configBandWidth: 0
	};
	this.motion = {
		type: 0,
		speed: 0
	};
	this.wheel = {
		id: 0,
		pulse: 0,
		pulsePrev: -1,
		event: 0,
		state: 0,
		count: 0,
		move: false,
		moveCount: 0,
		next: false,
		nextLeftWheel: 0,
		nextRightWheel: 0,
		nextPulse: 0
	};
	this.sound = {
		flag: 0,
		event: 0,
		state: 0,
		count: 0
	};
	this.lineTracer = {
		id: 0,
		event: 0,
		state: 0,
		count: 0
	};
	this.event = {
		freeFallId: -1,
		tapId: -1,
		tilt: -4,
		serialId: -1,
		pulseCount: -1,
		batteryState: -1
	};
	this.command = {
		serialId: 0,
	};
	this.port = {
		ackId: -1,
		serial: false
	};
	this.serial = {
		sendId: 0,
		sendPrevId: 0
	};
	this.packetSent = 0;
	this.packetReceived = 0;
	this.timerId = undefined;
	this.sensorMonitor = {
		count: 0,
		index: 0,
		duration: 0,
		averageIndex: 0,
		signalStrength: { current: 0, sum: 0.0, values: new Array(100) },
		leftProximity: { current: 0, sum: 0.0, values: new Array(100) },
		rightProximity: { current: 0, sum: 0.0, values: new Array(100) },
		leftFloor: { current: 0, sum: 0.0, values: new Array(100) },
		rightFloor: { current: 0, sum: 0.0, values: new Array(100) },
		accelerationX: { current: 0, sum: 0.0, values: new Array(100) },
		accelerationY: { current: 0, sum: 0.0, values: new Array(100) },
		accelerationZ: { current: 0, sum: 0.0, values: new Array(100) },
		light: { current: 0, sum: 0.0, values: new Array(100) },
		temperature: { current: 0, sum: 0.0, values: new Array(100) },
		inputA: { current: 0, sum: 0.0, values: new Array(100) },
		inputB: { current: 0, sum: 0.0, values: new Array(100) },
		reset: function() {
			this.signalStrength.current = 0;
			this.signalStrength.sum = 0.0;
			this.leftProximity.current = 0;
			this.leftProximity.sum = 0.0;
			this.rightProximity.current = 0;
			this.rightProximity.sum = 0.0;
			this.leftFloor.current = 0;
			this.leftFloor.sum = 0.0;
			this.rightFloor.current = 0;
			this.rightFloor.sum = 0.0;
			this.accelerationX.current = 0;
			this.accelerationX.sum = 0.0;
			this.accelerationY.current = 0;
			this.accelerationY.sum = 0.0;
			this.accelerationZ.current = 0;
			this.accelerationZ.sum = 0.0;
			this.light.current = 0;
			this.light.sum = 0.0;
			this.temperature.current = 0;
			this.temperature.sum = 0.0;
			this.inputA.current = 0;
			this.inputA.sum = 0.0;
			this.inputB.current = 0;
			this.inputB.sum = 0.0;
			this.count = 0;
			this.index = 0;
			this.averageIndex = 0;
		},
		put: function(device, value) {
			if(this.duration > 0) {
				device.sum += value;
				if(this.averageIndex >= this.duration - 1) {
					device.current = parseInt(device.sum * 100 / this.duration) / 100;
					device.sum = 0.0;
					device.values[this.index] = device.current;
				}
			} else {
				device.current = value;
				device.values[this.index] = value;
			}
		},
		update: function() {
			if(this.duration > 0) {
				++ this.averageIndex;
				if(this.averageIndex >= this.duration) {
					this.averageIndex = 0;
					if(this.count < 100) ++ this.count;
					++ this.index;
					if(this.index >= 100) this.index = 0;
				}
			} else {
				if(this.count < 100) ++ this.count;
				++ this.index;
				if(this.index >= 100) this.index = 0;
			}
		}
	};
	this.sensorNames = [
		'proximity sensor and floor sensor', 'acceleration sensor', 'light sensor', 'signal strength and temperature sensor', 'input A/B'
	];
	this.updateIdentity();
}

Controller.prototype.updateIdentity = function() {
	this.identity = Math.floor(Math.random() * 49); // 0 ~ 48
};

Controller.prototype.getIdentitySize = function() {
	return 2;
};

Controller.prototype.getIdentityHex = function(index) {
	if(index == 0) return HEX_LEDS[Math.floor(this.identity / 7)];
	else return HEX_LEDS[this.identity % 7];
};

Controller.prototype.getIdentityRgb = function(index) {
	if(index == 0) return RGB_LEDS[Math.floor(this.identity / 7)];
	else return RGB_LEDS[this.identity % 7];
};

Controller.prototype.getIdentityX = function(index) {
	return (index == 0) ? '79px' : '51px'
};

Controller.prototype.getIdentityY = function(index) {
	return (index == 0) ? '61px' : '75px'
};

Controller.prototype.toHex = function(number) {
	var value = parseInt(number);
	if(value < 0) value += 0x100;

	value = value.toString(16).toUpperCase();
	if(value.length > 1) return value;
	else return '0' + value;
};

Controller.prototype.toHex2 = function(number) {
	var value = parseInt(number);
	if(value < 0) value += 0x10000;
	
	value = value.toString(16).toUpperCase();
	var result = '';
	for(var i = value.length; i < 4; ++i) {
		result += '0';
	}
	return result + value;
};

Controller.prototype.calculateInnerSpeed = function(speed, radius) {
	return speed * (radius - HamsterS.WHEEL_CENTER_DISTANCE) / (radius + HamsterS.WHEEL_CENTER_DISTANCE);
};

Controller.prototype.calculateSwingPulse = function(deg, radius, degToPulse) {
	return Math.round(deg * degToPulse * (radius + HamsterS.WHEEL_CENTER_DISTANCE) / HamsterS.WHEEL_CENTER_DISTANCE);
};

Controller.prototype.cancelTimeout = function() {
	if(this.timerId !== undefined) clearTimeout(this.timerId);
	this.timerId = undefined;
};

Controller.prototype.load = function(connector, route, callback) {
	callback();
};

Controller.prototype.getOrCreateSerialArray = function(target) {
	if(target.serial == undefined) target.serial = new Array(19);
	return target.serial;
};

Controller.prototype.handleLocalData = function(data) { // data: string
	if(data.length != 53) return;
	
	var sensory = this.sensory;
	sensory.map1 = 0x80007fe0;
	sensory.map2 = 0x00000000;

	this.packetReceived = 0;
	var str = data.slice(0, 1);
	var value = parseInt(str, 16);
	if(value == 1) { // normal
		var motoring = this.motoring;
		var sensorMonitor = this.sensorMonitor;
		var event = this.event;
		// pulse count
		str = data.slice(2, 6);
		value = parseInt(str, 16);
		if(value != event.pulseCount) {
			if(event.pulseCount != -1) {
				sensory.map2 |= 0x00001000;
			}
			event.pulseCount = value;
			sensory.pulseCount = value;
		}
		// left proximity
		str = data.slice(6, 8);
		value = parseInt(str, 16);
		sensory.leftProximity = value;
		sensorMonitor.put(sensorMonitor.leftProximity, value);
		// right proximity
		str = data.slice(8, 10);
		value = parseInt(str, 16);
		sensory.rightProximity = value;
		sensorMonitor.put(sensorMonitor.rightProximity, value);
		// flag
		str = data.slice(38, 40);
		var value2 = parseInt(str, 16);
		if((value2 & 0x01) == 0) {
			// light
			str = data.slice(10, 14);
			value = parseInt(str, 16);
			sensory.light = value;
		} else {
			// temperature
			str = data.slice(10, 12);
			value = parseInt(str, 16);
			if(value > 0x7f) value -= 0x100;
			value = value / 2.0 + 23;
			sensory.temperature = parseInt(value);
		}
		sensorMonitor.put(sensorMonitor.light, sensory.light);
		sensorMonitor.put(sensorMonitor.temperature, sensory.temperature);
		// left floor
		str = data.slice(14, 16);
		value = parseInt(str, 16);
		sensory.leftFloor = value;
		sensorMonitor.put(sensorMonitor.leftFloor, value);
		// right floor
		str = data.slice(16, 18);
		value = parseInt(str, 16);
		sensory.rightFloor = value;
		sensorMonitor.put(sensorMonitor.rightFloor, value);
		// acceleration x
		var acceleration = sensory.acceleration;
		str = data.slice(18, 22);
		value = parseInt(str, 16);
		if(value > 0x7fff) value -= 0x10000;
		acceleration.x = value;
		sensorMonitor.put(sensorMonitor.accelerationX, value);
		// acceleration y
		str = data.slice(22, 26);
		value = parseInt(str, 16);
		if(value > 0x7fff) value -= 0x10000;
		acceleration.y = value;
		sensorMonitor.put(sensorMonitor.accelerationY, value);
		// acceleration z
		str = data.slice(26, 30);
		value = parseInt(str, 16);
		if(value > 0x7fff) value -= 0x10000;
		acceleration.z = value;
		sensorMonitor.put(sensorMonitor.accelerationZ, value);
		// tilt
		if(acceleration.z < 8192 && acceleration.x > 8192 && acceleration.y > -4096 && acceleration.y < 4096) value = 1;
		else if(acceleration.z < 8192 && acceleration.x < -8192 && acceleration.y > -4096 && acceleration.y < 4096) value = -1;
		else if(acceleration.z < 8192 && acceleration.y > 8192 && acceleration.x > -4096 && acceleration.x < 4096) value = 2;
		else if(acceleration.z < 8192 && acceleration.y < -8192 && acceleration.x > -4096 && acceleration.x < 4096) value = -2;
		else if(acceleration.z > 12288 && acceleration.x > -8192 && acceleration.x < 8192 && acceleration.y > -8192 && acceleration.y < 8192) value = 3;
		else if(acceleration.z < -12288 && acceleration.x > -4096 && acceleration.x < 4096 && acceleration.y > -4096 && acceleration.y < 4096) value = -3;
		else value = 0;
		if(value != event.tilt) {
			if(event.tilt != -4) {
				sensory.map1 |= 0x00000008;
			}
			event.tilt = value;
			sensory.tilt = value;
		}
		// input a
		str = data.slice(30, 32);
		value = parseInt(str, 16);
		if(motoring.ioModeA == HamsterS.IO_MODE_VOLTAGE_INPUT) {
			value = value * 3.6 / 255;
			value = value.toFixed(2);
		}
		sensory.inputA = value;
		sensorMonitor.put(sensorMonitor.inputA, value);
		// input b
		str = data.slice(32, 34);
		value = parseInt(str, 16);
		if(motoring.ioModeB == HamsterS.IO_MODE_VOLTAGE_INPUT) {
			value = value * 3.6 / 255;
			value = value.toFixed(2);
		}
		sensory.inputB = value;
		sensorMonitor.put(sensorMonitor.inputB, value);
		// internal state
		str = data.slice(34, 36);
		value = parseInt(str, 16);
		// free fall
		var id = (value >> 6) & 0x03;
		if(id != event.freeFallId) {
			if(event.freeFallId != -1) {
				sensory.map2 |= 0x00008000;
			}
			event.freeFallId = id;
		}
		// tap
		var wheel = this.wheel;
		id = (value >> 4) & 0x03;
		if(id != event.tapId) {
			if(event.tapId != -1 && !wheel.move) {
				sensory.map2 |= 0x00004000;
			}
			event.tapId = id;
		}
		// wheel state
		var state = (value >> 2) & 0x03;
//		console.log('target pulse = ' + wheel.pulse + ' pulse count = ' + sensory.pulseCount + ' state = ' + state);
//		if(wheel.event == 1) {
//			if(state == 3) {
//				wheel.event = 2;
//			} else if(state == 2) {
//				if(++wheel.count > 5) wheel.event = 2;
//			}
//		}
		if(wheel.event == 1) {
//			if(state != wheel.state || wheel.count > 5) {
			if(state == 2) {
				if(wheel.pulse != sensory.pulseCount) {
					if(wheel.pulse < 15) {
						if(++wheel.count < 10) state = 3;
					} else {
						state = 3;
					}
				}
			} else if(state == 3 && wheel.pulse == sensory.pulseCount) {
				state = 2;
			}
			if(state != wheel.state) {
				wheel.state = state;
				sensory.wheelState = state;
				sensory.map2 |= 0x00000800;
			}
			if(state == 2) {
				wheel.event = 0;
				wheel.count = 0;
				this.motion.type = HamsterS.MOTION_NONE;
//				console.log('DONE');
			}
//			}
		}
		if(wheel.event == -1) {
			wheel.state = state;
			sensory.wheelState = 2;
			sensory.map2 |= 0x00000800;
			wheel.event = 0;
			wheel.count = 0;
			this.motion.type = HamsterS.MOTION_NONE;
		}
		// sound state
		state = (value >> 1) & 0x01;
		var sound = this.sound;
		if(sound.event == 1) {
			if(state == 0) {
				if(++sound.count > 5) sound.event = 2;
			} else {
				sound.event = 2;
			}
		}
		if(sound.event == 2) {
			if(state != sound.state || sound.count > 5) {
				sound.state = state;
				sensory.soundState = state;
				sensory.map2 |= 0x00000400;
				if(state == 0) {
					sound.event = 0;
					sound.count = 0;
				}
			}
		}
		// signal strength
		str = data.slice(36, 38);
		value = parseInt(str, 16);
		value -= 0x100;
		sensory.signalStrength = value;
		sensorMonitor.put(sensorMonitor.signalStrength, value);
		// linetracer state
		state = (value2 >> 6) & 0x03;
		if((state & 0x02) != 0) {
			var lineTracer = this.lineTracer;
			if(lineTracer.event == 1) {
				if(state == 0x02) {
					if(++lineTracer.count > 5) lineTracer.event = 2;
				} else {
					lineTracer.event = 2;
				}
			}
			if(lineTracer.event == 2) {
				if(state != lineTracer.state || lineTracer.count > 5) {
					lineTracer.state = state;
					sensory.lineTracerState = (state << 5);
					sensory.map1 |= 0x00000010;
					if(state == 0x02) {
						lineTracer.event = 0;
						lineTracer.count = 0;
					}
				}
			}
		}
		// battery state
		state = (value2 >> 1) & 0x03;
		if(state == 0) state = 2; // normal
		else if(state >= 2) state = 0; // empty
		if(state != event.batteryState) {
			if(event.batteryState != -1) {
				sensory.map1 |= 0x00000004;
			}
			event.batteryState = state;
			sensory.batteryState = state;
		}
		var port = this.port;
		id = (value2 >> 5) & 0x01;
		if(id != port.ackId) {
			if(port.ackId != -1) {
				switch(motoring.ioModeA) {
					case HamsterS.IO_MODE_SERIAL_9600:
					case HamsterS.IO_MODE_SERIAL_14400:
					case HamsterS.IO_MODE_SERIAL_19200:
					case HamsterS.IO_MODE_SERIAL_28800:
					case HamsterS.IO_MODE_SERIAL_38400:
					case HamsterS.IO_MODE_SERIAL_57600:
					case HamsterS.IO_MODE_SERIAL_76800:
					case HamsterS.IO_MODE_SERIAL_115200:
						port.serial = true;
						break;
					default:
						port.serial = false;
						break;
				}
			}
			port.ackId = id;
		}
		
		sensorMonitor.update();
		this.packetReceived = PACKET_NORMAL;
		this.handleExtensionSensory(sensory);
		return sensory;
	} else if(value == 2) { // serial
		str = data.slice(1, 2);
		var id = parseInt(str, 16);
		var event = this.event;
		if(id != event.serialId) {
			if(event.serialId != -1) {
				sensory.map2 |= 0x00002000;
				str = data.slice(2, 4);
				var len = parseInt(str, 16);
				if(len > 18) len = 18;
				var serial = this.getOrCreateSerialArray(sensory);
				serial[0] = len;
				for(var i = 1, j = 4; i <= len; ++i, j += 2) {
					str = data.slice(j, j + 2);
					serial[i] = parseInt(str, 16);
				}
				for(var i = len + 1; i <= 18; ++i) {
					serial[i] = 0;
				}
			}
			event.serialId = id;
		}
		this.packetReceived = PACKET_SERIAL;
		this.handleExtensionSensory(sensory);
		return sensory;
	}
};

Controller.prototype.handleExtensionSensory = function(sensory) {
	var serial = this.serial;
	if(serial.sendId != serial.sendPrevId) {
		serial.sendPrevId = serial.sendId;
		sensory.map2 |= 0x00000200;
	}
};

Controller.prototype.write = function(data) {
	if(!data) return this.motoring;
	
	var map1 = data.map1;
	var map2 = data.map2;
	var writeData = data.writeData;
	var motoring = this.motoring;
	
	if(map1 & 0x00080000) motoring.ioModeA = writeData[13][0];
	if(map1 & 0x00040000) motoring.ioModeB = writeData[14][0];
	
	// left wheel
	var t = motoring.leftWheel + writeData[0][0];
	if(t < -100) t = -100;
	else if(t > 100) t = 100;
	motoring.leftWheel = t;
	// right wheel
	t = motoring.rightWheel + writeData[1][0];
	if(t < -100) t = -100;
	else if(t > 100) t = 100;
	motoring.rightWheel = t;
	// left rgb
	t = motoring.leftRgb.r + writeData[2][0];
	if(t < 0) t = 0;
	else if(t > 255) t = 255;
	motoring.leftRgb.r = t;
	t = motoring.leftRgb.g + writeData[2][1];
	if(t < 0) t = 0;
	else if(t > 255) t = 255;
	motoring.leftRgb.g = t;
	t = motoring.leftRgb.b + writeData[2][2];
	if(t < 0) t = 0;
	else if(t > 255) t = 255;
	motoring.leftRgb.b = t;
	// right rgb
	t = motoring.rightRgb.r + writeData[3][0];
	if(t < 0) t = 0;
	else if(t > 255) t = 255;
	motoring.rightRgb.r = t;
	t = motoring.rightRgb.g + writeData[3][1];
	if(t < 0) t = 0;
	else if(t > 255) t = 255;
	motoring.rightRgb.g = t;
	t = motoring.rightRgb.b + writeData[3][2];
	if(t < 0) t = 0;
	else if(t > 255) t = 255;
	motoring.rightRgb.b = t;
	// buzzer;
	t = writeData[4][0];
	if(t == 0) {
		motoring.buzzer = 0;
	} else {
		t = motoring.buzzer + t;
		if(t < 0.1) t = 0.1;
		else if(t > 6500.0) t = 6500.0;
		motoring.buzzer = t;
	}
	// output a
	t = writeData[5][0];
	switch(motoring.ioModeA) {
		case HamsterS.IO_MODE_DIGITAL_OUTPUT:
			break;
		case HamsterS.IO_MODE_PWM_OUTPUT:
			t += motoring.outputA;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case HamsterS.IO_MODE_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputA;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputA;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputA = t;
	// output b
	t = writeData[6][0];
	switch(motoring.ioModeB) {
		case HamsterS.IO_MODE_DIGITAL_OUTPUT:
			break;
		case HamsterS.IO_MODE_PWM_OUTPUT:
			t += motoring.outputB;
			if(t < 0) t = 0;
			else if(t > 100) t = 100;
			break;
		case HamsterS.IO_MODE_SERVO_OUTPUT:
			if(t != 0) {
				t += motoring.outputB;
				if(t < 1) t = 1;
				else if(t > 180) t = 180;
			}
			break;
		default:
			t += motoring.outputB;
			if(t < 0) t = 0;
			else if(t > 255) t = 255;
			break;
	}
	motoring.outputB = t;
	// commands
	if(map2 & 0x20000000) motoring.pulse = writeData[7][0];
	if(map1 & 0x00400000) motoring.note = writeData[8][0];
	if(map2 & 0x10000000) motoring.sound = writeData[9][0];
	if(map1 & 0x00200000) motoring.lineTracerMode = writeData[10][0];
	if(map2 & 0x08000000) motoring.lineTracerGain = writeData[11][0];
	if(map1 & 0x00100000) motoring.lineTracerSpeed = writeData[12][0];
	if(map2 & 0x04000000) {
		var serial = this.getOrCreateSerialArray(motoring);
		var w = writeData[15];
		for(var i = 0; i < 19; ++i) {
			serial[i] = w[i];
		}
	}
	if(map2 & 0x02000000) {
		motoring.motion.type = parseInt(writeData[16][0]);
		motoring.motion.unit = parseInt(writeData[16][1]);
		motoring.motion.speed = writeData[16][2];
		motoring.motion.value = writeData[16][3];
		motoring.motion.radius = writeData[16][4];
	}
	if(map2 & 0x01000000) motoring.motorMode = writeData[17][0];
	if(map1 & 0x00020000) motoring.configProximity = writeData[18][0];
	if(map1 & 0x00010000) motoring.configGravity = writeData[19][0];
	if(map1 & 0x00008000) motoring.configBandWidth = writeData[20][0];
	
	motoring.map1 |= map1;
	motoring.map2 |= map2;
	return motoring;
};

Controller.prototype.requestLocalData = function(address) {
	var self = this;
	var motoring = self.motoring;
	var port = self.port;
	
	// serial
	if(port.serial && (motoring.map2 & 0x04000000) && self.packetSent != PACKET_SERIAL) {
		motoring.map2 &= 0xfbffffff;
		self.packetSent = PACKET_SERIAL;
		self.serial.sendId = (self.serial.sendId % 255) + 1;
		
		var serial = motoring.serial;
		if(serial) {
			var len = serial[0];
			if(len > 0) {
				if(len > 18) len = 18;
				var command = self.command;
				command.serialId = (command.serialId % 15) + 1;
				var tmp = 0x20 | (command.serialId & 0x0f);
				var str = self.toHex(tmp);
				str += self.toHex(len);
				for(var i = 1; i <= len; ++i) {
					str += self.toHex(serial[i]);
				}
				for(var i = len + 1; i <= 18; ++i) {
					str += '00';
				}
				str += '-';
				str += address;
				str += '\r';
				return str;
			}
		}
	}
	
	var motion = self.motion;
	var wheel = self.wheel;
	var leftWheel = motoring.leftWheel, rightWheel = motoring.rightWheel;
	if(motoring.map2 & 0x02000000) {
		self.cancelTimeout();
		motion.type = parseInt(motoring.motion.type);
		if(motion.type < HamsterS.MOTION_NONE || motion.type > HamsterS.MOTION_SWING_RIGHT_PEN_RIGHT_BACKWARD) {
			motion.type = HamsterS.MOTION_NONE;
		}
		if(motion.type != HamsterS.MOTION_NONE) {
			motion.speed = motoring.motion.speed;
			wheel.pulse = 0;
			var unit = parseInt(motoring.motion.unit);
			var value = motoring.motion.value;
			if(unit == HamsterS.UNIT_SEC) {
				if(value == 0) {
					leftWheel = 0;
					rightWheel = 0;
					wheel.count = 0;
					wheel.event = -1;
					motion.type = HamsterS.MOTION_NONE;
				} else {
					self.timerId = setTimeout(function() {
						self.cancelTimeout();
						leftWheel = 0;
						rightWheel = 0;
						wheel.count = 0;
						wheel.event = -1;
						motion.type = HamsterS.MOTION_NONE;
					}, value * 1000);
				}
			} else {
				if(unit == HamsterS.UNIT_PULSE) {
					wheel.pulse = Math.round(value);
				} else if(unit == HamsterS.UNIT_CM_DEG) {
					switch(motion.type) {
						case HamsterS.MOTION_MOVE_FORWARD:
						case HamsterS.MOTION_MOVE_BACKWARD:
							wheel.pulse = Math.round(value * HamsterS.CM_TO_PULSE);
							break;
						case HamsterS.MOTION_TURN_LEFT:
						case HamsterS.MOTION_TURN_RIGHT:
							wheel.pulse = Math.round(value * HamsterS.DEG_TO_PULSE);
							break;
						case HamsterS.MOTION_PIVOT_LEFT_FORWARD:
						case HamsterS.MOTION_PIVOT_LEFT_BACKWARD:
						case HamsterS.MOTION_PIVOT_RIGHT_FORWARD:
						case HamsterS.MOTION_PIVOT_RIGHT_BACKWARD:
							wheel.pulse = Math.round(value * HamsterS.DEG_TO_PULSE * 2);
							break;
						case HamsterS.MOTION_SWING_LEFT_FORWARD:
						case HamsterS.MOTION_SWING_LEFT_BACKWARD:
						case HamsterS.MOTION_SWING_RIGHT_FORWARD:
						case HamsterS.MOTION_SWING_RIGHT_BACKWARD:
							wheel.pulse = self.calculateSwingPulse(value, motoring.motion.radius, HamsterS.DEG_TO_PULSE);
							break;
						case HamsterS.MOTION_PIVOT_LEFT_PEN_FORWARD:
						case HamsterS.MOTION_PIVOT_LEFT_PEN_BACKWARD:
						case HamsterS.MOTION_PIVOT_RIGHT_PEN_FORWARD:
						case HamsterS.MOTION_PIVOT_RIGHT_PEN_BACKWARD:
							wheel.pulse = self.calculateSwingPulse(value, HamsterS.PEN_CENTER_DISTANCE, HamsterS.DEG_TO_PULSE_PIVOT_PEN);
							break;
						case HamsterS.MOTION_SWING_LEFT_PEN_LEFT_FORWARD:
						case HamsterS.MOTION_SWING_LEFT_PEN_LEFT_BACKWARD:
						case HamsterS.MOTION_SWING_RIGHT_PEN_RIGHT_FORWARD:
						case HamsterS.MOTION_SWING_RIGHT_PEN_RIGHT_BACKWARD:
							wheel.pulse = self.calculateSwingPulse(value, motoring.motion.radius + HamsterS.PEN_CENTER_DISTANCE, HamsterS.DEG_TO_PULSE);
							break;
						case HamsterS.MOTION_SWING_LEFT_PEN_RIGHT_FORWARD:
						case HamsterS.MOTION_SWING_LEFT_PEN_RIGHT_BACKWARD:
						case HamsterS.MOTION_SWING_RIGHT_PEN_LEFT_FORWARD:
						case HamsterS.MOTION_SWING_RIGHT_PEN_LEFT_BACKWARD:
							if(motoring.motion.radius >= HamsterS.PEN_CENTER_DISTANCE) {
								wheel.pulse = self.calculateSwingPulse(value, motoring.motion.radius - HamsterS.PEN_CENTER_DISTANCE, HamsterS.DEG_TO_PULSE);
							} else {
								wheel.pulse = self.calculateSwingPulse(value, HamsterS.PEN_CENTER_DISTANCE - motoring.motion.radius, HamsterS.DEG_TO_PULSE);
							}
							break;
						default:
							motion.type = HamsterS.MOTION_NONE;
							break;
					}
				}
				if(wheel.pulse == 0) {
					leftWheel = 0;
					rightWheel = 0;
					wheel.count = 0;
					wheel.event = -1;
					motion.type = HamsterS.MOTION_NONE;
				}
			}
		}
	}
	if(motion.type == HamsterS.MOTION_NONE) wheel.pulse = motoring.pulse;
	var speed = motion.speed;
	switch(motion.type) {
		case HamsterS.MOTION_MOVE_FORWARD:
			if(speed == 0) speed = context.getMovementSpeed(MODULE);
			leftWheel = speed;
			rightWheel = speed;
			break;
		case HamsterS.MOTION_MOVE_BACKWARD:
			if(speed == 0) speed = context.getMovementSpeed(MODULE);
			leftWheel = -speed;
			rightWheel = -speed;
			break;
		case HamsterS.MOTION_TURN_LEFT:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -speed;
			rightWheel = speed;
			break;
		case HamsterS.MOTION_TURN_RIGHT:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = speed;
			rightWheel = -speed;
			break;
		case HamsterS.MOTION_PIVOT_LEFT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = 0;
			rightWheel = speed;
			break;
		case HamsterS.MOTION_PIVOT_LEFT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = 0;
			rightWheel = -speed;
			break;
		case HamsterS.MOTION_PIVOT_RIGHT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = speed;
			rightWheel = 0;
			break;
		case HamsterS.MOTION_PIVOT_RIGHT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -speed;
			rightWheel = 0;
			break;
		case HamsterS.MOTION_SWING_LEFT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = self.calculateInnerSpeed(speed, motoring.motion.radius);
			rightWheel = speed;
			break;
		case HamsterS.MOTION_SWING_LEFT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -self.calculateInnerSpeed(speed, motoring.motion.radius);
			rightWheel = -speed;
			break;
		case HamsterS.MOTION_SWING_RIGHT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = speed;
			rightWheel = self.calculateInnerSpeed(speed, motoring.motion.radius);
			break;
		case HamsterS.MOTION_SWING_RIGHT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -speed;
			rightWheel = -self.calculateInnerSpeed(speed, motoring.motion.radius);
			break;
		case HamsterS.MOTION_PIVOT_LEFT_PEN_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE);
			rightWheel = speed;
			break;
		case HamsterS.MOTION_PIVOT_LEFT_PEN_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE);
			rightWheel = -speed;
			break;
		case HamsterS.MOTION_PIVOT_RIGHT_PEN_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = speed;
			rightWheel = self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE);
			break;
		case HamsterS.MOTION_PIVOT_RIGHT_PEN_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -speed;
			rightWheel = -self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE);
			break;
		case HamsterS.MOTION_SWING_LEFT_PEN_LEFT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = self.calculateInnerSpeed(speed, motoring.motion.radius + HamsterS.PEN_CENTER_DISTANCE);
			rightWheel = speed;
			break;
		case HamsterS.MOTION_SWING_LEFT_PEN_LEFT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -self.calculateInnerSpeed(speed, motoring.motion.radius + HamsterS.PEN_CENTER_DISTANCE);
			rightWheel = -speed;
			break;
		case HamsterS.MOTION_SWING_LEFT_PEN_RIGHT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			if(motoring.motion.radius >= HamsterS.PEN_CENTER_DISTANCE) {
				leftWheel = speed;
				rightWheel = self.calculateInnerSpeed(speed, motoring.motion.radius - HamsterS.PEN_CENTER_DISTANCE);
			} else {
				leftWheel = -self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE - motoring.motion.radius);
				rightWheel = -speed;
			}
			break;
		case HamsterS.MOTION_SWING_LEFT_PEN_RIGHT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			if(motoring.motion.radius >= HamsterS.PEN_CENTER_DISTANCE) {
				leftWheel = -speed;
				rightWheel = -self.calculateInnerSpeed(speed, motoring.motion.radius - HamsterS.PEN_CENTER_DISTANCE);
			} else {
				leftWheel = self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE - motoring.motion.radius);
				rightWheel = speed;
			}
			break;
		case HamsterS.MOTION_SWING_RIGHT_PEN_LEFT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			if(motoring.motion.radius >= HamsterS.PEN_CENTER_DISTANCE) {
				leftWheel = self.calculateInnerSpeed(speed, motoring.motion.radius - HamsterS.PEN_CENTER_DISTANCE);
				rightWheel = speed;
			} else {
				leftWheel = -speed;
				rightWheel = -self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE - motoring.motion.radius);
			}
			break;
		case HamsterS.MOTION_SWING_RIGHT_PEN_LEFT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			if(motoring.motion.radius >= HamsterS.PEN_CENTER_DISTANCE) {
				leftWheel = -self.calculateInnerSpeed(speed, motoring.motion.radius - HamsterS.PEN_CENTER_DISTANCE);
				rightWheel = -speed;
			} else {
				leftWheel = speed;
				rightWheel = self.calculateInnerSpeed(speed, HamsterS.PEN_CENTER_DISTANCE - motoring.motion.radius);
			}
			break;
		case HamsterS.MOTION_SWING_RIGHT_PEN_RIGHT_FORWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = speed;
			rightWheel = self.calculateInnerSpeed(speed, motoring.motion.radius + HamsterS.PEN_CENTER_DISTANCE);
			break;
		case HamsterS.MOTION_SWING_RIGHT_PEN_RIGHT_BACKWARD:
			if(speed == 0) speed = context.getTurningSpeed(MODULE);
			leftWheel = -speed;
			rightWheel = -self.calculateInnerSpeed(speed, motoring.motion.radius + HamsterS.PEN_CENTER_DISTANCE);
			break;
	}
	
	if(((motoring.map2 & 0x02000000) || (motoring.map2 & 0x20000000)) && wheel.pulse > 0) {
		wheel.nextLeftWheel = leftWheel;
		wheel.nextRightWheel = rightWheel;
		wheel.nextPulse = wheel.pulse;
		wheel.next = true;
		leftWheel = 0;
		rightWheel = 0;
		wheel.pulse = 0;
	} else if(wheel.next) {
		wheel.next = false;
		leftWheel = wheel.nextLeftWheel;
		rightWheel = wheel.nextRightWheel;
		wheel.pulse = wheel.nextPulse;
		motoring.map2 |= 0x20000000;
	}
	
	var str = '10';
	if(motoring.motorMode == HamsterS.MOTOR_WAVE) str = '11';
    else if(motoring.motorMode == HamsterS.MOTOR_HALF) str = '12';
	else if(motoring.motorMode == HamsterS.MOTOR_FULL) str = '13';
	if(leftWheel == 0 && rightWheel == 0) {
		if(wheel.move && ++wheel.moveCount > 5) {
			wheel.move = false;
		}
	} else {
		wheel.move = true;
		wheel.moveCount = 0;
	}
	if(leftWheel < 0) str += self.toHex(leftWheel * 1.27 - 0.5);
	else str += self.toHex(leftWheel * 1.27 + 0.5);
	if(rightWheel < 0) str += self.toHex(rightWheel * 1.27 - 0.5);
	else str += self.toHex(rightWheel * 1.27 + 0.5);
	if(router.isExtraButtonActive()) {
		var rgb = self.getIdentityRgb(0);
		str += self.toHex(rgb[0]);
		str += self.toHex(rgb[1]);
		str += self.toHex(rgb[2]);
		rgb = self.getIdentityRgb(1);
		str += self.toHex(rgb[0]);
		str += self.toHex(rgb[1]);
		str += self.toHex(rgb[2]);
	} else {
		str += self.toHex(motoring.leftRgb.r);
		str += self.toHex(motoring.leftRgb.g);
		str += self.toHex(motoring.leftRgb.b);
		str += self.toHex(motoring.rightRgb.r);
		str += self.toHex(motoring.rightRgb.g);
		str += self.toHex(motoring.rightRgb.b);
	}
	if((motoring.map2 & 0x02000000) || (motoring.map2 & 0x20000000)) {
		if(wheel.pulse != 0 || wheel.pulsePrev != 0) {
			wheel.id = (wheel.id % 255) + 1;
		}
		wheel.count = 0;
		if(wheel.pulse > 0) {
			wheel.event = 1;
		} else if(wheel.event != -1) {
			wheel.event = 0;
		}
		wheel.pulsePrev = wheel.pulse;
	}
	str += self.toHex(wheel.id);
	str += self.toHex2(wheel.pulse);
	var lineTracer = self.lineTracer;
	var tmp = motoring.lineTracerMode & 0x0f;
	if(tmp >= HamsterS.LINE_TRACER_MODE_WHITE_LEFT_SENSOR) tmp += 1; // for white line
	if(motoring.map1 & 0x00200000) {
		lineTracer.count = 0;
		if(tmp > 0) {
			lineTracer.id = (lineTracer.id % 15) + 1;
			lineTracer.event = 1;
		} else {
			lineTracer.event = 0;
		}
	}
	tmp |= (lineTracer.id & 0x0f) << 4;
	str += self.toHex(tmp);
	tmp = (motoring.lineTracerSpeed & 0x0f) << 4;
	tmp |= motoring.lineTracerGain & 0x0f;
	str += self.toHex(tmp);
	tmp = (motoring.configProximity & 0x07) << 5;
	tmp |= (motoring.configBandWidth & 0x07) << 2;
	tmp |= (motoring.configGravity & 0x03);
	str += self.toHex(tmp);
	if(motoring.ioModeA >= HamsterS.IO_MODE_SERIAL_9600) {
		tmp = motoring.ioModeA;
	} else {
		var v = motoring.ioModeA;
		if(v == HamsterS.IO_MODE_VOLTAGE_INPUT) v = HamsterS.IO_MODE_ANALOG_INPUT_ABSOLUTE;
		tmp = (v & 0x0f) << 4;
		v = motoring.ioModeB;
		if(v == HamsterS.IO_MODE_VOLTAGE_INPUT) v = HamsterS.IO_MODE_ANALOG_INPUT_ABSOLUTE;
		tmp |= (v & 0x0f);
	}
	str += self.toHex(tmp);
	str += self.toHex(motoring.outputA);
	str += self.toHex(motoring.outputB);
	
	// sound
	var sound = self.sound;
	var sndid = 0;
	switch(motoring.sound) {
		case HamsterS.SOUND_OFF: sndid = 0x00; break;
		case HamsterS.SOUND_BEEP: sndid = 0x01; break;
		case HamsterS.SOUND_RANDOM_BEEP: sndid = 0x05; break;
		case HamsterS.SOUND_NOISE: sndid = 0x07; break;
		case HamsterS.SOUND_SIREN: sndid = 0x09; break;
		case HamsterS.SOUND_ENGINE: sndid = 0x0b; break;
		case HamsterS.SOUND_CHOP: sndid = 0x12; break;
		case HamsterS.SOUND_ROBOT: sndid = 0x20; break;
		case HamsterS.SOUND_DIBIDIBIDIP: sndid = 0x21; break;
		case HamsterS.SOUND_GOOD_JOB: sndid = 0x23; break;
		case HamsterS.SOUND_HAPPY: sndid = 0x30; break;
		case HamsterS.SOUND_ANGRY: sndid = 0x31; break;
		case HamsterS.SOUND_SAD: sndid = 0x32; break;
		case HamsterS.SOUND_SLEEP: sndid = 0x33; break;
		case HamsterS.SOUND_MARCH: sndid = 0x34; break;
		case HamsterS.SOUND_BIRTHDAY: sndid = 0x35; break;
	}
	if(motoring.map2 & 0x10000000) {
		sound.count = 0;
		if(sndid > 0) {
			sound.flag ^= 0x80;
			sound.event = 1;
		} else {
			sound.event = 0;
		}
	}
	if(sndid > 0) { // sound
		str += '00';
		str += self.toHex(sndid | sound.flag);
	} else if(motoring.note > 0) { // note
		str += '01';
		str += self.toHex(motoring.note);
	} else { // buzzer
		tmp = parseInt(motoring.buzzer * 10) + 512;
		str += self.toHex2(tmp);
	}
	str += '-';
	str += address;
	str += '\r';
	self.packetSent = PACKET_NORMAL;
	return str;
};

Controller.prototype.clear = function() {
	var motoring = this.motoring;
	if(this.packetSent == PACKET_NORMAL) {
		motoring.map1 = 0x80000000;
		motoring.map2 &= 0x04000000;
	}
	motoring.leftWheel = 0;
	motoring.rightWheel = 0;
	motoring.leftRgb.r = 0;
	motoring.leftRgb.g = 0;
	motoring.leftRgb.b = 0;
	motoring.rightRgb.r = 0;
	motoring.rightRgb.g = 0;
	motoring.rightRgb.b = 0;
	motoring.buzzer = 0.0;
	motoring.outputA = 0;
	motoring.outputB = 0;
	
	var sensory = this.sensory;
	switch(this.packetReceived) {
		case PACKET_NORMAL:
			sensory.map1 = 0x80007fe0;
			sensory.map2 &= 0x00002000;
			break;
		case PACKET_SERIAL:
			sensory.map2 &= 0xffffdfff;
			break;
	}
	sensory.tilt = 0;
	sensory.pulseCount = 0;
	sensory.wheelState = 0;
	sensory.soundState = 0;
	sensory.lineTracerState = 0;
	sensory.batteryState = 2;
};

Controller.prototype.reset = function() {
	this.clear();
	this.cancelTimeout();
	
	var motoring = this.motoring;
	motoring.pulse = 0;
	motoring.note = 0;
	motoring.sound = 0;
	motoring.lineTracerMode = 0;
	motoring.lineTracerGain = 4;
	motoring.lineTracerSpeed = 5;
	motoring.ioModeA = 0;
	motoring.ioModeB = 0;
	motoring.serial = undefined;
	motoring.motion.type = 0;
	motoring.motion.unit = 0;
	motoring.motion.speed = 0;
	motoring.motion.value = 0;
	motoring.motion.radius = 0;
	motoring.motorMode = 0;
	motoring.configProximity = 2;
	motoring.configGravity = 0;
	motoring.configBandWidth = 0;
	
	var sensory = this.sensory;
	sensory.serial = undefined;
	
	var motion = this.motion;
	motion.type = 0;
	motion.speed = 0;
	var wheel = this.wheel;
	wheel.pulse = 0;
	wheel.pulsePrev = -1;
	wheel.event = 0;
	wheel.state = 0;
	wheel.count = 0;
	wheel.move = false;
	wheel.moveCount = 0;
	wheel.next = false;
	wheel.nextLeftWheel = 0;
	wheel.nextRightWheel = 0;
	wheel.nextPulse = 0;
	var sound = this.sound;
	sound.event = 0;
	sound.state = 0;
	sound.count = 0;
	var lineTracer = this.lineTracer;
	lineTracer.event = 0;
	lineTracer.state = 0;
	lineTracer.count = 0;
	var event = this.event;
	event.freeFallId = -1;
	event.tapId = -1;
	event.tilt = -4;
	event.serialId = -1;
	event.pulseCount = -1;
	event.batteryState = -1;
	this.serial.sendId = 0;
	this.serial.sendPrevId = 0;
	var port = this.port;
	port.ackId = -1;
	port.serial = false;
	this.packetSent = 0;
	this.packetReceived = 0;
};

Controller.prototype.getSensorMonitor = function() {
	return this.sensorMonitor;
};

Controller.prototype.getSensorNames = function() {
	return this.sensorNames;
};

Controller.prototype.setDuration = function(duration) {
	var sensorMonitor = this.sensorMonitor;
	sensorMonitor.reset();
	sensorMonitor.duration = duration;
};

Controller.prototype.getIdentityHexRgb = function() {
};

module.exports.create = function() {
	return new Controller();
};