'use strict';
var context = require('context');
var HamsterS = require('../hamsterS');
var factory = require('../hamsterS-roboid');
const MODULE = 'hamsterS';

function Component() {
}

Component.prototype.createRobot = function(index) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.sensory = {
		version: 0x01,
		company: 0x02,
		model: 0x0E,
		variation: 0x01,
		network: 0x00,
		protocol: 'json',
		group: 'hamster',
		module: 'hamsterS',
		index: index,
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		freeFall: 0,
		freeFallId: 0,
		tap: 0,
		tapId: 0,
		tilt: 0,
		readSerial: undefined,
		readSerialId: 0,
		pulseCount: 0,
		wheelState: -1,
		wheelStateId: 0,
		soundState: -1,
		soundStateId: 0,
		lineTracerState: 0,
		lineTracerStateId: 0,
		batteryState: 2,
		handFound: false
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(HamsterS.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(HamsterS.RIGHT_WHEEL),
		leftRgb: roboid.findDeviceById(HamsterS.LEFT_RGB),
		rightRgb: roboid.findDeviceById(HamsterS.RIGHT_RGB),
		buzzer: roboid.findDeviceById(HamsterS.BUZZER),
		outputA: roboid.findDeviceById(HamsterS.OUTPUT_A),
		outputB: roboid.findDeviceById(HamsterS.OUTPUT_B),
		pulse: roboid.findDeviceById(HamsterS.PULSE),
		note: roboid.findDeviceById(HamsterS.NOTE),
		sound: roboid.findDeviceById(HamsterS.SOUND),
		lineTracerMode: roboid.findDeviceById(HamsterS.LINE_TRACER_MODE),
		lineTracerGain: roboid.findDeviceById(HamsterS.LINE_TRACER_GAIN),
		lineTracerSpeed: roboid.findDeviceById(HamsterS.LINE_TRACER_SPEED),
		ioModeA: roboid.findDeviceById(HamsterS.IO_MODE_A),
		ioModeB: roboid.findDeviceById(HamsterS.IO_MODE_B),
		writeSerial: roboid.findDeviceById(HamsterS.WRITE_SERIAL),
		motion: roboid.findDeviceById(HamsterS.MOTION),
		motorMode: roboid.findDeviceById(HamsterS.MOTOR_MODE),
		configProximity: roboid.findDeviceById(HamsterS.CONFIG_PROXIMITY),
		configGravity: roboid.findDeviceById(HamsterS.CONFIG_GRAVITY),
		configBandWidth: roboid.findDeviceById(HamsterS.CONFIG_BAND_WIDTH)
	};
	robot.pulseId = -1;
	robot.sound = 0;
	robot.soundRepeat = 1;
	robot.soundId = -1;
	robot.lineTracerModeId = -1;
	robot.writeSerialId = -1;
	robot.motionId = -1;
	return robot;
};

Component.prototype.runSound = function(robot, sound, count) {
	if(typeof count != 'number') count = 1;
	if(count < 0) count = -1;
	if(count) {
		robot.sound = sound;
		robot.soundRepeat = count;
		robot.motoring.sound.write(sound);
	}
};

Component.prototype.handleRemoteData = function(robot, data) {
	var motoring = robot.motoring;
	if(data.leftWheel !== undefined) motoring.leftWheel.write(data.leftWheel);
	if(data.rightWheel !== undefined) motoring.rightWheel.write(data.rightWheel);
	if(data.leftRed !== undefined) motoring.leftRgb.write(0, data.leftRed);
	if(data.leftGreen !== undefined) motoring.leftRgb.write(1, data.leftGreen);
	if(data.leftBlue !== undefined) motoring.leftRgb.write(2, data.leftBlue);
	if(data.rightRed !== undefined) motoring.rightRgb.write(0, data.rightRed);
	if(data.rightGreen !== undefined) motoring.rightRgb.write(1, data.rightGreen);
	if(data.rightBlue !== undefined) motoring.rightRgb.write(2, data.rightBlue);
	if(data.buzzer !== undefined) motoring.buzzer.write(data.buzzer);
	if(data.outputA !== undefined) motoring.outputA.write(data.outputA);
	if(data.outputB !== undefined) motoring.outputB.write(data.outputB);
	if(data.pulseId !== undefined) {
		var t = data.pulseId;
		if(t != robot.pulseId) {
			robot.pulseId = t;
			if(data.pulse !== undefined) motoring.pulse.write(data.pulse);
		}
	}
	if(data.note !== undefined) motoring.note.write(data.note);
	if(data.soundId !== undefined) {
		var t = data.soundId;
		if(t != robot.soundId) {
			robot.soundId = t;
			if(data.sound !== undefined && data.soundRepeat !== undefined) {
				this.runSound(robot, data.sound, data.soundRepeat);
			}
		}
	}
	if(data.lineTracerModeId !== undefined) {
		var t = data.lineTracerModeId;
		if(t != robot.lineTracerModeId) {
			robot.lineTracerModeId = t;
			if(data.lineTracerMode !== undefined) motoring.lineTracerMode.write(data.lineTracerMode);
		}
	}
	if(data.lineTracerGain !== undefined) motoring.lineTracerGain.write(data.lineTracerGain);
	if(data.lineTracerSpeed !== undefined) motoring.lineTracerSpeed.write(data.lineTracerSpeed);
	if(data.ioModeA !== undefined) motoring.ioModeA.write(data.ioModeA);
	if(data.ioModeB !== undefined) motoring.ioModeB.write(data.ioModeB);
	if(data.writeSerialId !== undefined) {
		var t = data.writeSerialId;
		if(t != robot.writeSerialId) {
			robot.writeSerialId = t;
			if(data.writeSerial !== undefined) motoring.writeSerial.write(data.writeSerial);
		}
	}
	if(data.motionId !== undefined) {
		var t = data.motionId;
		if(t != robot.motionId) {
			robot.motionId = t;
			if(data.motionType !== undefined) motoring.motion.write(0, data.motionType);
			if(data.motionUnit !== undefined) motoring.motion.write(1, data.motionUnit);
			if(data.motionSpeed !== undefined) motoring.motion.write(2, data.motionSpeed);
			if(data.motionValue !== undefined) motoring.motion.write(3, data.motionValue);
			if(data.motionRadius !== undefined) motoring.motion.write(4, data.motionRadius);
		}
	}
	
	var sensory = robot.sensory;
	sensory.freeFall = 0;
	sensory.tap = 0;
};

Component.prototype.prepareSensory = function(robot, data) {
	var sensory = robot.sensory;
	sensory.signalStrength = data.signalStrength;
	sensory.leftProximity = data.leftProximity;
	sensory.rightProximity = data.rightProximity;
	sensory.leftFloor = data.leftFloor;
	sensory.rightFloor = data.rightFloor;
	sensory.accelerationX = data.acceleration.x;
	sensory.accelerationY = data.acceleration.y;
	sensory.accelerationZ = data.acceleration.z;
	sensory.light = data.light;
	sensory.temperature = data.temperature;
	sensory.inputA = data.inputA;
	sensory.inputB = data.inputB;
	if(data.map1 & 0x00000010) {
		sensory.lineTracerState = data.lineTracerState;
		sensory.lineTracerStateId = (sensory.lineTracerStateId % 255) + 1;
	}
	if(data.map1 & 0x00000008) {
		sensory.tilt = data.tilt;
	}
	if(data.map1 & 0x00000004) {
		sensory.batteryState = data.batteryState;
	}
	var threshold = context.getHandFoundThreshold(MODULE);
	sensory.handFound = sensory.leftProximity > threshold || sensory.rightProximity > threshold;
	
	if(data.map2 & 0x00008000) {
		sensory.freeFall = 1;
		sensory.freeFallId = (sensory.freeFallId % 255) + 1;
	}
	if(data.map2 & 0x00004000) {
		sensory.tap = 1;
		sensory.tapId = (sensory.tapId % 255) + 1;
	}
	if(data.map2 & 0x00002000) {
		sensory.readSerial = data.serial;
		sensory.readSerialId = (sensory.readSerialId % 255) + 1;
	}
	if(data.map2 & 0x00001000) {
		sensory.pulseCount = data.pulseCount;
	}
	if(data.map2 & 0x00000800) {
		sensory.wheelState = data.wheelState;
		sensory.wheelStateId = (sensory.wheelStateId % 255) + 1;
	}
	if(data.map2 & 0x00000400) {
		if(data.soundState == 0) {
			if(robot.sound > 0) {
				if(robot.soundRepeat < 0) {
					this.runSound(robot, robot.sound, -1);
				} else if(robot.soundRepeat > 1) {
					robot.soundRepeat --;
					this.runSound(robot, robot.sound, robot.soundRepeat);
				} else {
					robot.sound = 0;
					robot.soundRepeat = 1;
					sensory.soundState = data.soundState;
					sensory.soundStateId = (sensory.soundStateId % 255) + 1;
				}
			} else {
				robot.sound = 0;
				robot.soundRepeat = 1;
				sensory.soundState = data.soundState;
				sensory.soundStateId = (sensory.soundStateId % 255) + 1;
			}
		} else if(data.soundState != sensory.soundState) {
			sensory.soundState = data.soundState;
			sensory.soundStateId = (sensory.soundStateId % 255) + 1;
		}
	}
	if(data.map2 & 0x00000200) {
		sensory.serialState = 1;
		sensory.serialStateId = (sensory.serialStateId % 255) + 1;
	}
};

Component.prototype.prepareSensoryMulti = function(sensory, key, robot, data) {
	this.prepareSensory(robot, data);
	var robotsensory = robot.sensory;
	sensory[key + 'signalStrength'] = robotsensory.signalStrength;
	sensory[key + 'leftProximity'] = robotsensory.leftProximity;
	sensory[key + 'rightProximity'] = robotsensory.rightProximity;
	sensory[key + 'leftFloor'] = robotsensory.leftFloor;
	sensory[key + 'rightFloor'] = robotsensory.rightFloor;
	sensory[key + 'accelerationX'] = robotsensory.accelerationX;
	sensory[key + 'accelerationY'] = robotsensory.accelerationY;
	sensory[key + 'accelerationZ'] = robotsensory.accelerationZ;
	sensory[key + 'light'] = robotsensory.light;
	sensory[key + 'temperature'] = robotsensory.temperature;
	sensory[key + 'inputA'] = robotsensory.inputA;
	sensory[key + 'inputB'] = robotsensory.inputB;
	sensory[key + 'freeFall'] = robotsensory.freeFall;
	sensory[key + 'freeFallId'] = robotsensory.freeFallId;
	sensory[key + 'tap'] = robotsensory.tap;
	sensory[key + 'tapId'] = robotsensory.tapId;
	sensory[key + 'tilt'] = robotsensory.tilt;
	sensory[key + 'readSerial'] = robotsensory.readSerial;
	sensory[key + 'readSerialId'] = robotsensory.readSerialId;
	sensory[key + 'pulseCount'] = robotsensory.pulseCount;
	sensory[key + 'wheelState'] = robotsensory.wheelState;
	sensory[key + 'wheelStateId'] = robotsensory.wheelStateId;
	sensory[key + 'soundState'] = robotsensory.soundState;
	sensory[key + 'soundStateId'] = robotsensory.soundStateId;
	sensory[key + 'lineTracerState'] = robotsensory.lineTracerState;
	sensory[key + 'lineTracerStateId'] = robotsensory.lineTracerStateId;
	sensory[key + 'batteryState'] = robotsensory.batteryState;
	sensory[key + 'handFound'] = robotsensory.handFound;
};

Component.prototype.updateSensory = function(robot) {
	var sensory = robot.sensory;
	sensory.wheelState = -1;
	sensory.soundState = -1;
	sensory.lineTracerState = 0;
};

Component.prototype.reset = function(robot) {
	robot.sound = 0;
	robot.soundRepeat = 1;
	robot.roboid.reset();
};

module.exports = new Component();