'use strict';
var HamsterS = require('./hamsterS');

function HamsterSRoboid(parent) {
	var io = require('roboid.io');
	var Device = io.Device;
	var DeviceType = io.DeviceType;
	var DataType = io.DataType;
	
	var sensory = {
		devices: [],
		readData: [],
		callbacks: []
	};
	var motoring = {
		map1: 0,
		map2: 0,
		devices: [],
		readData: [],
		writeData: [],
		callbacks: []
	};
	var writeMotoringFlag1 = 0;
	var writeMotoringFlag2 = 0;
	
	this.bind = function(device, type, data, callback) {
		if(type == DeviceType.SENSOR || type == DeviceType.EVENT) {
			sensory.devices.push(device);
			sensory.readData.push(data.read);
			sensory.callbacks.push(callback);
		} else {
			motoring.devices.push(device);
			motoring.readData.push(data.read);
			motoring.writeData.push(data.write);
			motoring.callbacks.push(callback);
		}
	};
	
	var devices = {};
	devices[HamsterS.LEFT_WHEEL] = new Device(this, HamsterS.LEFT_WHEEL, 'LeftWheel', DeviceType.EFFECTOR, DataType.FLOAT, 1, -100, 100, 0);
	devices[HamsterS.RIGHT_WHEEL] = new Device(this, HamsterS.RIGHT_WHEEL, 'RightWheel', DeviceType.EFFECTOR, DataType.FLOAT, 1, -100, 100, 0);
	devices[HamsterS.LEFT_RGB] = new Device(this, HamsterS.LEFT_RGB, 'LeftRgb', DeviceType.EFFECTOR, DataType.INTEGER, 3, 0, 255, 0);
	devices[HamsterS.RIGHT_RGB] = new Device(this, HamsterS.RIGHT_RGB, 'RightRgb', DeviceType.EFFECTOR, DataType.INTEGER, 3, 0, 255, 0);
	devices[HamsterS.BUZZER] = new Device(this, HamsterS.BUZZER, 'Buzzer', DeviceType.EFFECTOR, DataType.FLOAT, 1, 0, 6500.0, 0);
	devices[HamsterS.OUTPUT_A] = new Device(this, HamsterS.OUTPUT_A, 'OutputA', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.OUTPUT_B] = new Device(this, HamsterS.OUTPUT_B, 'OutputB', DeviceType.EFFECTOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.PULSE] = new Device(this, HamsterS.PULSE, 'Pulse', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 65535, 0);
	devices[HamsterS.NOTE] = new Device(this, HamsterS.NOTE, 'Note', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 88, 0);
	devices[HamsterS.SOUND] = new Device(this, HamsterS.SOUND, 'Sound', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 127, 0);
	devices[HamsterS.LINE_TRACER_MODE] = new Device(this, HamsterS.LINE_TRACER_MODE, 'LineTracerMode', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 15, 0);
	devices[HamsterS.LINE_TRACER_GAIN] = new Device(this, HamsterS.LINE_TRACER_GAIN, 'LineTracerGain', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 10, 4);
	devices[HamsterS.LINE_TRACER_SPEED] = new Device(this, HamsterS.LINE_TRACER_SPEED, 'LineTracerSpeed', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 10, 5);
	devices[HamsterS.IO_MODE_A] = new Device(this, HamsterS.IO_MODE_A, 'IoModeA', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.IO_MODE_B] = new Device(this, HamsterS.IO_MODE_B, 'IoModeB', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.WRITE_SERIAL] = new Device(this, HamsterS.WRITE_SERIAL, 'WriteSerial', DeviceType.COMMAND, DataType.INTEGER, 19, 0, 255, 0);
	devices[HamsterS.MOTION] = new Device(this, HamsterS.MOTION, 'Motion', DeviceType.COMMAND, DataType.FLOAT, 5, 0, 2147483647, 0);
	devices[HamsterS.MOTOR_MODE] = new Device(this, HamsterS.MOTOR_MODE, 'MotorMode', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 3, 0);
	devices[HamsterS.CONFIG_PROXIMITY] = new Device(this, HamsterS.CONFIG_PROXIMITY, 'ConfigProximity', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 7, 2);
	devices[HamsterS.CONFIG_GRAVITY] = new Device(this, HamsterS.CONFIG_GRAVITY, 'ConfigGravity', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 3, 0);
	devices[HamsterS.CONFIG_BAND_WIDTH] = new Device(this, HamsterS.CONFIG_BAND_WIDTH, 'ConfigBandWidth', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 7, 0);
	
	devices[HamsterS.SIGNAL_STRENGTH] = new Device(this, HamsterS.SIGNAL_STRENGTH, 'SignalStrength', DeviceType.SENSOR, DataType.INTEGER, 1, -128, 0, 0);
	devices[HamsterS.LEFT_PROXIMITY] = new Device(this, HamsterS.LEFT_PROXIMITY, 'LeftProximity', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.RIGHT_PROXIMITY] = new Device(this, HamsterS.RIGHT_PROXIMITY, 'RightProximity', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.LEFT_FLOOR] = new Device(this, HamsterS.LEFT_FLOOR, 'LeftFloor', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.RIGHT_FLOOR] = new Device(this, HamsterS.RIGHT_FLOOR, 'RightFloor', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.ACCELERATION] = new Device(this, HamsterS.ACCELERATION, 'Acceleration', DeviceType.SENSOR, DataType.INTEGER, 3, -32768, 32767, 0);
	devices[HamsterS.LIGHT] = new Device(this, HamsterS.LIGHT, 'Light', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 65535, 0);
	devices[HamsterS.TEMPERATURE] = new Device(this, HamsterS.TEMPERATURE, 'Temperature', DeviceType.SENSOR, DataType.INTEGER, 1, -41, 87, 0);
	devices[HamsterS.INPUT_A] = new Device(this, HamsterS.INPUT_A, 'InputA', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.INPUT_B] = new Device(this, HamsterS.INPUT_B, 'InputB', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.FREE_FALL] = new Device(this, HamsterS.FREE_FALL, 'FreeFall', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[HamsterS.TAP] = new Device(this, HamsterS.TAP, 'Tap', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[HamsterS.TILT] = new Device(this, HamsterS.TILT, 'Tilt', DeviceType.EVENT, DataType.INTEGER, 1, -3, 3, 0);
	devices[HamsterS.READ_SERIAL] = new Device(this, HamsterS.READ_SERIAL, 'ReadSerial', DeviceType.EVENT, DataType.INTEGER, 19, 0, 255, 0);
	devices[HamsterS.PULSE_COUNT] = new Device(this, HamsterS.PULSE_COUNT, 'PulseCount', DeviceType.EVENT, DataType.INTEGER, 1, 0, 65535, 0);
	devices[HamsterS.WHEEL_STATE] = new Device(this, HamsterS.WHEEL_STATE, 'WheelState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 3, 0);
	devices[HamsterS.SOUND_STATE] = new Device(this, HamsterS.SOUND_STATE, 'SoundState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[HamsterS.LINE_TRACER_STATE] = new Device(this, HamsterS.LINE_TRACER_STATE, 'LineTracerState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 255, 0);
	devices[HamsterS.BATTERY_STATE] = new Device(this, HamsterS.BATTERY_STATE, 'BatteryState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 2, 2);
	devices[HamsterS.SERIAL_STATE] = new Device(this, HamsterS.SERIAL_STATE, 'SerialState', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	
	this.findDeviceByName = function(name) {
		name = '' + name;
		name = name.toLowerCase();
		var dot = name.search('\\.');
		if(dot < 0) {
			for(var i in devices) {
				if(name == devices[i].getName().toLowerCase())
					return devices[i];
			}
		}
		return undefined;
	};
	
	this.findDeviceById = function(deviceId) {
		var device = devices[deviceId];
		if(device) {
			return device;
		}
		return undefined;
	};
	
	this.addDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].addDeviceDataChangedListener(listener);
		}
	};
	
	this.removeDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].removeDeviceDataChangedListener(listener);
		}
	};
	
	this.clearDeviceDataChangedListeners = function() {
		for(var i in devices) {
			devices[i].clearDeviceDataChangedListeners();
		}
	};
	
	this.onMotoringDeviceWritten = function(device) {
		switch(device.getId()) {
			case HamsterS.PULSE:
				writeMotoringFlag2 |= 0x20000000;
				break;
			case HamsterS.NOTE:
				writeMotoringFlag1 |= 0x00400000;
				break;
			case HamsterS.SOUND:
				writeMotoringFlag2 |= 0x10000000;
				break;
			case HamsterS.LINE_TRACER_MODE:
				writeMotoringFlag1 |= 0x00200000;
				break;
			case HamsterS.LINE_TRACER_GAIN:
				writeMotoringFlag2 |= 0x08000000;
				break;
			case HamsterS.LINE_TRACER_SPEED:
				writeMotoringFlag1 |= 0x00100000;
				break;
			case HamsterS.IO_MODE_A:
				writeMotoringFlag1 |= 0x00080000;
				break;
			case HamsterS.IO_MODE_B:
				writeMotoringFlag1 |= 0x00040000;
				break;
			case HamsterS.WRITE_SERIAL:
				writeMotoringFlag2 |= 0x04000000;
				break;
			case HamsterS.MOTION:
				writeMotoringFlag2 |= 0x02000000;
				break;
			case HamsterS.MOTOR_MODE:
				writeMotoringFlag2 |= 0x01000000;
				break;
			case HamsterS.CONFIG_PROXIMITY:
				writeMotoringFlag1 |= 0x00020000;
				break;
			case HamsterS.CONFIG_GRAVITY:
				writeMotoringFlag1 |= 0x00010000;
				break;
			case HamsterS.CONFIG_BAND_WIDTH:
				writeMotoringFlag1 |= 0x00008000;
				break;
		}
	};
	
	this.handleSensory = function(data) {
		var map1 = data.map1;
		var map2 = data.map2;
		var readData = sensory.readData;
		readData[0][0] = data.signalStrength;
		readData[1][0] = data.leftProximity;
		readData[2][0] = data.rightProximity;
		readData[3][0] = data.leftFloor;
		readData[4][0] = data.rightFloor;
		readData[5][0] = data.acceleration.x;
		readData[5][1] = data.acceleration.y;
		readData[5][2] = data.acceleration.z;
		readData[6][0] = data.light;
		readData[7][0] = data.temperature;
		readData[8][0] = data.inputA;
		readData[9][0] = data.inputB;
		
		var callbacks = sensory.callbacks;
		var i;
		for(i = 0; i < 10; ++i) {
			callbacks[i].fire();
		}
		
		if(map2 & 0x00008000) {
			callbacks[10].fire(); // free fall
		}
		if(map2 & 0x00004000) {
			callbacks[11].fire(); // tap
		}
		if(map1 & 0x00000008) {
			readData[12][0] = data.tilt;
			callbacks[12].fire();
		}
		if(map2 & 0x00002000) {
			var r = readData[13];
			var serial = data.serial;
			if(serial) {
				for(var i = 0; i < 19; ++i) {
					r[i] = serial[i];
				}
				callbacks[13].fire();
			}
		}
		if(map2 & 0x00001000) {
			readData[14][0] = data.pulseCount;
			callbacks[14].fire();
		}
		if(map2 & 0x00000800) {
			readData[15][0] = data.wheelState;
			callbacks[15].fire();
		}
		if(map2 & 0x00000400) {
			readData[16][0] = data.soundState;
			callbacks[16].fire();
		}
		if(map1 & 0x00000010) {
			readData[17][0] = data.lineTracerState;
			callbacks[17].fire();
		}
		if(map1 & 0x00000004) {
			readData[18][0] = data.batteryState;
			callbacks[18].fire();
		}
		if(map2 & 0x00000200) { // serial state
			callbacks[19].fire();
		}
		
		for(i = 0; i < 10; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map2 & 0x00008000) {
			callbacks[10].notifyDeviceDataChanged();
		}
		if(map2 & 0x00004000) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000008) {
			callbacks[12].notifyDeviceDataChanged();
		}
		if(map2 & 0x00002000) {
			callbacks[13].notifyDeviceDataChanged();
		}
		if(map2 & 0x00001000) {
			callbacks[14].notifyDeviceDataChanged();
		}
		if(map2 & 0x00000800) {
			callbacks[15].notifyDeviceDataChanged();
		}
		if(map2 & 0x00000400) {
			callbacks[16].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000010) {
			callbacks[17].notifyDeviceDataChanged();
		}
		if(map1 & 0x00000004) {
			callbacks[18].notifyDeviceDataChanged();
		}
		if(map2 & 0x00000200) {
			callbacks[19].notifyDeviceDataChanged();
		}
	};
	
	this.handleMotoring = function(data) {
		var map1 = data.map1;
		var map2 = data.map2;
		var readData = motoring.readData;
		readData[0][0] = data.leftWheel;
		readData[1][0] = data.rightWheel;
		readData[2][0] = data.leftRgb.r;
		readData[2][1] = data.leftRgb.g;
		readData[2][2] = data.leftRgb.b;
		readData[3][0] = data.rightRgb.r;
		readData[3][1] = data.rightRgb.g;
		readData[3][2] = data.rightRgb.b;
		readData[4][0] = data.buzzer;
		readData[5][0] = data.outputA;
		readData[6][0] = data.outputB;
		
		var callbacks = motoring.callbacks;
		var i;
		for(i = 0; i < 7; ++i) {
			callbacks[i].fire();
		}
		
		if(map2 & 0x20000000) {
			readData[7][0] = data.pulse;
			callbacks[7].fire();
		}
		if(map1 & 0x00400000) {
			readData[8][0] = data.note;
			callbacks[8].fire();
		}
		if(map2 & 0x10000000) {
			readData[9][0] = data.sound;
			callbacks[9].fire();
		}
		if(map1 & 0x00200000) {
			readData[10][0] = data.lineTracerMode;
			callbacks[10].fire();
		}
		if(map2 & 0x08000000) {
			readData[11][0] = data.lineTracerGain;
			callbacks[11].fire();
		}
		if(map1 & 0x00100000) {
			readData[12][0] = data.lineTracerSpeed;
			callbacks[12].fire();
		}
		if(map1 & 0x00080000) {
			readData[13][0] = data.ioModeA;
			callbacks[13].fire();
		}
		if(map1 & 0x00040000) {
			readData[14][0] = data.ioModeB;
			callbacks[14].fire();
		}
		if(map2 & 0x04000000) {
			var r = readData[15];
			var serial = data.serial;
			if(serial) {
				for(var i = 0; i < 19; ++i) {
					r[i] = serial[i];
				}
				callbacks[15].fire();
			}
		}
		if(map2 & 0x02000000) {
			readData[16][0] = data.motion.type;
			readData[16][1] = data.motion.unit;
			readData[16][2] = data.motion.speed;
			readData[16][3] = data.motion.value;
			readData[16][4] = data.motion.radius;
			callbacks[16].fire();
		}
		if(map2 & 0x01000000) {
			readData[17][0] = data.motorMode;
			callbacks[17].fire();
		}
		if(map1 & 0x00020000) {
			readData[18][0] = data.configProximity;
			callbacks[18].fire();
		}
		if(map1 & 0x00010000) {
			readData[19][0] = data.configGravity;
			callbacks[19].fire();
		}
		if(map1 & 0x00008000) {
			readData[20][0] = data.configBandWidth;
			callbacks[20].fire();
		}
		
		for(i = 0; i < 7; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map2 & 0x20000000) {
			callbacks[7].notifyDeviceDataChanged();
		}
		if(map1 & 0x00400000) {
			callbacks[8].notifyDeviceDataChanged();
		}
		if(map2 & 0x10000000) {
			callbacks[9].notifyDeviceDataChanged();
		}
		if(map1 & 0x00200000) {
			callbacks[10].notifyDeviceDataChanged();
		}
		if(map2 & 0x08000000) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map1 & 0x00100000) {
			callbacks[12].notifyDeviceDataChanged();
		}
		if(map1 & 0x00080000) {
			callbacks[13].notifyDeviceDataChanged();
		}
		if(map1 & 0x00040000) {
			callbacks[14].notifyDeviceDataChanged();
		}
		if(map2 & 0x04000000) {
			callbacks[15].notifyDeviceDataChanged();
		}
		if(map2 & 0x02000000) {
			callbacks[16].notifyDeviceDataChanged();
		}
		if(map2 & 0x01000000) {
			callbacks[17].notifyDeviceDataChanged();
		}
		if(map1 & 0x00020000) {
			callbacks[18].notifyDeviceDataChanged();
		}
		if(map1 & 0x00010000) {
			callbacks[19].notifyDeviceDataChanged();
		}
		if(map1 & 0x00008000) {
			callbacks[20].notifyDeviceDataChanged();
		}
	};
	
	this.requestMotoring = function() {
		motoring.map1 = 0xfc000000 | writeMotoringFlag1;
		motoring.map2 = 0xc0000000 | writeMotoringFlag2;
		writeMotoringFlag1 = 0;
		writeMotoringFlag2 = 0;
		return motoring;
	};
	
	this.updateSensory = function() {
		var callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.updateMotoring = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.reset = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
		callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
	};
}

HamsterSRoboid.prototype.getId = function() {
	return HamsterS.ID;
};

HamsterSRoboid.prototype.getName = function() {
	return 'HamsterS';
};

module.exports.create = function(parent) {
	return new HamsterSRoboid(parent);
};