'use strict';
var router = require('router');
var HamsterS = require('../hamsterS');
var factory = require('../hamsterS-roboid');
var SerialQueue = require('../serial-queue');
const MODULE = 'hamsterS';
const HEADER = { 'Content-Type': 'text/plain' };

function Component() {
}

Component.prototype.createRobot = function(state) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.connectionState = state;
	robot.sensory = {
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		tiltId: 0,
		tilt: 0,
		tapId: 0,
		freeFallId: 0,
		readSerialId: 0,
		readSerial: '',
		pulseCountId: 0,
		pulseCount: 0,
		wheelStateId: 0,
		wheelState: 0,
		soundStateId: 0,
		soundState: 0,
		lineTracerStateId: 0,
		lineTracerState: 0,
		batteryStateId: 0,
		batteryState: 2
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(HamsterS.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(HamsterS.RIGHT_WHEEL),
		leftRgb: roboid.findDeviceById(HamsterS.LEFT_RGB),
		rightRgb: roboid.findDeviceById(HamsterS.RIGHT_RGB),
		buzzer: roboid.findDeviceById(HamsterS.BUZZER),
		outputA: roboid.findDeviceById(HamsterS.OUTPUT_A),
		outputB: roboid.findDeviceById(HamsterS.OUTPUT_B),
		pulse: roboid.findDeviceById(HamsterS.PULSE),
		note: roboid.findDeviceById(HamsterS.NOTE),
		sound: roboid.findDeviceById(HamsterS.SOUND),
		lineTracerMode: roboid.findDeviceById(HamsterS.LINE_TRACER_MODE),
		lineTracerGain: roboid.findDeviceById(HamsterS.LINE_TRACER_GAIN),
		lineTracerSpeed: roboid.findDeviceById(HamsterS.LINE_TRACER_SPEED),
		ioModeA: roboid.findDeviceById(HamsterS.IO_MODE_A),
		ioModeB: roboid.findDeviceById(HamsterS.IO_MODE_B),
		writeSerial: roboid.findDeviceById(HamsterS.WRITE_SERIAL),
		motion: roboid.findDeviceById(HamsterS.MOTION),
		motorMode: roboid.findDeviceById(HamsterS.MOTOR_MODE),
		configProximity: roboid.findDeviceById(HamsterS.CONFIG_PROXIMITY),
		configGravity: roboid.findDeviceById(HamsterS.CONFIG_GRAVITY),
		configBandWidth: roboid.findDeviceById(HamsterS.CONFIG_BAND_WIDTH)
	};
	robot.soundId = 0;
	robot.soundRepeat = 1;
	robot.serialRate = HamsterS.IO_MODE_SERIAL_9600;
	robot.serialDelimiter = 0;
	robot.writeQueue = SerialQueue.createWriteQueue(64);
	robot.readQueue = SerialQueue.createReadQueue(64);
	return robot;
};

Component.prototype.start = function(express, robots) {
	var self = this;
	
	var getRoute = function(index) {
		if(router.multi) {
			return router.findRouteByIndex(MODULE, index);
		} else {
			var selectedRoute = router.selectedRoute;
			if(selectedRoute && selectedRoute.module == MODULE) {
				return selectedRoute;
			}
		}
	};
	
	var getRobot = function(index) {
		var key = router.multi ? MODULE + index : MODULE;
		var robot = robots[key];
		if(!robot) {
			var route = getRoute(index);
			if(route) {
				robot = self.createRobot(route.state);
				if(robot) {
					robots[key] = robot;
				}
			}
		}
		return robot;
	};
	
	express.get('/hamsterS/:index/wheels/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__wheels(getRobot(index), leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/hamsterS/:index/wheels/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__wheels(getRobot(index), speed, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/leftWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__leftWheel(getRobot(index), speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/rightWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseFloat(request.params.speed);
		self.__rightWheel(getRobot(index), speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/moveForward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveForward(getRobot(index), cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/moveForward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		self.__moveForward(getRobot(index), cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/moveBackward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveBackward(getRobot(index), cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/moveBackward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var cm = parseFloat(request.params.cm);
		self.__moveBackward(getRobot(index), cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/turnLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnLeft(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/turnLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__turnLeft(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/turnRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnRight(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/turnRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__turnRight(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/pivotLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotLeft(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/pivotLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__pivotLeft(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/pivotRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotRight(getRobot(index), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/pivotRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		self.__pivotRight(getRobot(index), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/swingLeft/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingLeft(getRobot(index), deg, cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/swingLeft/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingLeft(getRobot(index), deg, cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/swingRight/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingRight(getRobot(index), deg, cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/swingRight/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingRight(getRobot(index), deg, cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/:index/pulse/:pulse/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var pulse = parseFloat(request.params.pulse);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__pulse(getRobot(index), pulse, leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/hamsterS/:index/pulse/:pulse/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var pulse = parseFloat(request.params.pulse);
		var speed = parseFloat(request.params.speed);
		self.__pulse(getRobot(index), pulse, speed, speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/stop', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__stop(getRobot(index));
		response.end();
	});
	
	express.get('/hamsterS/:index/lineTracerMode/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var mode = parseInt(request.params.mode);
		self.__lineTracerMode(getRobot(index), mode);
		response.end();
	});
	
	express.get('/hamsterS/:index/lineTracerGain/:gain', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var gain = parseInt(request.params.gain);
		self.__lineTracerGain(getRobot(index), gain);
		response.end();
	});
	
	express.get('/hamsterS/:index/lineTracerSpeed/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var speed = parseInt(request.params.speed);
		self.__lineTracerSpeed(getRobot(index), speed);
		response.end();
	});
	
	express.get('/hamsterS/:index/rgbs/:leftRed/:leftGreen/:leftBlue/:rightRed/:rightGreen/:rightBlue', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var leftRed = parseInt(request.params.leftRed);
		var leftGreen = parseInt(request.params.leftGreen);
		var leftBlue = parseInt(request.params.leftBlue);
		var rightRed = parseInt(request.params.rightRed);
		var rightGreen = parseInt(request.params.rightGreen);
		var rightBlue = parseInt(request.params.rightBlue);
		self.__rgbs(getRobot(index), leftRed, leftGreen, leftBlue, rightRed, rightGreen, rightBlue);
		response.end();
	});
	
	express.get('/hamsterS/:index/rgbs/:leftRed/:leftGreen/:leftBlue/:rightRed/:rightGreen/:rightBlue', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var leftRed = parseInt(request.params.leftRed);
		var leftGreen = parseInt(request.params.leftGreen);
		var leftBlue = parseInt(request.params.leftBlue);
		var rightRed = parseInt(request.params.rightRed);
		var rightGreen = parseInt(request.params.rightGreen);
		var rightBlue = parseInt(request.params.rightBlue);
		self.__rgbs(getRobot(index), leftRed, leftGreen, leftBlue, rightRed, rightGreen, rightBlue);
		response.end();
	});
	
	express.get('/hamsterS/:index/rgbs/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__rgbs(getRobot(index), red, green, blue, red, green, blue);
		response.end();
	});
	
	express.get('/hamsterS/:index/rgbs/:leftGray/:rightGray', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var leftGray = parseInt(request.params.leftGray);
		var rightGray = parseInt(request.params.rightGray);
		self.__rgbs(getRobot(index), leftGray, leftGray, leftGray, rightGray, rightGray, rightGray);
		response.end();
	});
	
	express.get('/hamsterS/:index/rgbs/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var gray = parseInt(request.params.gray);
		self.__rgbs(getRobot(index), gray, gray, gray, gray, gray, gray);
		response.end();
	});
	
	express.get('/hamsterS/:index/leftRgb/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__leftRgb(getRobot(index), red, green, blue);
		response.end();
	});
	
	express.get('/hamsterS/:index/leftRgb/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var gray = parseInt(request.params.gray);
		self.__leftRgb(getRobot(index), gray, gray, gray);
		response.end();
	});
	
	express.get('/hamsterS/:index/rightRgb/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__rightRgb(getRobot(index), red, green, blue);
		response.end();
	});
	
	express.get('/hamsterS/:index/rightRgb/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var gray = parseInt(request.params.gray);
		self.__rightRgb(getRobot(index), gray, gray, gray);
		response.end();
	});
	
	express.get('/hamsterS/:index/buzzer/:hz', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var hz = parseFloat(request.params.hz);
		self.__buzzer(getRobot(index), hz);
		response.end();
	});
	
	express.get('/hamsterS/:index/note/:pitch', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var pitch = parseInt(request.params.pitch);
		self.__note(getRobot(index), pitch);
		response.end();
	});
	
	express.get('/hamsterS/:index/sound/:sound/:count', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var sound = parseInt(request.params.sound);
		var count = parseInt(request.params.count);
		self.__sound(getRobot(index), sound, count);
		response.end();
	});
	
	express.get('/hamsterS/:index/sound/:sound', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var sound = parseInt(request.params.sound);
		self.__sound(getRobot(index), sound, 1);
		response.end();
	});
	
	express.get('/hamsterS/:index/ioModeA/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var mode = parseInt(request.params.mode);
		self.__ioModeA(getRobot(index), mode);
		response.end();
	});
	
	express.get('/hamsterS/:index/ioModeB/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var mode = parseInt(request.params.mode);
		self.__ioModeB(getRobot(index), mode);
		response.end();
	});
	
	express.get('/hamsterS/:index/outputA/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var value = parseFloat(request.params.value);
		self.__outputA(getRobot(index), value);
		response.end();
	});
	
	express.get('/hamsterS/:index/outputB/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var value = parseFloat(request.params.value);
		self.__outputB(getRobot(index), value);
		response.end();
	});
	
	express.get('/hamsterS/:index/gripper/:action', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var action = parseInt(request.params.action);
		self.__gripper(getRobot(index), action);
		response.end();
	});
	
	express.get('/hamsterS/:index/openGripper', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__openGripper(getRobot(index));
		response.end();
	});
	
	express.get('/hamsterS/:index/closeGripper', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__closeGripper(getRobot(index));
		response.end();
	});
	
	express.get('/hamsterS/:index/releaseGripper', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__releaseGripper(getRobot(index));
		response.end();
	});
	
	express.get('/hamsterS/:index/writeSerial/:text', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		var text = parseInt(request.params.text);
		self.__writeSerial(getRobot(index), text);
		response.end();
	});
	
	express.get('/hamsterS/:index/signalStrength', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__signalStrength(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/leftProximity', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__leftProximity(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/rightProximity', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__rightProximity(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/leftFloor', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__leftFloor(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/rightFloor', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__rightFloor(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/accelerationX', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationX(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/accelerationY', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationY(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/accelerationZ', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__accelerationZ(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/light', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__light(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/temperature', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__temperature(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/inputA', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__inputA(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/inputB', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__inputB(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/pulseCount', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__pulseCount(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/pulseCountId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__pulseCountId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/tilt', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__tilt(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/tiltId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__tiltId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/tapId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__tapId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/freeFallId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__freeFallId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/wheelState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__wheelState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/wheelStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__wheelStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/soundState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__soundState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/soundStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__soundStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/lineTracerState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__lineTracerState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/lineTracerStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__lineTracerStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/batteryState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__batteryState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/batteryStateId', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__batteryStateId(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/sensors', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write(self.__sensors(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/connectionState', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		response.write('' + self.__connectionState(getRobot(index)));
		response.end();
	});
	
	express.get('/hamsterS/:index/reset', function(request, response) {
		response.writeHead(200, HEADER);
		var index = parseInt(request.params.index);
		self.__reset(getRobot(index), getRoute(index));
		response.end();
	});
	
	express.get('/hamsterS/wheels/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__wheels(getRobot(0), leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/hamsterS/wheels/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__wheels(getRobot(0), speed, speed);
		response.end();
	});
	
	express.get('/hamsterS/leftWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__leftWheel(getRobot(0), speed);
		response.end();
	});
	
	express.get('/hamsterS/rightWheel/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseFloat(request.params.speed);
		self.__rightWheel(getRobot(0), speed);
		response.end();
	});
	
	express.get('/hamsterS/moveForward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveForward(getRobot(0), cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/moveForward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		self.__moveForward(getRobot(0), cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/moveBackward/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__moveBackward(getRobot(0), cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/moveBackward/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var cm = parseFloat(request.params.cm);
		self.__moveBackward(getRobot(0), cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/turnLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnLeft(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/turnLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__turnLeft(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/turnRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__turnRight(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/turnRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__turnRight(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/pivotLeft/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotLeft(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/pivotLeft/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__pivotLeft(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/pivotRight/:deg/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var speed = parseFloat(request.params.speed);
		self.__pivotRight(getRobot(0), deg, speed);
		response.end();
	});
	
	express.get('/hamsterS/pivotRight/:deg', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		self.__pivotRight(getRobot(0), deg, 0);
		response.end();
	});
	
	express.get('/hamsterS/swingLeft/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingLeft(getRobot(0), deg, cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/swingLeft/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingLeft(getRobot(0), deg, cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/swingRight/:deg/:cm/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		var speed = parseFloat(request.params.speed);
		self.__swingRight(getRobot(0), deg, cm, speed);
		response.end();
	});
	
	express.get('/hamsterS/swingRight/:deg/:cm', function(request, response) {
		response.writeHead(200, HEADER);
		var deg = parseFloat(request.params.deg);
		var cm = parseFloat(request.params.cm);
		self.__swingRight(getRobot(0), deg, cm, 0);
		response.end();
	});
	
	express.get('/hamsterS/pulse/:pulse/:leftSpeed/:rightSpeed', function(request, response) {
		response.writeHead(200, HEADER);
		var pulse = parseFloat(request.params.pulse);
		var leftSpeed = parseFloat(request.params.leftSpeed);
		var rightSpeed = parseFloat(request.params.rightSpeed);
		self.__pulse(getRobot(0), pulse, leftSpeed, rightSpeed);
		response.end();
	});
	
	express.get('/hamsterS/pulse/:pulse/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var pulse = parseFloat(request.params.pulse);
		var speed = parseFloat(request.params.speed);
		self.__pulse(getRobot(0), pulse, speed, speed);
		response.end();
	});
	
	express.get('/hamsterS/stop', function(request, response) {
		response.writeHead(200, HEADER);
		self.__stop(getRobot(0));
		response.end();
	});
	
	express.get('/hamsterS/lineTracerMode/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var mode = parseInt(request.params.mode);
		self.__lineTracerMode(getRobot(0), mode);
		response.end();
	});
	
	express.get('/hamsterS/lineTracerGain/:gain', function(request, response) {
		response.writeHead(200, HEADER);
		var gain = parseInt(request.params.gain);
		self.__lineTracerGain(getRobot(0), gain);
		response.end();
	});
	
	express.get('/hamsterS/lineTracerSpeed/:speed', function(request, response) {
		response.writeHead(200, HEADER);
		var speed = parseInt(request.params.speed);
		self.__lineTracerSpeed(getRobot(0), speed);
		response.end();
	});
	
	express.get('/hamsterS/rgbs/:leftRed/:leftGreen/:leftBlue/:rightRed/:rightGreen/:rightBlue', function(request, response) {
		response.writeHead(200, HEADER);
		var leftRed = parseInt(request.params.leftRed);
		var leftGreen = parseInt(request.params.leftGreen);
		var leftBlue = parseInt(request.params.leftBlue);
		var rightRed = parseInt(request.params.rightRed);
		var rightGreen = parseInt(request.params.rightGreen);
		var rightBlue = parseInt(request.params.rightBlue);
		self.__rgbs(getRobot(0), leftRed, leftGreen, leftBlue, rightRed, rightGreen, rightBlue);
		response.end();
	});
	
	express.get('/hamsterS/rgbs/:leftRed/:leftGreen/:leftBlue/:rightRed/:rightGreen/:rightBlue', function(request, response) {
		response.writeHead(200, HEADER);
		var leftRed = parseInt(request.params.leftRed);
		var leftGreen = parseInt(request.params.leftGreen);
		var leftBlue = parseInt(request.params.leftBlue);
		var rightRed = parseInt(request.params.rightRed);
		var rightGreen = parseInt(request.params.rightGreen);
		var rightBlue = parseInt(request.params.rightBlue);
		self.__rgbs(getRobot(0), leftRed, leftGreen, leftBlue, rightRed, rightGreen, rightBlue);
		response.end();
	});
	
	express.get('/hamsterS/rgbs/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__rgbs(getRobot(0), red, green, blue, red, green, blue);
		response.end();
	});
	
	express.get('/hamsterS/rgbs/:leftGray/:rightGray', function(request, response) {
		response.writeHead(200, HEADER);
		var leftGray = parseInt(request.params.leftGray);
		var rightGray = parseInt(request.params.rightGray);
		self.__rgbs(getRobot(0), leftGray, leftGray, leftGray, rightGray, rightGray, rightGray);
		response.end();
	});
	
	express.get('/hamsterS/rgbs/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var gray = parseInt(request.params.gray);
		self.__rgbs(getRobot(0), gray, gray, gray, gray, gray, gray);
		response.end();
	});
	
	express.get('/hamsterS/leftRgb/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__leftRgb(getRobot(0), red, green, blue);
		response.end();
	});
	
	express.get('/hamsterS/leftRgb/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var gray = parseInt(request.params.gray);
		self.__leftRgb(getRobot(0), gray, gray, gray);
		response.end();
	});
	
	express.get('/hamsterS/rightRgb/:red/:green/:blue', function(request, response) {
		response.writeHead(200, HEADER);
		var red = parseInt(request.params.red);
		var green = parseInt(request.params.green);
		var blue = parseInt(request.params.blue);
		self.__rightRgb(getRobot(0), red, green, blue);
		response.end();
	});
	
	express.get('/hamsterS/rightRgb/:gray', function(request, response) {
		response.writeHead(200, HEADER);
		var gray = parseInt(request.params.gray);
		self.__rightRgb(getRobot(0), gray, gray, gray);
		response.end();
	});
	
	express.get('/hamsterS/buzzer/:hz', function(request, response) {
		response.writeHead(200, HEADER);
		var hz = parseFloat(request.params.hz);
		self.__buzzer(getRobot(0), hz);
		response.end();
	});
	
	express.get('/hamsterS/note/:pitch', function(request, response) {
		response.writeHead(200, HEADER);
		var pitch = parseInt(request.params.pitch);
		self.__note(getRobot(0), pitch);
		response.end();
	});
	
	express.get('/hamsterS/sound/:sound/:count', function(request, response) {
		response.writeHead(200, HEADER);
		var sound = parseInt(request.params.sound);
		var count = parseInt(request.params.count);
		self.__sound(getRobot(0), sound, count);
		response.end();
	});
	
	express.get('/hamsterS/sound/:sound', function(request, response) {
		response.writeHead(200, HEADER);
		var sound = parseInt(request.params.sound);
		self.__sound(getRobot(0), sound, 1);
		response.end();
	});
	
	express.get('/hamsterS/ioModeA/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var mode = parseInt(request.params.mode);
		self.__ioModeA(getRobot(0), mode);
		response.end();
	});
	
	express.get('/hamsterS/ioModeB/:mode', function(request, response) {
		response.writeHead(200, HEADER);
		var mode = parseInt(request.params.mode);
		self.__ioModeB(getRobot(0), mode);
		response.end();
	});
	
	express.get('/hamsterS/outputA/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var value = parseFloat(request.params.value);
		self.__outputA(getRobot(0), value);
		response.end();
	});
	
	express.get('/hamsterS/outputB/:value', function(request, response) {
		response.writeHead(200, HEADER);
		var value = parseFloat(request.params.value);
		self.__outputB(getRobot(0), value);
		response.end();
	});
	
	express.get('/hamsterS/gripper/:action', function(request, response) {
		response.writeHead(200, HEADER);
		var action = parseInt(request.params.action);
		self.__gripper(getRobot(0), action);
		response.end();
	});
	
	express.get('/hamsterS/openGripper', function(request, response) {
		response.writeHead(200, HEADER);
		self.__openGripper(getRobot(0));
		response.end();
	});
	
	express.get('/hamsterS/closeGripper', function(request, response) {
		response.writeHead(200, HEADER);
		self.__closeGripper(getRobot(0));
		response.end();
	});
	
	express.get('/hamsterS/releaseGripper', function(request, response) {
		response.writeHead(200, HEADER);
		self.__releaseGripper(getRobot(0));
		response.end();
	});
	
	express.get('/hamsterS/writeSerial/:text', function(request, response) {
		response.writeHead(200, HEADER);
		var text = parseInt(request.params.text);
		self.__writeSerial(getRobot(0), text);
		response.end();
	});
	
	express.get('/hamsterS/signalStrength', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__signalStrength(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/leftProximity', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__leftProximity(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/rightProximity', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__rightProximity(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/leftFloor', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__leftFloor(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/rightFloor', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__rightFloor(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/accelerationX', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationX(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/accelerationY', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationY(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/accelerationZ', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__accelerationZ(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/light', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__light(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/temperature', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__temperature(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/inputA', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__inputA(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/inputB', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__inputB(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/pulseCount', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__pulseCount(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/pulseCountId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__pulseCountId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/tilt', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__tilt(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/tiltId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__tiltId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/tapId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__tapId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/freeFallId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__freeFallId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/wheelState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__wheelState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/wheelStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__wheelStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/soundState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__soundState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/soundStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__soundStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/lineTracerState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__lineTracerState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/lineTracerStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__lineTracerStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/batteryState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__batteryState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/batteryStateId', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__batteryStateId(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/sensors', function(request, response) {
		response.writeHead(200, HEADER);
		response.write(self.__sensors(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/connectionState', function(request, response) {
		response.writeHead(200, HEADER);
		response.write('' + self.__connectionState(getRobot(0)));
		response.end();
	});
	
	express.get('/hamsterS/reset', function(request, response) {
		response.writeHead(200, HEADER);
		self.__reset(getRobot(0), getRoute(0));
		response.end();
	});
};

Component.prototype.__wheels = function(robot, leftSpeed, rightSpeed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		if(typeof leftSpeed == 'number') {
			motoring.leftWheel.write(leftSpeed);
		}
		if(typeof rightSpeed == 'number') {
			motoring.rightWheel.write(rightSpeed);
		}
	}
};

Component.prototype.__leftWheel = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		if(typeof speed == 'number') {
			motoring.leftWheel.write(speed);
		}
	}
};

Component.prototype.__rightWheel = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		if(typeof speed == 'number') {
			motoring.rightWheel.write(speed);
		}
	}
};

Component.prototype.__moveForward = function(robot, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof cm == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: cm
			motoring.motion.write(2, speed);
			if(cm < 0) {
				motoring.motion.write(3, -cm);
				motoring.motion.write(0, HamsterS.MOTION_MOVE_BACKWARD);
			} else {
				motoring.motion.write(3, cm);
				motoring.motion.write(0, HamsterS.MOTION_MOVE_FORWARD);
			}
		}
	}
};

Component.prototype.__moveBackward = function(robot, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof cm == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: cm
			motoring.motion.write(2, speed);
			if(cm < 0) {
				motoring.motion.write(3, -cm);
				motoring.motion.write(0, HamsterS.MOTION_MOVE_FORWARD);
			} else {
				motoring.motion.write(3, cm);
				motoring.motion.write(0, HamsterS.MOTION_MOVE_BACKWARD);
			}
		}
	}
};

Component.prototype.__turnLeft = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, HamsterS.MOTION_TURN_RIGHT);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, HamsterS.MOTION_TURN_LEFT);
			}
		}
	}
};

Component.prototype.__turnRight = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, HamsterS.MOTION_TURN_LEFT);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, HamsterS.MOTION_TURN_RIGHT);
			}
		}
	}
};

Component.prototype.__pivotLeft = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, HamsterS.MOTION_PIVOT_LEFT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, HamsterS.MOTION_PIVOT_LEFT_HEAD);
			}
		}
	}
};

Component.prototype.__pivotRight = function(robot, deg, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, HamsterS.MOTION_PIVOT_RIGHT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, HamsterS.MOTION_PIVOT_RIGHT_HEAD);
			}
		}
	}
};

Component.prototype.__swingLeft = function(robot, deg, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof cm == 'number') && cm >= 0 && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			motoring.motion.write(4, cm); // radius
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, HamsterS.MOTION_SWING_LEFT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, HamsterS.MOTION_SWING_LEFT_HEAD);
			}
		}
	}
};

Component.prototype.__swingRight = function(robot, deg, cm, speed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		motoring.pulse.write(0);
		motoring.lineTracerMode.write(0);
		if((typeof deg == 'number') && (typeof cm == 'number') && cm >= 0 && (typeof speed == 'number') && speed >= 0) {
			motoring.motion.write(1, 1); // unit: deg
			motoring.motion.write(2, speed);
			motoring.motion.write(4, cm); // radius
			if(deg < 0) {
				motoring.motion.write(3, -deg);
				motoring.motion.write(0, HamsterS.MOTION_SWING_RIGHT_TAIL);
			} else {
				motoring.motion.write(3, deg);
				motoring.motion.write(0, HamsterS.MOTION_SWING_RIGHT_HEAD);
			}
		}
	}
};

Component.prototype.__pulse = function(robot, pulse, leftSpeed, rightSpeed) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		motoring.motion.write(0);
		if(typeof leftSpeed == 'number') {
			motoring.leftWheel.write(leftSpeed);
		}
		if(typeof rightSpeed == 'number') {
			motoring.rightWheel.write(rightSpeed);
		}
		if(typeof pulse == 'number') {
			motoring.pulse.write(pulse);
		}
	}
};

Component.prototype.__stop = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.lineTracerMode.write(0);
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
	}
};

Component.prototype.__lineTracerMode = function(robot, mode) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.leftWheel.write(0);
		motoring.rightWheel.write(0);
		if(typeof mode == 'number') {
			motoring.lineTracerMode.write(mode);
		}
	}
};

Component.prototype.__lineTracerGain = function(robot, gain) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof gain == 'number') {
			motoring.lineTracerGain.write(gain);
		}
	}
};

Component.prototype.__lineTracerSpeed = function(robot, speed) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof speed == 'number') {
			motoring.lineTracerSpeed.write(speed);
		}
	}
};

Component.prototype.__rgbs = function(robot, leftRed, leftGreen, leftBlue, rightRed, rightGreen, rightBlue) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof leftRed == 'number') {
			motoring.leftRgb.write(0, leftRed);
		}
		if(typeof leftGreen == 'number') {
			motoring.leftRgb.write(1, leftGreen);
		}
		if(typeof leftBlue == 'number') {
			motoring.leftRgb.write(2, leftBlue);
		}
		if(typeof rightRed == 'number') {
			motoring.rightRgb.write(0, rightRed);
		}
		if(typeof rightGreen == 'number') {
			motoring.rightRgb.write(1, rightGreen);
		}
		if(typeof rightBlue == 'number') {
			motoring.rightRgb.write(2, rightBlue);
		}
	}
};

Component.prototype.__leftRgb = function(robot, red, green, blue) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof red == 'number') {
			motoring.leftRgb.write(0, red);
		}
		if(typeof green == 'number') {
			motoring.leftRgb.write(1, green);
		}
		if(typeof blue == 'number') {
			motoring.leftRgb.write(2, blue);
		}
	}
};

Component.prototype.__rightRgb = function(robot, red, green, blue) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof red == 'number') {
			motoring.rightRgb.write(0, red);
		}
		if(typeof green == 'number') {
			motoring.rightRgb.write(1, green);
		}
		if(typeof blue == 'number') {
			motoring.rightRgb.write(2, blue);
		}
	}
};

Component.prototype.__buzzer = function(robot, hz) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof hz == 'number') {
			motoring.buzzer.write(hz);
		}
		motoring.note.write(0);
	}
};

Component.prototype.__note = function(robot, pitch) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.buzzer.write(0);
		if(typeof pitch == 'number') {
			motoring.note.write(pitch);
		}
	}
};

Component.prototype.runSound = function(robot, sound, count) {
	if(typeof count != 'number') count = 1;
	if(count < 0) count = -1;
	if(count) {
		robot.soundId = sound;
		robot.soundRepeat = count;
		robot.motoring.sound.write(sound);
	}
};

Component.prototype.__sound = function(self, robot, sound, count) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.buzzer.write(0);
		motoring.note.write(0);
		if((typeof sound == 'number') && count) {
			self.runSound(robot, sound, count);
		}
	}
};

Component.prototype.__ioModeA = function(robot, mode) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof mode == 'number') {
			motoring.ioModeA.write(mode);
		}
	}
};

Component.prototype.__ioModeB = function(robot, mode) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof mode == 'number') {
			motoring.ioModeB.write(mode);
		}
	}
};

Component.prototype.__outputA = function(robot, value) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof value == 'number') {
			motoring.outputA.write(value);
		}
	}
};

Component.prototype.__outputB = function(robot, value) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof value == 'number') {
			motoring.outputB.write(value);
		}
	}
};

Component.prototype.__gripper = function(robot, action) {
	if(robot) {
		var motoring = robot.motoring;
		if(typeof action == 'number') {
			motoring.ioModeA.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
			motoring.ioModeB.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
			if(action > 0) {
				motoring.outputA.write(1);
				motoring.outputB.write(0);
			} else if(action < 0) {
				motoring.outputA.write(0);
				motoring.outputB.write(1);
			} else {
				motoring.outputA.write(0);
				motoring.outputB.write(0);
			}
		}
	}
};

Component.prototype.__openGripper = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.ioModeA.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
		motoring.ioModeB.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
		motoring.outputA.write(1);
		motoring.outputB.write(0);
	}
};

Component.prototype.__closeGripper = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.ioModeA.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
		motoring.ioModeB.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
		motoring.outputA.write(0);
		motoring.outputB.write(1);
	}
};

Component.prototype.__releaseGripper = function(robot) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.ioModeA.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
		motoring.ioModeB.write(HamsterS.IO_MODE_DIGITAL_OUTPUT);
		motoring.outputA.write(0);
		motoring.outputB.write(0);
	}
};

Component.prototype.__writeSerial = function(robot, text) {
	if(robot) {
		var motoring = robot.motoring;
		motoring.ioModeA.write(robot.serialRate);
		motoring.ioModeB.write(robot.serialRate);
		var queue = robot.writeQueue;
		queue.push(text, false);
		var data = queue.pop();
		if(data) {
			motoring.writeSerial.write(data);
		}
	}
};

Component.prototype.__signalStrength = function(robot) {
	if(robot) {
		return robot.sensory.signalStrength;
	}
};

Component.prototype.__leftProximity = function(robot) {
	if(robot) {
		return robot.sensory.leftProximity;
	}
};

Component.prototype.__rightProximity = function(robot) {
	if(robot) {
		return robot.sensory.rightProximity;
	}
};

Component.prototype.__leftFloor = function(robot) {
	if(robot) {
		return robot.sensory.leftFloor;
	}
};

Component.prototype.__rightFloor = function(robot) {
	if(robot) {
		return robot.sensory.rightFloor;
	}
};

Component.prototype.__accelerationX = function(robot) {
	if(robot) {
		return robot.sensory.accelerationX;
	}
};

Component.prototype.__accelerationY = function(robot) {
	if(robot) {
		return robot.sensory.accelerationY;
	}
};

Component.prototype.__accelerationZ = function(robot) {
	if(robot) {
		return robot.sensory.accelerationZ;
	}
};

Component.prototype.__light = function(robot) {
	if(robot) {
		return robot.sensory.light;
	}
};

Component.prototype.__temperature = function(robot) {
	if(robot) {
		return robot.sensory.temperature;
	}
};

Component.prototype.__inputA = function(robot) {
	if(robot) {
		return robot.sensory.inputA;
	}
};

Component.prototype.__inputB = function(robot) {
	if(robot) {
		return robot.sensory.inputB;
	}
};

Component.prototype.__pulseCount = function(robot) {
	if(robot) {
		return robot.sensory.pulseCount;
	}
};

Component.prototype.__pulseCountId = function(robot) {
	if(robot) {
		return robot.sensory.pulseCountId;
	}
};

Component.prototype.__tilt = function(robot) {
	if(robot) {
		return robot.sensory.tilt;
	}
};

Component.prototype.__tiltId = function(robot) {
	if(robot) {
		return robot.sensory.tiltId;
	}
};

Component.prototype.__tapId = function(robot) {
	if(robot) {
		return robot.sensory.tapId;
	}
};

Component.prototype.__freeFallId = function(robot) {
	if(robot) {
		return robot.sensory.freeFallId;
	}
};

Component.prototype.__wheelState = function(robot) {
	if(robot) {
		return robot.sensory.wheelState;
	}
};

Component.prototype.__wheelStateId = function(robot) {
	if(robot) {
		return robot.sensory.wheelStateId;
	}
};

Component.prototype.__soundState = function(robot) {
	if(robot) {
		return robot.sensory.soundState;
	}
};

Component.prototype.__soundStateId = function(robot) {
	if(robot) {
		return robot.sensory.soundStateId;
	}
};

Component.prototype.__lineTracerState = function(robot) {
	if(robot) {
		return robot.sensory.lineTracerState;
	}
};

Component.prototype.__lineTracerStateId = function(robot) {
	if(robot) {
		return robot.sensory.lineTracerStateId;
	}
};

Component.prototype.__batteryState = function(robot) {
	if(robot) {
		return robot.sensory.batteryState;
	}
};

Component.prototype.__batteryStateId = function(robot) {
	if(robot) {
		return robot.sensory.batteryStateId;
	}
};

Component.prototype.__sensors = function(robot) {
	if(robot) {
		try {
			return JSON.stringify(robot.sensory);
		} catch (e) {
		}
	}
};

Component.prototype.__connectionState = function(robot) {
	if(robot) {
		return robot.connectionState;
	}
};

Component.prototype.__reset = function(robot, route) {
	if(robot) {
		robot.soundId = 0;
		robot.soundRepeat = 1;
		robot.serialRate = HamsterS.IO_MODE_SERIAL_9600;
		robot.serialDelimiter = 0;
		robot.writeQueue.reset();
		robot.readQueue.reset();
	}
	if(route) {
		router.resetRoute(route);
	}
};

Component.prototype.setState = function(robot, state) {
	robot.connectionState = state;
};

Component.prototype.handleSensory = function(robot, data, state) {
	robot.connectionState = state;
	var sensory = robot.sensory;
	sensory.signalStrength = data.signalStrength;
	sensory.leftProximity = data.leftProximity;
	sensory.rightProximity = data.rightProximity;
	sensory.leftFloor = data.leftFloor;
	sensory.rightFloor = data.rightFloor;
	sensory.accelerationX = data.acceleration.x;
	sensory.accelerationY = data.acceleration.y;
	sensory.accelerationZ = data.acceleration.z;
	sensory.light = data.light;
	sensory.temperature = data.temperature;
	sensory.inputA = data.inputA;
	sensory.inputB = data.inputB;
	if(data.map1 & 0x00000010) {
		sensory.lineTracerState = data.lineTracerState;
		sensory.lineTracerStateId = (sensory.lineTracerStateId % 255) + 1;
	}
	if(data.map1 & 0x00000008) {
		sensory.tilt = data.tilt;
		sensory.tiltId = (sensory.tiltId % 255) + 1;
	}
	if(data.map1 & 0x00000004) {
		sensory.batteryState = data.batteryState;
		sensory.batteryStateId = (sensory.batteryStateId % 255) + 1;
	}
	if(data.map2 & 0x00008000) {
		sensory.freeFallId = (sensory.freeFallId % 255) + 1;
	}
	if(data.map2 & 0x00004000) {
		sensory.tapId = (sensory.tapId % 255) + 1;
	}
	if(data.map2 & 0x00002000) {
		sensory.readSerial = data.serial;
		sensory.readSerialId = (sensory.readSerialId % 255) + 1;
	}
	if(data.map2 & 0x00001000) {
		sensory.pulseCount = data.pulseCount;
		sensory.pulseCountId = (sensory.pulseCountId % 255) + 1;
	}
	if(data.map2 & 0x00000800) {
		sensory.wheelState = data.wheelState;
		sensory.wheelStateId = (sensory.wheelStateId % 255) + 1;
	}
	if(data.map2 & 0x00000400) {
		if(data.soundState == 0) {
			if(robot.soundId > 0) {
				if(robot.soundRepeat < 0) {
					this.runSound(robot, robot.soundId, -1);
				} else if(robot.soundRepeat > 1) {
					robot.soundRepeat --;
					this.runSound(robot, robot.soundId, robot.soundRepeat);
				} else {
					robot.soundId = 0;
					robot.soundRepeat = 1;
					sensory.soundState = data.soundState;
					sensory.soundStateId = (sensory.soundStateId % 255) + 1;
				}
			} else {
				robot.soundId = 0;
				robot.soundRepeat = 1;
				sensory.soundState = data.soundState;
				sensory.soundStateId = (sensory.soundStateId % 255) + 1;
			}
		} else if(data.soundState != sensory.soundState) {
			sensory.soundState = data.soundState;
			sensory.soundStateId = (sensory.soundStateId % 255) + 1;
		}
	}
};

module.exports = new Component();