'use strict';
var context = require('context');
var HamsterS = require('../hamsterS');
var factory = require('../hamsterS-roboid');
const MODULE = 'hamsterS';

const SPEED2GAINS = { 1: 6, 2: 6, 3: 5, 4: 5, 5: 4, 6: 4, 7: 3, 8: 3 };
const LED2RGB = {
	0: [0, 0, 0],
	1: [0, 0, 255],
	2: [0, 255, 0],
	3: [0, 255, 255],
	4: [255, 0, 0],
	5: [255, 0, 255],
	6: [255, 255, 0],
	7: [255, 255, 255]
};

function Component() {
}

Component.prototype.createRobot = function(index) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	robot.sensory = {
		module: 'hamster', // for compatibility
		realModule: MODULE,
		index: index,
		map: 0,
		map2: 0,
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		tilt: 0,
		serial: '',
		pulseCount: 0,
		wheelState: 0,
		soundState: 0,
		lineTracerState: 0,
		batteryState: 2,
		handFound: false
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(HamsterS.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(HamsterS.RIGHT_WHEEL),
		leftRgb: roboid.findDeviceById(HamsterS.LEFT_RGB),
		rightRgb: roboid.findDeviceById(HamsterS.RIGHT_RGB),
		buzzer: roboid.findDeviceById(HamsterS.BUZZER),
		outputA: roboid.findDeviceById(HamsterS.OUTPUT_A),
		outputB: roboid.findDeviceById(HamsterS.OUTPUT_B),
		pulse: roboid.findDeviceById(HamsterS.PULSE),
		note: roboid.findDeviceById(HamsterS.NOTE),
		sound: roboid.findDeviceById(HamsterS.SOUND),
		lineTracerMode: roboid.findDeviceById(HamsterS.LINE_TRACER_MODE),
		lineTracerGain: roboid.findDeviceById(HamsterS.LINE_TRACER_GAIN),
		lineTracerSpeed: roboid.findDeviceById(HamsterS.LINE_TRACER_SPEED),
		ioModeA: roboid.findDeviceById(HamsterS.IO_MODE_A),
		ioModeB: roboid.findDeviceById(HamsterS.IO_MODE_B),
		writeSerial: roboid.findDeviceById(HamsterS.WRITE_SERIAL),
		motion: roboid.findDeviceById(HamsterS.MOTION),
		motorMode: roboid.findDeviceById(HamsterS.MOTOR_MODE),
		configProximity: roboid.findDeviceById(HamsterS.CONFIG_PROXIMITY),
		configGravity: roboid.findDeviceById(HamsterS.CONFIG_GRAVITY),
		configBandWidth: roboid.findDeviceById(HamsterS.CONFIG_BAND_WIDTH)
	};
	return robot;
};

Component.prototype.handleRemoteData = function(robot, data) {
	var motoring = robot.motoring;
	if(data.leftWheel !== undefined) motoring.leftWheel.write(data.leftWheel);
	if(data.rightWheel !== undefined) motoring.rightWheel.write(data.rightWheel);
	if(data.leftRed !== undefined) motoring.leftRgb.write(0, data.leftRed);
	if(data.leftGreen !== undefined) motoring.leftRgb.write(1, data.leftGreen);
	if(data.leftBlue !== undefined) motoring.leftRgb.write(2, data.leftBlue);
	if(data.rightRed !== undefined) motoring.rightRgb.write(0, data.rightRed);
	if(data.rightGreen !== undefined) motoring.rightRgb.write(1, data.rightGreen);
	if(data.rightBlue !== undefined) motoring.rightRgb.write(2, data.rightBlue);
	if(data.buzzer !== undefined) motoring.buzzer.write(data.buzzer);
	if(data.outputA !== undefined) motoring.outputA.write(data.outputA);
	if(data.outputB !== undefined) motoring.outputB.write(data.outputB);
	var map1 = data.map, map2 = data.map2;
	if((map2 & 0x20000000) && (data.pulse !== undefined)) motoring.pulse.write(data.pulse);
	if((map1 & 0x00400000) && (data.note !== undefined)) motoring.note.write(data.note);
	if((map2 & 0x10000000) && (data.sound !== undefined)) motoring.sound.write(data.sound);
	if((map1 & 0x00200000) && (data.lineTracerMode !== undefined)) motoring.lineTracerMode.write(data.lineTracerMode);
	if((map2 & 0x08000000) && (data.lineTracerGain !== undefined)) motoring.lineTracerGain.write(data.lineTracerGain);
	if((map1 & 0x00100000) && (data.lineTracerSpeed !== undefined)) motoring.lineTracerSpeed.write(data.lineTracerSpeed);
	if((map1 & 0x00080000) && (data.ioModeA !== undefined)) motoring.ioModeA.write(data.ioModeA);
	if((map1 & 0x00040000) && (data.ioModeB !== undefined)) motoring.ioModeB.write(data.ioModeB);
	if((map2 & 0x04000000) && (data.serial !== undefined)) motoring.writeSerial.write(data.serial);
	if(map2 & 0x02000000) {
		if(data.motionType !== undefined) {
			var motionType = data.motionType;
			var value = 0;
			if(motionType == 101) {
				motionType = 1;
				value = context.getMovementDistance(MODULE);
			} else if(motionType == 102) {
				motionType = 2;
				value = context.getMovementDistance(MODULE);
			} else if(motionType == 103) {
				motionType = 3;
				value = context.getTurningAngle(MODULE);
			} else if(motionType == 104) {
				motionType = 4;
				value = context.getTurningAngle(MODULE);
			} else if(data.motionValue != undefined) {
				value = data.motionValue;
			}
			motoring.motion.write(0, motionType);
			if(data.motionUnit !== undefined) motoring.motion.write(1, data.motionUnit);
			if(data.motionSpeed !== undefined) motoring.motion.write(2, data.motionSpeed);
			if(data.motionValue !== undefined) motoring.motion.write(3, value);
			if(data.motionRadius !== undefined) motoring.motion.write(4, data.motionRadius);
		}
	}
};

Component.prototype.handleRemoteDataHamster = function(robot, data) {
	var motoring = robot.motoring;
	if(data.motion == 1) {
		var speed = context.getMovementSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 2) {
		var speed = context.getMovementSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(-speed);
	} else if(data.motion == 3) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(-speed);
		motoring.rightWheel.write(speed);
	} else if(data.motion == 4) {
		var speed = context.getTurningSpeed(MODULE);
		motoring.leftWheel.write(speed);
		motoring.rightWheel.write(-speed);
	} else {
		if(data.leftWheel !== undefined) motoring.leftWheel.write(data.leftWheel);
		if(data.rightWheel !== undefined) motoring.rightWheel.write(data.rightWheel);
	}
	if(data.buzzer !== undefined) motoring.buzzer.write(data.buzzer);
	if(data.outputA !== undefined) motoring.outputA.write(data.outputA);
	if(data.outputB !== undefined) motoring.outputB.write(data.outputB);
	var map = data.map;
	if((map & 0x01000000) && (data.leftLed !== undefined)) {
		var rgb = LED2RGB[data.leftLed];
		if(rgb) motoring.leftRgb.write(rgb);
	}
	if((map & 0x00800000) && (data.rightLed !== undefined)) {
		var rgb = LED2RGB[data.rightLed];
		if(rgb) motoring.rightRgb.write(rgb);
	}
	motoring.pulse.write(0);
	if((map & 0x00400000) && (data.note !== undefined)) motoring.note.write(data.note);
	motoring.sound.write(0);
	if((map & 0x00200000) && (data.lineTracerMode !== undefined)) motoring.lineTracerMode.write(data.lineTracerMode);
	if((map & 0x00100000) && (data.lineTracerSpeed !== undefined)) {
		motoring.lineTracerSpeed.write(data.lineTracerSpeed);
		motoring.lineTracerGain.write(SPEED2GAINS[data.lineTracerSpeed]);
	}
	if((map & 0x00080000) && (data.ioModeA !== undefined)) motoring.ioModeA.write(data.ioModeA);
	if((map & 0x00040000) && (data.ioModeB !== undefined)) motoring.ioModeB.write(data.ioModeB);
	motoring.motion.write(0, 0);
	motoring.motion.write(1, 0);
	motoring.motion.write(2, 0);
	motoring.motion.write(3, 0);
	motoring.motion.write(4, 0);
};

Component.prototype.handleSensory = function(packet, robot, data) {
	var sensory = robot.sensory;
	sensory.map = data.map1;
	sensory.map2 = data.map2;
	sensory.signalStrength = data.signalStrength;
	sensory.leftProximity = data.leftProximity;
	sensory.rightProximity = data.rightProximity;
	sensory.leftFloor = data.leftFloor;
	sensory.rightFloor = data.rightFloor;
	sensory.accelerationX = data.acceleration.x;
	sensory.accelerationY = data.acceleration.y;
	sensory.accelerationZ = data.acceleration.z;
	sensory.light = data.light;
	sensory.temperature = data.temperature;
	sensory.inputA = data.inputA;
	sensory.inputB = data.inputB;
	if(data.map1 & 0x00000008) sensory.tilt = data.tilt;
	if(data.map2 & 0x00002000) sensory.serial = data.serial;
	if(data.map2 & 0x00001000) sensory.pulseCount = data.pulseCount;
	if(data.map2 & 0x00000800) sensory.wheelState = data.wheelState;
	if(data.map2 & 0x00000400) sensory.soundState = data.soundState;
	if(data.map1 & 0x00000010) sensory.lineTracerState = data.lineTracerState;
	if(data.map1 & 0x00000004) sensory.batteryState = data.batteryState;
	var threshold = context.getHandFoundThreshold(MODULE);
	sensory.handFound = sensory.leftProximity > threshold || sensory.rightProximity > threshold;
	packet.robot = sensory;
};

module.exports = new Component();