'use strict';
var router = require('router');
var component = require('./component');
const MODULE = 'hamsterS';

function Extension() {
	this.robots = {};
	this.connectionPacket = {
		type: 0,
		module: 'hamster', // for compatibility
		state: 0
	};
	this.masterVersion = 2;
}

Extension.prototype.getRobot = function(index) {
	if(index >= 0) {
		var robot = this.robots[index];
		if(!robot) {
			robot = component.createRobot(index);
			if(robot) {
				this.robots[index] = robot;
			}
		}
		return robot;
	}
};

Extension.prototype.start = function(connection) {
	var self = this;
	self.connection = connection;
	connection.on('message', function(message) {
		if(message.type === 'utf8') {
			self.handleRemoteData(message.utf8Data);
		}
	});
};

Extension.prototype.stop = function() {
	this.connection = undefined;
	router.reset();
	this.robots = {};
};

Extension.prototype.setState = function(route) {
	if(route == router.firstRoute && route.module == MODULE && this.connection) {
		this.connectionPacket.state = router.getConnectionState(route.state);
		try {
			this.connection.send(JSON.stringify(this.connectionPacket));
		} catch (e) {
		}
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleRemoteDataByModule = function(data) {
	var robot = this.getRobot(data.index);
	if(robot) {
		if(data.module == MODULE) {
			component.handleRemoteData(robot, data);
		} else if(data.module == 'hamster') {
			component.handleRemoteDataHamster(robot, data);
		}
	}
};

Extension.prototype.handleRemoteData = function(utf8Data) {
	try {
		var received = JSON.parse(utf8Data);
		for(var i in received) {
			this.handleRemoteDataByModule(received[i]);
		}
		this.masterVersion = received.version || 0;
	} catch (e) {
	}
};

Extension.prototype.handleSensory = function(data, route) {
	if(route.module == MODULE) {
		var robot = this.getRobot(route.index);
		if(robot) {
			var packet = {};
			component.handleSensory(packet, robot, data);
			if(this.masterVersion == 0) {
				packet = packet.robot;
			}
			packet.type = 1;
			packet.version = 1;
			if(this.connection) {
				try {
					this.connection.send(JSON.stringify(packet));
				} catch (e) {
				}
			}
		}
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	if(route.module == MODULE) {
		var robot = this.getRobot(route.index);
		if(robot) {
			var roboid = robot.roboid;
			roboid.handleMotoring(data);
			roboid.updateMotoring();
		}
	}
};

Extension.prototype.requestMotoring = function(route) {
	if(route.module == MODULE) {
		var robot = this.getRobot(route.index);
		if(robot) {
			return robot.roboid.requestMotoring();
		}
	}
};

Extension.prototype.reset = function(route) {
	if(route.module == MODULE) {
		var robot = this.getRobot(route.index);
		if(robot) {
			robot.roboid.reset();
		}
	}
};

module.exports.create = function() {
	return new Extension();
};