'use strict';

function WriteQueue(size) {
	this.setSize(size);
	this.output = new Array(19);
}

WriteQueue.prototype.setSize = function(size) {
	this.buffer = new Array(size);
	this.mask = size - 1;
	this.provider = 0;
	this.consumer = 0;
};

WriteQueue.prototype.reset = function() {
	this.provider = 0;
	this.consumer = 0;
};

WriteQueue.prototype.push = function(str, line) {
	var buffer = this.buffer;
	var mask = this.mask;
	var provider = this.provider;
	var consumer = this.consumer;
	
	var len = str.length;
	if(len > 0) {
		for(var i = 0; i < len; ++i) {
			if(((provider - consumer) & mask) == mask) { // full
				consumer = (consumer + 1) & mask;
			}
			buffer[provider] = str.charCodeAt(i);
			provider = (provider + 1) & mask;
		}
	}
	if(line) {
		if(((provider - consumer) & mask) == mask) { // full
			consumer = (consumer + 1) & mask;
		}
		buffer[provider] = 0x0D;
		provider = (provider + 1) & mask;
	}
	this.provider = provider;
	this.consumer = consumer;
};

WriteQueue.prototype.pop = function() {
	var provider = this.provider;
	var consumer = this.consumer;
	if(provider == consumer) return undefined; // empty
	
	var buffer = this.buffer;
	var mask = this.mask;
	var output = this.output;
	var len = (provider - consumer) & mask;
	if(len > 18) len = 18;
	
	output[0] = len;
	var i = 1;
	for(; i <= len && consumer != provider; ++i) {
		output[i] = buffer[consumer];
		consumer = (consumer + 1) & mask;
	}
	for(; i <= 18; ++i) {
		output[i] = 0;
	}
	this.consumer = consumer;
	return output;
};

function ReadQueue(size) {
	this.setSize(size);
}

ReadQueue.prototype.setSize = function(size) {
	this.buffer = new Array(size);
	this.mask = size - 1;
	this.provider = 0;
	this.consumer = 0;
};

ReadQueue.prototype.reset = function() {
	this.provider = 0;
	this.consumer = 0;
};

ReadQueue.prototype.push = function(packet) {
	var len = packet[0];
	if(len > 0) {
		if(len > 18) len = 18;
		
		var buffer = this.buffer;
		var mask = this.mask;
		var provider = this.provider;
		var consumer = this.consumer;
		for(var i = 1; i <= len; ++i) {
			if(((provider - consumer) & mask) == mask) { // full
				consumer = (consumer + 1) & mask;
			}
			buffer[provider] = packet[i];
			provider = (provider + 1) & mask;
		}
		this.provider = provider;
		this.consumer = consumer;
	}
};

ReadQueue.prototype.pop = function(delimiter) {
	var provider = this.provider;
	var consumer = this.consumer;
	if(provider == consumer) return undefined; // empty

	var buffer = this.buffer;
	var mask = this.mask;
	if(delimiter == 0) {
		var str = '';
		while(consumer != provider) {
			str += String.fromCharCode(buffer[consumer]);
			consumer = (consumer + 1) & mask;
		}
		this.consumer = consumer;
		return str;
	} else {
		var found = -1;
		while(consumer != provider) {
			if(buffer[consumer] == delimiter) {
				found = consumer;
				break;
			}
			consumer = (consumer + 1) & mask;
		}
		if(found >= 0) {
			consumer = this.consumer;
			var str = '';
			while(consumer != found) {
				str += String.fromCharCode(buffer[consumer]);
				consumer = (consumer + 1) & mask;
			}
			this.consumer = (consumer + 1) & mask;
			return str;
		}
	}
};

module.exports.createWriteQueue = function(size) {
	return new WriteQueue(size);
};

module.exports.createReadQueue = function(size) {
	return new ReadQueue(size);
};