'use strict';
var context = require('context');
var router = require('router');
var HamsterS = require('../hamsterS');
var factory = require('../hamsterS-roboid');
const GROUP = 'hamster';
const MODULE = 'hamsterS';

function Component() {
}

Component.prototype.createRobot = function(index, state) {
	var robot = {};
	var roboid = robot.roboid = factory.create();
	state = router.getConnectionState(state);
	robot.sensory = {
		group: GROUP,
		module: MODULE,
		model: Hamster.ID,
		index: index,
		connectionState: state,
		map1: 0,
		map2: 0,
		signalStrength: 0,
		leftProximity: 0,
		rightProximity: 0,
		leftFloor: 0,
		rightFloor: 0,
		accelerationX: 0,
		accelerationY: 0,
		accelerationZ: 0,
		light: 0,
		temperature: 0,
		inputA: 0,
		inputB: 0,
		tilt: 0,
		serial: undefined,
		pulseCount: 0,
		wheelState: 0,
		soundState: 0,
		lineTracerState: 0,
		batteryState: 2,
		handFound: false
	};
	robot.motoring = {
		leftWheel: roboid.findDeviceById(HamsterS.LEFT_WHEEL),
		rightWheel: roboid.findDeviceById(HamsterS.RIGHT_WHEEL),
		leftRgb: roboid.findDeviceById(HamsterS.LEFT_RGB),
		rightRgb: roboid.findDeviceById(HamsterS.RIGHT_RGB),
		buzzer: roboid.findDeviceById(HamsterS.BUZZER),
		outputA: roboid.findDeviceById(HamsterS.OUTPUT_A),
		outputB: roboid.findDeviceById(HamsterS.OUTPUT_B),
		pulse: roboid.findDeviceById(HamsterS.PULSE),
		note: roboid.findDeviceById(HamsterS.NOTE),
		sound: roboid.findDeviceById(HamsterS.SOUND),
		lineTracerMode: roboid.findDeviceById(HamsterS.LINE_TRACER_MODE),
		lineTracerGain: roboid.findDeviceById(HamsterS.LINE_TRACER_GAIN),
		lineTracerSpeed: roboid.findDeviceById(HamsterS.LINE_TRACER_SPEED),
		ioModeA: roboid.findDeviceById(HamsterS.IO_MODE_A),
		ioModeB: roboid.findDeviceById(HamsterS.IO_MODE_B),
		writeSerial: roboid.findDeviceById(HamsterS.WRITE_SERIAL),
		motion: roboid.findDeviceById(HamsterS.MOTION),
		motorMode: roboid.findDeviceById(HamsterS.MOTOR_MODE),
		configProximity: roboid.findDeviceById(HamsterS.CONFIG_PROXIMITY),
		configGravity: roboid.findDeviceById(HamsterS.CONFIG_GRAVITY),
		configBandWidth: roboid.findDeviceById(HamsterS.CONFIG_BAND_WIDTH)
	};
	return robot;
};

Component.prototype.setState = function(robot, state) {
	robot.sensory.connectionState = router.getConnectionState(state);
};

Component.prototype.handleRemoteData = function(robot, data) {
	var motoring = robot.motoring;
	if(data.leftWheel !== undefined) motoring.leftWheel.write(data.leftWheel);
	if(data.rightWheel !== undefined) motoring.rightWheel.write(data.rightWheel);
	if(data.leftRed !== undefined) motoring.leftRgb.write(0, data.leftRed);
	if(data.leftGreen !== undefined) motoring.leftRgb.write(1, data.leftGreen);
	if(data.leftBlue !== undefined) motoring.leftRgb.write(2, data.leftBlue);
	if(data.rightRed !== undefined) motoring.rightRgb.write(0, data.rightRed);
	if(data.rightGreen !== undefined) motoring.rightRgb.write(1, data.rightGreen);
	if(data.rightBlue !== undefined) motoring.rightRgb.write(2, data.rightBlue);
	if(data.buzzer !== undefined) motoring.buzzer.write(data.buzzer);
	if(data.outputA !== undefined) motoring.outputA.write(data.outputA);
	if(data.outputB !== undefined) motoring.outputB.write(data.outputB);
	var map1 = data.map1, map2 = data.map2;
	if((map2 & 0x20000000) && (data.pulse !== undefined)) motoring.pulse.write(data.pulse);
	if((map1 & 0x00400000) && (data.note !== undefined)) motoring.note.write(data.note);
	if((map2 & 0x10000000) && (data.sound !== undefined)) motoring.sound.write(data.sound);
	if((map1 & 0x00200000) && (data.lineTracerMode !== undefined)) motoring.lineTracerMode.write(data.lineTracerMode);
	if((map2 & 0x08000000) && (data.lineTracerGain !== undefined)) motoring.lineTracerGain.write(data.lineTracerGain);
	if((map1 & 0x00100000) && (data.lineTracerSpeed !== undefined)) motoring.lineTracerSpeed.write(data.lineTracerSpeed);
	if((map1 & 0x00080000) && (data.ioModeA !== undefined)) motoring.ioModeA.write(data.ioModeA);
	if((map1 & 0x00040000) && (data.ioModeB !== undefined)) motoring.ioModeB.write(data.ioModeB);
	if((map2 & 0x04000000) && (data.serial !== undefined)) motoring.writeSerial.write(data.serial);
	if(map2 & 0x02000000) {
		if(data.motionType !== undefined) {
			var motionType = data.motionType;
			var value = 0;
			if(motionType == 101) {
				motionType = 1;
				value = context.getMovementDistance(MODULE);
			} else if(motionType == 102) {
				motionType = 2;
				value = context.getMovementDistance(MODULE);
			} else if(motionType == 103) {
				motionType = 3;
				value = context.getTurningAngle(MODULE);
			} else if(motionType == 104) {
				motionType = 4;
				value = context.getTurningAngle(MODULE);
			} else if(data.motionValue != undefined) {
				value = data.motionValue;
			}
			motoring.motion.write(0, motionType);
			if(data.motionUnit !== undefined) motoring.motion.write(1, data.motionUnit);
			if(data.motionSpeed !== undefined) motoring.motion.write(2, data.motionSpeed);
			if(data.motionValue !== undefined) motoring.motion.write(3, value);
			if(data.motionRadius !== undefined) motoring.motion.write(4, data.motionRadius);
		}
	}
	if((map1 & 0x00020000) && (data.configProximity !== undefined)) motoring.configProximity.write(data.configProximity);
	if((map1 & 0x00010000) && (data.configGravity !== undefined)) motoring.configGravity.write(data.configGravity);
	if((map1 & 0x00008000) && (data.configBandWidth !== undefined)) motoring.configBandWidth.write(data.configBandWidth);
};

Component.prototype.handleSensory = function(robot, data, state, connection) {
	if(connection) {
		var sensory = robot.sensory;
		sensory.connectionState = router.getConnectionState(state);
		sensory.map1 = data.map1;
		sensory.map2 = data.map2;
		sensory.signalStrength = data.signalStrength;
		sensory.leftProximity = data.leftProximity;
		sensory.rightProximity = data.rightProximity;
		sensory.leftFloor = data.leftFloor;
		sensory.rightFloor = data.rightFloor;
		sensory.accelerationX = data.acceleration.x;
		sensory.accelerationY = data.acceleration.y;
		sensory.accelerationZ = data.acceleration.z;
		sensory.light = data.light;
		sensory.temperature = data.temperature;
		sensory.inputA = data.inputA;
		sensory.inputB = data.inputB;
		if(data.map1 & 0x00000008) sensory.tilt = data.tilt;
		if(data.map2 & 0x00002000) sensory.serial = data.serial;
		if(data.map2 & 0x00001000) sensory.pulseCount = data.pulseCount;
		if(data.map2 & 0x00000800) sensory.wheelState = data.wheelState;
		if(data.map2 & 0x00000400) sensory.soundState = data.soundState;
		if(data.map1 & 0x00000010) sensory.lineTracerState = data.lineTracerState;
		if(data.map1 & 0x00000004) sensory.batteryState = data.batteryState;
		
		var threshold = context.getHandFoundThreshold(MODULE);
		sensory.handFound = sensory.leftProximity > threshold || sensory.rightProximity > threshold;
		try {
			connection.send(JSON.stringify(sensory));
		} catch (e) {
		}
	}
};

module.exports = new Component();