'use strict';
var router = require('router');
var translator = require('translator');
var component = require('./component');
const MODULE = 'turtle';

function Extension() {
	this.robot = component.createRobot(0);
}

Extension.prototype.start = function(express) {
	var self = this;
	var robot = self.robot;
	var problemMessage = '';
	
	self.running = true;
	robot.clearWaits();
	var setup = function() {
		problemMessage = '_problem ' + translator.translate('Robot is not connected.') + '\n';
		component.updateLang(translator);
	};
	translator.on('turtle-scratch', function() {
		setup();
	});
	setup();
	
	if(express) {
		const HEADER = { 'Content-Type': 'text/plain' };
		
		express.get('/reset_all', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearWaits();
			robot.clearTimeouts();
			robot.reset();
			router.reset();
			response.end();
		});
		
		express.get('/poll', function(request, response) {
			if(!self.running) {
				response.set('Connection', 'close');
			}
			response.writeHead(200, HEADER);
			var waits = robot.getWaits();
			if(waits.length > 0) {
				response.write('_busy ' + waits.join(' ') + '\n');
			}
			if(self.state == 'connected') {
				robot.writeSingleSensory(response);
			} else {
				response.write(problemMessage);
			}
			response.end();
		});
		
		express.get('/turtleMoveForward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveForward(request);
			response.end();
		});
		
		express.get('/turtleMoveBackward/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveBackward(request);
			response.end();
		});
		
		express.get('/turtleTurn/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turn(request);
			response.end();
		});
		
		express.get('/turtleMoveForwardUnit/:id/:value/:unit', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveForwardUnit(request);
			response.end();
		});

		express.get('/turtleMoveBackwardUnit/:id/:value/:unit', function(request, response) {
			response.writeHead(200, HEADER);
			robot.moveBackwardUnit(request);
			response.end();
		});

		express.get('/turtleTurnUnitInPlace/:id/:direction/:value/:unit', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnUnitInPlace(request);
			response.end();
		});
		
		express.get('/turtleTurnUnitWithRadiusInDirection/:id/:direction/:value/:unit/:radius/:head', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnUnitWithRadiusInDirection(request);
			response.end();
		});
		
		express.get('/turtlePivotAroundWheelUnitInDirection/:id/:wheel/:value/:unit/:head', function(request, response) {
			response.writeHead(200, HEADER);
			robot.pivotAroundWheelUnitInDirection(request);
			response.end();
		});
		
		express.get('/turtleChangeWheelsByLeftRight/:left/:right', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeWheelsByLeftRight(request);
			response.end();
		});
		
		express.get('/turtleSetWheelsToLeftRight/:left/:right', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setWheelsToLeftRight(request);
			response.end();
		});
		
		express.get('/turtleChangeWheelBy/:wheel/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeWheelBy(request);
			response.end();
		});
		
		express.get('/turtleSetWheelTo/:wheel/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setWheelTo(request);
			response.end();
		});
		
		express.get('/turtleFollowLine/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.followLine(request);
			response.end();
		});
		
		express.get('/turtleFollowLineUntil/:id/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.followLineUntil(request);
			response.end();
		});
		
		express.get('/turtleFollowLineUntilBlack/:id/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.followLineUntilBlack(request);
			response.end();
		});
		
		express.get('/turtleCrossIntersection/:id', function(request, response) {
			response.writeHead(200, HEADER);
			robot.crossIntersection(request);
			response.end();
		});
		
		express.get('/turtleTurnAtIntersection/:id/:direction', function(request, response) {
			response.writeHead(200, HEADER);
			robot.turnAtIntersection(request);
			response.end();
		});
		
		express.get('/turtleSetFollowingSpeedTo/:speed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setFollowingSpeedTo(request);
			response.end();
		});
		
		express.get('/turtleStop', function(request, response) {
			response.writeHead(200, HEADER);
			robot.stop(request);
			response.end();
		});
		
		express.get('/turtleSetHeadLedTo/:color', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setHeadLedTo(request);
			response.end();
		});
		
		express.get('/turtleChangeHeadLedByRGB/:red/:green/:blue', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeHeadLedByRGB(request);
			response.end();
		});
		
		express.get('/turtleSetHeadLedToRGB/:red/:green/:blue', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setHeadLedToRGB(request);
			response.end();
		});
		
		express.get('/turtleClearHeadLed', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearHeadLed(request);
			response.end();
		});
		
		express.get('/turtlePlaySound/:sound', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSound(request);
			response.end();
		});
		
		express.get('/turtlePlaySoundTimes/:sound/:count', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSoundTimes(request);
			response.end();
		});
		
		express.get('/turtlePlaySoundTimesUntilDone/:id/:sound/:count', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playSoundTimesUntilDone(request);
			response.end();
		});
		
		express.get('/turtleChangeBuzzerBy/:hz', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeBuzzerBy(request);
			response.end();
		});
		
		express.get('/turtleSetBuzzerTo/:hz', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setBuzzerTo(request);
			response.end();
		});
		
		express.get('/turtleClearSound', function(request, response) {
			response.writeHead(200, HEADER);
			robot.clearSound(request);
			response.end();
		});
		
		express.get('/turtlePlayNote/:note/:octave', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNote(request);
			response.end();
		});
		
		express.get('/turtlePlayNoteForBeats/:id/:note/:octave/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.playNoteForBeats(request);
			response.end();
		});
		
		express.get('/turtleRestForBeats/:id/:beat', function(request, response) {
			response.writeHead(200, HEADER);
			robot.restForBeats(request);
			response.end();
		});
		
		express.get('/turtleChangeTempoBy/:bpm', function(request, response) {
			response.writeHead(200, HEADER);
			robot.changeTempoBy(request);
			response.end();
		});
		
		express.get('/turtleSetTempoTo/:bpm', function(request, response) {
			response.writeHead(200, HEADER);
			robot.setTempoTo(request);
			response.end();
		});
	}
};

Extension.prototype.stop = function() {
	this.running = false;
	translator.remove('turtle-scratch');
	router.reset();
};

Extension.prototype.setState = function(route) {
	if(route == router.selectedRoute && route.module == MODULE) {
		this.state = route.state;
	}
};

Extension.prototype.setOption = function(option) {
};

Extension.prototype.handleSensory = function(data, route) {
	if(route.module == MODULE) {
		this.robot.handleSensory(data);
	}
};

Extension.prototype.handleMotoring = function(data, route) {
	if(route.module == MODULE) {
		var roboid = this.robot.roboid;
		roboid.handleMotoring(data);
		roboid.updateMotoring();
	}
};

Extension.prototype.requestMotoring = function(route) {
	if(route.module == MODULE) {
		return this.robot.roboid.requestMotoring();
	}
};

Extension.prototype.reset = function(route) {
	if(route == router.selectedRoute && route.module == MODULE) {
		this.robot.roboid.reset();
		this.robot.motoring.reset();
	}
};

module.exports.create = function() {
	return new Extension();
};