'use strict';
var Turtle = require('./turtle');

function TurtleRoboid(parent) {
	var io = require('roboid.io');
	var Device = io.Device;
	var DeviceType = io.DeviceType;
	var DataType = io.DataType;
	
	var sensory = {
		devices: [],
		readData: [],
		callbacks: []
	};
	var motoring = {
		map: 0,
		devices: [],
		readData: [],
		writeData: [],
		callbacks: []
	};
	var writeMotoringFlag = 0;
	
	this.bind = function(device, type, data, callback) {
		if(type == DeviceType.SENSOR || type == DeviceType.EVENT) {
			sensory.devices.push(device);
			sensory.readData.push(data.read);
			sensory.callbacks.push(callback);
		} else {
			motoring.devices.push(device);
			motoring.readData.push(data.read);
			motoring.writeData.push(data.write);
			motoring.callbacks.push(callback);
		}
	};
	
	var devices = {};
	devices[Turtle.LEFT_WHEEL] = new Device(this, Turtle.LEFT_WHEEL, 'LeftWheel', DeviceType.EFFECTOR, DataType.FLOAT, 1, -400, 400, 0);
	devices[Turtle.RIGHT_WHEEL] = new Device(this, Turtle.RIGHT_WHEEL, 'RightWheel', DeviceType.EFFECTOR, DataType.FLOAT, 1, -400, 400, 0);
	devices[Turtle.LED] = new Device(this, Turtle.LED, 'Led', DeviceType.EFFECTOR, DataType.INTEGER, 3, 0, 255, 0);
	devices[Turtle.BUZZER] = new Device(this, Turtle.BUZZER, 'Buzzer', DeviceType.EFFECTOR, DataType.FLOAT, 1, 0, 167772.15, 0);
	devices[Turtle.PULSE] = new Device(this, Turtle.PULSE, 'Pulse', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 65535, 0);
	devices[Turtle.NOTE] = new Device(this, Turtle.NOTE, 'Note', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 88, 0);
	devices[Turtle.SOUND] = new Device(this, Turtle.SOUND, 'Sound', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 127, 0);
	devices[Turtle.LINE_TRACER_MODE] = new Device(this, Turtle.LINE_TRACER_MODE, 'LineTracerMode', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 127, 0);
	devices[Turtle.LINE_TRACER_GAIN] = new Device(this, Turtle.LINE_TRACER_GAIN, 'LineTracerGain', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 8, 5);
	devices[Turtle.LINE_TRACER_SPEED] = new Device(this, Turtle.LINE_TRACER_SPEED, 'LineTracerSpeed', DeviceType.COMMAND, DataType.INTEGER, 1, 1, 8, 5);
	devices[Turtle.LAMP] = new Device(this, Turtle.LAMP, 'Lamp', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 1);
	devices[Turtle.LOCK] = new Device(this, Turtle.LOCK, 'Lock', DeviceType.COMMAND, DataType.INTEGER, 1, 0, 1, 0);
	devices[Turtle.MOTION] = new Device(this, Turtle.MOTION, 'Motion', DeviceType.COMMAND, DataType.FLOAT, 5, 0, 2147483647, 0);
	devices[Turtle.SIGNAL_STRENGTH] = new Device(this, Turtle.SIGNAL_STRENGTH, 'SignalStrength', DeviceType.SENSOR, DataType.INTEGER, 1, -128, 0, 0);
	devices[Turtle.COLOR] = new Device(this, Turtle.COLOR, 'Color', DeviceType.SENSOR, DataType.INTEGER, 4, 0, 255, 0);
	devices[Turtle.FLOOR] = new Device(this, Turtle.FLOOR, 'Floor', DeviceType.SENSOR, DataType.INTEGER, 1, 0, 255, 0);
	devices[Turtle.ACCELERATION] = new Device(this, Turtle.ACCELERATION, 'Acceleration', DeviceType.SENSOR, DataType.INTEGER, 3, -32768, 32767, 0);
	devices[Turtle.TEMPERATURE] = new Device(this, Turtle.TEMPERATURE, 'Temperature', DeviceType.SENSOR, DataType.INTEGER, 1, -40, 88, 0);
	devices[Turtle.BUTTON] = new Device(this, Turtle.BUTTON, 'Button', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[Turtle.CLICKED] = new Device(this, Turtle.CLICKED, 'Clicked', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[Turtle.DOUBLE_CLICKED] = new Device(this, Turtle.DOUBLE_CLICKED, 'DoubleClicked', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[Turtle.LONG_PRESSED] = new Device(this, Turtle.LONG_PRESSED, 'LongPressed', DeviceType.EVENT, DataType.INTEGER, 0, 0, 0, 0);
	devices[Turtle.COLOR_NUMBER] = new Device(this, Turtle.COLOR_NUMBER, 'ColorNumber', DeviceType.EVENT, DataType.INTEGER, 1, -1, 8, -1);
	devices[Turtle.COLOR_PATTERN] = new Device(this, Turtle.COLOR_PATTERN, 'ColorPattern', DeviceType.EVENT, DataType.INTEGER, 1, -1, 88, -1);
	devices[Turtle.PULSE_COUNT] = new Device(this, Turtle.PULSE_COUNT, 'PulseCount', DeviceType.EVENT, DataType.INTEGER, 1, 0, 65535, 0);
	devices[Turtle.WHEEL_STATE] = new Device(this, Turtle.WHEEL_STATE, 'WheelState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[Turtle.SOUND_STATE] = new Device(this, Turtle.SOUND_STATE, 'SoundState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 1, 0);
	devices[Turtle.LINE_TRACER_STATE] = new Device(this, Turtle.LINE_TRACER_STATE, 'LineTracerState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 3, 0);
	devices[Turtle.TILT] = new Device(this, Turtle.TILT, 'Tilt', DeviceType.EVENT, DataType.INTEGER, 1, -3, 3, 0);
	devices[Turtle.BATTERY_STATE] = new Device(this, Turtle.BATTERY_STATE, 'BatteryState', DeviceType.EVENT, DataType.INTEGER, 1, 0, 2, 2);
	
	this.findDeviceByName = function(name) {
		name = '' + name;
		name = name.toLowerCase();
		var dot = name.search('\\.');
		if(dot < 0) {
			for(var i in devices) {
				if(name == devices[i].getName().toLowerCase())
					return devices[i];
			}
		}
		return undefined;
	};
	
	this.findDeviceById = function(deviceId) {
		var device = devices[deviceId];
		if(device) {
			return device;
		}
		return undefined;
	};
	
	this.addDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].addDeviceDataChangedListener(listener);
		}
	};
	
	this.removeDeviceDataChangedListener = function(listener) {
		for(var i in devices) {
			devices[i].removeDeviceDataChangedListener(listener);
		}
	};
	
	this.clearDeviceDataChangedListeners = function() {
		for(var i in devices) {
			devices[i].clearDeviceDataChangedListeners();
		}
	};
	
	this.onMotoringDeviceWritten = function(device) {
		switch(device.getId()) {
			case Turtle.PULSE:
				writeMotoringFlag |= 0x04000000;
				break;
			case Turtle.NOTE:
				writeMotoringFlag |= 0x02000000;
				break;
			case Turtle.SOUND:
				writeMotoringFlag |= 0x01000000;
				break;
			case Turtle.LINE_TRACER_MODE:
				writeMotoringFlag |= 0x00800000;
				break;
			case Turtle.LINE_TRACER_GAIN:
				writeMotoringFlag |= 0x00400000;
				break;
			case Turtle.LINE_TRACER_SPEED:
				writeMotoringFlag |= 0x00200000;
				break;
			case Turtle.LAMP:
				writeMotoringFlag |= 0x00100000;
				break;
			case Turtle.LOCK:
				writeMotoringFlag |= 0x00080000;
				break;
			case Turtle.MOTION:
				writeMotoringFlag |= 0x00040000;
				break;
		}
	};
	
	this.handleSensory = function(data) {
		var map = data.map;
		var readData = sensory.readData;
		readData[0][0] = data.signalStrength;
		readData[1][0] = data.color.r;
		readData[1][1] = data.color.g;
		readData[1][2] = data.color.b;
		readData[1][3] = data.color.c;
		readData[2][0] = data.floor;
		readData[3][0] = data.acceleration.x;
		readData[3][1] = data.acceleration.y;
		readData[3][2] = data.acceleration.z;
		readData[4][0] = data.temperature;
		
		var callbacks = sensory.callbacks;
		var i;
		for(i = 0; i < 5; ++i) {
			callbacks[i].fire();
		}
		
		if(map & 0x00001000) {
			readData[5][0] = data.button;
			callbacks[5].fire();
		}
		if(map & 0x00000800) {
			callbacks[6].fire();
		}
		if(map & 0x00000400) {
			callbacks[7].fire();
		}
		if(map & 0x00000200) {
			callbacks[8].fire();
		}
		if(map & 0x00000100) {
			readData[9][0] = data.colorNumber;
			callbacks[9].fire();
		}
		if(map & 0x00000080) {
			readData[10][0] = data.colorPattern;
			callbacks[10].fire();
		}
		if(map & 0x00000040) {
			readData[11][0] = data.pulseCount;
			callbacks[11].fire();
		}
		if(map & 0x00000020) {
			readData[12][0] = data.wheelState;
			callbacks[12].fire();
		}
		if(map & 0x00000010) {
			readData[13][0] = data.soundState;
			callbacks[13].fire();
		}
		if(map & 0x00000008) {
			readData[14][0] = data.lineTracerState;
			callbacks[14].fire();
		}
		if(map & 0x00000004) {
			readData[15][0] = data.tilt;
			callbacks[15].fire();
		}
		if(map & 0x00000002) {
			readData[16][0] = data.batteryState;
			callbacks[16].fire();
		}
		
		for(i = 0; i < 5; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map & 0x00001000) {
			callbacks[5].notifyDeviceDataChanged();
		}
		if(map & 0x00000800) {
			callbacks[6].notifyDeviceDataChanged();
		}
		if(map & 0x00000400) {
			callbacks[7].notifyDeviceDataChanged();
		}
		if(map & 0x00000200) {
			callbacks[8].notifyDeviceDataChanged();
		}
		if(map & 0x00000100) {
			callbacks[9].notifyDeviceDataChanged();
		}
		if(map & 0x00000080) {
			callbacks[10].notifyDeviceDataChanged();
		}
		if(map & 0x00000040) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map & 0x00000020) {
			callbacks[12].notifyDeviceDataChanged();
		}
		if(map & 0x00000010) {
			callbacks[13].notifyDeviceDataChanged();
		}
		if(map & 0x00000008) {
			callbacks[14].notifyDeviceDataChanged();
		}
		if(map & 0x00000004) {
			callbacks[15].notifyDeviceDataChanged();
		}
		if(map & 0x00000002) {
			callbacks[16].notifyDeviceDataChanged();
		}
	};
	
	this.handleMotoring = function(data) {
		var map = data.map;
		var readData = motoring.readData;
		readData[0][0] = data.leftWheel;
		readData[1][0] = data.rightWheel;
		readData[2][0] = data.led.r;
		readData[2][1] = data.led.g;
		readData[2][2] = data.led.b;
		readData[3][0] = data.buzzer;
		
		var callbacks = motoring.callbacks;
		var i;
		for(i = 0; i < 4; ++i) {
			callbacks[i].fire();
		}
		
		if(map & 0x04000000) {
			readData[4][0] = data.pulse;
			callbacks[4].fire();
		}
		if(map & 0x02000000) {
			readData[5][0] = data.note;
			callbacks[5].fire();
		}
		if(map & 0x01000000) {
			readData[6][0] = data.sound;
			callbacks[6].fire();
		}
		if(map & 0x00800000) {
			readData[7][0] = data.lineTracerMode;
			callbacks[7].fire();
		}
		if(map & 0x00400000) {
			readData[8][0] = data.lineTracerGain;
			callbacks[8].fire();
		}
		if(map & 0x00200000) {
			readData[9][0] = data.lineTracerSpeed;
			callbacks[9].fire();
		}
		if(map & 0x00100000) {
			readData[10][0] = data.lamp;
			callbacks[10].fire();
		}
		if(map & 0x00080000) {
			readData[11][0] = data.lock;
			callbacks[11].fire();
		}
		if(map & 0x00040000) {
			readData[12][0] = data.motion.type;
			readData[12][1] = data.motion.unit;
			readData[12][2] = data.motion.speed;
			readData[12][3] = data.motion.value;
			readData[12][4] = data.motion.radius;
			callbacks[12].fire();
		}
		
		for(i = 0; i < 4; ++i) {
			callbacks[i].notifyDeviceDataChanged();
		}
		
		if(map & 0x04000000) {
			callbacks[4].notifyDeviceDataChanged();
		}
		if(map & 0x02000000) {
			callbacks[5].notifyDeviceDataChanged();
		}
		if(map & 0x01000000) {
			callbacks[6].notifyDeviceDataChanged();
		}
		if(map & 0x00800000) {
			callbacks[7].notifyDeviceDataChanged();
		}
		if(map & 0x00400000) {
			callbacks[8].notifyDeviceDataChanged();
		}
		if(map & 0x00200000) {
			callbacks[9].notifyDeviceDataChanged();
		}
		if(map & 0x00100000) {
			callbacks[10].notifyDeviceDataChanged();
		}
		if(map & 0x00080000) {
			callbacks[11].notifyDeviceDataChanged();
		}
		if(map & 0x00040000) {
			callbacks[12].notifyDeviceDataChanged();
		}
	};
	
	this.requestMotoring = function() {
		motoring.map = 0xf8000000 | writeMotoringFlag;
		writeMotoringFlag = 0;
		return motoring;
	};
	
	this.updateSensory = function() {
		var callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.updateMotoring = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].update();
		}
	};
	
	this.reset = function() {
		var callbacks = motoring.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
		callbacks = sensory.callbacks;
		for(var i in callbacks) {
			callbacks[i].reset();
		}
	};
}

TurtleRoboid.prototype.getId = function() {
	return Turtle.ID;
};

TurtleRoboid.prototype.getName = function() {
	return 'Turtle';
};

module.exports.create = function(parent) {
	return new TurtleRoboid(parent);
};