'use strict';
var util = require('util');
var EventEmitter = require('events').EventEmitter;
var logger = require('logger').get();

function Chat() {
	EventEmitter.call(this);
	this.webSocketServer = undefined;
	this.httpServer = undefined;
	this.connections = {};
}

util.inherits(Chat, EventEmitter);

Chat.prototype.start = function(port) {
	var self = this;
	self.stop();
	if(typeof port == 'number') {
		var WebSocketServer = require('websocket').server;
		var http = require('http');
		var httpServer = http.createServer(function(request, response) {
			response.writeHead(200);
			response.end();
		});
		self.httpServer = httpServer;
		httpServer.listen(port, function() {
			logger.i('Chatting on port ' + port);
			self.emit('start');
		});
		var webSocketServer = new WebSocketServer({
			httpServer: httpServer,
			autoAcceptConnections: false
		});
		self.webSocketServer = webSocketServer;
		webSocketServer.on('request', function(request) {
			var connection = request.accept();
			connection.on('close', function(reasonCode, description) {
				self.remove(connection);
			});
			connection.on('message', function(message) {
				if(message.type === 'utf8') {
					try {
						var data = JSON.parse(message.utf8Data);
						if(data.command === 'register') {
							if(data.name !== undefined) {
								self.connections[data.name] = connection;
								self.emit('participants', self.connections);
							}
						} else if(data.command === 'send') {
							if(data.to !== undefined && data.message !== undefined) {
								var conn = self.connections[data.to];
								if(conn) {
									conn.send(message.utf8Data);
								}
							}
						} else if(data.command === 'broadcast') {
							if(data.message !== undefined) {
								var connections = self.connections;
								for(var i in connections) {
									connections[i].send(message.utf8Data);
								}
							}
						}
					} catch (e) {
					}
				}
			});
		});
	}
};

Chat.prototype.stop = function() {
	if(this.webSocketServer) {
		this.webSocketServer.shutDown();
		this.webSocketServer = undefined;
	}
	if(this.httpServer) {
		this.httpServer.removeAllListeners();
		this.httpServer.close();
		this.httpServer = undefined;
	}
	this.connections = {};
};

Chat.prototype.remove = function(connection) {
	var connections = this.connections;
	for(var i in connections) {
		if(connections[i] == connection) {
			delete connections[i];
			break;
		}
	}
	this.emit('participants', connections);
};

Chat.prototype.getIps = function() {
	var interfaces = require('os').networkInterfaces();
	var ips = '';
	for(var devName in interfaces) {
		var iface = interfaces[devName];
		var alias;
		for(var i = 0; i < iface.length; ++i) {
			alias = iface[i];
			if(alias.family === 'IPv4' && alias.address !== '127.0.0.1' && !alias.internal) {
				if(ips) ips += '<br/>';
				ips += alias.address;
			}
		}
	}
	return ips;
};

module.exports = new Chat();