'use strict';

function Context() {
	this.modules = {};
	this.components = {};
	this.settings = {
		hamster: {
			movementSpeed: 30,
			turningSpeed: 30,
			handFoundThreshold: 50
		},
		hamsterS: {
			movementSpeed: 30,
			movementDistance: 5,
			turningSpeed: 30,
			turningAngle: 90,
			handFoundThreshold: 50
		},
		turtle: {
			movementSpeed: 50,
			movementDistance: 6,
			turningSpeed: 50,
			turningAngle: 90
		}
	};
	this.ranges = {
		hamster: {
			movementSpeed: [0, 100],
			turningSpeed: [0, 100],
			handFoundThreshold: [0, 50]
		},
		hamsterS: {
			movementSpeed: [0, 100],
			turningSpeed: [0, 100],
			handFoundThreshold: [0, 50]
		},
		turtle: {
			movementSpeed: [0, 400],
			turningSpeed: [0, 400]
		}
	};
}

Context.prototype.register = function(config) {
	this.modules[config.model] = config.module;
};

Context.prototype.model2module = function(model) {
	return this.modules[model];
};

Context.prototype.getModules = function() {
	return this.modules;
};

Context.prototype.getComponents = function(program) {
	var modules = this.modules;
	var results = [], component;
	for(var i in modules) {
		component = this.getComponentByModule(program, modules[i]);
		if(component) results.push(component);
	}
	return results;
};

Context.prototype.getComponentByModule = function(program, module) {
	var key = module + '/' + program;
	var component = this.components[key];
	if(!component) {
		component = require('../../modules/' + key + '/component');
		component.module = module;
		this.components[key] = component;
	}
	return component;
};

Context.prototype.getComponentByModel = function(program, model) {
	return this.getComponentByModule(program, this.model2module(model));
};

Context.prototype.getComponentByRoute = function(program, route) {
	return this.getComponentByModule(program, route.module);
};

Context.prototype.getSettings = function() {
	return this.settings;
};

Context.prototype.setSetting = function(module, key, value) {
	value = parseFloat(value);
	if(typeof value == 'number') {
		var setting = this.settings[module];
		var range = this.ranges[module];
		if(setting && range) {
			if(setting[key] !== undefined) {
				var minmax = range[key];
				if(minmax) {
					if(value < minmax[0]) value = minmax[0];
					else if(value > minmax[1]) value = minmax[1];
				} else if(value < 0) {
					value = 0;
				}
				setting[key] = value;
			}
		}
	}
};

Context.prototype.getMovementSpeed = function(module) {
	var setting = this.settings[module];
	if(setting) {
		return setting.movementSpeed;
	}
};

Context.prototype.getMovementDistance = function(module) {
	var setting = this.settings[module];
	if(setting) {
		return setting.movementDistance;
	}
};

Context.prototype.getTurningSpeed = function(module) {
	var setting = this.settings[module];
	if(setting) {
		return setting.turningSpeed;
	}
};

Context.prototype.getTurningAngle = function(module) {
	var setting = this.settings[module];
	if(setting) {
		return setting.turningAngle;
	}
};

Context.prototype.getHandFoundThreshold = function(module) {
	var setting = this.settings[module];
	if(setting) {
		return setting.handFoundThreshold;
	}
};

Context.prototype.saveSettings = function() {
	var fs = require('fs');
	try {
		var data = JSON.stringify(this.settings);
		fs.writeFileSync(process.cwd() + '/modules/setting.cfg', data);
	} catch(e) {}
};

module.exports = new Context();