'use strict';
function Scanner() {
}

Scanner.prototype.startScan = function(router, configs, callback) {
	var logger = require('logger').get();
	var self = this;
	
	logger.i('scanning...');
	
	self.configs = {
		serial: [],
		bluetooth: [],
		ble: []
	};
	for(var i = 0, len = configs.length; i < len; ++i) {
		var config = configs[i];
		switch(config.type) {
			case 'serial': {
				self.configs.serial.push(config);
				break;
			}
			case 'bluetooth': {
				self.configs.bluetooth.push(config);
				break;
			}
			case 'ble': {
				self.configs.ble.push(config);
				break;
			}
		}
	};
	
	self.scanners = {};
	self.done = false;
	self.scanners.serial = require('./serial');
	self.scanners.serial.startScan(self.configs.serial, function(error, connector, config, property) {
		if(error) {
			logger.e(error);
		} else {
			var controller = require('../../../modules/' + config.module + '/controller').create();
			var route = router.found(controller, config, property);
			controller.load(connector, route, function() {
				router.connect(connector, route);
			});
			if(self.done == false) {
				self.done = true;
				if(callback) {
					callback('done');
				}
			}
		}
	});
};

Scanner.prototype.stopScan = function() {
	var logger = require('logger').get();
	var scanners = this.scanners;
	if(scanners) {
		for(var key in scanners) {
			scanners[key].stopScan();
		}
		this.scanners = undefined;
	}
	logger.i('scanning canceled');
};

module.exports = new Scanner();