'use strict';
function Scanner() {
}

Scanner.prototype.startScan = function(configs, callback) {
	var serialport = require('serialport');
	var self = this;

	self.scanning = true;
	self.found = false;
	self.connectors = {};
	self.scanCount = 0;
	self.closeConnectors();
	self.clearTimer();
	self.scan(serialport, configs, callback);
	self.timer = setInterval(function() {
		self.scan(serialport, configs, callback);
	}, 1000);
};

Scanner.prototype.stopScan = function() {
	this.scanning = false;
	this.clearTimer();
	this.closeConnectors();
};

Scanner.prototype.scan = function(serialport, configs, callback) {
	var self = this;

	if(!self.scanning) return;
	serialport.list(function(error, devices) {
		if(error) {
			if(callback) {
				callback(error);
			}
			return;
		}
		
		var vendor = 'Silicon Lab';
		if(self.scanCount < 5) self.scanCount ++;
		else {
			if(devices.some(function(device) {
				return device.manufacturer && device.manufacturer.indexOf(vendor) != -1;
			}) == false) {
				vendor = undefined;
			}
		}

		if(!self.scanning) return;
		devices.forEach(function(device) {
			if(!vendor || (device.manufacturer && device.manufacturer.indexOf(vendor) != -1)) {
				var comName = device.comName;
				var connector = self.connectors[comName];
				if(connector == undefined) {
					connector = require('../connector/serial').create();
					connector.open(comName, function(error, sp) {
						if(error) {
							if(callback) {
								callback(error);
							}
						} else {
							self.connectors[comName] = connector;
							sp.on('data', function(data) {
								if(data.slice(0, 2) == 'FF') {
									self.validate(connector, configs, data, function(result, config, property) {
										if(result) {
											sp.removeAllListeners('data');
											config.vendor = device.manufacturer;
											self.connectors[comName] = undefined;
											self.found = true;
											if(callback) {
												property.port = comName;
												property.dongle = {};
												property.dongle.canReset = (data.slice(2, 4) != '00');
												callback(null, connector, config, property);
											}
										}
									});
								} else {
									connector.send('FF\r');
								}
							});
						}
					});
				}
			}
		});
	});
	if(self.found) {
		self.stopScan();
	}
};

Scanner.prototype.clearTimer = function() {
	if(this.timer) {
		clearInterval(this.timer);
		this.timer = undefined;
	}
};

Scanner.prototype.closeConnectors = function() {
	var connectors = this.connectors;
	if(connectors) {
		var connector;
		for(var key in connectors) {
			connector = connectors[key];
			if(connector) {
				connector.close();
			}
		}
	}
	this.connectors = {};
};

Scanner.prototype.validate = function(connector, configs, data, callback) {
	if(callback) {
		var info = data.split(',');
		if(info && info.length >= 5) {
			for(var i = 0, len = configs.length; i < len; ++i) {
				var config = configs[i];
				var property = connector.validateInfo(info, config);
				if(property) {
					callback(true, config, property);
					return;
				}
			}
			callback(false);
		}
	}
};

module.exports = new Scanner();