'use strict';
var router = require('router');

function Server() {
}

Server.prototype.start = function(path, port, option, callback) {
	var http = require('http');
	var fs = require('fs');
	var logger = require('logger').get();
	var self = this;
	
	var httpServer = http.createServer(function(request, response) {
		response.writeHead(200);
		response.end();
	});
	httpServer.on('listening', function(e) {
		self.httpServer = httpServer;
		logger.i('Listening on port ' + port);
		
		var server = require('socket.io')(httpServer);
		server.set('transports', ['websocket',
			'flashsocket',
			'htmlfile',
			'xhr-polling',
			'jsonp-polling',
			'polling']);
		
		self.server = server;
		server.on('connection', function(socket) {
			var connection = socket;
			if(callback) callback('connected');
			
			connection.on('disconnect', function(socket) {
				if(callback) callback('disconnected');
				if(self.extension) self.extension.disconnected(this);
			});
			
			connection.on('close', function(reasonCode, description) {
				if(callback) callback('closed');
				if(self.extension) self.extension.closed(this);
			});
			
			var extension = require(path).create();
			if(extension) {
				self.extension = extension;
				extension.start(server, connection);
				extension.setOption(option);
				var route = router.getRepresentativeRoute();
				if(route) extension.setState(route);
				router.setExtension(extension);
			}
		});
	});
	httpServer.listen(port);
};

Server.prototype.stop = function() {
	if(this.extension) {
		this.extension.close();
	}
	if(this.server) {
		this.server.close();
		this.server = undefined;
	}
	if(this.httpServer) {
		this.httpServer.removeAllListeners();
		this.httpServer.close();
		this.httpServer = undefined;
	}
	if(this.extension) {
		router.setExtension(undefined);
		this.extension.stop();
		this.extension = undefined;
	}
};

Server.prototype.setState = function(route) {
	if(this.extension) {
		this.extension.setState(route);
	}
};

Server.prototype.setOption = function(option) {
	if(this.extension) {
		this.extension.setOption(option);
	}
};

module.exports = new Server();