'use strict';
var router = require('router');

function Server() {
}

Server.prototype.start = function(path, port, option, callback) {
	var version = '1.6.1';
	var http = require('http');
	var url = 'http://hamster.school/entry/version.txt';
	http.get(url, function(res) {
		var data = '';
		res.on('data', function (chunk) {
			data += chunk;
		});
		res.on('end', function() {
			version = data;
		});
	}).on('error', function() {
	});
	
	var https = require('https');
	var fs = require('fs');
	var logger = require('logger').get();
	var self = this;

	var base = './node_modules/server/entry/';
	var httpServer = https.createServer({
		key: fs.readFileSync(base + '1.a'),
		cert: fs.readFileSync(base + '2.b'),
		ca: [fs.readFileSync(base + '3.c'), fs.readFileSync(base + '4.d'), fs.readFileSync(base + '5.e')]
	}, function(request, response) {
		response.writeHead(200);
		response.end();
	});
	httpServer.on('listening', function(e) {
		self.httpServer = httpServer;
		logger.i('Listening on port ' + port);
		
		var server = require('socket.io')(httpServer);
		server.set('transports', ['websocket',
		    'flashsocket',
	      	'htmlfile',
	      	'xhr-polling',
	      	'jsonp-polling',
	      	'polling']);

		self.server = server;
		server.on('connection', function(socket) {
			var connection = socket;
			if(callback) callback('connected');
			
			connection.on('disconnect', function(socket) {
				if(callback) callback('disconnected');
				if(self.extension) self.extension.disconnected(this);
			});
			
			connection.on('close', function(reasonCode, description) {
				if(callback) callback('closed');
				if(self.extension) self.extension.closed(this);
			});
			
			var extension = require(path).create();
			if(extension) {
				self.extension = extension;
				extension.start(server, connection, version);
				extension.setOption(option);
				var route = router.getRepresentativeRoute();
				if(route) extension.setState(route);
				router.setExtension(extension);
			}
		});
	});
	httpServer.listen(port);
};

Server.prototype.stop = function() {
	if(this.extension) {
		this.extension.close();
	}
	if(this.server) {
		this.server.close();
		this.server = undefined;
	}
	if(this.httpServer) {
		this.httpServer.removeAllListeners();
		this.httpServer.close();
		this.httpServer = undefined;
	}
	if(this.extension) {
		router.setExtension(undefined);
		this.extension.stop();
		this.extension = undefined;
	}
};

Server.prototype.setState = function(route) {
	if(this.extension) {
		this.extension.setState(route);
	}
};

Server.prototype.setOption = function(option) {
	if(this.extension) {
		this.extension.setOption(option);
	}
};

module.exports = new Server();