'use strict';
var router = require('router');

function Server() {
}

Server.prototype.start = function(path, port, option, callback) {
	var http = require('http');
	var express = require('express')();
	var logger = require('logger').get();
	var self = this;
	
	express.get('/crossdomain.xml', function(request, response) {
		response.writeHead(200, { 'Content-Type': 'text/plain' });
		response.write('<cross-domain-policy><allow-access-from domain=\"*\" to-ports=\"<' + port + '>\"/></cross-domain-policy>\0');
		response.end();
	});
	
	var extension = require(path).create();
	if(extension) {
		self.extension = extension;
		extension.start(express);
		extension.setOption(option);
		var route = router.getRepresentativeRoute();
		if(route) extension.setState(route);
		router.setExtension(extension);
	}
	self.server = http.createServer(express);
	self.server.listen(port, function() {
		logger.i('Listening on port ' + port);
	});
};

Server.prototype.stop = function() {
	if(this.server) {
		this.server.close();
		this.server = undefined;
	}
	if(this.extension) {
		router.setExtension(undefined);
		this.extension.stop();
		this.extension = undefined;
	}
};

Server.prototype.setState = function(route) {
	if(this.extension) {
		this.extension.setState(route);
	}
};

Server.prototype.setOption = function(option) {
	if(this.extension) {
		this.extension.setOption(option);
	}
};

module.exports = new Server();