'use strict';
var router = require('router');

function Server() {
}

Server.prototype.start = function(path, port, option, callback) {
	var WebSocketServer = require('websocket').server;
	var http = require('http');
	var logger = require('logger').get();
	var self = this;

	var httpServer = http.createServer(function(request, response) {
		response.writeHead(200);
		response.end();
	});
	self.httpServer = httpServer;
	httpServer.listen(port, function() {
		logger.i('Listening on port ' + port);
	});
	var server = new WebSocketServer({
		httpServer: httpServer,
		autoAcceptConnections: false
	});
	self.server = server;
	server.on('request', function(request) {
		var connection = request.accept();
		if(callback) callback('connected');
		
		connection.on('close', function(reasonCode, description) {
			if(self.extension) self.extension.stop();
			if(callback) callback('disconnected');
		});
		var extension = require(path).create();
		if(extension) {
			self.extension = extension;
			extension.start(connection);
			extension.setOption(option);
			var route = router.getRepresentativeRoute();
			if(route) extension.setState(route);
			router.setExtension(extension);
		}
	});
};

Server.prototype.stop = function() {
	if(this.server) {
		this.server.shutDown();
		this.server = undefined;
	}
	if(this.httpServer) {
		this.httpServer.removeAllListeners();
		this.httpServer.close();
		this.httpServer = undefined;
	}
	if(this.extension) {
		router.setExtension(undefined);
		this.extension.stop();
		this.extension = undefined;
	}
};

Server.prototype.setState = function(route) {
	if(this.extension) {
		this.extension.setState(route);
	}
};

Server.prototype.setOption = function(option) {
	if(this.extension) {
		this.extension.setOption(option);
	}
};

module.exports = new Server();