'use strict';
function Translator() {
	var langs = {
		'ko': 'ko',
		'uz': 'uz'
	};
	var lang = window.navigator.userLanguage || window.navigator.language;
	if(lang) {
		lang = lang.toLowerCase();
		var index = lang.indexOf('-');
		if(index >= 0) lang = lang.substring(0, index);
	}
	lang = langs[lang];
	if(lang == undefined)
		lang = 'en';

	var Localize = require('localize');
	var loc = new Localize('locales');
	loc.setLocale(lang);
	loc.throwOnMissingTranslation(false);
	var listeners = {};
	
	this.getLanguage = function() {
		return lang;
	};
	
	this.setLanguage = function(newLang) {
		if(lang != newLang) {
			lang = newLang;
			loc.setLocale(lang);
			var listener;
			for(var i in listeners) {
				listener = listeners[i];
				if(typeof listener === 'function') {
					listener();
				}
			}
		}
	};
	
	this.on = function(key, callback) {
		listeners[key] = callback;
	};
	
	this.remove = function(key) {
		if(key in listeners) {
			delete listeners[key];
		}
	};
	
	this.clear = function() {
		listeners = {};
	};
	
	this.translate = function() {
		return loc.translate.apply(this, arguments);
	};
}

module.exports = new Translator();