'use strict';
(function() {
	var fs = require('fs');
	var translator = require('translator');
	var chat = require('chat');
	var lang = translator.getLanguage();
	
	var ui = {
		started: false,
		updateLabel: function() {
			$('#lang').val(lang);
			$('#win-title').text(translator.translate('Chatting Server'));
			$('#server-message .title').text(translator.translate('Server started.'));
			$('#ip-title').text(translator.translate('IPs of this computer'));
			$('#client-title').text(translator.translate('Participants'));
			$('#server-port .title').text(translator.translate('Port'));
			$('#start-server').text(translator.translate('Start Server'));
			$('#stop-server').text(translator.translate('Stop Server'));
		},
		updateMessage: function(started) {
			if(started === undefined) started = this.started;
			else this.started = started;
			if(started) $('#server-message').show();
			else $('#server-message').hide();
		},
		updateParticipants: function(connections) {
			var names = '';
			for(var name in connections) {
				if(names) names += '<br/>';
				names += name;
			}
			$('#client-list').html(names);
		}
	};
	$('#lang').change(function() {
		lang = $(this).val();
		translator.setLanguage(lang);
		ui.updateLabel();
		ui.updateMessage();
	});
	$('#console').click(function() {
		require('nw.gui').Window.get().showDevTools();
	});
	
	var chatServer = {
		PORT: 60000,
		start: function() {
			var port = parseInt($('#server-port-number').val());
			chat.start(port);
		},
		stop: function() {
			$('#stop-server').hide();
			$('#start-server').show();
			chat.stop();
			ui.updateMessage(false);
		}
	};
	
	$('#server-port-number').val(chatServer.PORT);
	ui.updateLabel();
	
	$(window).unload(function() {
		chatServer.stop();
		
		var config = { open: 0 };
		var path = process.cwd() + '/package/dashboard.cfg';
		try {
			var data = JSON.stringify(config);
			fs.writeFileSync(path, data);
		} catch (e) {
		}
	});
	$('#start-server').click(function() {
		chatServer.start();
	});
	$('#stop-server').click(function() {
		chatServer.stop();
	});
	
	$('#ip-list').html(chat.getIps());
	chat.on('start', function() {
		$('#start-server').hide();
		$('#stop-server').show();
		ui.updateMessage(true);
	});
	chat.on('participants', function(connections) {
		ui.updateParticipants(connections);
	});
}());