'use strict';
(function() {
	var editor = CodeMirror.fromTextArea(document.getElementById("code"), {
		lineNumbers: true,
		extraKeys: { "Ctrl-Space": "autocomplete" },
		mode: { name: "javascript", globalVars: true }
	});
	var gui = require('nw.gui');
	var translator = require('translator');
	var execute = undefined;
	
	var setup = function() {
		$('#win-title').text(translator.translate('JavaScript'));
		$('#console').text(translator.translate('Console'));
		$('#open').text(translator.translate('Open File'));
		$('#save').text(translator.translate('Save File'));
		$('#clear').text(translator.translate('Clear'));
		$('#stop').text(translator.translate('Stop'));
		$('#run').text(translator.translate('Run'));
		//$('#message').text(translator.translate('Auto Completion: Ctrl + Space'));
	};
	
	translator.on('js', function() {
		setup();
	});
	setup();
	
	function chooseFile(name, callback) {
		var chooser = $(name);
		chooser.unbind('change');
		chooser.change(function(ev) {
			var value = $(this).val();
			$(this).val('');
			if(callback) {
				callback(value);
			}
		});
		chooser.trigger('click');
	}

	$('#console').click(function() {
		gui.Window.get().showDevTools();
	});
	$('#open').click(function() {
		chooseFile('#openDialog', function(path) {
			if(path) {
				var fs = require('fs');
				try {
					fs.readFile(path, 'utf8', function(err, data) {
						if(err) {
							console.log(err);
						} else {
							editor.doc.setValue(data);
						}
					});
				} catch (e) {
				}
			}
		});
	});
	$('#save').click(function() {
		chooseFile('#saveDialog', function(path) {
			if(path) {
				var fs = require('fs');
				try {
					var val = editor.doc.getValue();
					fs.writeFile(path, val, 'utf8', function(err) {
						if(err) {
							console.log(err);
						}
					});
				} catch (e) {
				}
			}
		});
	});
	$('#clear').click(function() {
		editor.doc.setValue('');
	});
	$('#run').click(function() {
		$('#run').hide();
		$('#stop').show();
		runCode(editor.doc.getValue());
	});
	$('#stop').click(function() {
		$('#stop').hide();
		$('#run').show();
		stopCode();
	});
	
	$(window).unload(function() {
		__runner.close();
		translator.remove('js');
	});
	__runner.open('ws://127.0.0.1:54417');
}());