'use strict';
(function() {
	const VERSION = 'v1.7.2';
	// initialize options
	var options = {};
	var gui = require('nw.gui');
	gui.App.argv.forEach(function(item, index) {
		if(item == '-console') {
			options.console = true;
		}
	});
	
	// show devtools if console
	var win = gui.Window.get();
	if(options.console) {
		win.showDevTools();
	}
	
	// os
	var isLinux = false;
	var isLinux32 = false;
	var isOsx = false;
	if(process.platform === 'win32') {
		isLinux = false;
		isLinux32 = false;
	} else if(process.platform === 'darwin') {
		isLinux = false;
		isLinux32 = false;
		isOsx = true;
	} else {
		isLinux = true;
		if(process.arch.search('32') < 0) isLinux32 = false;
		else isLinux32 = true;
	}
	
	const ENTRY_OFFLINE_INCLUDED = true;
	
	// language
	var translator = require('translator');
	var lang = translator.getLanguage();
	console.log('language: ' + lang);
	
	// logger
	var logger;
	if(isLinux) {
		logger = {
			v: function(str) {
				console.log(str);
			},
			i: function(str) {
				console.info(str);
			},
			w: function(str) {
				console.warn(str);
			},
			e: function(str) {
				console.error(str);
			}
		};
	} else {
		logger = {
			v: function(str) {
				console.log(str);
			},
			i: function(str) {
				console.info('%c' + str, 'color: dodgerblue');
			},
			w: function(str) {
				console.warn('%c' + str, 'color: orange');
			},
			e: function(str) {
				console.error('%c' + str, 'color: red');
			}
		};
	}
	require('logger').set(logger);
	
	var context = require('context');
	var router = require('router');
	var server = require('server');
//	var dashboard = require('dashboard');
	
	var configs = [];
	var components = {};
	var alignments = {};
	var monitors = {};
	var intervalIds = {};
	
	// ui & control
	var ui = {
		enableProgram: function(id, img1, img2) {
			var target = $('#' + id);
			var targetImage = $('#' + id + ' img');
			target.off('hover');
			target.hover(function() {
				$(this).css('cursor', 'pointer');
				$(this).css('opacity', '0.5');
			}, function() {
				$(this).css('cursor', 'auto');
				$(this).css('opacity', '1');
			});
			target.css('color', '#fff');
			targetImage.css('background-image', 'url(images/bg_item_white.png)');
			if(img1 && img2) {
				var img = img1;
				intervalIds[id] = setInterval(function() {
					if(img == img1) img = img2;
					else img = img1;
					targetImage.attr('src', 'images/' + img);
				}, 500);
			}
		},
		disableProgram: function(id, img) {
			var target = $('#' + id);
			var targetImage = $('#' + id + ' img');
			target.off('hover');
			target.hover(function() {
				$(this).css('cursor', 'not-allowed');
				$(this).css('opacity', '1');
			}, function() {
				$(this).css('cursor', 'auto');
				$(this).css('opacity', '1');
			});
			if(id in intervalIds) {
				clearInterval(intervalIds[id]);
			}
			target.css('color', '#705D4B');
			targetImage.css('background-image', 'url(images/bg_item_dark.png)');
			if(img) {
				targetImage.attr('src', 'images/' + img);
			}
		},
		disableTopMenu: function(id) {
			var target = $('#' + id);
			var targetImage = $('#' + id + ' img');
			target.off('hover');
			target.hover(function() {
				$(this).css('cursor', 'not-allowed');
				$(this).css('opacity', '1');
			}, function() {
				$(this).css('cursor', 'auto');
				$(this).css('opacity', '1');
			});
			if(id in intervalIds) {
				clearInterval(intervalIds[id]);
			}
			target.css('color', '#101F2B');
			targetImage.css('background-image', 'url(images/bg_item_dark.png)');
		},
		showStartScan: function() {
			$('#start-scan').show();
			$('#stop-scan').hide();
		},
		showStopScan: function() {
			$('#start-scan').hide();
			$('#stop-scan').show();
		},
		getId: function(route) {
			return route.config.module + '_' + route.index;
		},
		reverseAddress: function(address) {
			return address.slice(10, 12) + ':' + address.slice(8, 10) + ':' + address.slice(6, 8) + ':' + address.slice(4, 6) + ':' + address.slice(2, 4) + ':' + address.slice(0, 2)
		},
		addRobot: function(route) {
			var config = route.config;
			var id = ui.getId(route);
			var contents = '<div id="' + id + '" class="robot">';
			route.icon = '../modules/' + config.module + '/' + config.icon;
			route.iconSelected = '../modules/' + config.module + '/' + config.iconSelected;
			contents += '<img class="icon" src="../modules/' + config.module + '/' + config.icon + '" />';
			contents += '<div class="change-robot"><img src="images/change-robot.png" /></div>';
			contents += '<div class="index">' + route.index + '</div>';
			contents += '<div class="identity-refresh"><img src="images/identity-refresh.png" /></div>';
			contents += '<div class="identity1 identity-led"></div>';
			contents += '<div class="identity2 identity-led"></div>';
			contents += '<div class="description">';
			contents += '<div class="title">' + translator.translate(config.name) + ' ' + route.id + '</div>';
			contents += '<div class="port">' + route.port + '</div>';
			contents += '<div class="address">' + ui.reverseAddress(route.address) + '</div>';
			contents += '<div class="state"></div>';
			contents += '<div class="dongle-reset"><img src="images/reset-dongle.png" /></div>';
			contents += '</div></div>';
			ui.setState(route);
			$('#robot-list').append(contents);
			ui.updateChangeRobot(id, route);
			var target = $('#' + id + ' .dongle-reset');
			target.off();
			target.click(function(event) {
				event.stopPropagation();
				if(route.dongle) {
					route.dongle.doReset = true;
				}
			});
			target = $('#' + id + ' .change-robot');
			target.off();
			target.click(function(event) {
				event.stopPropagation();
				if(route.hwModule == 'hamsterS') {
					var oldModule = route.hwModule;
					var oldIndex = route.index;
					if(route.module == 'hamsterS') {
						route.module = 'hamster';
						route.iconFileName = 'hamster.png';
						route.iconSelectedFileName = 'hamster-selected.png';
					} else if(route.module == 'hamster') {
						route.module = 'hamsterS';
						route.iconFileName = 'hamsterS.png';
						route.iconSelectedFileName = 'hamsterS-selected.png';
					}
					route.controller = require('../modules/' + route.module + '/controller').create();
					ui.updateRobot(oldModule + '_' + oldIndex, route);
					ui.updateIdentity(route);
				}
			});
			target = $('#' + id + ' .identity-refresh');
			target.off();
			target.click(function(event) {
				event.stopPropagation();
				route.controller.updateIdentity();
				ui.updateIdentity(route);
			});
			ui.updateIdentity(route);
			target = $('#' + id);
			target.off();
			target.click(function() {
				if(router.multi) return;
				ui.selectRobot(route);
				server.restartForSingleRobot(route);
				server.setState(route);
				components['alignment'].update(route);
				components['monitor'].update(route);
				components['setting'].update(route);
			});
			target.hover(function() {
				$(this).css('cursor', 'pointer');
			}, function() {
				$(this).css('cursor', 'auto');
			});
		},
		removeRobot: function(route) {
			var target = $('#' + ui.getId(route));
			if(target) {
				target.remove();
			}
		},
		updateRobot: function(id, route) {
			var config = route.config;
			route.icon = '../modules/' + route.module + '/' + (route.iconFileName || config.icon);
			route.iconSelected = '../modules/' + route.module + '/' + (route.iconSelectedFileName || config.iconSelected);
			if(route.selectedIcon) {
				$('#' + id + ' .icon').attr('src', route.iconSelected);
			} else {
				$('#' + id + ' .icon').attr('src', route.icon);
			}
			$('#' + id + ' .index').text(route.index);
			$('#' + id + ' .title').text(translator.translate(config.name) + ' ' + route.id);
			$('#' + id + ' .port').text(route.port);
			$('#' + id + ' .address').text(ui.reverseAddress(route.address));
			ui.setState(route);
			ui.updateChangeRobot(id, route);
			$('#' + id).attr('id', ui.getId(route));
		},
		updateSingleRobotIcon: function() {
			var routes = router.getRoutes();
			var selectedRoute = router.selectedRoute;
			var route, id, image;
			for(var i = 0, len = routes.length; i < len; ++i) {
				route = routes[i];
				if(route == selectedRoute) {
					if(!route.selectedIcon) {
						route.selectedIcon = true;
						id = ui.getId(route);
						image = $('#' + id + ' .icon');
						image.attr('src', route.iconSelected);
					}
				} else {
					if(route.selectedIcon) {
						route.selectedIcon = false;
						id = ui.getId(route);
						image = $('#' + id + ' .icon');
						image.attr('src', route.icon);
					}
				}
			}
		},
		updateAllRobotIcons: function() {
			var routes = router.getRoutes();
			var route, id, image;
			for(var i = 0, len = routes.length; i < len; ++i) {
				route = routes[i];
				if(!route.selectedIcon) {
					route.selectedIcon = true;
					id = ui.getId(route);
					image = $('#' + id + ' .icon');
					image.attr('src', route.iconSelected);
				}
			}
		},
		selectRobot: function(route) {
			router.selectedRoute = route;
			if(router.multi) {
				ui.updateAllRobotIcons();
			} else {
				ui.updateSingleRobotIcon();
			}
		},
		updateDongle: function(route) {
			var id = ui.getId(route);
			if(route.state == 'connected' && route.dongle && route.dongle.canReset) {
				$('#' + id + ' .dongle-reset').show();
			} else {
				$('#' + id + ' .dongle-reset').hide();
			}
		},
		updateChangeRobot: function(id, route) {
			if(route.config.robotChangeable && router.isExtraButtonActive()) {
				$('#' + id + ' .change-robot').show();
			} else {
				$('#' + id + ' .change-robot').hide();
			}
		},
		updateChangeRobots: function() {
			var routes = router.getRoutes();
			var route, id;
			for(var i in routes) {
				route = routes[i];
				id = ui.getId(route);
				ui.updateChangeRobot(id, route);
			}
		},
		updateIdentity: function(route) {
			var id = ui.getId(route);
			var identity1 = $('#' + id + ' .identity1');
			var identity2 = $('#' + id + ' .identity2');
			var refresh = $('#' + id + ' .identity-refresh');
			if(route.controller && router.isExtraButtonActive() && route.state == 'connected') {
				var sz = route.controller.getIdentitySize();
				if(sz > 0) {
					identity1.css('left', route.controller.getIdentityX(0));
					identity1.css('top', route.controller.getIdentityY(0));
					identity1.css('background-color', route.controller.getIdentityHex(0));
					identity1.show();
					if(sz > 1) {
						identity2.css('left', route.controller.getIdentityX(1));
						identity2.css('top', route.controller.getIdentityY(1));
						identity2.css('background-color', route.controller.getIdentityHex(1));
						identity2.show();
					} else {
						identity2.hide();
					}
					refresh.show();
					return;
				}
			}
			identity1.hide();
			identity2.hide();
			refresh.hide();
		},
		updateIdentities: function() {
			var routes = router.getRoutes();
			for(var i in routes) {
				ui.updateIdentity(routes[i]);
			}
		},
		setState: function(route) {
			var id = ui.getId(route);
			var label, image;
			switch(route.state) {
				case 'connecting': {
					label = 'Connecting';
					image = 'images/bg_item_orange.png';
					break;
				}
				case 'connected': {
					label = 'Connected';
					image = 'images/bg_item_blue.png';
					break;
				}
				case 'lost': {
					label = 'Connecting';
					image = 'images/bg_item_orange.png';
					break;
				}
				case 'disconnected': {
					label = 'Disconnected';
					image = 'images/bg_item_pink.png';
					break;
				}
			}
			$('#' + id + ' .state').text(translator.translate(label));
			$('#' + id + ' .icon').css('background-image', 'url(' + image +')');
			ui.updateIdentity(route);
			ui.updateDongle(route);
		},
		openScratch: function(level) {
			var route = router.getRepresentativeRoute();
			if(route) {
				var path = require('path');
				var fs = require('fs');
				var filename = translator.translate('Untitled') + '.sb2';
				var dir = process.cwd() + '/modules/';
				if(router.multi) {
					if(router.numberOfActiveModules > 1) {
						dir += 'all/scratch/' + lang + '/level' + level + '/multi';
					} else {
						dir += route.module + '/scratch/' + lang + '/level' + level + '/multi';
					}
				} else {
					dir += route.module + '/scratch/' + lang + '/level' + level;
				}
				server.setState(route);
				var srcFile = path.join(dir, filename);
				var dstFile = path.join(process.cwd() + '/workspace/scratch', filename);
				var rs = fs.createReadStream(srcFile);
				rs.on('error', function(error) {
					complete(error);
				});
				var ws = fs.createWriteStream(dstFile);
				ws.on('error', function(error) {
					complete(error);
				});
				ws.on('close', function() {
					complete();
				});
				rs.pipe(ws);
				
				function complete(error) {
					if(error) {
						console.error(translator.translate('An error occurred while trying to open $[1].', filename));
						return;
					}
					gui.Shell.openItem(dstFile);
				}
			}
		},
		openScratchX: function(level) {
			var route = router.getRepresentativeRoute();
			if(route) {
				var url = 'http://scratchx.org/?lang=' + lang + '&level=' + level + '&url=http://hamsterschool.github.io/scratchx/';
				if(router.multi) {
					if(router.numberOfActiveModules > 1) {
						url += 'all-multi172.js#scratch';
					} else {
						url += route.module + '-multi172.js#scratch';
					}
				} else {
					url += route.module + '172.js#scratch';
				}
				if(isLinux) {
					var exec = require('child_process').exec;
					exec('google-chrome --start-maximized --disable-translate --app="' + url + '"');
				} else {
					if(ui.winScratchX) {
						if(url == ui.prevScratchXUrl)
							ui.winScratchX.reload();
						else
							ui.winScratchX.window.location.href = url;
					} else {
						var title = translator.translate('ScratchX');
						var winOptions = {
							title: title,
							toolbar: false,
							icon: 'package/images/scratch.png'
						};
						ui.winScratchX = gui.Window.open(url, winOptions);
						ui.winScratchX.on('maximize', function() {
							ui.winScratchX.show();
						});
						ui.winScratchX.hide();
						ui.winScratchX.maximize();
						ui.winScratchX.on('closed', function() {
							ui.winScratchX = undefined;
							ui.prevScratchXUrl = undefined;
							console.log('ScratchX window closed');
						});
					}
				}
				ui.prevScratchXUrl = url;
			}
		},
		openScratch3Offline: function() {
			var route = router.getRepresentativeRoute();
			if(route) {
				if(isOsx) {
					var filePath = process.cwd() + '/package/scratch3/Scratch Desktop.app';
					gui.Shell.openItem(filePath);
				} else if(!ui.winScratch3) {
					ui.winScratch3 = true;
					ui.updateScratch3Message();
					
					var filePath = process.cwd() + '/package/scratch3/Scratch Desktop.exe';
					var child = require('child_process').execFile(filePath, function(error, stdout, stderr) {
						if(error) {
							ui.winScratch3 = false;
							ui.updateScratch3Message();
						}
					});
					child.on('exit', function(code) {
						ui.winScratch3 = false;
						ui.updateScratch3Message();
					});
				}
			}
		},
		openEntryOnline: function() {
			var route = router.getRepresentativeRoute();
			if(route) {
				var entryLang = 'en';
				if(lang == 'ko' || lang == 'vn') entryLang = lang;
				var url = 'http://www.playentry.org/ws?lang=' + entryLang + '#!/';
				if(ui.winEntryOnline) {
					if(url == ui.prevEntryOnlineUrl)
						ui.winEntryOnline.reload();
					else
						ui.winEntryOnline.window.location.href = url;
				} else {
					var title = translator.translate('Entry');
					var winOptions = {
						title: title,
						toolbar: false,
						icon: 'package/images/entry.png',
						'inject-js-end': 'modules/all/entry/plugin.js'
					};
					ui.winEntryOnline = gui.Window.open(url, winOptions);
					ui.winEntryOnline.on('maximize', function() {
						ui.winEntryOnline.show();
					});
					if(!isLinux) ui.winEntryOnline.hide();
					ui.winEntryOnline.maximize();
					ui.winEntryOnline.on('closed', function() {
						ui.winEntryOnline = undefined;
						ui.prevEntryOnlineUrl = undefined;
						console.log('Entry Online window closed');
					});
				}
				ui.prevEntryOnlineUrl = url;
			}
		},
		openEntryOffline: function() {
			var route = router.getRepresentativeRoute();
			if(route) {
				if(ENTRY_OFFLINE_INCLUDED) {
					var filePath = process.cwd() + '/package/entry/Entry';
					if(isOsx) filePath += '.app';
					else filePath += '.exe';
					gui.Shell.openItem(filePath);
				} else {
					var path = require('path');
					var fs = require('fs');
					var filename = 'Untitled.ent';
					var dir = process.cwd() + '/modules/all/entry/' + lang;
					var srcFile = path.join(dir, filename);
					var dstFile = path.join(process.cwd() + '/workspace/entry', filename);
					var rs = fs.createReadStream(srcFile);
					rs.on('error', function(error) {
						complete(error);
					});
					var ws = fs.createWriteStream(dstFile);
					ws.on('error', function(error) {
						complete(error);
					});
					ws.on('close', function() {
						complete();
					});
					rs.pipe(ws);
					
					function complete(error) {
						if(error) {
							console.error(translator.translate('An error occurred while trying to open $[1].', filename));
							return;
						}
						gui.Shell.openItem(dstFile);
					}
				}
			}
		},
		openSnap: function(level) {
			if(router.selectedRoute) {
				if(!ui.winSnap) {
					var winOptions = {
						title: 'Snap',
						toolbar: false,
						icon: 'package/images/snap.png'
					};
					var url = "http://snap.berkeley.edu/snapsource/snap.html#open:http://127.0.0.1:60417";
//					ui.winSnap = gui.Window.open(url, winOptions);
//					ui.winSnap.on('maximize', function() {
//						ui.winSnap.show();
//					});
//					ui.winSnap.hide();
//					ui.winSnap.maximize();
//					ui.winSnap.on('closed', function() {
//						ui.winSnap = undefined;
//					});
					gui.Shell.openItem(url);
				}
			}
		},
		openPlayBot: function() {
			if(router.selectedRoute) {
				if(!ui.winPlaybot) {
					var winOptions = {
						title: 'PlayBot.. The Platform for Software Education..',
						toolbar: false,
						icon: 'package/images/playbot.png',
						'inject-js-end': 'modules/all/playbot/plugin.js'
					};
					ui.winPlaybot = gui.Window.open('http://playbot.spaceii.com/playbot.htm', winOptions);
					ui.winPlaybot.on('maximize', function() {
						ui.winPlaybot.show();
					});
					if(!isLinux) ui.winPlaybot.hide();
					ui.winPlaybot.maximize();
					ui.winPlaybot.on('closed', function() {
						ui.winPlaybot = undefined;
						console.log('PlayBot window closed');
					});
				}
			}
		},
		openJavaScript: function() {
			if(router.getRepresentativeRoute()) {
				if(!ui.winJavaScript) {
					var title = translator.translate('JavaScript');
					var winOptions = {
						title: title,
						toolbar: false,
						width: 640,
						height: 536,
						icon: 'package/images/js.png'
					};
					ui.winJavaScript = gui.Window.open('js.html', winOptions);
					ui.winJavaScript.on('closed', function() {
						ui.winJavaScript = undefined;
						console.log('JavaScript window closed');
					});
				}
			}
		},
		/*openDashboard: function() {
			if(!dashboard.isOpen()) {
				var title = translator.translate('Chatting Server');
				var winOptions = {
					title: title,
					toolbar: false,
					resizable: false,
					width: 440,
					height: 192,
					'new-instance': true,
					icon: 'package/images/chat.png'
				};
				ui.winDashboard = gui.Window.open('dashboard.html', winOptions);
				dashboard.setOpen(true);
			}
		},*/
		updateMessage: function(msg) {
			if(msg == undefined) {
				msg = ui.activeProgram;
			} else if(msg == 'alignment-close' || msg == 'setting-close' || msg == 'monitor-close') {
				msg = ui.activeProgram = ui.prevProgram;
				ui.prevProgram = undefined;
			} else {
				if(msg.indexOf('alignment') >= 0 || msg.indexOf('setting') >= 0 || msg.indexOf('monitor') >= 0) {
					ui.prevProgram = ui.activeProgram;
				}
				ui.activeProgram = msg;
			}
			if(msg == 'scratch3-offline') $('#message').text(translator.translate('Scratch') + ' 3 ' + translator.translate('Offline'));
			else if(msg == 'scratch-online') $('#message').text(translator.translate('ScratchX') + ' ' + translator.translate('Online'));
			else if(msg == 'scratch-offline') $('#message').text(translator.translate('Scratch') + ' 2 ' + translator.translate('Offline'));
			else if(msg == 'entry-online') $('#message').text(translator.translate('Entry') + ' ' + translator.translate('Online'));
			else if(msg == 'entry-offline') $('#message').text(translator.translate('Entry') + ' ' + translator.translate('Offline'));
			else if(msg == 'snap-online') $('#message').text(translator.translate('Snap') + ' ' + translator.translate('Online'));
			else if(msg == 'playbot-online') $('#message').text(translator.translate('PlayBot') + ' ' + translator.translate('Online'));
			else if(msg == 'javascript') $('#message').text(translator.translate('JavaScript'));
			else if(msg == 'server') $('#message').text(translator.translate('Server'));
			else if(msg == 'alignment-hamster') $('#message').text(translator.translate('Hamster') + ' ' + translator.translate('Calibration'));
			else if(msg == 'alignment-turtle') $('#message').text(translator.translate('Turtle') + ' ' + translator.translate('Calibration'));
			else if(msg == 'alignment-cheese') $('#message').text(translator.translate('Cheese Stick') + ' ' + translator.translate('Calibration'));
			else if(msg == 'setting') $('#message').text(translator.translate('Default Values'));
			else if(msg == 'monitor') $('#message').text(translator.translate('Sensor Monitor'));
			else $('#message').text('');
		},
		updateServerMessage: function(msg, port) {
			if(msg == undefined) {
				msg = ui.activeServer;
			} else {
				ui.activeServer = msg;
			}
			if(port == undefined) {
				port = ui.activeServerPort;
			} else {
				ui.activeServerPort = port;
			}
			if(msg == 'http') $('#server-message').text(translator.translate('$[1] server started. Port: ', translator.translate('Http')) + port);
			else if(msg == 'websocket') $('#server-message').text(translator.translate('$[1] server started. Port: ', translator.translate('WebSocket')) + port);
			else if(msg == 'socketio') $('#server-message').text(translator.translate('$[1] server started. Port: ', translator.translate('Socket.io')) + port);
			else $('#server-message').text('');
		},
		updateScratch3Message: function() {
			if(ui.winScratch3) {
				$('#scratch3-message').text(translator.translate('The Scratch 3 is running.'));
			} else {
				$('#scratch3-message').text('');
			}
		},
		updateLanguage: function() {
			$('#lang').val(lang);
			$('#win-title').text(translator.translate('Robot Coding') + ' ' + VERSION);
			$('#start-scan .title').text(translator.translate('Start Scanning'));
			$('#stop-scan .message').text(translator.translate('Scanning...'));
			$('#stop-scan .title').text(translator.translate('Cancel'));
			$('#scratch3-offline .title').html(translator.translate('Scratch') + ' 3<br/>' + translator.translate('Offline'));
			$('#scratch-online .title').html(translator.translate('ScratchX') + '<br/>' + translator.translate('Online'));
			$('#scratch-offline .title').html(translator.translate('Scratch') + ' 2<br/>' + translator.translate('Offline'));
			$('#entry-online .title').html(translator.translate('Entry') + '<br/>' + translator.translate('Online'));
			$('#entry-offline .title').html(translator.translate('Entry') + '<br/>' + translator.translate('Offline'));
//			$('#snap-online .title').html(translator.translate('Snap') + '<br/>' + translator.translate('Online'));
			$('#playbot-online .title').html(translator.translate('PlayBot') + '<br/>' + translator.translate('Online'));
			$('#javascript .title').text(translator.translate('JavaScript'));
			$('#server .title').html(translator.translate('Server'));
			$('#alignment .title').html(translator.translate('Robot Calibration'));
			$('#setting .title').html(translator.translate('Default Values'));
			$('#monitor .title').html(translator.translate('Sensor Monitor'));
			$('#console .title').html(translator.translate('Console'));
//			$('#chatting .title').html(translator.translate('Chatting<br/>Server'));
			$('#scratch-back .title').text(translator.translate('Back'));
			$('#scratch-single .title').text(translator.translate('Single Robot'));
			$('#scratch-multi .title').text(translator.translate('Multiple Robots'));
			$('#scratch-level1 .title').text(translator.translate('Level 1'));
			$('#scratch-level2 .title').text(translator.translate('Level 2'));
			$('#scratch-level3 .title').text(translator.translate('Level 3'));
			$('#scratch3-back .title').text(translator.translate('Back'));
			$('#scratch3-single .title').text(translator.translate('Single Robot'));
			$('#scratch3-multi .title').text(translator.translate('Multiple Robots'));
			$('#scratch3-level1 .title').text(translator.translate('Level 1'));
			$('#scratch3-level2 .title').text(translator.translate('Level 2'));
			$('#scratch3-level3 .title').text(translator.translate('Level 3'));
			$('#entry-back .title').text(translator.translate('Back'));
			$('#entry-single .title').text(translator.translate('Single Robot'));
			$('#entry-multi .title').text(translator.translate('Multiple Robots'));
			$('#entry-basic .title').text(translator.translate('New File'));
//			$('#snap-back .title').text(translator.translate('Back'));
//			$('#snap-single .title').text(translator.translate('Single Robot'));
//			$('#snap-multi .title').text(translator.translate('Multiple Robots'));
//			$('#snap-level1 .title').text(translator.translate('Level 1'));
//			$('#snap-level2 .title').text(translator.translate('Level 2'));
//			$('#snap-level3 .title').text(translator.translate('Level 3'));
			$('#js-back .title').text(translator.translate('Back'));
			$('#js-single .title').text(translator.translate('Single Robot'));
			$('#js-multi .title').text(translator.translate('Multiple Robots'));
			$('#server-back .title').text(translator.translate('Back'));
			$('#server-port .title').text(translator.translate('Port'));
			$('#server-single .title').text(translator.translate('Single Robot'));
			$('#server-multi .title').text(translator.translate('Multiple Robots'));
			$('#server-http .title').text(translator.translate('Http'));
			$('#server-websocket .title').text(translator.translate('WebSocket'));
			$('#server-socketio .title').text(translator.translate('Socket.io'));
			$('#alignment-close .title').text(translator.translate('Close'));
			$('#alignment-start').text(translator.translate('Start'));
			$('#alignment-stop').text(translator.translate('Stop'));
			$('#alignment-save').text(translator.translate('Save'));
			$('#alignment-hamster-robot .title').text(translator.translate('Robot:'));
			$('#alignment-hamster-speed .title').text(translator.translate('Speed:'));
			$('#alignment-hamster-direction .title').text(translator.translate('Direction:'));
			$('#alignment-hamsterS-message .title').text(translator.translate('HamsterS has no value to calibrate.'));
			$('#alignment-turtle-robot .title').text(translator.translate('Robot:'));
			$('#alignment-turtle-type .title').text(translator.translate('Type:'));
			$('#alignment-turtle-value .title').text(translator.translate('Value:'));
			$('#alignment-turtle-distance .title').text(translator.translate('Distance:'));
			$('#alignment-turtle-message .title').html(translator.translate('Click the + or - button of the \'Value\'<br/>according to the shape of the line<br/>drawn by the Turtle.<br/>Please calibrate until<br/>the endpoints meet each other.'));
			$('#alignment-cheese-message .title').text(translator.translate('Cheese Stick has no value to calibrate.'));
			var combo = $('#alignment-turtle-type .combo');
			var val = combo.val();
			var contents = '<option value="right">' + translator.translate('Right Turn') + '</option>';
			contents += '<option value="left">' + translator.translate('Left Turn') + '</option>';
			contents += '<option value="distance">' + translator.translate('Wheel Distance') + '</option>';
			combo.html(contents);
			combo.val(val);
			combo = $('#monitor-robot .combo');
			val = combo.val();
			contents = '<option value=0>' + translator.translate('all values') + '</option>';
			contents += '<option value=10>' + translator.translate('average (0.25 secs)') + '</option>';
			contents += '<option value=20>' + translator.translate('average (0.5 secs)') + '</option>';
			contents += '<option value=40>' + translator.translate('average (1 sec)') + '</option>';
			contents += '<option value=80>' + translator.translate('average (2 secs)') + '</option>';
			contents += '<option value=200>' + translator.translate('average (5 secs)') + '</option>';
			contents += '<option value=400>' + translator.translate('average (10 secs)') + '</option>';
			contents += '<option value=800>' + translator.translate('average (20 secs)') + '</option>';
			contents += '<option value=2400>' + translator.translate('average (1 min)') + '</option>';
			combo.html(contents);
			combo.val(val);
			$('#setting-robot .title').text(translator.translate('Robot Category'));
			combo = $('#setting-robot .combo');
			contents = '<option value="hamster">' + translator.translate('Hamster') + '</option>';
			contents += '<option value="hamsterS">' + translator.translate('HamsterS') + '</option>';
			contents += '<option value="turtle">' + translator.translate('Turtle') + '</option>';
			combo.html(contents);
			$('#setting-hamster-movementSpeed .title').text(translator.translate('Movement Speed'));
			$('#setting-hamster-movementSpeed .range').text(translator.translate('0 - 100'));
			$('#setting-hamster-turningSpeed .title').text(translator.translate('Turning Speed'));
			$('#setting-hamster-turningSpeed .range').text(translator.translate('0 - 100'));
			$('#setting-hamster-handFoundThreshold .title').text(translator.translate('Hand Found Threshold'));
			$('#setting-hamster-handFoundThreshold .range').text(translator.translate('0 - 50'));
			$('#setting-hamsterS-movementSpeed .title').text(translator.translate('Movement Speed'));
			$('#setting-hamsterS-movementSpeed .range').text(translator.translate('0 - 100'));
			$('#setting-hamsterS-movementDistance .title').text(translator.translate('Movement Distance'));
			$('#setting-hamsterS-turningSpeed .title').text(translator.translate('Turning Speed'));
			$('#setting-hamsterS-turningSpeed .range').text(translator.translate('0 - 100'));
			$('#setting-hamsterS-turningAngle .title').text(translator.translate('Turning Angle'));
			$('#setting-hamsterS-turningAngle .unit').text(translator.translate('degrees'));
			$('#setting-hamsterS-handFoundThreshold .title').text(translator.translate('Hand Found Threshold'));
			$('#setting-hamsterS-handFoundThreshold .range').text(translator.translate('0 - 50'));
			$('#setting-turtle-movementSpeed .title').text(translator.translate('Movement Speed'));
			$('#setting-turtle-movementSpeed .range').text(translator.translate('0 - 400'));
			$('#setting-turtle-movementDistance .title').text(translator.translate('Movement Distance'));
			$('#setting-turtle-turningSpeed .title').text(translator.translate('Turning Speed'));
			$('#setting-turtle-turningSpeed .range').text(translator.translate('0 - 400'));
			$('#setting-turtle-turningAngle .title').text(translator.translate('Turning Angle'));
			$('#setting-turtle-turningAngle .unit').text(translator.translate('degrees'));
			$('#setting-close .title').text(translator.translate('Close'));
			$('#setting-cancel').text(translator.translate('Cancel'));
			$('#setting-save').text(translator.translate('OK'));
			$('#monitor-close .title').text(translator.translate('Close'));
			$('#monitor-robot .title').text(translator.translate('Robot:'));
			$('#monitor-sensor .title').text(translator.translate('Sensor:'));
			$('#monitor-pause').text(translator.translate('Pause'));
			$('#monitor-resume').text(translator.translate('Resume'));
			$('#monitor-save').text(translator.translate('Save'));
			$('#monitor-save-image').text(translator.translate('Save Graph'));
			$('#monitor-save-text').text(translator.translate('Save Data'));
			$('#monitor-save-cancel').text(translator.translate('Cancel'));
			ui.updateMessage();
			ui.updateServerMessage();
			ui.updateScratch3Message();
		}
	};
	alignments['hamster'] = {
		activate: function(route) {
			var self = this;
			self.controller = route.controller;
			ui.updateMessage('alignment-hamster');
			var target = $('#alignment-hamster-speed-down10');
			target.off('click');
			target.click(function() {
				self.changeSpeed(-10);
			});
			target = $('#alignment-hamster-speed-down');
			target.off('click');
			target.click(function() {
				self.changeSpeed(-1);
			});
			target = $('#alignment-hamster-speed-up');
			target.off('click');
			target.click(function() {
				self.changeSpeed(1);
			});
			target = $('#alignment-hamster-speed-up10');
			target.off('click');
			target.click(function() {
				self.changeSpeed(10);
			});
			target = $('#alignment-hamster-direction-left10');
			target.off('click');
			target.click(function() {
				self.changeBalance(-10);
			});
			target = $('#alignment-hamster-direction-left');
			target.off('click');
			target.click(function() {
				self.changeBalance(-1);
			});
			target = $('#alignment-hamster-direction-right');
			target.off('click');
			target.click(function() {
				self.changeBalance(1);
			});
			target = $('#alignment-hamster-direction-right10');
			target.off('click');
			target.click(function() {
				self.changeBalance(10);
			});
			target = $('#alignment-start');
			target.off('click');
			target.click(function() {
				$('#alignment-start').hide();
				$('#alignment-stop').show();
				if(self.controller) self.controller.startAlignmentMovement();
			});
			target = $('#alignment-stop');
			target.off('click');
			target.click(function() {
				$('#alignment-start').show();
				$('#alignment-stop').hide();
				if(self.controller) self.controller.stopAlignmentMovement();
			});
			target = $('#alignment-save');
			target.off('click');
			target.click(function() {
				if(self.controller) {
					self.controller.saveAlignment(self.balance);
					self.balance = 0;
					$('#alignment-hamster-direction-value').text(self.balance);
				}
			});
			var name = route.config.name + '[' + route.index + ']';
			$('#alignment-hamster-robot .name').text(name);
			self.reset();
			if(self.controller) self.controller.activateAlignment();
		},
		deactivate: function() {
			this.reset();
			if(this.controller) this.controller.deactivateAlignment();
			this.controller = undefined;
		},
		changeSpeed: function(delta) {
			this.speed += delta;
			if(this.speed < 0) this.speed = 0;
			else if(this.speed > 100) this.speed = 100;
			$('#alignment-hamster-speed-value').text(this.speed);
			if(this.controller) this.controller.setAlignmentSpeed(this.speed);
		},
		changeBalance: function(delta) {
			this.balance += delta;
			if(this.balance < -100) this.balance = -100;
			else if(this.balance > 100) this.balance = 100;
			$('#alignment-hamster-direction-value').text(this.balance);
			if(this.controller) this.controller.setAlignmentBalance(this.balance);
		},
		reset: function() {
			this.speed = 30;
			this.balance = 0;
			$('#alignment-hamster-speed-value').text(this.speed);
			$('#alignment-hamster-direction-value').text(this.balance);
			$('#alignment-start').show();
			$('#alignment-stop').hide();
			$('#alignment-save').show();
			if(this.controller) this.controller.resetAlignment();
		}
	};
	alignments['hamsterS'] = {
		activate: function(route) {
			ui.updateMessage('alignment-hamsterS');
			$('#alignment-start').hide();
			$('#alignment-stop').hide();
			$('#alignment-save').hide();
		},
		deactivate: function() {
		}
	};
	alignments['turtle'] = {
		activate: function(route) {
			var self = this;
			self.controller = route.controller;
			ui.updateMessage('alignment-turtle');
			var target = $('#alignment-turtle-type .combo');
			target.off('change');
			target.change(function() {
				var index = $(this).children('option').index($(this).children('option:selected'));
				self.changeType(index);
			});
			target = $('#alignment-turtle-value-down10');
			target.off('click');
			target.click(function() {
				self.changeValue(-10);
			});
			target = $('#alignment-turtle-value-down');
			target.off('click');
			target.click(function() {
				self.changeValue(-1);
			});
			target = $('#alignment-turtle-value-up');
			target.off('click');
			target.click(function() {
				self.changeValue(1);
			});
			target = $('#alignment-turtle-value-up10');
			target.off('click');
			target.click(function() {
				self.changeValue(10);
			});
			target = $('#alignment-start');
			target.off('click');
			target.click(function() {
				$('#alignment-start').hide();
				$('#alignment-stop').show();
				$('#alignment-save').hide();
				if(self.controller) self.controller.startAlignmentMovement(function() {
					$('#alignment-start').show();
					$('#alignment-stop').hide();
					$('#alignment-save').show();
				});
			});
			target = $('#alignment-stop');
			target.off('click');
			target.click(function() {
				$('#alignment-start').show();
				$('#alignment-stop').hide();
				$('#alignment-save').show();
				if(self.controller) self.controller.stopAlignmentMovement();
			});
			target = $('#alignment-save');
			target.off('click');
			target.click(function() {
				if(self.controller) {
					$('#alignment-start').hide();
					$('#alignment-save').hide();
					var distance = $('#alignment-turtle-distance-value').val();
					self.controller.saveAlignment(self.value, distance, function() {
						$('#alignment-start').show();
						$('#alignment-save').show();
						self.value = 0;
						$('#alignment-turtle-value-value').text(self.value);
						if(self.controller) $('#alignment-turtle-distance-value').val(self.controller.getAlignmentDistance());
					});
				}
			});
			var name = route.config.name + '[' + route.index + ']';
			$('#alignment-turtle-robot .name').text(name);
			self.reset();
			if(self.controller) self.controller.activateAlignment();
		},
		deactivate: function() {
			this.reset();
			if(this.controller) this.controller.deactivateAlignment();
			this.controller = undefined;
		},
		changeType: function(index) {
			this.value = 0;
			$('#alignment-turtle-value-value').text(this.value);
			if(index == 0) { // right turn
				$('#alignment-turtle-help-minus').attr('src', 'images/turtle_right_much.png');
				$('#alignment-turtle-help-plus').attr('src', 'images/turtle_right_less.png');
				$('#alignment-turtle-value').show();
				$('#alignment-turtle-help-angle').show();
				$('#alignment-turtle-distance').hide();
				$('#alignment-turtle-help-distance').hide();
				$('#alignment-turtle-message').show();
			} else if(index == 1) { // left turn
				$('#alignment-turtle-help-minus').attr('src', 'images/turtle_left_much.png');
				$('#alignment-turtle-help-plus').attr('src', 'images/turtle_left_less.png');
				$('#alignment-turtle-value').show();
				$('#alignment-turtle-help-angle').show();
				$('#alignment-turtle-distance').hide();
				$('#alignment-turtle-help-distance').hide();
				$('#alignment-turtle-message').show();
			} else { // distance
				$('#alignment-turtle-value').hide();
				$('#alignment-turtle-help-angle').hide();
				$('#alignment-turtle-distance').show();
				$('#alignment-turtle-help-distance').show();
				$('#alignment-turtle-message').hide();
			}
			if(this.controller) {
				this.controller.setAlignmentDelta(0);
				this.controller.setAlignmentType(index);
			}
		},
		changeValue: function(delta) {
			this.value += delta;
			if(this.value < -100) this.value = -100;
			else if(this.value > 100) this.value = 100;
			$('#alignment-turtle-value-value').text(this.value);
			if(this.controller) this.controller.setAlignmentDelta(this.value);
		},
		reset: function() {
			this.speed = 50;
			this.value = 0;
			$('#alignment-turtle-type .combo').val('right');
			$('#alignment-turtle-value-value').text(this.value);
			if(this.controller) $('#alignment-turtle-distance-value').val(this.controller.getAlignmentDistance());
			$('#alignment-turtle-help-minus').attr('src', 'images/turtle_right_much.png');
			$('#alignment-turtle-help-plus').attr('src', 'images/turtle_right_less.png');
			$('#alignment-turtle-value').show();
			$('#alignment-turtle-help-angle').show();
			$('#alignment-turtle-distance').hide();
			$('#alignment-turtle-help-distance').hide();
			$('#alignment-turtle-message').show();
			$('#alignment-start').show();
			$('#alignment-stop').hide();
			$('#alignment-save').show();
			if(this.controller) this.controller.resetAlignment();
		}
	};
	alignments['cheese'] = {
		activate: function(route) {
			ui.updateMessage('alignment-cheese');
			$('#alignment-start').hide();
			$('#alignment-stop').hide();
			$('#alignment-save').hide();
		},
		deactivate: function() {
		}
	};
	monitors['hamster'] = [
		{
			range: [0, 150],
			sub: [50, 100],
			base: 200,
			scale: 200.0/150,
			sensors: [
				{ literal: 'left proximity', device: 'leftProximity', color: '#D32F2F' },
				{ literal: 'right proximity', device: 'rightProximity', color: '#303F9F' },
				{ literal: 'left floor', device: 'leftFloor', color: '#7B1FA2' },
				{ literal: 'right floor', device: 'rightFloor', color: '#0097A7' }
			]
		},
		{
			range: [-32768, 32768],
			sub: [-16384, 16384],
			base: 100,
			scale: 200.0/65536,
			sensors: [
				{ literal: 'x acceleration', device: 'accelerationX', color: '#D32F2F' },
				{ literal: 'y acceleration', device: 'accelerationY', color: '#0097A7' },
				{ literal: 'z acceleration', device: 'accelerationZ', color: '#303F9F' }
			]
		},
		{
			range: [0, 1000],
			sub: [250, 500, 750],
			base: 200,
			scale: 200.0/1000,
			sensors: [
				{ literal: 'light', device: 'light', color: '#455A64' }
			]
		},
		{
			range: [-100, 100],
			sub: [-50, 50],
			base: 100,
			scale: 200.0/200,
			sensors: [
				{ literal: 'signal strength', device: 'signalStrength', color: '#E64A19' },
				{ literal: 'temperature', device: 'temperature', color: '#512DA8' }
			]
		},
		{
			range: [0, 250],
			sub: [50, 100, 150, 200],
			base: 200,
			scale: 200.0/250,
			sensors: [
				{ literal: 'input A', device: 'inputA', color: '#D32F2F' },
				{ literal: 'input B', device: 'inputB', color: '#303F9F' }
			]
		}
	];
	monitors['hamsterS'] = [
		{
			range: [0, 150],
			sub: [50, 100],
			base: 200,
			scale: 200.0/150,
			sensors: [
				{ literal: 'left proximity', device: 'leftProximity', color: '#D32F2F' },
				{ literal: 'right proximity', device: 'rightProximity', color: '#303F9F' },
				{ literal: 'left floor', device: 'leftFloor', color: '#7B1FA2' },
				{ literal: 'right floor', device: 'rightFloor', color: '#0097A7' }
			]
		},
		{
			range: [-32768, 32768],
			sub: [-16384, 16384],
			base: 100,
			scale: 200.0/65536,
			sensors: [
				{ literal: 'x acceleration', device: 'accelerationX', color: '#D32F2F' },
				{ literal: 'y acceleration', device: 'accelerationY', color: '#0097A7' },
				{ literal: 'z acceleration', device: 'accelerationZ', color: '#303F9F' }
			]
		},
		{
			range: [0, 1000],
			sub: [250, 500, 750],
			base: 200,
			scale: 200.0/1000,
			sensors: [
				{ literal: 'light', device: 'light', color: '#455A64' }
			]
		},
		{
			range: [-100, 100],
			sub: [-50, 50],
			base: 100,
			scale: 200.0/200,
			sensors: [
				{ literal: 'signal strength', device: 'signalStrength', color: '#E64A19' },
				{ literal: 'temperature', device: 'temperature', color: '#512DA8' }
			]
		},
		{
			range: [0, 250],
			sub: [50, 100, 150, 200],
			base: 200,
			scale: 200.0/250,
			sensors: [
				{ literal: 'input A', device: 'inputA', color: '#D32F2F' },
				{ literal: 'input B', device: 'inputB', color: '#303F9F' }
			]
		}
	];
	monitors['turtle'] = [
		{
			range: [0, 150],
			sub: [50, 100],
			base: 200,
			scale: 200.0/150,
			sensors: [
				{ literal: 'floor', device: 'floor', color: '#7B1FA2' }
			]
		},
		{
			range: [-3, 9],
			sub: [3, 6],
			base: 150,
			scale: 200.0/12,
			sensors: [
				{ literal: 'button', device: 'button', color: '#D32F2F' },
				{ literal: 'color number', device: 'colorNumber', color: '#303F9F' }
			]
		},
		{
			range: [-32768, 32768],
			sub: [-16384, 16384],
			base: 100,
			scale: 200.0/65536,
			sensors: [
				{ literal: 'x acceleration', device: 'accelerationX', color: '#D32F2F' },
				{ literal: 'y acceleration', device: 'accelerationY', color: '#0097A7' },
				{ literal: 'z acceleration', device: 'accelerationZ', color: '#303F9F' }
			]
		},
		{
			range: [-100, 100],
			sub: [-50, 50],
			base: 100,
			scale: 200.0/200,
			sensors: [
				{ literal: 'signal strength', device: 'signalStrength', color: '#E64A19' },
				{ literal: 'temperature', device: 'temperature', color: '#512DA8' }
			]
		}
	];
	monitors['cheese'] = [
		{
			range: [-32768, 32768],
			sub: [-16384, 16384],
			base: 100,
			scale: 200.0/65536,
			sensors: [
				{ literal: 'x acceleration', device: 'accelerationX', color: '#D32F2F' },
				{ literal: 'y acceleration', device: 'accelerationY', color: '#0097A7' },
				{ literal: 'z acceleration', device: 'accelerationZ', color: '#303F9F' }
			]
		},
		{
			range: [-1, 1],
			sub: [],
			base: 100,
			scale: 200.0/2,
			sensors: [
				{ literal: 'free fall', device: 'freeFall', color: '#D32F2F' },
				{ literal: 'tap', device: 'tap', color: '#303F9F' }
			]
		},
		{
			range: [-100, 100],
			sub: [-50, 50],
			base: 100,
			scale: 200.0/200,
			sensors: [
				{ literal: 'signal strength', device: 'signalStrength', color: '#E64A19' }
			]
		},
		{
			range: [0, 65535],
			sub: [20000, 40000],
			base: 200,
			scale: 200.0/65535,
			sensors: [
				{ literal: 'step count', device: 'stepCount', color: '#7B1FA2' }
			]
		},
		{
			range: [0, 2],
			sub: [1],
			base: 200,
			scale: 200.0/2,
			sensors: [
				{ literal: 'battery state', device: 'batteryState', color: '#303F9F' }
			]
		},
		{
			range: [0, 255],
			sub: [100, 200],
			base: 200,
			scale: 200.0/255,
			sensors: [
				{ literal: 'Sa', device: 'inputSA', color: '#D32F2F' },
				{ literal: 'Sb', device: 'inputSB', color: '#0097A7' },
				{ literal: 'Sc', device: 'inputSC', color: '#303F9F' }
			]
		},
		{
			range: [0, 255],
			sub: [100, 200],
			base: 200,
			scale: 200.0/255,
			sensors: [
				{ literal: 'La', device: 'inputLA', color: '#D32F2F' },
				{ literal: 'Lb', device: 'inputLB', color: '#0097A7' },
				{ literal: 'Lc', device: 'inputLC', color: '#303F9F' }
			]
		},
		{
			range: [0, 65535],
			sub: [20000, 40000],
			base: 200,
			scale: 200.0/65535,
			sensors: [
				{ literal: 'ultrasonic', device: 'ultrasonic', color: '#D32F2F' },
				{ literal: 'encoder', device: 'encoder', color: '#303F9F' }
			]
		},
		{
			range: [0, 100],
			sub: [50],
			base: 200,
			scale: 200.0/100,
			sensors: [
				{ literal: 'temperature', device: 'temperature', color: '#D32F2F' },
				{ literal: 'humidity', device: 'humidity', color: '#303F9F' }
			]
		}
	];
	$('#lang').change(function() {
		lang = $(this).val();
		translator.setLanguage(lang);
		ui.updateLanguage();
		var routes = router.getRoutes();
		var route, config, id;
		for(var i in routes) {
			route = routes[i];
			ui.setState(route);
			config = route.config;
			id = ui.getId(route);
			$('#' + id + ' .title').text(translator.translate(config.name) + ' ' + route.id);
		}
		components['monitor'].updateLanguage();
	});
	ui.updateLanguage();
	$('#start-scan .scan-button').click(function() {
		router.startScan(configs);
		ui.showStopScan();
	});
	$('#stop-scan .scan-button').click(function() {
		router.stopScan();
		ui.showStartScan();
	});
	ui.showStopScan();
	components['scratch'] = {
		handler: {
			online: false,
			callback: function(state) {
				var online = components.scratch.handler.online;
				if(state == 'connected') {
					if(online) logger.i('ScratchX connected.');
					else logger.i('Scratch2 connected.');
				} else if(state == 'disconnected') {
					if(online) logger.w('ScratchX disconnected.');
					else logger.w('Scratch2 disconnected.');
				}
			},
			needsRestart: function() {
				return false;
			},
			getServerName: function() {
				if(this.online) return 'websocket';
				else return 'scratch';
			},
			getModule: function() {
				if(router.multi && router.numberOfActiveModules > 1) {
					return 'all';
				} else {
					var route = router.getRepresentativeRoute();
					if(route) return route.module;
				}
			},
			getExtension: function() {
				if(router.multi) {
					if(this.online) return '/scratchx/extension-multi';
					else return '/scratch/extension-multi';
				} else {
					if(this.online) return '/scratchx/extension';
					else return '/scratch/extension';
				}
			},
			getPort: function() {
				if(this.online) return 51417;
				else if(router.multi) return 50418;
				else return 50417;
			},
			getCallback: function() {
				return this.callback;
			},
			open: function(level) {
				if(this.online) ui.openScratchX(level);
				else ui.openScratch(level);
			}
		},
		enable: function() {
			var self = this;
			ui.enableProgram('scratch-online', 'scratch-online1.png', 'scratch-online2.png');
			if(!isLinux) ui.enableProgram('scratch-offline', 'scratch-offline1.png', 'scratch-offline2.png');
			var target = $('#scratch-online');
			target.off('click');
			target.click(function() {
				router.multi = false;
				self.handler.online = true;
				ui.updateSingleRobotIcon();
				$('#program-panel').hide();
				$('#scratch-panel').show();
				ui.updateMessage('scratch-online');
				router.setMajorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				router.stopScan();
				ui.showStartScan();
			});
			if(!isLinux) {
				target = $('#scratch-offline');
				target.off('click');
				target.click(function() {
					router.multi = false;
					self.handler.online = false;
					ui.updateSingleRobotIcon();
					$('#program-panel').hide();
					$('#scratch-panel').show();
					ui.updateMessage('scratch-offline');
					router.setMajorExtraButtonActive(false);
					ui.updateIdentities();
					ui.updateChangeRobots();
					router.stopScan();
					ui.showStartScan();
				});
			}
			target = $('#scratch-back');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#program-panel').show();
				$('#scratch-panel').hide();
				$('#scratch-single img').attr('src', 'images/robot-single.png');
				$('#scratch-multi img').attr('src', 'images/robot-multi.png');
				$('#scratch-level1').hide();
				$('#scratch-level2').hide();
				$('#scratch-level3').hide();
				router.setMajorExtraButtonActive(true);
				ui.updateIdentities();
				ui.updateChangeRobots();
				ui.updateMessage('');
				server.stop();
			});
			target = $('#scratch-single');
			target.off('click');
			target.click(function() {
				router.multi = false;
				router.reset();
				ui.updateSingleRobotIcon();
				$('#scratch-single img').attr('src', 'images/robot-single-checked.png');
				$('#scratch-multi img').attr('src', 'images/robot-multi.png');
				$('#scratch-level1').show();
				$('#scratch-level2').show();
				$('#scratch-level3').show();
				server.start(self.handler);
			});
			target = $('#scratch-multi');
			target.off('click');
			target.click(function() {
				router.multi = true;
				router.reset();
				ui.updateAllRobotIcons();
				$('#scratch-single img').attr('src', 'images/robot-single.png');
				$('#scratch-multi img').attr('src', 'images/robot-multi-checked.png');
				$('#scratch-level1').show();
				$('#scratch-level2').show();
				$('#scratch-level3').show();
				server.start(self.handler);
			});
			target = $('#scratch-level1');
			target.off('click');
			target.click(function() {
				self.handler.open(1);
			});
			target = $('#scratch-level2');
			target.off('click');
			target.click(function() {
				self.handler.open(2);
			});
			target = $('#scratch-level3');
			target.off('click');
			target.click(function() {
				self.handler.open(3);
			});
		},
		disable: function() {
			ui.disableProgram('scratch-online', 'scratch-online1.png');
			ui.disableProgram('scratch-offline', 'scratch-offline1.png');
			$('#scratch-online').off('click');
			$('#scratch-offline').off('click');
			$('#scratch-panel').hide();
			$('#scratch-single img').attr('src', 'images/robot-single.png');
			$('#scratch-multi img').attr('src', 'images/robot-multi.png');
			$('#scratch-level1').hide();
			$('#scratch-level2').hide();
			$('#scratch-level3').hide();
		}
	};
	components['scratch3'] = {
		option: {
			level: 3
		},
		handler: {
			//online: false,
			callback: function(state) {
				//var online = components.scratch3.handler.online;
				if(state == 'connected') {
					//if(online) logger.i('Scratch3-online connected.');
					//else
					logger.i('Scratch3-offline connected.');
				} else if(state == 'disconnected') {
					//if(online) logger.w('Scratch3-online disconnected.');
					//else
					logger.w('Scratch3-offline disconnected.');
				}
			},
			needsRestart: function() {
				return false;
			},
			getServerName: function() {
				return 'websocket';
			},
			getModule: function() {
				return 'all';
			},
			getExtension: function() {
				//if(this.online) return '/scratch3/extension-online';
				//else
				return '/scratch3/extension-offline';
			},
			getPort: function() {
				return 55417;
			},
			getCallback: function() {
				return this.callback;
			},
			open: function(level) {
				//if(this.online) ui.openScratch3(level);
				//else
				ui.openScratch3Offline();
				var option = components.scratch3.option;
				option.level = level;
				server.setOption(option);
			}
		},
		enable: function() {
			var self = this;
			//ui.enableProgram('scratch3-online', 'scratch3-online1.png', 'scratch3-online2.png');
			if(!isLinux) ui.enableProgram('scratch3-offline', 'scratch3-offline1.png', 'scratch3-offline2.png');
			var target;
			/*var target = $('#scratch3-online');
			target.off('click');
			target.click(function() {
				router.multi = false;
				self.handler.online = true;
				ui.updateSingleRobotIcon();
				$('#program-panel').hide();
				$('#scratch3-panel').show();
				ui.updateMessage('scratch3-online');
				router.setMajorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				router.stopScan();
				ui.showStartScan();
			});*/
			if(!isLinux) {
				target = $('#scratch3-offline');
				target.off('click');
				target.click(function() {
					router.multi = false;
					//self.handler.online = false;
					ui.updateSingleRobotIcon();
					$('#program-panel').hide();
					$('#scratch3-panel').show();
					ui.updateMessage('scratch3-offline');
					router.setMajorExtraButtonActive(false);
					ui.updateIdentities();
					ui.updateChangeRobots();
					router.stopScan();
					ui.showStartScan();
				});
			}
			target = $('#scratch3-back');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#program-panel').show();
				$('#scratch3-panel').hide();
				$('#scratch3-single img').attr('src', 'images/robot-single.png');
				$('#scratch3-multi img').attr('src', 'images/robot-multi.png');
				$('#scratch3-level1').hide();
				$('#scratch3-level2').hide();
				$('#scratch3-level3').hide();
				router.setMajorExtraButtonActive(true);
				ui.updateIdentities();
				ui.updateChangeRobots();
				ui.updateMessage('');
				server.stop();
			});
			target = $('#scratch3-single');
			target.off('click');
			target.click(function() {
				router.multi = false;
				router.reset();
				ui.updateSingleRobotIcon();
				$('#scratch3-single img').attr('src', 'images/robot-single-checked.png');
				$('#scratch3-multi img').attr('src', 'images/robot-multi.png');
				$('#scratch3-level1').show();
				$('#scratch3-level2').show();
				$('#scratch3-level3').show();
				server.start(self.handler, self.option);
				server.setOption(self.option);
				server.setState(router.getRepresentativeRoute());
			});
			target = $('#scratch3-multi');
			target.off('click');
			target.click(function() {
				router.multi = true;
				router.reset();
				ui.updateAllRobotIcons();
				$('#scratch3-single img').attr('src', 'images/robot-single.png');
				$('#scratch3-multi img').attr('src', 'images/robot-multi-checked.png');
				$('#scratch3-level1').show();
				$('#scratch3-level2').show();
				$('#scratch3-level3').show();
				server.start(self.handler, self.option);
				server.setOption(self.option);
				server.setState(router.getRepresentativeRoute());
			});
			target = $('#scratch3-level1');
			target.off('click');
			target.click(function() {
				self.handler.open(1);
			});
			target = $('#scratch3-level2');
			target.off('click');
			target.click(function() {
				self.handler.open(2);
			});
			target = $('#scratch3-level3');
			target.off('click');
			target.click(function() {
				self.handler.open(3);
			});
		},
		disable: function() {
			//ui.disableProgram('scratch3-online', 'scratch3-online1.png');
			ui.disableProgram('scratch3-offline', 'scratch3-offline1.png');
			//$('#scratch3-online').off('click');
			$('#scratch3-offline').off('click');
			$('#scratch3-panel').hide();
			$('#scratch3-single img').attr('src', 'images/robot-single.png');
			$('#scratch3-multi img').attr('src', 'images/robot-multi.png');
			$('#scratch3-level1').hide();
			$('#scratch3-level2').hide();
			$('#scratch3-level3').hide();
		},
		update: function() {
			server.setOption(this.option);
		}
	};
	components['entry'] = {
		handler: {
			online: false,
			callback: function(state) {
				var online = components.entry.handler.online;
				if(state == 'connected') {
					if(online) logger.i('Entry-online connected.');
					else logger.i('Entry-offline connected.');
				} else if(state == 'disconnected') {
					if(online) logger.w('Entry-online disconnected.');
					else logger.w('Entry-offline disconnected.');
				}
			},
			needsRestart: function() {
				return false;
			},
			getServerName: function() {
				if(this.online) return 'entry-online';
				else return 'entry-offline-new';
			},
			getModule: function() {
				return 'all';
			},
			getExtension: function() {
				if(router.multi) {
					if(this.online) return '/entry/extension-online-multi';
					else return '/entry/extension-offline-new-multi';
				} else {
					if(this.online) return '/entry/extension-online';
					else return '/entry/extension-offline-new';
				}
			},
			getPort: function() {
				return 23518;
			},
			getCallback: function() {
				return this.callback;
			},
			open: function() {
				if(this.online) ui.openEntryOnline();
				else ui.openEntryOffline();
			}
		},
		enable: function() {
			var self = this;
			ui.enableProgram('entry-online', 'entry-online1.png', 'entry-online2.png');
			if(!isLinux) ui.enableProgram('entry-offline', 'entry-offline1.png', 'entry-offline2.png');
			var target = $('#entry-online');
			target.off('click');
			target.click(function() {
				router.multi = false;
				self.handler.online = true;
				ui.updateSingleRobotIcon();
				$('#program-panel').hide();
				$('#entry-panel').show();
				ui.updateMessage('entry-online');
				router.setMajorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				router.stopScan();
				ui.showStartScan();
			});
			if(!isLinux) {
				target = $('#entry-offline');
				target.off('click');
				target.click(function() {
					router.multi = false;
					self.handler.online = false;
					ui.updateSingleRobotIcon();
					$('#program-panel').hide();
					$('#entry-panel').show();
					ui.updateMessage('entry-offline');
					router.setMajorExtraButtonActive(false);
					ui.updateIdentities();
					ui.updateChangeRobots();
					router.stopScan();
					ui.showStartScan();
				});
			}
			target = $('#entry-back');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#program-panel').show();
				$('#entry-panel').hide();
				$('#entry-single img').attr('src', 'images/robot-single.png');
				$('#entry-multi img').attr('src', 'images/robot-multi.png');
				$('#entry-basic').hide();
				router.setMajorExtraButtonActive(true);
				ui.updateIdentities();
				ui.updateChangeRobots();
				ui.updateMessage('');
				server.stop();
			});
			target = $('#entry-single');
			target.off('click');
			target.click(function() {
				router.multi = false;
				router.reset();
				ui.updateSingleRobotIcon();
				$('#entry-single img').attr('src', 'images/robot-single-checked.png');
				$('#entry-multi img').attr('src', 'images/robot-multi.png');
				$('#entry-basic').show();
				$('#entry-extension').show();
				server.start(self.handler);
			});
			target = $('#entry-multi');
			target.off('click');
			target.click(function() {
				router.multi = true;
				router.reset();
				ui.updateAllRobotIcons();
				$('#entry-single img').attr('src', 'images/robot-single.png');
				$('#entry-multi img').attr('src', 'images/robot-multi-checked.png');
				$('#entry-basic').show();
				$('#entry-extension').show();
				server.start(self.handler);
			});
			target = $('#entry-basic');
			target.off('click');
			target.click(function() {
				self.handler.open();
			});
		},
		disable: function() {
			ui.disableProgram('entry-online', 'entry-online1.png');
			ui.disableProgram('entry-offline', 'entry-offline1.png');
			$('#entry-online').off('click');
			$('#entry-offline').off('click');
			$('#entry-panel').hide();
			$('#entry-single img').attr('src', 'images/robot-single.png');
			$('#entry-multi img').attr('src', 'images/robot-multi.png');
			$('#entry-basic').hide();
		}
	};
	components['playbot'] = {
		handler: {
			callback: function(state) {
				if(state == 'connected') {
					logger.i('PlayBot connected.');
				} else if(state == 'disconnected') {
					logger.w('PlayBot disconnected.');
					router.setMajorExtraButtonActive(true);
					ui.updateIdentities();
					ui.updateChangeRobots();
				}
			},
			needsRestart: function() {
				return false;
			},
			getServerName: function() {
				return 'websocket';
			},
			getModule: function() {
				return 'all';
			},
			getExtension: function() {
				return '/playbot/extension';
			},
			getPort: function() {
				return 53417;
			},
			getCallback: function() {
				return this.callback;
			}
		},
		enable: function() {
			var self = this;
			ui.enableProgram('playbot-online', 'playbot-online1.png', 'playbot-online2.png');
			var target = $('#playbot-online');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				ui.updateMessage('playbot-online');
				router.setMajorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				router.stopScan();
				ui.showStartScan();
				server.start(self.handler);
				ui.openPlayBot();
			});
		},
		disable: function() {
			ui.disableProgram('playbot-online', 'playbot-online.png');
			$('#playbot-online').off('click');
		}
	};
	components['javascript'] = {
		handler: {
			callback: function(state) {
				if(state == 'connected') {
					logger.i('JavaScript connected.');
				} else if(state == 'disconnected') {
					logger.w('JavaScript disconnected.');
				}
			},
			needsRestart: function() {
				return false;
			},
			getServerName: function() {
				return 'websocket';
			},
			getModule: function() {
				return 'all';
			},
			getExtension: function() {
				return '/websocket/extension';
			},
			getPort: function() {
				return 54417;
			},
			getCallback: function() {
				return this.callback;
			}
		},
		enable: function() {
			var self = this;
			ui.enableProgram('javascript');
			var target = $('#javascript');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#program-panel').hide();
				$('#js-panel').show();
				ui.updateMessage('javascript');
				router.setMajorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				router.stopScan();
				ui.showStartScan();
			});
			target = $('#js-back');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#program-panel').show();
				$('#js-panel').hide();
				$('#js-single img').attr('src', 'images/robot-single.png');
				$('#js-multi img').attr('src', 'images/robot-multi.png');
				router.setMajorExtraButtonActive(true);
				ui.updateIdentities();
				ui.updateChangeRobots();
				ui.updateMessage('');
				server.stop();
				if(ui.winJavaScript) {
					ui.winJavaScript.close();
					ui.winJavaScript = undefined;
				}
			});
			target = $('#js-single');
			target.off('click');
			target.click(function() {
				router.multi = false;
				router.reset();
				ui.updateSingleRobotIcon();
				$('#js-single img').attr('src', 'images/robot-single-checked.png');
				$('#js-multi img').attr('src', 'images/robot-multi.png');
				server.start(self.handler);
				ui.openJavaScript();
			});
			target = $('#js-multi');
			target.off('click');
			target.click(function() {
				router.multi = true;
				router.reset();
				ui.updateAllRobotIcons();
				$('#js-single img').attr('src', 'images/robot-single.png');
				$('#js-multi img').attr('src', 'images/robot-multi-checked.png');
				server.start(self.handler);
				ui.openJavaScript();
			});
		},
		disable: function() {
			ui.disableProgram('javascript');
			$('#javascript').off('click');
			$('#js-panel').hide();
			$('#js-single img').attr('src', 'images/robot-single.png');
			$('#js-multi img').attr('src', 'images/robot-multi.png');
		}
	};
	components['server'] = {
		handler: {
			selectedServer: undefined,
			port: 58000,
			callback: function(state) {
				var server = components.server.handler.selectedServer;
				if(server) {
					if(state == 'connected') {
						logger.i(server + ' client connected.');
					} else if(state == 'disconnected') {
						logger.w(server + ' client disconnected.');
					}
				}
			},
			needsRestart: function() {
				return false;
			},
			getServerName: function() {
				return this.selectedServer;
			},
			getModule: function() {
				return 'all';
			},
			getExtension: function() {
				var server = this.selectedServer;
				if(server) {
					return '/' + server + '/extension';
				}
			},
			getPort: function() {
				return this.port;
			},
			getCallback: function() {
				return this.callback;
			},
			start: function(selection) {
				if(!selection) selection = this.selectedServer;
				if(selection) {
					this.selectedServer = selection;
					try {
						this.port = parseInt($('#server-port-number').val());
						server.start(this);
						ui.updateServerMessage(selection, this.port);
					} catch (e) {
					}
				}
			},
			clear: function() {
				this.selectedServer = undefined;
			}
		},
		enable: function() {
			var self = this;
			ui.enableProgram('server');
			var target = $('#server');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#program-panel').hide();
				$('#server-panel').show();
				$('#server-port-number').val('58000');
				ui.updateMessage('server');
				router.setMajorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				router.stopScan();
				ui.showStartScan();
				self.handler.clear();
			});
			target = $('#server-back');
			target.off('click');
			target.click(function() {
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#program-panel').show();
				$('#server-panel').hide();
				$('#server-single img').attr('src', 'images/robot-single.png');
				$('#server-multi img').attr('src', 'images/robot-multi.png');
				$('#server-http').hide();
				$('#server-websocket').hide();
				$('#server-socketio').hide();
				router.setMajorExtraButtonActive(true);
				ui.updateIdentities();
				ui.updateChangeRobots();
				ui.updateMessage('');
				ui.updateServerMessage('');
				server.stop();
				self.handler.clear();
			});
			target = $('#server-single');
			target.off('click');
			target.click(function() {
				router.multi = false;
				router.reset();
				ui.updateSingleRobotIcon();
				$('#server-single img').attr('src', 'images/robot-single-checked.png');
				$('#server-multi img').attr('src', 'images/robot-multi.png');
				$('#server-http').show();
				$('#server-websocket').show();
//				$('#server-socketio').show();
				self.handler.start();
			});
			target = $('#server-multi');
			target.off('click');
			target.click(function() {
				router.multi = true;
				router.reset();
				ui.updateAllRobotIcons();
				$('#server-single img').attr('src', 'images/robot-single.png');
				$('#server-multi img').attr('src', 'images/robot-multi-checked.png');
				$('#server-http').show();
				$('#server-websocket').show();
//				$('#server-socketio').show();
				self.handler.start();
			});
			target = $('#server-http');
			target.off('click');
			target.click(function() {
				self.handler.start('http');
			});
			target = $('#server-websocket');
			target.off('click');
			target.click(function() {
				self.handler.start('websocket');
			});
			target = $('#server-socketio');
			target.off('click');
			target.click(function() {
				self.handler.start('socketio');
			});
		},
		disable: function() {
			ui.disableProgram('server');
			$('#server').off('click');
			$('#server-panel').hide();
			$('#server-single img').attr('src', 'images/robot-single.png');
			$('#server-multi img').attr('src', 'images/robot-multi.png');
			$('#server-http').hide();
			$('#server-websocket').hide();
			$('#server-socketio').hide();
			ui.updateServerMessage('');
		}
	};
	components['alignment'] = {
		enable: function() {
			var self = this;
			ui.enableProgram('alignment');
			var target = $('#alignment');
			target.off('click');
			target.click(function() {
				self.multi = router.multi;
				router.multi = false;
				ui.updateSingleRobotIcon();
				$('#alignment').hide();
				$('#alignment-panel').show();
				router.stopScan();
				ui.showStartScan();
				self.shown = true;
				var current = self.current;
				if(current) {
					$('#alignment-panel-' + current.module).show();
					alignments[current.module].activate(current.route);
				}
				router.setMinorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
			});
			target = $('#alignment-close');
			target.off('click');
			target.click(function() {
				if(self.multi) {
					router.multi = true;
					ui.updateAllRobotIcons();
				} else {
					router.multi = false;
					ui.updateSingleRobotIcon();
				}
				$('#alignment').show();
				$('#alignment-panel').hide();
				ui.updateMessage('alignment-close');
				self.shown = false;
				if(self.current) {
					var module = self.current.module;
					$('#alignment-panel-' + module).hide();
					alignments[module].deactivate();
				}
				router.setMinorExtraButtonActive(true);
				ui.updateIdentities();
				ui.updateChangeRobots();
			});
		},
		disable: function() {
			ui.disableTopMenu('alignment');
			$('#alignment').off('click');
			$('#alignment').show();
			$('#alignment-panel').hide();
			this.shown = false;
			if(this.current) {
				var module = this.current.module;
				$('#alignment-panel-' + module).hide();
				alignments[module].deactivate();
				this.current = undefined;
			}
		},
		update: function(route) {
			var current = this.current;
			if(current && route.hwModule == current.module && route.address == current.address && route == current.route) return;
			if(this.shown) {
				if(current) {
					if(current.module != route.hwModule) {
						$('#alignment-panel-' + current.module).hide();
						$('#alignment-panel-' + route.hwModule).show();
					}
					alignments[current.module].deactivate();
				} else {
					$('#alignment-panel-' + route.hwModule).show();
				}
				alignments[route.hwModule].activate(route);
			}
			this.current = {
				module: route.hwModule,
				address: route.address,
				route: route
			};
		},
		check: function() {
			var route = router.selectedRoute;
			if(route) {
				this.enable();
				this.update(route);
			} else {
				this.disable();
			}
		}
	};
	components['setting'] = {
		selected: 'hamster',
		enable: function() {
			var self = this;
			var target = $('#setting');
			target.off('click');
			target.click(function() {
//				self.multi = router.multi;
//				router.multi = false;
//				ui.updateSingleRobotIcon();
				ui.updateMessage('setting');
				$('#setting').hide();
				$('#setting-panel').show();
//				router.stopScan();
//				ui.showStartScan();
				var settings = context.getSettings();
				var setting, box;
				for(var i in settings) {
					setting = settings[i];
					for(var j in setting) {
						box = $('#setting-' + i + '-' + j + '-value');
						if(box) box.val(setting[j]);
					}
				}
				router.setMinorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				self.update(router.selectedRoute);
			});
			target = $('#setting-close');
			target.off('click');
			target.click(function() {
				self.close();
			});
			target = $('#setting-cancel');
			target.off('click');
			target.click(function() {
				self.close();
			});
			target = $('#setting-save');
			target.off('click');
			target.click(function() {
				var settings = context.getSettings();
				var setting, box;
				for(var i in settings) {
					setting = settings[i];
					for(var j in setting) {
						box = $('#setting-' + i + '-' + j + '-value');
						if(box) {
							context.setSetting(i, j, box.val());
							box.val(setting[j]);
						}
					}
				}
				context.saveSettings();
				self.close();
			});
			target = $('#setting-robot .combo');
			target.off('change');
			target.change(function() {
				var selection = $(this).val();
				$('#setting-' + self.selected).hide();
				$('#setting-' + selection).show();
				self.selected = selection;
			});
		},
		close: function() {
//			if(this.multi) {
//				router.multi = true;
//				ui.updateAllRobotIcons();
//			} else {
//				router.multi = false;
//				ui.updateSingleRobotIcon();
//			}
			ui.updateMessage('setting-close');
			$('#setting').show();
			$('#setting-panel').hide();
			router.setMinorExtraButtonActive(true);
			ui.updateIdentities();
			ui.updateChangeRobots();
		},
		update: function(route) {
			if(route && this.selected != route.module) {
				$('#setting-' + this.selected).hide();
				$('#setting-' + route.module).show();
				this.selected = route.module;
				$('#setting-robot .combo').val(this.selected);
			}
		},
		check: function() {
			this.update(router.selectedRoute);
		}
	};
	const GRAPH_LEFT = 50;
	const GRAPH_TOP = 60;
	const GRAPH_BOTTOM = 260;
	const TIME_AXIS = {
		0: { unit: 'secs', label: ['0.5', '1', '1.5', '2', '2.5'] },
		10: { unit: 'secs', label: ['5', '10', '15', '20', '25'] },
		20: { unit: 'secs', label: ['10', '20', '30', '40', '50'] },
		40: { unit: 'secs', label: ['20', '40', '60', '80', '100'] },
		80: { unit: 'secs', label: ['40', '80', '120', '160', '200'] },
		200: { unit: 'secs', label: ['100', '200', '300', '400', '500'] },
		400: { unit: 'secs', label: ['200', '400', '600', '800', '1000'] },
		800: { unit: 'secs', label: ['400', '800', '1200', '1600', '2000'] },
		2400: { unit: 'mins', label: ['20', '40', '60', '80', '100'] }
	};
	
	components['monitor'] = {
		graph: undefined,
		context: undefined,
		timeUnit: 'secs',
		averageSelection: 0,
		sensorSelection: 0,
		position: [ [ 16, 16], [166, 16], [16, 36], [166, 36] ],
		enable: function() {
			var self = this;
			ui.enableProgram('monitor');
			var target = $('#monitor');
			target.off('click');
			target.click(function() {
				self.multi = router.multi;
				router.multi = false;
				ui.updateSingleRobotIcon();
				ui.updateMessage('monitor');
				self.averageSelection = 0;
				$('#monitor-robot .combo').val(0);
				$('#monitor').hide();
				$('#monitor-panel').show();
				$('#monitor-pause').show();
				$('#monitor-resume').hide();
				router.setMinorExtraButtonActive(false);
				ui.updateIdentities();
				ui.updateChangeRobots();
				router.stopScan();
				ui.showStartScan();
				self.hoverX = -1;
				self.shown = true;
				self.running = true;
				self.graphData = undefined;
				self.dumpData = undefined;
				self.updateRobot();
				self.updateLanguage();
				self.graph = document.getElementById('graph');
				self.context = self.graph.getContext('2d');
				self.requestFrame();
				
				self.graph.addEventListener('mousemove', function(event) {
					var rect = self.graph.getBoundingClientRect();
					self.hoverX = event.clientX - rect.left;
				});
				self.graph.addEventListener('mouseout', function(event) {
					self.hoverX = -1;
				});
			});
			target = $('#monitor-close');
			target.off('click');
			target.click(function() {
				if(self.multi) {
					router.multi = true;
					ui.updateAllRobotIcons();
				} else {
					router.multi = false;
					ui.updateSingleRobotIcon();
				}
				$('#monitor').show();
				$('#monitor-panel').hide();
				ui.updateMessage('monitor-close');
				self.hoverX = -1;
				self.shown = false;
				self.running = false;
				self.graphData = undefined;
				self.dumpData = undefined;
				router.setMinorExtraButtonActive(true);
				ui.updateIdentities();
				ui.updateChangeRobots();
			});
			target = $('#monitor-robot .combo');
			target.off('change');
			target.change(function() {
				var selection = $(this).val();
				self.averageSelection = parseInt(selection);
				self.timeUnit = translator.translate(TIME_AXIS[self.averageSelection].unit);
				if(self.route) {
					self.route.controller.setDuration(self.averageSelection);
				}
				self.resume();
				self.cancelSave();
			});
			target = $('#monitor-sensor .combo');
			target.off('change');
			target.change(function() {
				var selection = $(this).val();
				self.sensorSelection = parseInt(selection);
				self.resume();
				self.cancelSave();
			});
			target = $('#monitor-resume');
			target.off('click');
			target.click(function() {
				self.resume();
			});
			target = $('#monitor-pause');
			target.off('click');
			target.click(function() {
				self.pause();
			});
			target = $('#monitor-save');
			target.off('click');
			target.click(function() {
				var img = self.graph.toDataURL('image/png');
				var data = img.replace(/^data:image\/\w+;base64,/, '');
				self.graphData = new Buffer(data, 'base64');
				if(self.running) {
					self.pause();
				}
				$('#monitor-buttons').hide();
				$('#monitor-save-menu').show();
			});
			target = $('#monitor-save-image');
			target.off('click');
			target.click(function() {
				if(self.graphData) {
					self.chooseFile('#saveDialogImage', function(path) {
						if(path) {
							var fs = require('fs');
							fs.writeFile(path, self.graphData, function() {
								self.graphData = undefined;
								$('#monitor-save-menu').hide();
								$('#monitor-buttons').show();
							});
						}
					});
				}
			});
			target = $('#monitor-save-text');
			target.off('click');
			target.click(function() {
				if(self.dumpData) {
					self.chooseFile('#saveDialogText', function(path) {
						if(path) {
							var data = self.dumpData;
							if(data && data.length > 0) {
								var text = 'seconds', t = 0.025;
								if(self.averageSelection > 0) t = self.averageSelection / 40.0;
								var values = data[0];
								for(var i = 0, j = 0; i < values.length; ++i, j += t) {
									text += ' ' + (parseInt(j * 1000 + 0.5) / 1000.0);
								}
								text += '\r\n';
								var sensors = monitors[self.route.module][self.sensorSelection].sensors;
								for(var i in data) {
									values = data[i];
									text += sensors[i].device;
									for(var j = 0; j < values.length; ++j) {
										text += ' ' + values[j];
									}
									text += '\r\n';
								}
								var fs = require('fs');
								fs.writeFile(path, text, function() {
									self.graphData = undefined;
									$('#monitor-save-menu').hide();
									$('#monitor-buttons').show();
								});
							}
						}
					});
				}
			});
			target = $('#monitor-save-cancel');
			target.off('click');
			target.click(function() {
				self.cancelSave();
			});
		},
		disable: function() {
			ui.disableTopMenu('monitor');
			$('#monitor').off('click');
			$('#monitor').show();
			$('#monitor-panel').hide();
			this.hoverX = -1;
			this.shown = false;
			this.running = false;
			this.route = undefined;
		},
		copyData: function() {
			if(this.route) {
				var sensorMonitor = this.route.controller.getSensorMonitor();
				var sensors = monitors[this.route.module][this.sensorSelection].sensors;
				var data = new Array();
				var arr, sensor, values;
				for(var i in sensors) {
					arr = new Array();
					sensor = sensors[i];
					values = sensorMonitor[sensor.device].values;
					if(sensorMonitor.count < 100) {
						for(var j = 0; j < sensorMonitor.count; ++j) {
							arr.push(values[j]);
						}
					} else {
						for(var j = sensorMonitor.index; j < 100; ++j) {
							arr.push(values[j]);
						}
						for(var j = 0; j < sensorMonitor.index; ++j) {
							arr.push(values[j]);
						}
					}
					data.push(arr);
				}
				this.dumpData = data;
			}
		},
		resume: function() {
			if(this.running) return;
			this.running = true;
			this.dumpData = undefined;
			$('#monitor-pause').show();
			$('#monitor-resume').hide();
			this.requestFrame();
		},
		pause: function() {
			if(!this.running) return;
			this.running = false;
			this.copyData();
			$('#monitor-pause').hide();
			$('#monitor-resume').show();
		},
		cancelSave: function() {
			this.graphData = undefined;
			$('#monitor-save-menu').hide();
			$('#monitor-buttons').show();
		},
		chooseFile: function(name, callback) {
			var chooser = $(name);
			chooser.unbind('change');
			chooser.change(function(ev) {
				var value = $(this).val();
				$(this).val('');
				if(callback) {
					callback(value);
				}
			});
			chooser.trigger('click');
		},
		requestFrame: function() {
			var self = this;
			requestAnimationFrame(function() {
				try {
					var context = self.context;
					context.fillStyle = '#ffffff';
					context.fillRect(0, 0, self.graph.width, self.graph.height);
					
					context.strokeStyle = '#aaaaaa';
					context.lineWidth = 2;
					context.beginPath();
					context.moveTo(GRAPH_LEFT - 1, GRAPH_TOP);
					context.lineTo(GRAPH_LEFT - 1, GRAPH_BOTTOM);
					context.stroke();
					context.closePath();
					
					context.save();
					context.setLineDash([5, 3]);
					context.lineWidth = 1;
					context.beginPath();
					context.moveTo(GRAPH_LEFT + 40, GRAPH_TOP);
					context.lineTo(GRAPH_LEFT + 40, GRAPH_BOTTOM);
					context.moveTo(GRAPH_LEFT + 80, GRAPH_TOP);
					context.lineTo(GRAPH_LEFT + 80, GRAPH_BOTTOM);
					context.moveTo(GRAPH_LEFT + 120, GRAPH_TOP);
					context.lineTo(GRAPH_LEFT + 120, GRAPH_BOTTOM);
					context.moveTo(GRAPH_LEFT + 160, GRAPH_TOP);
					context.lineTo(GRAPH_LEFT + 160, GRAPH_BOTTOM);
					context.moveTo(GRAPH_LEFT + 200, GRAPH_TOP);
					context.lineTo(GRAPH_LEFT + 200, GRAPH_BOTTOM);
					context.moveTo(GRAPH_LEFT, GRAPH_TOP);
					context.lineTo(GRAPH_LEFT + 216, GRAPH_TOP);
					context.moveTo(GRAPH_LEFT, GRAPH_BOTTOM);
					context.lineTo(GRAPH_LEFT + 216, GRAPH_BOTTOM);
					context.stroke();
					context.closePath();
					context.restore();
					
					context.font = 'bold 13px Arial,Dotum';
					context.fillStyle = '#000000';
					context.textAlign = 'center';
					context.textBaseline = 'middle';
					var y = GRAPH_BOTTOM + 12;
					var label = TIME_AXIS[self.averageSelection].label;
					context.fillText('0', GRAPH_LEFT, y);
					context.fillText(label[0], GRAPH_LEFT + 40, y);
					context.fillText(label[1], GRAPH_LEFT + 80, y);
					context.fillText(label[2], GRAPH_LEFT + 120, y);
					context.fillText(label[3], GRAPH_LEFT + 160, y);
					context.fillText(label[4], GRAPH_LEFT + 200, y);
					context.fillText(self.timeUnit, GRAPH_LEFT + 232, y);
					
					if(self.route) {
						var target = monitors[self.route.module][self.sensorSelection];
						
						context.textAlign = 'right';
						context.textBaseline = 'middle';
						context.save();
						context.setLineDash([5, 3]);
						context.lineWidth = 1;
						context.beginPath();
						var base = GRAPH_TOP + target.base;
						var sub = target.sub, y;
						for(var i in sub) {
							y = base - sub[i] * target.scale;
							context.moveTo(GRAPH_LEFT, y);
							context.lineTo(GRAPH_LEFT + 216, y);
							context.fillText(sub[i], GRAPH_LEFT - 8, y);
						}
						context.stroke();
						context.closePath();
						context.restore();
						
						context.lineWidth = 2;
						context.beginPath();
						context.moveTo(GRAPH_LEFT, base);
						context.lineTo(GRAPH_LEFT + 216, base);
						context.stroke();
						context.closePath();
						
						context.fillText(target.range[1], GRAPH_LEFT - 8, GRAPH_TOP);
						context.fillText(target.range[0], GRAPH_LEFT - 8, GRAPH_BOTTOM);
						if(target.range[0] < 0) context.fillText('0', GRAPH_LEFT - 8, base);
						
						context.textAlign = 'left';
						var sensorMonitor = self.route.controller.getSensorMonitor();
						var sensors = target.sensors;
						var position = self.position
						var sensor, data, values;
						if(self.dumpData) {
							data = self.dumpData;
							for(var i in data) {
								sensor = sensors[i];
								values = data[i];
								context.fillStyle = sensor.color;
								context.fillText(sensor.name + ' ' + values[values.length-1], position[i][0], position[i][1]);
								context.fillText(sensor.name, GRAPH_LEFT + 204, base - values[values.length-1] * target.scale);
								context.strokeStyle = sensor.color;
								context.beginPath();
								context.moveTo(GRAPH_LEFT, base - values[0] * target.scale);
								for(var j in values) {
									context.lineTo(GRAPH_LEFT + j * 2, base - values[j] * target.scale);
								}
								context.stroke();
								context.closePath();
							}
						} else {
							for(var i in sensors) {
								sensor = sensors[i];
								data = sensorMonitor[sensor.device];
								values = data.values;
								context.fillStyle = sensor.color;
								context.fillText(sensor.name + ' ' + data.current, position[i][0], position[i][1]);
								context.fillText(sensor.name, GRAPH_LEFT + 204, base - data.current * target.scale);
								context.strokeStyle = sensor.color;
								context.beginPath();
								if(sensorMonitor.count < 100) {
									context.moveTo(GRAPH_LEFT, base - values[0] * target.scale);
									for(var j = 1; j < sensorMonitor.count; ++j) {
										context.lineTo(GRAPH_LEFT + j * 2, base - values[j] * target.scale);
									}
								} else {
									context.moveTo(GRAPH_LEFT, base - values[sensorMonitor.index] * target.scale);
									var index = 1;
									for(var j = sensorMonitor.index + 1; j < 100; ++j) {
										context.lineTo(GRAPH_LEFT + index * 2, base - values[j] * target.scale);
										++ index;
									}
									for(var j = 0; j < sensorMonitor.index; ++j) {
										context.lineTo(GRAPH_LEFT + index * 2, base - values[j] * target.scale);
										++ index;
									}
								}
								context.stroke();
								context.closePath();
							}
						}
						if(self.hoverX >= 0) {
							var numIndex = -1;
							if(self.dumpData) {
								data = self.dumpData;
								if(data.length > 0) {
									numIndex = data[0].length;
								}
							} else {
								numIndex = sensorMonitor.count;
							}
							var x = self.hoverX;
							var rx = x - GRAPH_LEFT;
							var index = Math.round(rx / 2);
							if(index >= 0 && index < numIndex) {
								context.textBaseline = 'middle';
								context.save();
								context.strokeStyle = '#ff7a00';
								context.setLineDash([5, 3]);
								context.lineWidth = 1;
								context.beginPath();
								context.moveTo(x, GRAPH_TOP);
								context.lineTo(x, GRAPH_BOTTOM);
								context.stroke();
								context.closePath();
								context.restore();
								
								y = GRAPH_TOP;
								if(self.dumpData) {
									data = self.dumpData;
									for(var i in data) {
										values = data[i];
										context.fillStyle = sensors[i].color;
										context.fillText(' ' + values[index], x, y);
										y += 16;
									}
								} else {
									for(var i in sensors) {
										sensor = sensors[i];
										values = sensorMonitor[sensor.device].values;
										context.fillStyle = sensor.color;
										context.fillText(' ' + values[(sensorMonitor.index + index) % numIndex], x, y);
										y += 16;
									}
								}
							}
						}
					}
				} catch (e) {
				}
				if(self.shown) {
					self.requestFrame();
				}
			});
		},
		updateRobot: function() {
			if(this.route) {
				var name = this.route.config.name + '[' + this.route.index + ']';
				$('#monitor-robot .name').text(name);
				this.route.controller.setDuration(this.averageSelection);
			}
			this.sensorSelection = 0;
			this.resume();
			this.cancelSave();
		},
		updateLanguage: function() {
			this.timeUnit = translator.translate(TIME_AXIS[this.averageSelection].unit);
			if(this.route) {
				var combo = $('#monitor-sensor .combo');
				var list = this.route.controller.getSensorNames();
				var contents = '';
				for(var i in list) {
					contents += '<option value="' + i + '">' + translator.translate(list[i]) + '</option>';
				}
				combo.html(contents);
				combo.val(this.sensorSelection);
				
				var config = monitors[this.route.module];
				var sensors, sensor;
				for(var i in config) {
					sensors = config[i].sensors;
					for(var j in sensors) {
						sensor = sensors[j];
						sensor.name = translator.translate(sensor.literal);
					}
				}
			}
		},
		update: function(route) {
			this.route = route;
			if(this.shown) {
				this.updateRobot();
				this.updateLanguage();
			}
		},
		check: function() {
			var route = router.selectedRoute;
			if(route) {
				this.enable();
				this.update(route);
			} else {
				this.disable();
			}
		}
	};
	components['console'] = {
		enable: function() {
			ui.enableProgram('console');
			var target = $('#console');
			target.off('click');
			target.click(function() {
				win.showDevTools();
			});
		},
		disable: function() {
			ui.disableTopMenu('console');
			$('#console').off('click');
		}
	};
/*	components['chatting'] = {
		enable: function() {
			var target = $('#chatting');
			target.off('click');
			target.click(function() {
				ui.openDashboard();
			});
			dashboard.setOpen(false);
		}
	};*/
	components['console'].enable();
//	components['chatting'].enable();
	
	win.on('close', function() {
		this.hide();
		router.close();
		if(ui.winScratchX) {
			ui.winScratchX.close(true);
			ui.winScratchX = undefined;
		}
		if(ui.winEntryOnline) {
			ui.winEntryOnline.close(true);
			ui.winEntryOnline = undefined;
		}
		//if(ui.winSnap) {
		//	ui.winSnap.close(true);
		//	ui.winSnap = undefined;
		//}
		if(ui.winPlaybot) {
			ui.winPlaybot.close(true);
			ui.winPlaybot = undefined;
		}
		if(ui.winJavaScript) {
			ui.winJavaScript.close(true);
			ui.winJavaScript = undefined;
		}
/*		if(ui.winDashboard) {
			ui.winDashboard.close(true);
			ui.winDashboard = undefined;
		}*/
		translator.clear();
		this.close(true);
	});
	
	// configuration
	var fs = require('fs');
	fs.readdir('./modules', function(error, files) {
		if(error) {
			logger.e(error);
			return;
		}
		files.filter(function(file) {
			return /(?:\.([^.]+))?$/.exec(file)[1] == 'json';
		}).forEach(function(file) {
			try {
				var config = fs.readFileSync('./modules/' + file);
				config = JSON.parse(config);
				configs.push(config);
				context.register(config);
			} catch(e) {}
		});
		try {
			var settings = fs.readFileSync('./modules/setting.cfg');
			settings = JSON.parse(settings);
			var setting;
			for(var i in settings) {
				setting = settings[i];
				for(var j in setting) {
					context.setSetting(i, j, setting[j]);
				}
			}
		} catch(e) {}
		router.startScan(configs);
	});
	
	router.on('found', function(route) {
		if(route) {
			ui.addRobot(route);
			if(router.selectedRoute == undefined) {
				ui.selectRobot(route);
				if(!isLinux32) components['scratch'].enable();
				components['scratch3'].enable();
				components['entry'].enable();
//				components['snap'].enable();
				components['playbot'].enable();
				components['javascript'].enable();
				components['server'].enable();
			} else if(router.multi) {
				ui.updateAllRobotIcons();
				server.restartForMultiRobots();
			}
			components['alignment'].check();
			components['monitor'].check();
			components['setting'].check();
			components['scratch3'].update();
		}
	});
	router.on('update', function(oldModule, oldIndex, route) {
		if(route) {
			var id = oldModule + '_' + oldIndex;
			route.iconFileName = undefined;
			route.iconSelectedFileName = undefined;
			ui.updateRobot(id, route);
			ui.updateIdentity(route);
			ui.updateDongle(route);
			if(router.selectedRoute == undefined) {
				ui.selectRobot(route);
				if(!isLinux32) components['scratch'].enable();
				components['scratch3'].enable();
				components['entry'].enable();
//				components['snap'].enable();
				components['playbot'].enable();
				components['javascript'].enable();
				components['server'].enable();
			} else if(router.multi) {
				server.restartForMultiRobots();
			} else if(route == router.selectedRoute && route.module != oldModule) {
				ui.selectRobot(route);
				server.restartForSingleRobot(route);
				server.setState(route);
			}
			components['alignment'].check();
			components['monitor'].check();
			components['setting'].check();
			components['scratch3'].update();
		}
	});
	router.on('state', function(route) {
		if(route.state == 'disconnected') {
			ui.removeRobot(route);
			if(route == router.selectedRoute) {
				var firstRoute = router.firstRoute;
				if(firstRoute) {
					ui.selectRobot(firstRoute);
					if(router.multi) {
						server.restartForMultiRobots();
					} else if(firstRoute.config.module != route.config.module) {
						server.restartForSingleRobot(firstRoute);
						server.setState(firstRoute);
					}
					route = firstRoute;
				} else {
					server.stop();
					router.selectedRoute = undefined;
					router.multi = false;
					components['scratch'].disable();
					components['scratch3'].disable();
					components['entry'].disable();
//					components['snap'].disable();
					components['playbot'].disable();
					components['javascript'].disable();
					components['server'].disable();
					ui.updateSingleRobotIcon();
					$('#program-panel').show();
					ui.updateMessage('');
					router.setMajorExtraButtonActive(true);
					router.setMinorExtraButtonActive(true);
					ui.updateIdentities();
					ui.updateChangeRobots();
				}
			} else if(router.multi) {
				server.restartForMultiRobots();
			}
			components['alignment'].check();
			components['monitor'].check();
			components['setting'].check();
			components['scratch3'].update();
		} else {
			ui.setState(route);
		}
		server.setState(route);
	});
	router.on('scan-done', function() {
		ui.showStartScan();
	});
	components['setting'].enable();
}());
